"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class JobStatus(str, Enum):
    r"""The status of the job."""

    DRAFT = "draft"
    INTERNAL = "internal"
    PUBLISHED = "published"
    COMPLETED = "completed"
    PLACED = "placed"
    ON_HOLD = "on-hold"
    PRIVATE = "private"
    ACCEPTING_CANDIDATES = "accepting_candidates"
    OPEN = "open"
    CLOSED = "closed"
    ARCHIVED = "archived"
