"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ConnectorStatus(str, Enum):
    r"""Status of the connector. Connectors with status live or beta are callable."""

    LIVE = "live"
    BETA = "beta"
    EARLY_ACCESS = "early-access"
    DEVELOPMENT = "development"
    CONSIDERING = "considering"
