"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


Value5TypedDict = TypeAliasType("Value5TypedDict", Union[str, int, float])


Value5 = TypeAliasType("Value5", Union[str, int, float])


SimpleFormFieldOptionValueTypedDict = TypeAliasType(
    "SimpleFormFieldOptionValueTypedDict",
    Union[str, int, float, bool, List[Value5TypedDict]],
)


SimpleFormFieldOptionValue = TypeAliasType(
    "SimpleFormFieldOptionValue", Union[str, int, float, bool, List[Value5]]
)


class OptionType(str, Enum):
    SIMPLE = "simple"


class SimpleFormFieldOptionTypedDict(TypedDict):
    label: str
    option_type: OptionType
    value: NotRequired[SimpleFormFieldOptionValueTypedDict]


class SimpleFormFieldOption(BaseModel):
    label: str

    option_type: OptionType

    value: Optional[SimpleFormFieldOptionValue] = None
