"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentStatus(str, Enum):
    r"""Status of payment"""

    DRAFT = "draft"
    AUTHORISED = "authorised"
    REJECTED = "rejected"
    PAID = "paid"
    VOIDED = "voided"
    DELETED = "deleted"
