"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CreditOrDebit(str, Enum):
    r"""Whether the amount is a credit or debit."""

    CREDIT = "credit"
    DEBIT = "debit"
