"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.eventlogs import EventLogs
from apideck_unify.webhooks import Webhooks


class WebhookSDK(BaseSDK):
    webhooks: Webhooks
    event_logs: EventLogs

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.webhooks = Webhooks(self.sdk_configuration)
        self.event_logs = EventLogs(self.sdk_configuration)
