"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class FilesSearchTypedDict(TypedDict):
    query: str
    r"""The query to search for. May match across multiple fields."""
    drive_id: NotRequired[str]
    r"""ID of the drive to filter on"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class FilesSearch(BaseModel):
    query: str
    r"""The query to search for. May match across multiple fields."""

    drive_id: Optional[str] = None
    r"""ID of the drive to filter on"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""
