"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class AssigneeInputTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""


class AssigneeInput(BaseModel):
    id: str
    r"""A unique identifier for an object."""
