"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apiresourcecoverage_sdk import APIResourceCoverageSDK
from apideck_unify.apiresources import APIResources
from apideck_unify.apis import Apis
from apideck_unify.connectordocs import ConnectorDocs
from apideck_unify.connectorresources import ConnectorResources
from apideck_unify.connectors import Connectors


class ConnectorSDK(BaseSDK):
    connectors: Connectors
    connector_docs: ConnectorDocs
    connector_resources: ConnectorResources
    apis: Apis
    api_resources: APIResources
    api_resource_coverage: APIResourceCoverageSDK

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.connectors = Connectors(self.sdk_configuration)
        self.connector_docs = ConnectorDocs(self.sdk_configuration)
        self.connector_resources = ConnectorResources(self.sdk_configuration)
        self.apis = Apis(self.sdk_configuration)
        self.api_resources = APIResources(self.sdk_configuration)
        self.api_resource_coverage = APIResourceCoverageSDK(self.sdk_configuration)
