"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EcommerceCustomersFilterTypedDict(TypedDict):
    email: NotRequired[str]
    r"""Customer email address to filter on"""
    phone_number: NotRequired[str]
    r"""Customer phone number to filter on"""


class EcommerceCustomersFilter(BaseModel):
    email: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Customer email address to filter on"""

    phone_number: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Customer phone number to filter on"""
