"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from apideck_unify import models, utils
from apideck_unify._hooks import SDKHooks
from apideck_unify.models import internal
from apideck_unify.types import OptionalNullable, UNSET
import httpx
import importlib
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from apideck_unify.accounting import Accounting
    from apideck_unify.ats import Ats
    from apideck_unify.connector_sdk import ConnectorSDK
    from apideck_unify.crm import Crm
    from apideck_unify.ecommerce import Ecommerce
    from apideck_unify.filestorage import FileStorage
    from apideck_unify.hris import Hris
    from apideck_unify.issuetracking import IssueTracking
    from apideck_unify.sms import Sms
    from apideck_unify.vault import Vault
    from apideck_unify.webhook_sdk import WebhookSDK


class Apideck(BaseSDK):
    r"""Apideck: The Apideck OpenAPI Spec: SDK Optimized
    https://developers.apideck.com - Apideck Developer Docs
    """

    accounting: "Accounting"
    ats: "Ats"
    crm: "Crm"
    ecommerce: "Ecommerce"
    file_storage: "FileStorage"
    hris: "Hris"
    sms: "Sms"
    issue_tracking: "IssueTracking"
    connector: "ConnectorSDK"
    vault: "Vault"
    webhook: "WebhookSDK"
    _sub_sdk_map = {
        "accounting": ("apideck_unify.accounting", "Accounting"),
        "ats": ("apideck_unify.ats", "Ats"),
        "crm": ("apideck_unify.crm", "Crm"),
        "ecommerce": ("apideck_unify.ecommerce", "Ecommerce"),
        "file_storage": ("apideck_unify.filestorage", "FileStorage"),
        "hris": ("apideck_unify.hris", "Hris"),
        "sms": ("apideck_unify.sms", "Sms"),
        "issue_tracking": ("apideck_unify.issuetracking", "IssueTracking"),
        "connector": ("apideck_unify.connector_sdk", "ConnectorSDK"),
        "vault": ("apideck_unify.vault", "Vault"),
        "webhook": ("apideck_unify.webhook_sdk", "WebhookSDK"),
    }

    def __init__(
        self,
        api_key: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key: The api_key required for authentication
        :param consumer_id: Configures the consumer_id parameter for all supported operations
        :param app_id: Configures the app_id parameter for all supported operations
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(api_key=api_key())
        else:
            security = models.Security(api_key=api_key)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        _globals = internal.Globals(
            consumer_id=utils.get_global_from_env(
                consumer_id, "APIDECK_CONSUMER_ID", str
            ),
            app_id=utils.get_global_from_env(app_id, "APIDECK_APP_ID", str),
        )

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                globals=_globals,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = importlib.import_module(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None
