"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.collections import Collections
from apideck_unify.collectiontags import CollectionTags
from apideck_unify.collectionticketcomments import CollectionTicketComments
from apideck_unify.collectiontickets import CollectionTickets
from apideck_unify.collectionusers import CollectionUsers


class IssueTracking(BaseSDK):
    collections: Collections
    collection_tickets: CollectionTickets
    collection_ticket_comments: CollectionTicketComments
    collection_users: CollectionUsers
    collection_tags: CollectionTags

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.collections = Collections(self.sdk_configuration)
        self.collection_tickets = CollectionTickets(self.sdk_configuration)
        self.collection_ticket_comments = CollectionTicketComments(
            self.sdk_configuration
        )
        self.collection_users = CollectionUsers(self.sdk_configuration)
        self.collection_tags = CollectionTags(self.sdk_configuration)
