"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivitiesFilterTypedDict(TypedDict):
    company_id: NotRequired[str]
    r"""Company ID to filter on"""
    owner_id: NotRequired[str]
    r"""Owner ID to filter on"""
    contact_id: NotRequired[str]
    r"""Primary contact ID to filter on"""
    updated_since: NotRequired[datetime]
    type: NotRequired[str]
    r"""Type to filter on"""


class ActivitiesFilter(BaseModel):
    company_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Company ID to filter on"""

    owner_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Owner ID to filter on"""

    contact_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Primary contact ID to filter on"""

    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None

    type: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Type to filter on"""
