"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SortDirection(str, Enum):
    r"""The direction in which to sort the results"""

    ASC = "asc"
    DESC = "desc"
