"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EcommerceProductsFilterTypedDict(TypedDict):
    name: NotRequired[str]
    r"""Product name to filter on"""
    updated_since: NotRequired[str]
    r"""Minimum date the product was last modified"""
    created_since: NotRequired[str]
    r"""Minimum date the product was created"""


class EcommerceProductsFilter(BaseModel):
    name: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Product name to filter on"""

    updated_since: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Minimum date the product was last modified"""

    created_since: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Minimum date the product was created"""
