"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class OpportunitiesFilterTypedDict(TypedDict):
    title: NotRequired[str]
    r"""Title of the opportunity to filter on"""
    status: NotRequired[str]
    r"""Status to filter on"""
    monetary_amount: NotRequired[float]
    r"""Monetary amount to filter on"""
    win_probability: NotRequired[float]
    r"""Win probability to filter on"""
    company_id: NotRequired[str]
    r"""Company ID to filter on"""
    owner_id: NotRequired[str]
    r"""Owner ID to filter on"""
    primary_contact_id: NotRequired[str]
    r"""Primary contact ID to filter on"""


class OpportunitiesFilter(BaseModel):
    title: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Title of the opportunity to filter on"""

    status: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Status to filter on"""

    monetary_amount: Annotated[Optional[float], FieldMetadata(query=True)] = None
    r"""Monetary amount to filter on"""

    win_probability: Annotated[Optional[float], FieldMetadata(query=True)] = None
    r"""Win probability to filter on"""

    company_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Company ID to filter on"""

    owner_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Owner ID to filter on"""

    primary_contact_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Primary contact ID to filter on"""
