"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class UnifiedIDTypedDict(TypedDict):
    r"""A object containing a unique identifier for the resource that was created, updated, or deleted."""

    id: str
    r"""The unique identifier of the resource"""


class UnifiedID(BaseModel):
    r"""A object containing a unique identifier for the resource that was created, updated, or deleted."""

    id: str
    r"""The unique identifier of the resource"""
