"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins

if TYPE_CHECKING:
    from .accounting_agedcreditorsoneop import (
        AccountingAgedCreditorsOneGlobals,
        AccountingAgedCreditorsOneGlobalsTypedDict,
        AccountingAgedCreditorsOneRequest,
        AccountingAgedCreditorsOneRequestTypedDict,
        AccountingAgedCreditorsOneResponse,
        AccountingAgedCreditorsOneResponseTypedDict,
    )
    from .accounting_ageddebtorsoneop import (
        AccountingAgedDebtorsOneGlobals,
        AccountingAgedDebtorsOneGlobalsTypedDict,
        AccountingAgedDebtorsOneRequest,
        AccountingAgedDebtorsOneRequestTypedDict,
        AccountingAgedDebtorsOneResponse,
        AccountingAgedDebtorsOneResponseTypedDict,
    )
    from .accounting_attachmentsallop import (
        AccountingAttachmentsAllGlobals,
        AccountingAttachmentsAllGlobalsTypedDict,
        AccountingAttachmentsAllRequest,
        AccountingAttachmentsAllRequestTypedDict,
        AccountingAttachmentsAllResponse,
        AccountingAttachmentsAllResponseTypedDict,
    )
    from .accounting_attachmentsdeleteop import (
        AccountingAttachmentsDeleteGlobals,
        AccountingAttachmentsDeleteGlobalsTypedDict,
        AccountingAttachmentsDeleteRequest,
        AccountingAttachmentsDeleteRequestTypedDict,
        AccountingAttachmentsDeleteResponse,
        AccountingAttachmentsDeleteResponseTypedDict,
    )
    from .accounting_attachmentsdownloadop import (
        AccountingAttachmentsDownloadGlobals,
        AccountingAttachmentsDownloadGlobalsTypedDict,
        AccountingAttachmentsDownloadRequest,
        AccountingAttachmentsDownloadRequestTypedDict,
        AccountingAttachmentsDownloadResponse,
        AccountingAttachmentsDownloadResponseTypedDict,
    )
    from .accounting_attachmentsoneop import (
        AccountingAttachmentsOneGlobals,
        AccountingAttachmentsOneGlobalsTypedDict,
        AccountingAttachmentsOneRequest,
        AccountingAttachmentsOneRequestTypedDict,
        AccountingAttachmentsOneResponse,
        AccountingAttachmentsOneResponseTypedDict,
    )
    from .accounting_attachmentsuploadop import (
        ACCOUNTING_ATTACHMENTS_UPLOAD_OP_SERVERS,
        AccountingAttachmentsUploadGlobals,
        AccountingAttachmentsUploadGlobalsTypedDict,
        AccountingAttachmentsUploadRequest,
        AccountingAttachmentsUploadRequestTypedDict,
        AccountingAttachmentsUploadResponse,
        AccountingAttachmentsUploadResponseTypedDict,
    )
    from .accounting_balancesheetoneop import (
        AccountingBalanceSheetOneGlobals,
        AccountingBalanceSheetOneGlobalsTypedDict,
        AccountingBalanceSheetOneRequest,
        AccountingBalanceSheetOneRequestTypedDict,
        AccountingBalanceSheetOneResponse,
        AccountingBalanceSheetOneResponseTypedDict,
    )
    from .accounting_bankfeedaccountsaddop import (
        AccountingBankFeedAccountsAddGlobals,
        AccountingBankFeedAccountsAddGlobalsTypedDict,
        AccountingBankFeedAccountsAddRequest,
        AccountingBankFeedAccountsAddRequestTypedDict,
        AccountingBankFeedAccountsAddResponse,
        AccountingBankFeedAccountsAddResponseTypedDict,
    )
    from .accounting_bankfeedaccountsallop import (
        AccountingBankFeedAccountsAllGlobals,
        AccountingBankFeedAccountsAllGlobalsTypedDict,
        AccountingBankFeedAccountsAllRequest,
        AccountingBankFeedAccountsAllRequestTypedDict,
        AccountingBankFeedAccountsAllResponse,
        AccountingBankFeedAccountsAllResponseTypedDict,
    )
    from .accounting_bankfeedaccountsdeleteop import (
        AccountingBankFeedAccountsDeleteGlobals,
        AccountingBankFeedAccountsDeleteGlobalsTypedDict,
        AccountingBankFeedAccountsDeleteRequest,
        AccountingBankFeedAccountsDeleteRequestTypedDict,
        AccountingBankFeedAccountsDeleteResponse,
        AccountingBankFeedAccountsDeleteResponseTypedDict,
    )
    from .accounting_bankfeedaccountsoneop import (
        AccountingBankFeedAccountsOneGlobals,
        AccountingBankFeedAccountsOneGlobalsTypedDict,
        AccountingBankFeedAccountsOneRequest,
        AccountingBankFeedAccountsOneRequestTypedDict,
        AccountingBankFeedAccountsOneResponse,
        AccountingBankFeedAccountsOneResponseTypedDict,
    )
    from .accounting_bankfeedaccountsupdateop import (
        AccountingBankFeedAccountsUpdateGlobals,
        AccountingBankFeedAccountsUpdateGlobalsTypedDict,
        AccountingBankFeedAccountsUpdateRequest,
        AccountingBankFeedAccountsUpdateRequestTypedDict,
        AccountingBankFeedAccountsUpdateResponse,
        AccountingBankFeedAccountsUpdateResponseTypedDict,
    )
    from .accounting_bankfeedstatementsaddop import (
        AccountingBankFeedStatementsAddGlobals,
        AccountingBankFeedStatementsAddGlobalsTypedDict,
        AccountingBankFeedStatementsAddRequest,
        AccountingBankFeedStatementsAddRequestTypedDict,
        AccountingBankFeedStatementsAddResponse,
        AccountingBankFeedStatementsAddResponseTypedDict,
    )
    from .accounting_bankfeedstatementsallop import (
        AccountingBankFeedStatementsAllGlobals,
        AccountingBankFeedStatementsAllGlobalsTypedDict,
        AccountingBankFeedStatementsAllRequest,
        AccountingBankFeedStatementsAllRequestTypedDict,
        AccountingBankFeedStatementsAllResponse,
        AccountingBankFeedStatementsAllResponseTypedDict,
    )
    from .accounting_bankfeedstatementsdeleteop import (
        AccountingBankFeedStatementsDeleteGlobals,
        AccountingBankFeedStatementsDeleteGlobalsTypedDict,
        AccountingBankFeedStatementsDeleteRequest,
        AccountingBankFeedStatementsDeleteRequestTypedDict,
        AccountingBankFeedStatementsDeleteResponse,
        AccountingBankFeedStatementsDeleteResponseTypedDict,
    )
    from .accounting_bankfeedstatementsoneop import (
        AccountingBankFeedStatementsOneGlobals,
        AccountingBankFeedStatementsOneGlobalsTypedDict,
        AccountingBankFeedStatementsOneRequest,
        AccountingBankFeedStatementsOneRequestTypedDict,
        AccountingBankFeedStatementsOneResponse,
        AccountingBankFeedStatementsOneResponseTypedDict,
    )
    from .accounting_bankfeedstatementsupdateop import (
        AccountingBankFeedStatementsUpdateGlobals,
        AccountingBankFeedStatementsUpdateGlobalsTypedDict,
        AccountingBankFeedStatementsUpdateRequest,
        AccountingBankFeedStatementsUpdateRequestTypedDict,
        AccountingBankFeedStatementsUpdateResponse,
        AccountingBankFeedStatementsUpdateResponseTypedDict,
    )
    from .accounting_billpaymentsaddop import (
        AccountingBillPaymentsAddGlobals,
        AccountingBillPaymentsAddGlobalsTypedDict,
        AccountingBillPaymentsAddRequest,
        AccountingBillPaymentsAddRequestTypedDict,
        AccountingBillPaymentsAddResponse,
        AccountingBillPaymentsAddResponseTypedDict,
    )
    from .accounting_billpaymentsallop import (
        AccountingBillPaymentsAllGlobals,
        AccountingBillPaymentsAllGlobalsTypedDict,
        AccountingBillPaymentsAllRequest,
        AccountingBillPaymentsAllRequestTypedDict,
        AccountingBillPaymentsAllResponse,
        AccountingBillPaymentsAllResponseTypedDict,
    )
    from .accounting_billpaymentsdeleteop import (
        AccountingBillPaymentsDeleteGlobals,
        AccountingBillPaymentsDeleteGlobalsTypedDict,
        AccountingBillPaymentsDeleteRequest,
        AccountingBillPaymentsDeleteRequestTypedDict,
        AccountingBillPaymentsDeleteResponse,
        AccountingBillPaymentsDeleteResponseTypedDict,
    )
    from .accounting_billpaymentsoneop import (
        AccountingBillPaymentsOneGlobals,
        AccountingBillPaymentsOneGlobalsTypedDict,
        AccountingBillPaymentsOneRequest,
        AccountingBillPaymentsOneRequestTypedDict,
        AccountingBillPaymentsOneResponse,
        AccountingBillPaymentsOneResponseTypedDict,
    )
    from .accounting_billpaymentsupdateop import (
        AccountingBillPaymentsUpdateGlobals,
        AccountingBillPaymentsUpdateGlobalsTypedDict,
        AccountingBillPaymentsUpdateRequest,
        AccountingBillPaymentsUpdateRequestTypedDict,
        AccountingBillPaymentsUpdateResponse,
        AccountingBillPaymentsUpdateResponseTypedDict,
    )
    from .accounting_billsaddop import (
        AccountingBillsAddGlobals,
        AccountingBillsAddGlobalsTypedDict,
        AccountingBillsAddRequest,
        AccountingBillsAddRequestTypedDict,
        AccountingBillsAddResponse,
        AccountingBillsAddResponseTypedDict,
    )
    from .accounting_billsallop import (
        AccountingBillsAllGlobals,
        AccountingBillsAllGlobalsTypedDict,
        AccountingBillsAllRequest,
        AccountingBillsAllRequestTypedDict,
        AccountingBillsAllResponse,
        AccountingBillsAllResponseTypedDict,
    )
    from .accounting_billsdeleteop import (
        AccountingBillsDeleteGlobals,
        AccountingBillsDeleteGlobalsTypedDict,
        AccountingBillsDeleteRequest,
        AccountingBillsDeleteRequestTypedDict,
        AccountingBillsDeleteResponse,
        AccountingBillsDeleteResponseTypedDict,
    )
    from .accounting_billsoneop import (
        AccountingBillsOneGlobals,
        AccountingBillsOneGlobalsTypedDict,
        AccountingBillsOneRequest,
        AccountingBillsOneRequestTypedDict,
        AccountingBillsOneResponse,
        AccountingBillsOneResponseTypedDict,
    )
    from .accounting_billsupdateop import (
        AccountingBillsUpdateGlobals,
        AccountingBillsUpdateGlobalsTypedDict,
        AccountingBillsUpdateRequest,
        AccountingBillsUpdateRequestTypedDict,
        AccountingBillsUpdateResponse,
        AccountingBillsUpdateResponseTypedDict,
    )
    from .accounting_categoriesallop import (
        AccountingCategoriesAllGlobals,
        AccountingCategoriesAllGlobalsTypedDict,
        AccountingCategoriesAllRequest,
        AccountingCategoriesAllRequestTypedDict,
        AccountingCategoriesAllResponse,
        AccountingCategoriesAllResponseTypedDict,
    )
    from .accounting_categoriesoneop import (
        AccountingCategoriesOneGlobals,
        AccountingCategoriesOneGlobalsTypedDict,
        AccountingCategoriesOneRequest,
        AccountingCategoriesOneRequestTypedDict,
        AccountingCategoriesOneResponse,
        AccountingCategoriesOneResponseTypedDict,
    )
    from .accounting_companyinfooneop import (
        AccountingCompanyInfoOneGlobals,
        AccountingCompanyInfoOneGlobalsTypedDict,
        AccountingCompanyInfoOneRequest,
        AccountingCompanyInfoOneRequestTypedDict,
        AccountingCompanyInfoOneResponse,
        AccountingCompanyInfoOneResponseTypedDict,
    )
    from .accounting_creditnotesaddop import (
        AccountingCreditNotesAddGlobals,
        AccountingCreditNotesAddGlobalsTypedDict,
        AccountingCreditNotesAddRequest,
        AccountingCreditNotesAddRequestTypedDict,
        AccountingCreditNotesAddResponse,
        AccountingCreditNotesAddResponseTypedDict,
    )
    from .accounting_creditnotesallop import (
        AccountingCreditNotesAllGlobals,
        AccountingCreditNotesAllGlobalsTypedDict,
        AccountingCreditNotesAllRequest,
        AccountingCreditNotesAllRequestTypedDict,
        AccountingCreditNotesAllResponse,
        AccountingCreditNotesAllResponseTypedDict,
    )
    from .accounting_creditnotesdeleteop import (
        AccountingCreditNotesDeleteGlobals,
        AccountingCreditNotesDeleteGlobalsTypedDict,
        AccountingCreditNotesDeleteRequest,
        AccountingCreditNotesDeleteRequestTypedDict,
        AccountingCreditNotesDeleteResponse,
        AccountingCreditNotesDeleteResponseTypedDict,
    )
    from .accounting_creditnotesoneop import (
        AccountingCreditNotesOneGlobals,
        AccountingCreditNotesOneGlobalsTypedDict,
        AccountingCreditNotesOneRequest,
        AccountingCreditNotesOneRequestTypedDict,
        AccountingCreditNotesOneResponse,
        AccountingCreditNotesOneResponseTypedDict,
    )
    from .accounting_creditnotesupdateop import (
        AccountingCreditNotesUpdateGlobals,
        AccountingCreditNotesUpdateGlobalsTypedDict,
        AccountingCreditNotesUpdateRequest,
        AccountingCreditNotesUpdateRequestTypedDict,
        AccountingCreditNotesUpdateResponse,
        AccountingCreditNotesUpdateResponseTypedDict,
    )
    from .accounting_customersaddop import (
        AccountingCustomersAddGlobals,
        AccountingCustomersAddGlobalsTypedDict,
        AccountingCustomersAddRequest,
        AccountingCustomersAddRequestTypedDict,
        AccountingCustomersAddResponse,
        AccountingCustomersAddResponseTypedDict,
    )
    from .accounting_customersallop import (
        AccountingCustomersAllGlobals,
        AccountingCustomersAllGlobalsTypedDict,
        AccountingCustomersAllRequest,
        AccountingCustomersAllRequestTypedDict,
        AccountingCustomersAllResponse,
        AccountingCustomersAllResponseTypedDict,
    )
    from .accounting_customersdeleteop import (
        AccountingCustomersDeleteGlobals,
        AccountingCustomersDeleteGlobalsTypedDict,
        AccountingCustomersDeleteRequest,
        AccountingCustomersDeleteRequestTypedDict,
        AccountingCustomersDeleteResponse,
        AccountingCustomersDeleteResponseTypedDict,
    )
    from .accounting_customersoneop import (
        AccountingCustomersOneGlobals,
        AccountingCustomersOneGlobalsTypedDict,
        AccountingCustomersOneRequest,
        AccountingCustomersOneRequestTypedDict,
        AccountingCustomersOneResponse,
        AccountingCustomersOneResponseTypedDict,
    )
    from .accounting_customersupdateop import (
        AccountingCustomersUpdateGlobals,
        AccountingCustomersUpdateGlobalsTypedDict,
        AccountingCustomersUpdateRequest,
        AccountingCustomersUpdateRequestTypedDict,
        AccountingCustomersUpdateResponse,
        AccountingCustomersUpdateResponseTypedDict,
    )
    from .accounting_departmentsaddop import (
        AccountingDepartmentsAddGlobals,
        AccountingDepartmentsAddGlobalsTypedDict,
        AccountingDepartmentsAddRequest,
        AccountingDepartmentsAddRequestTypedDict,
        AccountingDepartmentsAddResponse,
        AccountingDepartmentsAddResponseTypedDict,
    )
    from .accounting_departmentsallop import (
        AccountingDepartmentsAllGlobals,
        AccountingDepartmentsAllGlobalsTypedDict,
        AccountingDepartmentsAllRequest,
        AccountingDepartmentsAllRequestTypedDict,
        AccountingDepartmentsAllResponse,
        AccountingDepartmentsAllResponseTypedDict,
    )
    from .accounting_departmentsdeleteop import (
        AccountingDepartmentsDeleteGlobals,
        AccountingDepartmentsDeleteGlobalsTypedDict,
        AccountingDepartmentsDeleteRequest,
        AccountingDepartmentsDeleteRequestTypedDict,
        AccountingDepartmentsDeleteResponse,
        AccountingDepartmentsDeleteResponseTypedDict,
    )
    from .accounting_departmentsoneop import (
        AccountingDepartmentsOneGlobals,
        AccountingDepartmentsOneGlobalsTypedDict,
        AccountingDepartmentsOneRequest,
        AccountingDepartmentsOneRequestTypedDict,
        AccountingDepartmentsOneResponse,
        AccountingDepartmentsOneResponseTypedDict,
    )
    from .accounting_departmentsupdateop import (
        AccountingDepartmentsUpdateGlobals,
        AccountingDepartmentsUpdateGlobalsTypedDict,
        AccountingDepartmentsUpdateRequest,
        AccountingDepartmentsUpdateRequestTypedDict,
        AccountingDepartmentsUpdateResponse,
        AccountingDepartmentsUpdateResponseTypedDict,
    )
    from .accounting_expensesaddop import (
        AccountingExpensesAddGlobals,
        AccountingExpensesAddGlobalsTypedDict,
        AccountingExpensesAddRequest,
        AccountingExpensesAddRequestTypedDict,
        AccountingExpensesAddResponse,
        AccountingExpensesAddResponseTypedDict,
    )
    from .accounting_expensesallop import (
        AccountingExpensesAllGlobals,
        AccountingExpensesAllGlobalsTypedDict,
        AccountingExpensesAllRequest,
        AccountingExpensesAllRequestTypedDict,
        AccountingExpensesAllResponse,
        AccountingExpensesAllResponseTypedDict,
    )
    from .accounting_expensesdeleteop import (
        AccountingExpensesDeleteGlobals,
        AccountingExpensesDeleteGlobalsTypedDict,
        AccountingExpensesDeleteRequest,
        AccountingExpensesDeleteRequestTypedDict,
        AccountingExpensesDeleteResponse,
        AccountingExpensesDeleteResponseTypedDict,
    )
    from .accounting_expensesoneop import (
        AccountingExpensesOneGlobals,
        AccountingExpensesOneGlobalsTypedDict,
        AccountingExpensesOneRequest,
        AccountingExpensesOneRequestTypedDict,
        AccountingExpensesOneResponse,
        AccountingExpensesOneResponseTypedDict,
    )
    from .accounting_expensesupdateop import (
        AccountingExpensesUpdateGlobals,
        AccountingExpensesUpdateGlobalsTypedDict,
        AccountingExpensesUpdateRequest,
        AccountingExpensesUpdateRequestTypedDict,
        AccountingExpensesUpdateResponse,
        AccountingExpensesUpdateResponseTypedDict,
    )
    from .accounting_invoiceitemsaddop import (
        AccountingInvoiceItemsAddGlobals,
        AccountingInvoiceItemsAddGlobalsTypedDict,
        AccountingInvoiceItemsAddRequest,
        AccountingInvoiceItemsAddRequestTypedDict,
        AccountingInvoiceItemsAddResponse,
        AccountingInvoiceItemsAddResponseTypedDict,
    )
    from .accounting_invoiceitemsallop import (
        AccountingInvoiceItemsAllGlobals,
        AccountingInvoiceItemsAllGlobalsTypedDict,
        AccountingInvoiceItemsAllRequest,
        AccountingInvoiceItemsAllRequestTypedDict,
        AccountingInvoiceItemsAllResponse,
        AccountingInvoiceItemsAllResponseTypedDict,
    )
    from .accounting_invoiceitemsdeleteop import (
        AccountingInvoiceItemsDeleteGlobals,
        AccountingInvoiceItemsDeleteGlobalsTypedDict,
        AccountingInvoiceItemsDeleteRequest,
        AccountingInvoiceItemsDeleteRequestTypedDict,
        AccountingInvoiceItemsDeleteResponse,
        AccountingInvoiceItemsDeleteResponseTypedDict,
    )
    from .accounting_invoiceitemsoneop import (
        AccountingInvoiceItemsOneGlobals,
        AccountingInvoiceItemsOneGlobalsTypedDict,
        AccountingInvoiceItemsOneRequest,
        AccountingInvoiceItemsOneRequestTypedDict,
        AccountingInvoiceItemsOneResponse,
        AccountingInvoiceItemsOneResponseTypedDict,
    )
    from .accounting_invoiceitemsupdateop import (
        AccountingInvoiceItemsUpdateGlobals,
        AccountingInvoiceItemsUpdateGlobalsTypedDict,
        AccountingInvoiceItemsUpdateRequest,
        AccountingInvoiceItemsUpdateRequestTypedDict,
        AccountingInvoiceItemsUpdateResponse,
        AccountingInvoiceItemsUpdateResponseTypedDict,
    )
    from .accounting_invoicesaddop import (
        AccountingInvoicesAddGlobals,
        AccountingInvoicesAddGlobalsTypedDict,
        AccountingInvoicesAddRequest,
        AccountingInvoicesAddRequestTypedDict,
        AccountingInvoicesAddResponse,
        AccountingInvoicesAddResponseTypedDict,
    )
    from .accounting_invoicesallop import (
        AccountingInvoicesAllGlobals,
        AccountingInvoicesAllGlobalsTypedDict,
        AccountingInvoicesAllRequest,
        AccountingInvoicesAllRequestTypedDict,
        AccountingInvoicesAllResponse,
        AccountingInvoicesAllResponseTypedDict,
    )
    from .accounting_invoicesdeleteop import (
        AccountingInvoicesDeleteGlobals,
        AccountingInvoicesDeleteGlobalsTypedDict,
        AccountingInvoicesDeleteRequest,
        AccountingInvoicesDeleteRequestTypedDict,
        AccountingInvoicesDeleteResponse,
        AccountingInvoicesDeleteResponseTypedDict,
    )
    from .accounting_invoicesoneop import (
        AccountingInvoicesOneGlobals,
        AccountingInvoicesOneGlobalsTypedDict,
        AccountingInvoicesOneRequest,
        AccountingInvoicesOneRequestTypedDict,
        AccountingInvoicesOneResponse,
        AccountingInvoicesOneResponseTypedDict,
    )
    from .accounting_invoicesupdateop import (
        AccountingInvoicesUpdateGlobals,
        AccountingInvoicesUpdateGlobalsTypedDict,
        AccountingInvoicesUpdateRequest,
        AccountingInvoicesUpdateRequestTypedDict,
        AccountingInvoicesUpdateResponse,
        AccountingInvoicesUpdateResponseTypedDict,
    )
    from .accounting_journalentriesaddop import (
        AccountingJournalEntriesAddGlobals,
        AccountingJournalEntriesAddGlobalsTypedDict,
        AccountingJournalEntriesAddRequest,
        AccountingJournalEntriesAddRequestTypedDict,
        AccountingJournalEntriesAddResponse,
        AccountingJournalEntriesAddResponseTypedDict,
    )
    from .accounting_journalentriesallop import (
        AccountingJournalEntriesAllGlobals,
        AccountingJournalEntriesAllGlobalsTypedDict,
        AccountingJournalEntriesAllRequest,
        AccountingJournalEntriesAllRequestTypedDict,
        AccountingJournalEntriesAllResponse,
        AccountingJournalEntriesAllResponseTypedDict,
    )
    from .accounting_journalentriesdeleteop import (
        AccountingJournalEntriesDeleteGlobals,
        AccountingJournalEntriesDeleteGlobalsTypedDict,
        AccountingJournalEntriesDeleteRequest,
        AccountingJournalEntriesDeleteRequestTypedDict,
        AccountingJournalEntriesDeleteResponse,
        AccountingJournalEntriesDeleteResponseTypedDict,
    )
    from .accounting_journalentriesoneop import (
        AccountingJournalEntriesOneGlobals,
        AccountingJournalEntriesOneGlobalsTypedDict,
        AccountingJournalEntriesOneRequest,
        AccountingJournalEntriesOneRequestTypedDict,
        AccountingJournalEntriesOneResponse,
        AccountingJournalEntriesOneResponseTypedDict,
    )
    from .accounting_journalentriesupdateop import (
        AccountingJournalEntriesUpdateGlobals,
        AccountingJournalEntriesUpdateGlobalsTypedDict,
        AccountingJournalEntriesUpdateRequest,
        AccountingJournalEntriesUpdateRequestTypedDict,
        AccountingJournalEntriesUpdateResponse,
        AccountingJournalEntriesUpdateResponseTypedDict,
    )
    from .accounting_ledgeraccountsaddop import (
        AccountingLedgerAccountsAddGlobals,
        AccountingLedgerAccountsAddGlobalsTypedDict,
        AccountingLedgerAccountsAddRequest,
        AccountingLedgerAccountsAddRequestTypedDict,
        AccountingLedgerAccountsAddResponse,
        AccountingLedgerAccountsAddResponseTypedDict,
    )
    from .accounting_ledgeraccountsallop import (
        AccountingLedgerAccountsAllGlobals,
        AccountingLedgerAccountsAllGlobalsTypedDict,
        AccountingLedgerAccountsAllRequest,
        AccountingLedgerAccountsAllRequestTypedDict,
        AccountingLedgerAccountsAllResponse,
        AccountingLedgerAccountsAllResponseTypedDict,
    )
    from .accounting_ledgeraccountsdeleteop import (
        AccountingLedgerAccountsDeleteGlobals,
        AccountingLedgerAccountsDeleteGlobalsTypedDict,
        AccountingLedgerAccountsDeleteRequest,
        AccountingLedgerAccountsDeleteRequestTypedDict,
        AccountingLedgerAccountsDeleteResponse,
        AccountingLedgerAccountsDeleteResponseTypedDict,
    )
    from .accounting_ledgeraccountsoneop import (
        AccountingLedgerAccountsOneGlobals,
        AccountingLedgerAccountsOneGlobalsTypedDict,
        AccountingLedgerAccountsOneRequest,
        AccountingLedgerAccountsOneRequestTypedDict,
        AccountingLedgerAccountsOneResponse,
        AccountingLedgerAccountsOneResponseTypedDict,
    )
    from .accounting_ledgeraccountsupdateop import (
        AccountingLedgerAccountsUpdateGlobals,
        AccountingLedgerAccountsUpdateGlobalsTypedDict,
        AccountingLedgerAccountsUpdateRequest,
        AccountingLedgerAccountsUpdateRequestTypedDict,
        AccountingLedgerAccountsUpdateResponse,
        AccountingLedgerAccountsUpdateResponseTypedDict,
    )
    from .accounting_locationsaddop import (
        AccountingLocationsAddGlobals,
        AccountingLocationsAddGlobalsTypedDict,
        AccountingLocationsAddRequest,
        AccountingLocationsAddRequestTypedDict,
        AccountingLocationsAddResponse,
        AccountingLocationsAddResponseTypedDict,
    )
    from .accounting_locationsallop import (
        AccountingLocationsAllGlobals,
        AccountingLocationsAllGlobalsTypedDict,
        AccountingLocationsAllRequest,
        AccountingLocationsAllRequestTypedDict,
        AccountingLocationsAllResponse,
        AccountingLocationsAllResponseTypedDict,
    )
    from .accounting_locationsdeleteop import (
        AccountingLocationsDeleteGlobals,
        AccountingLocationsDeleteGlobalsTypedDict,
        AccountingLocationsDeleteRequest,
        AccountingLocationsDeleteRequestTypedDict,
        AccountingLocationsDeleteResponse,
        AccountingLocationsDeleteResponseTypedDict,
    )
    from .accounting_locationsoneop import (
        AccountingLocationsOneGlobals,
        AccountingLocationsOneGlobalsTypedDict,
        AccountingLocationsOneRequest,
        AccountingLocationsOneRequestTypedDict,
        AccountingLocationsOneResponse,
        AccountingLocationsOneResponseTypedDict,
    )
    from .accounting_locationsupdateop import (
        AccountingLocationsUpdateGlobals,
        AccountingLocationsUpdateGlobalsTypedDict,
        AccountingLocationsUpdateRequest,
        AccountingLocationsUpdateRequestTypedDict,
        AccountingLocationsUpdateResponse,
        AccountingLocationsUpdateResponseTypedDict,
    )
    from .accounting_paymentsaddop import (
        AccountingPaymentsAddGlobals,
        AccountingPaymentsAddGlobalsTypedDict,
        AccountingPaymentsAddRequest,
        AccountingPaymentsAddRequestTypedDict,
        AccountingPaymentsAddResponse,
        AccountingPaymentsAddResponseTypedDict,
    )
    from .accounting_paymentsallop import (
        AccountingPaymentsAllGlobals,
        AccountingPaymentsAllGlobalsTypedDict,
        AccountingPaymentsAllRequest,
        AccountingPaymentsAllRequestTypedDict,
        AccountingPaymentsAllResponse,
        AccountingPaymentsAllResponseTypedDict,
    )
    from .accounting_paymentsdeleteop import (
        AccountingPaymentsDeleteGlobals,
        AccountingPaymentsDeleteGlobalsTypedDict,
        AccountingPaymentsDeleteRequest,
        AccountingPaymentsDeleteRequestTypedDict,
        AccountingPaymentsDeleteResponse,
        AccountingPaymentsDeleteResponseTypedDict,
    )
    from .accounting_paymentsoneop import (
        AccountingPaymentsOneGlobals,
        AccountingPaymentsOneGlobalsTypedDict,
        AccountingPaymentsOneRequest,
        AccountingPaymentsOneRequestTypedDict,
        AccountingPaymentsOneResponse,
        AccountingPaymentsOneResponseTypedDict,
    )
    from .accounting_paymentsupdateop import (
        AccountingPaymentsUpdateGlobals,
        AccountingPaymentsUpdateGlobalsTypedDict,
        AccountingPaymentsUpdateRequest,
        AccountingPaymentsUpdateRequestTypedDict,
        AccountingPaymentsUpdateResponse,
        AccountingPaymentsUpdateResponseTypedDict,
    )
    from .accounting_profitandlossoneop import (
        AccountingProfitAndLossOneGlobals,
        AccountingProfitAndLossOneGlobalsTypedDict,
        AccountingProfitAndLossOneRequest,
        AccountingProfitAndLossOneRequestTypedDict,
        AccountingProfitAndLossOneResponse,
        AccountingProfitAndLossOneResponseTypedDict,
    )
    from .accounting_purchaseordersaddop import (
        AccountingPurchaseOrdersAddGlobals,
        AccountingPurchaseOrdersAddGlobalsTypedDict,
        AccountingPurchaseOrdersAddRequest,
        AccountingPurchaseOrdersAddRequestTypedDict,
        AccountingPurchaseOrdersAddResponse,
        AccountingPurchaseOrdersAddResponseTypedDict,
    )
    from .accounting_purchaseordersallop import (
        AccountingPurchaseOrdersAllGlobals,
        AccountingPurchaseOrdersAllGlobalsTypedDict,
        AccountingPurchaseOrdersAllRequest,
        AccountingPurchaseOrdersAllRequestTypedDict,
        AccountingPurchaseOrdersAllResponse,
        AccountingPurchaseOrdersAllResponseTypedDict,
    )
    from .accounting_purchaseordersdeleteop import (
        AccountingPurchaseOrdersDeleteGlobals,
        AccountingPurchaseOrdersDeleteGlobalsTypedDict,
        AccountingPurchaseOrdersDeleteRequest,
        AccountingPurchaseOrdersDeleteRequestTypedDict,
        AccountingPurchaseOrdersDeleteResponse,
        AccountingPurchaseOrdersDeleteResponseTypedDict,
    )
    from .accounting_purchaseordersoneop import (
        AccountingPurchaseOrdersOneGlobals,
        AccountingPurchaseOrdersOneGlobalsTypedDict,
        AccountingPurchaseOrdersOneRequest,
        AccountingPurchaseOrdersOneRequestTypedDict,
        AccountingPurchaseOrdersOneResponse,
        AccountingPurchaseOrdersOneResponseTypedDict,
    )
    from .accounting_purchaseordersupdateop import (
        AccountingPurchaseOrdersUpdateGlobals,
        AccountingPurchaseOrdersUpdateGlobalsTypedDict,
        AccountingPurchaseOrdersUpdateRequest,
        AccountingPurchaseOrdersUpdateRequestTypedDict,
        AccountingPurchaseOrdersUpdateResponse,
        AccountingPurchaseOrdersUpdateResponseTypedDict,
    )
    from .accounting_quotesaddop import (
        AccountingQuotesAddGlobals,
        AccountingQuotesAddGlobalsTypedDict,
        AccountingQuotesAddRequest,
        AccountingQuotesAddRequestTypedDict,
        AccountingQuotesAddResponse,
        AccountingQuotesAddResponseTypedDict,
    )
    from .accounting_quotesallop import (
        AccountingQuotesAllGlobals,
        AccountingQuotesAllGlobalsTypedDict,
        AccountingQuotesAllRequest,
        AccountingQuotesAllRequestTypedDict,
        AccountingQuotesAllResponse,
        AccountingQuotesAllResponseTypedDict,
    )
    from .accounting_quotesdeleteop import (
        AccountingQuotesDeleteGlobals,
        AccountingQuotesDeleteGlobalsTypedDict,
        AccountingQuotesDeleteRequest,
        AccountingQuotesDeleteRequestTypedDict,
        AccountingQuotesDeleteResponse,
        AccountingQuotesDeleteResponseTypedDict,
    )
    from .accounting_quotesoneop import (
        AccountingQuotesOneGlobals,
        AccountingQuotesOneGlobalsTypedDict,
        AccountingQuotesOneRequest,
        AccountingQuotesOneRequestTypedDict,
        AccountingQuotesOneResponse,
        AccountingQuotesOneResponseTypedDict,
    )
    from .accounting_quotesupdateop import (
        AccountingQuotesUpdateGlobals,
        AccountingQuotesUpdateGlobalsTypedDict,
        AccountingQuotesUpdateRequest,
        AccountingQuotesUpdateRequestTypedDict,
        AccountingQuotesUpdateResponse,
        AccountingQuotesUpdateResponseTypedDict,
    )
    from .accounting_subsidiariesaddop import (
        AccountingSubsidiariesAddGlobals,
        AccountingSubsidiariesAddGlobalsTypedDict,
        AccountingSubsidiariesAddRequest,
        AccountingSubsidiariesAddRequestTypedDict,
        AccountingSubsidiariesAddResponse,
        AccountingSubsidiariesAddResponseTypedDict,
    )
    from .accounting_subsidiariesallop import (
        AccountingSubsidiariesAllGlobals,
        AccountingSubsidiariesAllGlobalsTypedDict,
        AccountingSubsidiariesAllRequest,
        AccountingSubsidiariesAllRequestTypedDict,
        AccountingSubsidiariesAllResponse,
        AccountingSubsidiariesAllResponseTypedDict,
    )
    from .accounting_subsidiariesdeleteop import (
        AccountingSubsidiariesDeleteGlobals,
        AccountingSubsidiariesDeleteGlobalsTypedDict,
        AccountingSubsidiariesDeleteRequest,
        AccountingSubsidiariesDeleteRequestTypedDict,
        AccountingSubsidiariesDeleteResponse,
        AccountingSubsidiariesDeleteResponseTypedDict,
    )
    from .accounting_subsidiariesoneop import (
        AccountingSubsidiariesOneGlobals,
        AccountingSubsidiariesOneGlobalsTypedDict,
        AccountingSubsidiariesOneRequest,
        AccountingSubsidiariesOneRequestTypedDict,
        AccountingSubsidiariesOneResponse,
        AccountingSubsidiariesOneResponseTypedDict,
    )
    from .accounting_subsidiariesupdateop import (
        AccountingSubsidiariesUpdateGlobals,
        AccountingSubsidiariesUpdateGlobalsTypedDict,
        AccountingSubsidiariesUpdateRequest,
        AccountingSubsidiariesUpdateRequestTypedDict,
        AccountingSubsidiariesUpdateResponse,
        AccountingSubsidiariesUpdateResponseTypedDict,
    )
    from .accounting_suppliersaddop import (
        AccountingSuppliersAddGlobals,
        AccountingSuppliersAddGlobalsTypedDict,
        AccountingSuppliersAddRequest,
        AccountingSuppliersAddRequestTypedDict,
        AccountingSuppliersAddResponse,
        AccountingSuppliersAddResponseTypedDict,
    )
    from .accounting_suppliersallop import (
        AccountingSuppliersAllGlobals,
        AccountingSuppliersAllGlobalsTypedDict,
        AccountingSuppliersAllRequest,
        AccountingSuppliersAllRequestTypedDict,
        AccountingSuppliersAllResponse,
        AccountingSuppliersAllResponseTypedDict,
    )
    from .accounting_suppliersdeleteop import (
        AccountingSuppliersDeleteGlobals,
        AccountingSuppliersDeleteGlobalsTypedDict,
        AccountingSuppliersDeleteRequest,
        AccountingSuppliersDeleteRequestTypedDict,
        AccountingSuppliersDeleteResponse,
        AccountingSuppliersDeleteResponseTypedDict,
    )
    from .accounting_suppliersoneop import (
        AccountingSuppliersOneGlobals,
        AccountingSuppliersOneGlobalsTypedDict,
        AccountingSuppliersOneRequest,
        AccountingSuppliersOneRequestTypedDict,
        AccountingSuppliersOneResponse,
        AccountingSuppliersOneResponseTypedDict,
    )
    from .accounting_suppliersupdateop import (
        AccountingSuppliersUpdateGlobals,
        AccountingSuppliersUpdateGlobalsTypedDict,
        AccountingSuppliersUpdateRequest,
        AccountingSuppliersUpdateRequestTypedDict,
        AccountingSuppliersUpdateResponse,
        AccountingSuppliersUpdateResponseTypedDict,
    )
    from .accounting_taxratesaddop import (
        AccountingTaxRatesAddGlobals,
        AccountingTaxRatesAddGlobalsTypedDict,
        AccountingTaxRatesAddRequest,
        AccountingTaxRatesAddRequestTypedDict,
        AccountingTaxRatesAddResponse,
        AccountingTaxRatesAddResponseTypedDict,
    )
    from .accounting_taxratesallop import (
        AccountingTaxRatesAllGlobals,
        AccountingTaxRatesAllGlobalsTypedDict,
        AccountingTaxRatesAllRequest,
        AccountingTaxRatesAllRequestTypedDict,
        AccountingTaxRatesAllResponse,
        AccountingTaxRatesAllResponseTypedDict,
    )
    from .accounting_taxratesdeleteop import (
        AccountingTaxRatesDeleteGlobals,
        AccountingTaxRatesDeleteGlobalsTypedDict,
        AccountingTaxRatesDeleteRequest,
        AccountingTaxRatesDeleteRequestTypedDict,
        AccountingTaxRatesDeleteResponse,
        AccountingTaxRatesDeleteResponseTypedDict,
    )
    from .accounting_taxratesoneop import (
        AccountingTaxRatesOneGlobals,
        AccountingTaxRatesOneGlobalsTypedDict,
        AccountingTaxRatesOneRequest,
        AccountingTaxRatesOneRequestTypedDict,
        AccountingTaxRatesOneResponse,
        AccountingTaxRatesOneResponseTypedDict,
    )
    from .accounting_taxratesupdateop import (
        AccountingTaxRatesUpdateGlobals,
        AccountingTaxRatesUpdateGlobalsTypedDict,
        AccountingTaxRatesUpdateRequest,
        AccountingTaxRatesUpdateRequestTypedDict,
        AccountingTaxRatesUpdateResponse,
        AccountingTaxRatesUpdateResponseTypedDict,
    )
    from .accounting_trackingcategoriesaddop import (
        AccountingTrackingCategoriesAddGlobals,
        AccountingTrackingCategoriesAddGlobalsTypedDict,
        AccountingTrackingCategoriesAddRequest,
        AccountingTrackingCategoriesAddRequestTypedDict,
        AccountingTrackingCategoriesAddResponse,
        AccountingTrackingCategoriesAddResponseTypedDict,
    )
    from .accounting_trackingcategoriesallop import (
        AccountingTrackingCategoriesAllGlobals,
        AccountingTrackingCategoriesAllGlobalsTypedDict,
        AccountingTrackingCategoriesAllRequest,
        AccountingTrackingCategoriesAllRequestTypedDict,
        AccountingTrackingCategoriesAllResponse,
        AccountingTrackingCategoriesAllResponseTypedDict,
    )
    from .accounting_trackingcategoriesdeleteop import (
        AccountingTrackingCategoriesDeleteGlobals,
        AccountingTrackingCategoriesDeleteGlobalsTypedDict,
        AccountingTrackingCategoriesDeleteRequest,
        AccountingTrackingCategoriesDeleteRequestTypedDict,
        AccountingTrackingCategoriesDeleteResponse,
        AccountingTrackingCategoriesDeleteResponseTypedDict,
    )
    from .accounting_trackingcategoriesoneop import (
        AccountingTrackingCategoriesOneGlobals,
        AccountingTrackingCategoriesOneGlobalsTypedDict,
        AccountingTrackingCategoriesOneRequest,
        AccountingTrackingCategoriesOneRequestTypedDict,
        AccountingTrackingCategoriesOneResponse,
        AccountingTrackingCategoriesOneResponseTypedDict,
    )
    from .accounting_trackingcategoriesupdateop import (
        AccountingTrackingCategoriesUpdateGlobals,
        AccountingTrackingCategoriesUpdateGlobalsTypedDict,
        AccountingTrackingCategoriesUpdateRequest,
        AccountingTrackingCategoriesUpdateRequestTypedDict,
        AccountingTrackingCategoriesUpdateResponse,
        AccountingTrackingCategoriesUpdateResponseTypedDict,
    )
    from .accountingdepartment import (
        AccountingDepartment,
        AccountingDepartmentInput,
        AccountingDepartmentInputTypedDict,
        AccountingDepartmentTypedDict,
        DepartmentStatus,
    )
    from .accountingdepartmentsfilter import (
        AccountingDepartmentsFilter,
        AccountingDepartmentsFilterTypedDict,
    )
    from .accountinglocation import (
        AccountingLocation,
        AccountingLocationInput,
        AccountingLocationInputTypedDict,
        AccountingLocationTypedDict,
        LocationStatus,
    )
    from .accountinglocationsfilter import (
        AccountingLocationsFilter,
        AccountingLocationsFilterTypedDict,
    )
    from .activitiesfilter import ActivitiesFilter, ActivitiesFilterTypedDict
    from .activitiessort import (
        ActivitiesSort,
        ActivitiesSortBy,
        ActivitiesSortTypedDict,
    )
    from .activity import (
        Activity,
        ActivityInput,
        ActivityInputTypedDict,
        ActivityType,
        ActivityTypedDict,
        ShowAs,
    )
    from .activityattendee import (
        ActivityAttendee,
        ActivityAttendeeInput,
        ActivityAttendeeInputTypedDict,
        ActivityAttendeeStatus,
        ActivityAttendeeTypedDict,
    )
    from .address import Address, AddressTypedDict, Type
    from .agedcreditors import AgedCreditors, AgedCreditorsTypedDict
    from .ageddebtors import AgedDebtors, AgedDebtorsTypedDict
    from .agedreportfilter import AgedReportFilter, AgedReportFilterTypedDict
    from .allocation import (
        Allocation,
        AllocationInput,
        AllocationInputTypedDict,
        AllocationType,
        AllocationTypedDict,
    )
    from .api import API, APIType, APITypedDict, Resources, ResourcesTypedDict
    from .apideckerror import ApideckError
    from .apierror import APIError
    from .apiresource import (
        APIResource,
        APIResourceTypedDict,
        LinkedResources,
        LinkedResourcesTypedDict,
        Schema,
        SchemaTypedDict,
    )
    from .apiresourcecoverage import (
        APIResourceCoverage,
        APIResourceCoverageTypedDict,
        Coverage,
        CoverageTypedDict,
    )
    from .apisfilter import ApisFilter, ApisFilterTypedDict
    from .apistatus import APIStatus
    from .applicant import (
        Applicant,
        ApplicantGender,
        ApplicantInput,
        ApplicantInputTypedDict,
        ApplicantType,
        ApplicantTypedDict,
        SocialLinks,
        SocialLinksTypedDict,
        Websites,
        WebsitesTypedDict,
    )
    from .applicantsfilter import ApplicantsFilter, ApplicantsFilterTypedDict
    from .application import (
        Application,
        ApplicationInput,
        ApplicationInputTypedDict,
        ApplicationStatus,
        ApplicationTypedDict,
        Stage,
        StageTypedDict,
    )
    from .assignee import Assignee, AssigneeTypedDict
    from .assignee_input import AssigneeInput, AssigneeInputTypedDict
    from .ats_applicantsaddop import (
        AtsApplicantsAddGlobals,
        AtsApplicantsAddGlobalsTypedDict,
        AtsApplicantsAddRequest,
        AtsApplicantsAddRequestTypedDict,
        AtsApplicantsAddResponse,
        AtsApplicantsAddResponseTypedDict,
    )
    from .ats_applicantsallop import (
        AtsApplicantsAllGlobals,
        AtsApplicantsAllGlobalsTypedDict,
        AtsApplicantsAllRequest,
        AtsApplicantsAllRequestTypedDict,
        AtsApplicantsAllResponse,
        AtsApplicantsAllResponseTypedDict,
    )
    from .ats_applicantsdeleteop import (
        AtsApplicantsDeleteGlobals,
        AtsApplicantsDeleteGlobalsTypedDict,
        AtsApplicantsDeleteRequest,
        AtsApplicantsDeleteRequestTypedDict,
        AtsApplicantsDeleteResponse,
        AtsApplicantsDeleteResponseTypedDict,
    )
    from .ats_applicantsoneop import (
        AtsApplicantsOneGlobals,
        AtsApplicantsOneGlobalsTypedDict,
        AtsApplicantsOneRequest,
        AtsApplicantsOneRequestTypedDict,
        AtsApplicantsOneResponse,
        AtsApplicantsOneResponseTypedDict,
    )
    from .ats_applicantsupdateop import (
        AtsApplicantsUpdateGlobals,
        AtsApplicantsUpdateGlobalsTypedDict,
        AtsApplicantsUpdateRequest,
        AtsApplicantsUpdateRequestTypedDict,
        AtsApplicantsUpdateResponse,
        AtsApplicantsUpdateResponseTypedDict,
    )
    from .ats_applicationsaddop import (
        AtsApplicationsAddGlobals,
        AtsApplicationsAddGlobalsTypedDict,
        AtsApplicationsAddRequest,
        AtsApplicationsAddRequestTypedDict,
        AtsApplicationsAddResponse,
        AtsApplicationsAddResponseTypedDict,
    )
    from .ats_applicationsallop import (
        AtsApplicationsAllGlobals,
        AtsApplicationsAllGlobalsTypedDict,
        AtsApplicationsAllRequest,
        AtsApplicationsAllRequestTypedDict,
        AtsApplicationsAllResponse,
        AtsApplicationsAllResponseTypedDict,
    )
    from .ats_applicationsdeleteop import (
        AtsApplicationsDeleteGlobals,
        AtsApplicationsDeleteGlobalsTypedDict,
        AtsApplicationsDeleteRequest,
        AtsApplicationsDeleteRequestTypedDict,
        AtsApplicationsDeleteResponse,
        AtsApplicationsDeleteResponseTypedDict,
    )
    from .ats_applicationsoneop import (
        AtsApplicationsOneGlobals,
        AtsApplicationsOneGlobalsTypedDict,
        AtsApplicationsOneRequest,
        AtsApplicationsOneRequestTypedDict,
        AtsApplicationsOneResponse,
        AtsApplicationsOneResponseTypedDict,
    )
    from .ats_applicationsupdateop import (
        AtsApplicationsUpdateGlobals,
        AtsApplicationsUpdateGlobalsTypedDict,
        AtsApplicationsUpdateRequest,
        AtsApplicationsUpdateRequestTypedDict,
        AtsApplicationsUpdateResponse,
        AtsApplicationsUpdateResponseTypedDict,
    )
    from .ats_jobsallop import (
        AtsJobsAllGlobals,
        AtsJobsAllGlobalsTypedDict,
        AtsJobsAllRequest,
        AtsJobsAllRequestTypedDict,
        AtsJobsAllResponse,
        AtsJobsAllResponseTypedDict,
    )
    from .ats_jobsoneop import (
        AtsJobsOneGlobals,
        AtsJobsOneGlobalsTypedDict,
        AtsJobsOneRequest,
        AtsJobsOneRequestTypedDict,
        AtsJobsOneResponse,
        AtsJobsOneResponseTypedDict,
    )
    from .attachment import Attachment, AttachmentTypedDict
    from .attachmentreference import AttachmentReference, AttachmentReferenceTypedDict
    from .attachmentreferencetype import AttachmentReferenceType
    from .authtype import AuthType
    from .badrequestresponse import (
        BadRequestResponse,
        BadRequestResponseData,
        BadRequestResponseDetail,
    )
    from .balancebyperiod import BalanceByPeriod, BalanceByPeriodTypedDict
    from .balancebytransaction import (
        BalanceByTransaction,
        BalanceByTransactionTypedDict,
        TransactionType,
    )
    from .balancesheet import (
        BalanceSheet,
        BalanceSheetAssetsAccount,
        BalanceSheetAssetsAccountTypedDict,
        BalanceSheetEquityAccount,
        BalanceSheetEquityAccountTypedDict,
        BalanceSheetLiabilitiesAccount,
        BalanceSheetLiabilitiesAccountTypedDict,
        BalanceSheetTypedDict,
        BalanceSheetUncategorizedItemsAccount,
        BalanceSheetUncategorizedItemsAccountTypedDict,
        Reports,
        ReportsTypedDict,
    )
    from .balancesheetfilter import (
        BalanceSheetFilter,
        BalanceSheetFilterTypedDict,
        PeriodType,
    )
    from .bankaccount import AccountType, BankAccount, BankAccountTypedDict
    from .bankaccount1 import (
        BankAccount1,
        BankAccount1AccountType,
        BankAccount1TypedDict,
    )
    from .bankfeedaccount import (
        BankAccountType,
        BankFeedAccount,
        BankFeedAccountInput,
        BankFeedAccountInputTypedDict,
        BankFeedAccountTypedDict,
        FeedStatus,
    )
    from .bankfeedstatement import (
        BankFeedStatement,
        BankFeedStatementInput,
        BankFeedStatementInputTypedDict,
        BankFeedStatementTransactionType,
        BankFeedStatementTypedDict,
        StatementStatus,
        Transactions,
        TransactionsTypedDict,
    )
    from .benefit import Benefit, BenefitTypedDict
    from .bill import (
        AmortizationType,
        Bill,
        BillInput,
        BillInputTypedDict,
        BillStatus,
        BillTypedDict,
    )
    from .billlineitem import (
        BillLineItem,
        BillLineItemInput,
        BillLineItemInputTypedDict,
        BillLineItemType,
        BillLineItemTypedDict,
    )
    from .billpayment import (
        Allocations,
        AllocationsTypedDict,
        BillPayment,
        BillPaymentAllocationType,
        BillPaymentAllocations,
        BillPaymentAllocationsTypedDict,
        BillPaymentInput,
        BillPaymentInputTypedDict,
        BillPaymentType,
        BillPaymentTypedDict,
    )
    from .billsfilter import BillsFilter, BillsFilterTypedDict
    from .billssort import BillsSort, BillsSortTypedDict, By
    from .categoriesfilter import (
        CategoriesFilter,
        CategoriesFilterType,
        CategoriesFilterTypedDict,
    )
    from .category import Category, CategoryStatus, CategoryType, CategoryTypedDict
    from .collection import Collection, CollectionTypedDict
    from .collectionssort import (
        CollectionsSort,
        CollectionsSortBy,
        CollectionsSortTypedDict,
    )
    from .collectiontag import CollectionTag, CollectionTagTypedDict
    from .collectiontag_input import CollectionTagInput, CollectionTagInputTypedDict
    from .collectionticketcomment import (
        CollectionTicketComment,
        CollectionTicketCommentTypedDict,
    )
    from .collectionticketcomment_input import (
        CollectionTicketCommentInput,
        CollectionTicketCommentInputTypedDict,
    )
    from .collectionuser import CollectionUser, CollectionUserTypedDict
    from .commentssort import CommentsSort, CommentsSortBy, CommentsSortTypedDict
    from .companiesfilter import CompaniesFilter, CompaniesFilterTypedDict
    from .companiessort import CompaniesSort, CompaniesSortBy, CompaniesSortTypedDict
    from .company1 import (
        Company1,
        Company1Input,
        Company1InputTypedDict,
        Company1TypedDict,
        CompanyRowType,
        CompanyRowTypeTypedDict,
    )
    from .companyinfo import (
        CompanyInfo,
        CompanyInfoTypedDict,
        CompanyStatus,
        TheStartMonthOfFiscalYear,
    )
    from .compensation import Compensation, CompensationTypedDict
    from .connection import (
        Configuration,
        ConfigurationTypedDict,
        Connection,
        ConnectionConfiguration,
        ConnectionConfigurationTypedDict,
        ConnectionDefaults,
        ConnectionDefaultsTypedDict,
        ConnectionInput,
        ConnectionInputTypedDict,
        ConnectionStatus,
        ConnectionTypedDict,
        ConnectionValue,
        ConnectionValue5,
        ConnectionValue5TypedDict,
        ConnectionValueTypedDict,
        Defaults,
        DefaultsTypedDict,
        Target,
    )
    from .connectionimportdata import (
        ConnectionImportData,
        ConnectionImportDataTypedDict,
        Credentials,
        CredentialsTypedDict,
    )
    from .connectionstate import ConnectionState
    from .connector import (
        Connector,
        ConnectorAuthType,
        ConnectorOauthGrantType,
        ConnectorOauthScopes,
        ConnectorOauthScopesTypedDict,
        ConnectorTypedDict,
        OauthCredentialsSource,
        OauthScopes,
        OauthScopesTypedDict,
        TLSSupport,
        TLSSupportTypedDict,
        UnifiedApis,
        UnifiedApisTypedDict,
    )
    from .connector_apiresourcecoverageoneop import (
        ConnectorAPIResourceCoverageOneGlobals,
        ConnectorAPIResourceCoverageOneGlobalsTypedDict,
        ConnectorAPIResourceCoverageOneRequest,
        ConnectorAPIResourceCoverageOneRequestTypedDict,
        ConnectorAPIResourceCoverageOneResponse,
        ConnectorAPIResourceCoverageOneResponseTypedDict,
    )
    from .connector_apiresourcesoneop import (
        ConnectorAPIResourcesOneGlobals,
        ConnectorAPIResourcesOneGlobalsTypedDict,
        ConnectorAPIResourcesOneRequest,
        ConnectorAPIResourcesOneRequestTypedDict,
        ConnectorAPIResourcesOneResponse,
        ConnectorAPIResourcesOneResponseTypedDict,
    )
    from .connector_apisallop import (
        ConnectorApisAllGlobals,
        ConnectorApisAllGlobalsTypedDict,
        ConnectorApisAllRequest,
        ConnectorApisAllRequestTypedDict,
        ConnectorApisAllResponse,
        ConnectorApisAllResponseTypedDict,
    )
    from .connector_apisoneop import (
        ConnectorApisOneGlobals,
        ConnectorApisOneGlobalsTypedDict,
        ConnectorApisOneRequest,
        ConnectorApisOneRequestTypedDict,
        ConnectorApisOneResponse,
        ConnectorApisOneResponseTypedDict,
    )
    from .connector_connectordocsoneop import (
        ConnectorConnectorDocsOneGlobals,
        ConnectorConnectorDocsOneGlobalsTypedDict,
        ConnectorConnectorDocsOneRequest,
        ConnectorConnectorDocsOneRequestTypedDict,
        ConnectorConnectorDocsOneResponse,
        ConnectorConnectorDocsOneResponseTypedDict,
    )
    from .connector_connectorresourcesoneop import (
        ConnectorConnectorResourcesOneGlobals,
        ConnectorConnectorResourcesOneGlobalsTypedDict,
        ConnectorConnectorResourcesOneRequest,
        ConnectorConnectorResourcesOneRequestTypedDict,
        ConnectorConnectorResourcesOneResponse,
        ConnectorConnectorResourcesOneResponseTypedDict,
    )
    from .connector_connectorsallop import (
        ConnectorConnectorsAllGlobals,
        ConnectorConnectorsAllGlobalsTypedDict,
        ConnectorConnectorsAllRequest,
        ConnectorConnectorsAllRequestTypedDict,
        ConnectorConnectorsAllResponse,
        ConnectorConnectorsAllResponseTypedDict,
    )
    from .connector_connectorsoneop import (
        ConnectorConnectorsOneGlobals,
        ConnectorConnectorsOneGlobalsTypedDict,
        ConnectorConnectorsOneRequest,
        ConnectorConnectorsOneRequestTypedDict,
        ConnectorConnectorsOneResponse,
        ConnectorConnectorsOneResponseTypedDict,
    )
    from .connectordoc import Audience, ConnectorDoc, ConnectorDocTypedDict, Format
    from .connectorevent import ConnectorEvent, ConnectorEventTypedDict, EventSource
    from .connectorresource import ConnectorResource, ConnectorResourceTypedDict
    from .connectorsetting import (
        ConnectorSetting,
        ConnectorSettingType,
        ConnectorSettingTypedDict,
    )
    from .connectorsfilter import ConnectorsFilter, ConnectorsFilterTypedDict
    from .connectorstatus import ConnectorStatus
    from .consumer import Consumer, ConsumerTypedDict
    from .consumerconnection import ConsumerConnection, ConsumerConnectionTypedDict
    from .consumermetadata import ConsumerMetadata, ConsumerMetadataTypedDict
    from .consumerrequestcountsindaterangeresponse import (
        ConsumerRequestCountsInDateRangeResponse,
        ConsumerRequestCountsInDateRangeResponseData,
        ConsumerRequestCountsInDateRangeResponseDataTypedDict,
        ConsumerRequestCountsInDateRangeResponseTypedDict,
    )
    from .contact import (
        Contact,
        ContactGender,
        ContactInput,
        ContactInputTypedDict,
        ContactType,
        ContactTypedDict,
    )
    from .contactsfilter import ContactsFilter, ContactsFilterTypedDict
    from .contactssort import ContactsSort, ContactsSortBy, ContactsSortTypedDict
    from .copyfolderrequest import CopyFolderRequest, CopyFolderRequestTypedDict
    from .createaccountingdepartmentresponse import (
        CreateAccountingDepartmentResponse,
        CreateAccountingDepartmentResponseTypedDict,
    )
    from .createaccountinglocationresponse import (
        CreateAccountingLocationResponse,
        CreateAccountingLocationResponseTypedDict,
    )
    from .createactivityresponse import (
        CreateActivityResponse,
        CreateActivityResponseTypedDict,
    )
    from .createapplicantresponse import (
        CreateApplicantResponse,
        CreateApplicantResponseTypedDict,
    )
    from .createapplicationresponse import (
        CreateApplicationResponse,
        CreateApplicationResponseTypedDict,
    )
    from .createattachmentresponse import (
        CreateAttachmentResponse,
        CreateAttachmentResponseTypedDict,
    )
    from .createbankfeedaccountresponse import (
        CreateBankFeedAccountResponse,
        CreateBankFeedAccountResponseTypedDict,
    )
    from .createbankfeedstatementresponse import (
        CreateBankFeedStatementResponse,
        CreateBankFeedStatementResponseTypedDict,
    )
    from .createbillpaymentresponse import (
        CreateBillPaymentResponse,
        CreateBillPaymentResponseTypedDict,
    )
    from .createbillresponse import CreateBillResponse, CreateBillResponseTypedDict
    from .createcallbackstate import CreateCallbackState, CreateCallbackStateTypedDict
    from .createcallbackstateresponse import (
        CreateCallbackStateResponse,
        CreateCallbackStateResponseData,
        CreateCallbackStateResponseDataTypedDict,
        CreateCallbackStateResponseTypedDict,
    )
    from .createcommentresponse import (
        CreateCommentResponse,
        CreateCommentResponseTypedDict,
    )
    from .createcompanyresponse import (
        CreateCompanyResponse,
        CreateCompanyResponseTypedDict,
    )
    from .createconnectionresponse import (
        CreateConnectionResponse,
        CreateConnectionResponseTypedDict,
    )
    from .createconsumerrequest import (
        CreateConsumerRequest,
        CreateConsumerRequestTypedDict,
    )
    from .createconsumerresponse import (
        CreateConsumerResponse,
        CreateConsumerResponseTypedDict,
    )
    from .createcontactresponse import (
        CreateContactResponse,
        CreateContactResponseTypedDict,
    )
    from .createcreditnoteresponse import (
        CreateCreditNoteResponse,
        CreateCreditNoteResponseTypedDict,
    )
    from .createcustomerresponse import (
        CreateCustomerResponse,
        CreateCustomerResponseTypedDict,
    )
    from .createcustomobjectresponse import (
        CreateCustomObjectResponse,
        CreateCustomObjectResponseTypedDict,
    )
    from .createcustomobjectschemaresponse import (
        CreateCustomObjectSchemaResponse,
        CreateCustomObjectSchemaResponseTypedDict,
    )
    from .createdepartmentresponse import (
        CreateDepartmentResponse,
        CreateDepartmentResponseTypedDict,
    )
    from .createdrivegroupresponse import (
        CreateDriveGroupResponse,
        CreateDriveGroupResponseTypedDict,
    )
    from .createdriveresponse import CreateDriveResponse, CreateDriveResponseTypedDict
    from .createemployeeresponse import (
        CreateEmployeeResponse,
        CreateEmployeeResponseTypedDict,
    )
    from .createexpenseresponse import (
        CreateExpenseResponse,
        CreateExpenseResponseTypedDict,
    )
    from .createfolderrequest import CreateFolderRequest, CreateFolderRequestTypedDict
    from .createfolderresponse import (
        CreateFolderResponse,
        CreateFolderResponseTypedDict,
    )
    from .createhriscompanyresponse import (
        CreateHrisCompanyResponse,
        CreateHrisCompanyResponseTypedDict,
    )
    from .createinvoiceitemresponse import (
        CreateInvoiceItemResponse,
        CreateInvoiceItemResponseTypedDict,
    )
    from .createinvoiceresponse import (
        CreateInvoiceResponse,
        CreateInvoiceResponseTypedDict,
    )
    from .createjournalentryresponse import (
        CreateJournalEntryResponse,
        CreateJournalEntryResponseTypedDict,
    )
    from .createleadresponse import CreateLeadResponse, CreateLeadResponseTypedDict
    from .createledgeraccountresponse import (
        CreateLedgerAccountResponse,
        CreateLedgerAccountResponseTypedDict,
    )
    from .createmessageresponse import (
        CreateMessageResponse,
        CreateMessageResponseTypedDict,
    )
    from .createnoteresponse import CreateNoteResponse, CreateNoteResponseTypedDict
    from .createopportunityresponse import (
        CreateOpportunityResponse,
        CreateOpportunityResponseTypedDict,
    )
    from .createpaymentresponse import (
        CreatePaymentResponse,
        CreatePaymentResponseTypedDict,
    )
    from .createpipelineresponse import (
        CreatePipelineResponse,
        CreatePipelineResponseTypedDict,
    )
    from .createpurchaseorderresponse import (
        CreatePurchaseOrderResponse,
        CreatePurchaseOrderResponseTypedDict,
    )
    from .createquoteresponse import CreateQuoteResponse, CreateQuoteResponseTypedDict
    from .createsessionresponse import (
        CreateSessionResponse,
        CreateSessionResponseData,
        CreateSessionResponseDataTypedDict,
        CreateSessionResponseTypedDict,
    )
    from .createsharedlinkresponse import (
        CreateSharedLinkResponse,
        CreateSharedLinkResponseTypedDict,
    )
    from .createsubsidiaryresponse import (
        CreateSubsidiaryResponse,
        CreateSubsidiaryResponseTypedDict,
    )
    from .createsupplierresponse import (
        CreateSupplierResponse,
        CreateSupplierResponseTypedDict,
    )
    from .createtaxrateresponse import (
        CreateTaxRateResponse,
        CreateTaxRateResponseTypedDict,
    )
    from .createticketresponse import (
        CreateTicketResponse,
        CreateTicketResponseTypedDict,
    )
    from .createtimeoffrequestresponse import (
        CreateTimeOffRequestResponse,
        CreateTimeOffRequestResponseTypedDict,
    )
    from .createtrackingcategoryresponse import (
        CreateTrackingCategoryResponse,
        CreateTrackingCategoryResponseTypedDict,
    )
    from .createuploadsessionrequest import (
        CreateUploadSessionRequest,
        CreateUploadSessionRequestTypedDict,
    )
    from .createuploadsessionresponse import (
        CreateUploadSessionResponse,
        CreateUploadSessionResponseTypedDict,
    )
    from .createuserresponse import CreateUserResponse, CreateUserResponseTypedDict
    from .createwebhookrequest import (
        CreateWebhookRequest,
        CreateWebhookRequestTypedDict,
    )
    from .createwebhookresponse import (
        CreateWebhookResponse,
        CreateWebhookResponseTypedDict,
    )
    from .creditnote import (
        CreditNote,
        CreditNoteInput,
        CreditNoteInputTypedDict,
        CreditNoteStatus,
        CreditNoteType,
        CreditNoteTypedDict,
    )
    from .creditnotesfilter import CreditNotesFilter, CreditNotesFilterTypedDict
    from .creditnotessort import (
        CreditNotesSort,
        CreditNotesSortBy,
        CreditNotesSortTypedDict,
    )
    from .creditordebit import CreditOrDebit
    from .crm_activitiesaddop import (
        CrmActivitiesAddGlobals,
        CrmActivitiesAddGlobalsTypedDict,
        CrmActivitiesAddRequest,
        CrmActivitiesAddRequestTypedDict,
        CrmActivitiesAddResponse,
        CrmActivitiesAddResponseTypedDict,
    )
    from .crm_activitiesallop import (
        CrmActivitiesAllGlobals,
        CrmActivitiesAllGlobalsTypedDict,
        CrmActivitiesAllRequest,
        CrmActivitiesAllRequestTypedDict,
        CrmActivitiesAllResponse,
        CrmActivitiesAllResponseTypedDict,
    )
    from .crm_activitiesdeleteop import (
        CrmActivitiesDeleteGlobals,
        CrmActivitiesDeleteGlobalsTypedDict,
        CrmActivitiesDeleteRequest,
        CrmActivitiesDeleteRequestTypedDict,
        CrmActivitiesDeleteResponse,
        CrmActivitiesDeleteResponseTypedDict,
    )
    from .crm_activitiesoneop import (
        CrmActivitiesOneGlobals,
        CrmActivitiesOneGlobalsTypedDict,
        CrmActivitiesOneRequest,
        CrmActivitiesOneRequestTypedDict,
        CrmActivitiesOneResponse,
        CrmActivitiesOneResponseTypedDict,
    )
    from .crm_activitiesupdateop import (
        CrmActivitiesUpdateGlobals,
        CrmActivitiesUpdateGlobalsTypedDict,
        CrmActivitiesUpdateRequest,
        CrmActivitiesUpdateRequestTypedDict,
        CrmActivitiesUpdateResponse,
        CrmActivitiesUpdateResponseTypedDict,
    )
    from .crm_companiesaddop import (
        CrmCompaniesAddGlobals,
        CrmCompaniesAddGlobalsTypedDict,
        CrmCompaniesAddRequest,
        CrmCompaniesAddRequestTypedDict,
        CrmCompaniesAddResponse,
        CrmCompaniesAddResponseTypedDict,
    )
    from .crm_companiesallop import (
        CrmCompaniesAllGlobals,
        CrmCompaniesAllGlobalsTypedDict,
        CrmCompaniesAllRequest,
        CrmCompaniesAllRequestTypedDict,
        CrmCompaniesAllResponse,
        CrmCompaniesAllResponseTypedDict,
    )
    from .crm_companiesdeleteop import (
        CrmCompaniesDeleteGlobals,
        CrmCompaniesDeleteGlobalsTypedDict,
        CrmCompaniesDeleteRequest,
        CrmCompaniesDeleteRequestTypedDict,
        CrmCompaniesDeleteResponse,
        CrmCompaniesDeleteResponseTypedDict,
    )
    from .crm_companiesoneop import (
        CrmCompaniesOneGlobals,
        CrmCompaniesOneGlobalsTypedDict,
        CrmCompaniesOneRequest,
        CrmCompaniesOneRequestTypedDict,
        CrmCompaniesOneResponse,
        CrmCompaniesOneResponseTypedDict,
    )
    from .crm_companiesupdateop import (
        CrmCompaniesUpdateGlobals,
        CrmCompaniesUpdateGlobalsTypedDict,
        CrmCompaniesUpdateRequest,
        CrmCompaniesUpdateRequestTypedDict,
        CrmCompaniesUpdateResponse,
        CrmCompaniesUpdateResponseTypedDict,
    )
    from .crm_contactsaddop import (
        CrmContactsAddGlobals,
        CrmContactsAddGlobalsTypedDict,
        CrmContactsAddRequest,
        CrmContactsAddRequestTypedDict,
        CrmContactsAddResponse,
        CrmContactsAddResponseTypedDict,
    )
    from .crm_contactsallop import (
        CrmContactsAllGlobals,
        CrmContactsAllGlobalsTypedDict,
        CrmContactsAllRequest,
        CrmContactsAllRequestTypedDict,
        CrmContactsAllResponse,
        CrmContactsAllResponseTypedDict,
    )
    from .crm_contactsdeleteop import (
        CrmContactsDeleteGlobals,
        CrmContactsDeleteGlobalsTypedDict,
        CrmContactsDeleteRequest,
        CrmContactsDeleteRequestTypedDict,
        CrmContactsDeleteResponse,
        CrmContactsDeleteResponseTypedDict,
    )
    from .crm_contactsoneop import (
        CrmContactsOneGlobals,
        CrmContactsOneGlobalsTypedDict,
        CrmContactsOneRequest,
        CrmContactsOneRequestTypedDict,
        CrmContactsOneResponse,
        CrmContactsOneResponseTypedDict,
    )
    from .crm_contactsupdateop import (
        CrmContactsUpdateGlobals,
        CrmContactsUpdateGlobalsTypedDict,
        CrmContactsUpdateRequest,
        CrmContactsUpdateRequestTypedDict,
        CrmContactsUpdateResponse,
        CrmContactsUpdateResponseTypedDict,
    )
    from .crm_customobjectsaddop import (
        CrmCustomObjectsAddGlobals,
        CrmCustomObjectsAddGlobalsTypedDict,
        CrmCustomObjectsAddRequest,
        CrmCustomObjectsAddRequestTypedDict,
        CrmCustomObjectsAddResponse,
        CrmCustomObjectsAddResponseTypedDict,
    )
    from .crm_customobjectsallop import (
        CrmCustomObjectsAllGlobals,
        CrmCustomObjectsAllGlobalsTypedDict,
        CrmCustomObjectsAllRequest,
        CrmCustomObjectsAllRequestTypedDict,
        CrmCustomObjectsAllResponse,
        CrmCustomObjectsAllResponseTypedDict,
    )
    from .crm_customobjectschemasaddop import (
        CrmCustomObjectSchemasAddGlobals,
        CrmCustomObjectSchemasAddGlobalsTypedDict,
        CrmCustomObjectSchemasAddRequest,
        CrmCustomObjectSchemasAddRequestTypedDict,
        CrmCustomObjectSchemasAddResponse,
        CrmCustomObjectSchemasAddResponseTypedDict,
    )
    from .crm_customobjectschemasallop import (
        CrmCustomObjectSchemasAllGlobals,
        CrmCustomObjectSchemasAllGlobalsTypedDict,
        CrmCustomObjectSchemasAllRequest,
        CrmCustomObjectSchemasAllRequestTypedDict,
        CrmCustomObjectSchemasAllResponse,
        CrmCustomObjectSchemasAllResponseTypedDict,
    )
    from .crm_customobjectschemasdeleteop import (
        CrmCustomObjectSchemasDeleteGlobals,
        CrmCustomObjectSchemasDeleteGlobalsTypedDict,
        CrmCustomObjectSchemasDeleteRequest,
        CrmCustomObjectSchemasDeleteRequestTypedDict,
        CrmCustomObjectSchemasDeleteResponse,
        CrmCustomObjectSchemasDeleteResponseTypedDict,
    )
    from .crm_customobjectschemasoneop import (
        CrmCustomObjectSchemasOneGlobals,
        CrmCustomObjectSchemasOneGlobalsTypedDict,
        CrmCustomObjectSchemasOneRequest,
        CrmCustomObjectSchemasOneRequestTypedDict,
        CrmCustomObjectSchemasOneResponse,
        CrmCustomObjectSchemasOneResponseTypedDict,
    )
    from .crm_customobjectschemasupdateop import (
        CrmCustomObjectSchemasUpdateGlobals,
        CrmCustomObjectSchemasUpdateGlobalsTypedDict,
        CrmCustomObjectSchemasUpdateRequest,
        CrmCustomObjectSchemasUpdateRequestTypedDict,
        CrmCustomObjectSchemasUpdateResponse,
        CrmCustomObjectSchemasUpdateResponseTypedDict,
    )
    from .crm_customobjectsdeleteop import (
        CrmCustomObjectsDeleteGlobals,
        CrmCustomObjectsDeleteGlobalsTypedDict,
        CrmCustomObjectsDeleteRequest,
        CrmCustomObjectsDeleteRequestTypedDict,
        CrmCustomObjectsDeleteResponse,
        CrmCustomObjectsDeleteResponseTypedDict,
    )
    from .crm_customobjectsoneop import (
        CrmCustomObjectsOneGlobals,
        CrmCustomObjectsOneGlobalsTypedDict,
        CrmCustomObjectsOneRequest,
        CrmCustomObjectsOneRequestTypedDict,
        CrmCustomObjectsOneResponse,
        CrmCustomObjectsOneResponseTypedDict,
    )
    from .crm_customobjectsupdateop import (
        CrmCustomObjectsUpdateGlobals,
        CrmCustomObjectsUpdateGlobalsTypedDict,
        CrmCustomObjectsUpdateRequest,
        CrmCustomObjectsUpdateRequestTypedDict,
        CrmCustomObjectsUpdateResponse,
        CrmCustomObjectsUpdateResponseTypedDict,
    )
    from .crm_leadsaddop import (
        CrmLeadsAddGlobals,
        CrmLeadsAddGlobalsTypedDict,
        CrmLeadsAddRequest,
        CrmLeadsAddRequestTypedDict,
        CrmLeadsAddResponse,
        CrmLeadsAddResponseTypedDict,
    )
    from .crm_leadsallop import (
        CrmLeadsAllGlobals,
        CrmLeadsAllGlobalsTypedDict,
        CrmLeadsAllRequest,
        CrmLeadsAllRequestTypedDict,
        CrmLeadsAllResponse,
        CrmLeadsAllResponseTypedDict,
    )
    from .crm_leadsdeleteop import (
        CrmLeadsDeleteGlobals,
        CrmLeadsDeleteGlobalsTypedDict,
        CrmLeadsDeleteRequest,
        CrmLeadsDeleteRequestTypedDict,
        CrmLeadsDeleteResponse,
        CrmLeadsDeleteResponseTypedDict,
    )
    from .crm_leadsoneop import (
        CrmLeadsOneGlobals,
        CrmLeadsOneGlobalsTypedDict,
        CrmLeadsOneRequest,
        CrmLeadsOneRequestTypedDict,
        CrmLeadsOneResponse,
        CrmLeadsOneResponseTypedDict,
    )
    from .crm_leadsupdateop import (
        CrmLeadsUpdateGlobals,
        CrmLeadsUpdateGlobalsTypedDict,
        CrmLeadsUpdateRequest,
        CrmLeadsUpdateRequestTypedDict,
        CrmLeadsUpdateResponse,
        CrmLeadsUpdateResponseTypedDict,
    )
    from .crm_notesaddop import (
        CrmNotesAddGlobals,
        CrmNotesAddGlobalsTypedDict,
        CrmNotesAddRequest,
        CrmNotesAddRequestTypedDict,
        CrmNotesAddResponse,
        CrmNotesAddResponseTypedDict,
    )
    from .crm_notesallop import (
        CrmNotesAllGlobals,
        CrmNotesAllGlobalsTypedDict,
        CrmNotesAllRequest,
        CrmNotesAllRequestTypedDict,
        CrmNotesAllResponse,
        CrmNotesAllResponseTypedDict,
    )
    from .crm_notesdeleteop import (
        CrmNotesDeleteGlobals,
        CrmNotesDeleteGlobalsTypedDict,
        CrmNotesDeleteRequest,
        CrmNotesDeleteRequestTypedDict,
        CrmNotesDeleteResponse,
        CrmNotesDeleteResponseTypedDict,
    )
    from .crm_notesoneop import (
        CrmNotesOneGlobals,
        CrmNotesOneGlobalsTypedDict,
        CrmNotesOneRequest,
        CrmNotesOneRequestTypedDict,
        CrmNotesOneResponse,
        CrmNotesOneResponseTypedDict,
    )
    from .crm_notesupdateop import (
        CrmNotesUpdateGlobals,
        CrmNotesUpdateGlobalsTypedDict,
        CrmNotesUpdateRequest,
        CrmNotesUpdateRequestTypedDict,
        CrmNotesUpdateResponse,
        CrmNotesUpdateResponseTypedDict,
    )
    from .crm_opportunitiesaddop import (
        CrmOpportunitiesAddGlobals,
        CrmOpportunitiesAddGlobalsTypedDict,
        CrmOpportunitiesAddRequest,
        CrmOpportunitiesAddRequestTypedDict,
        CrmOpportunitiesAddResponse,
        CrmOpportunitiesAddResponseTypedDict,
    )
    from .crm_opportunitiesallop import (
        CrmOpportunitiesAllGlobals,
        CrmOpportunitiesAllGlobalsTypedDict,
        CrmOpportunitiesAllRequest,
        CrmOpportunitiesAllRequestTypedDict,
        CrmOpportunitiesAllResponse,
        CrmOpportunitiesAllResponseTypedDict,
    )
    from .crm_opportunitiesdeleteop import (
        CrmOpportunitiesDeleteGlobals,
        CrmOpportunitiesDeleteGlobalsTypedDict,
        CrmOpportunitiesDeleteRequest,
        CrmOpportunitiesDeleteRequestTypedDict,
        CrmOpportunitiesDeleteResponse,
        CrmOpportunitiesDeleteResponseTypedDict,
    )
    from .crm_opportunitiesoneop import (
        CrmOpportunitiesOneGlobals,
        CrmOpportunitiesOneGlobalsTypedDict,
        CrmOpportunitiesOneRequest,
        CrmOpportunitiesOneRequestTypedDict,
        CrmOpportunitiesOneResponse,
        CrmOpportunitiesOneResponseTypedDict,
    )
    from .crm_opportunitiesupdateop import (
        CrmOpportunitiesUpdateGlobals,
        CrmOpportunitiesUpdateGlobalsTypedDict,
        CrmOpportunitiesUpdateRequest,
        CrmOpportunitiesUpdateRequestTypedDict,
        CrmOpportunitiesUpdateResponse,
        CrmOpportunitiesUpdateResponseTypedDict,
    )
    from .crm_pipelinesaddop import (
        CrmPipelinesAddGlobals,
        CrmPipelinesAddGlobalsTypedDict,
        CrmPipelinesAddRequest,
        CrmPipelinesAddRequestTypedDict,
        CrmPipelinesAddResponse,
        CrmPipelinesAddResponseTypedDict,
    )
    from .crm_pipelinesallop import (
        CrmPipelinesAllGlobals,
        CrmPipelinesAllGlobalsTypedDict,
        CrmPipelinesAllRequest,
        CrmPipelinesAllRequestTypedDict,
        CrmPipelinesAllResponse,
        CrmPipelinesAllResponseTypedDict,
    )
    from .crm_pipelinesdeleteop import (
        CrmPipelinesDeleteGlobals,
        CrmPipelinesDeleteGlobalsTypedDict,
        CrmPipelinesDeleteRequest,
        CrmPipelinesDeleteRequestTypedDict,
        CrmPipelinesDeleteResponse,
        CrmPipelinesDeleteResponseTypedDict,
    )
    from .crm_pipelinesoneop import (
        CrmPipelinesOneGlobals,
        CrmPipelinesOneGlobalsTypedDict,
        CrmPipelinesOneRequest,
        CrmPipelinesOneRequestTypedDict,
        CrmPipelinesOneResponse,
        CrmPipelinesOneResponseTypedDict,
    )
    from .crm_pipelinesupdateop import (
        CrmPipelinesUpdateGlobals,
        CrmPipelinesUpdateGlobalsTypedDict,
        CrmPipelinesUpdateRequest,
        CrmPipelinesUpdateRequestTypedDict,
        CrmPipelinesUpdateResponse,
        CrmPipelinesUpdateResponseTypedDict,
    )
    from .crm_usersaddop import (
        CrmUsersAddGlobals,
        CrmUsersAddGlobalsTypedDict,
        CrmUsersAddRequest,
        CrmUsersAddRequestTypedDict,
        CrmUsersAddResponse,
        CrmUsersAddResponseTypedDict,
    )
    from .crm_usersallop import (
        CrmUsersAllGlobals,
        CrmUsersAllGlobalsTypedDict,
        CrmUsersAllRequest,
        CrmUsersAllRequestTypedDict,
        CrmUsersAllResponse,
        CrmUsersAllResponseTypedDict,
    )
    from .crm_usersdeleteop import (
        CrmUsersDeleteGlobals,
        CrmUsersDeleteGlobalsTypedDict,
        CrmUsersDeleteRequest,
        CrmUsersDeleteRequestTypedDict,
        CrmUsersDeleteResponse,
        CrmUsersDeleteResponseTypedDict,
    )
    from .crm_usersoneop import (
        CrmUsersOneGlobals,
        CrmUsersOneGlobalsTypedDict,
        CrmUsersOneRequest,
        CrmUsersOneRequestTypedDict,
        CrmUsersOneResponse,
        CrmUsersOneResponseTypedDict,
    )
    from .crm_usersupdateop import (
        CrmUsersUpdateGlobals,
        CrmUsersUpdateGlobalsTypedDict,
        CrmUsersUpdateRequest,
        CrmUsersUpdateRequestTypedDict,
        CrmUsersUpdateResponse,
        CrmUsersUpdateResponseTypedDict,
    )
    from .currency import Currency
    from .customer import (
        Customer,
        CustomerInput,
        CustomerInputTypedDict,
        CustomerStatusStatus,
        CustomerTypedDict,
    )
    from .customersfilter import (
        CustomersFilter,
        CustomersFilterStatus,
        CustomersFilterTypedDict,
    )
    from .customerssort import CustomersSort, CustomersSortBy, CustomersSortTypedDict
    from .customfield import (
        CustomField,
        CustomFieldTypedDict,
        Five,
        FiveTypedDict,
        Value,
        ValueTypedDict,
    )
    from .customfieldfinder import CustomFieldFinder, CustomFieldFinderTypedDict
    from .custommapping import CustomMapping, CustomMappingTypedDict
    from .custommapping_input import CustomMappingInput, CustomMappingInputTypedDict
    from .customobject import (
        CustomObject,
        CustomObjectFields,
        CustomObjectFieldsTypedDict,
        CustomObjectInput,
        CustomObjectInputTypedDict,
        CustomObjectTypedDict,
    )
    from .customobjectschema import (
        CustomObjectSchema,
        CustomObjectSchemaInput,
        CustomObjectSchemaInputTypedDict,
        CustomObjectSchemaOptions,
        CustomObjectSchemaOptionsTypedDict,
        CustomObjectSchemaType,
        CustomObjectSchemaTypedDict,
        Fields,
        FieldsTypedDict,
    )
    from .deduction import Deduction, DeductionTypedDict
    from .deleteaccountingdepartmentresponse import (
        DeleteAccountingDepartmentResponse,
        DeleteAccountingDepartmentResponseTypedDict,
    )
    from .deleteaccountinglocationresponse import (
        DeleteAccountingLocationResponse,
        DeleteAccountingLocationResponseTypedDict,
    )
    from .deleteactivityresponse import (
        DeleteActivityResponse,
        DeleteActivityResponseTypedDict,
    )
    from .deleteapplicantresponse import (
        DeleteApplicantResponse,
        DeleteApplicantResponseTypedDict,
    )
    from .deleteapplicationresponse import (
        DeleteApplicationResponse,
        DeleteApplicationResponseTypedDict,
    )
    from .deleteattachmentresponse import (
        DeleteAttachmentResponse,
        DeleteAttachmentResponseTypedDict,
    )
    from .deletebankfeedaccountresponse import (
        DeleteBankFeedAccountResponse,
        DeleteBankFeedAccountResponseTypedDict,
    )
    from .deletebankfeedstatementresponse import (
        DeleteBankFeedStatementResponse,
        DeleteBankFeedStatementResponseTypedDict,
    )
    from .deletebillpaymentresponse import (
        DeleteBillPaymentResponse,
        DeleteBillPaymentResponseTypedDict,
    )
    from .deletebillresponse import DeleteBillResponse, DeleteBillResponseTypedDict
    from .deletecommentresponse import (
        DeleteCommentResponse,
        DeleteCommentResponseTypedDict,
    )
    from .deletecompanyresponse import (
        DeleteCompanyResponse,
        DeleteCompanyResponseTypedDict,
    )
    from .deleteconsumerresponse import (
        DeleteConsumerResponse,
        DeleteConsumerResponseData,
        DeleteConsumerResponseDataTypedDict,
        DeleteConsumerResponseTypedDict,
    )
    from .deletecontactresponse import (
        DeleteContactResponse,
        DeleteContactResponseTypedDict,
    )
    from .deletecreditnoteresponse import (
        DeleteCreditNoteResponse,
        DeleteCreditNoteResponseTypedDict,
    )
    from .deletecustomerresponse import (
        DeleteCustomerResponse,
        DeleteCustomerResponseTypedDict,
    )
    from .deletecustomobjectresponse import (
        DeleteCustomObjectResponse,
        DeleteCustomObjectResponseTypedDict,
    )
    from .deletecustomobjectschemaresponse import (
        DeleteCustomObjectSchemaResponse,
        DeleteCustomObjectSchemaResponseTypedDict,
    )
    from .deletedepartmentresponse import (
        DeleteDepartmentResponse,
        DeleteDepartmentResponseTypedDict,
    )
    from .deletedrivegroupresponse import (
        DeleteDriveGroupResponse,
        DeleteDriveGroupResponseTypedDict,
    )
    from .deletedriveresponse import DeleteDriveResponse, DeleteDriveResponseTypedDict
    from .deleteemployeeresponse import (
        DeleteEmployeeResponse,
        DeleteEmployeeResponseTypedDict,
    )
    from .deleteexpenseresponse import (
        DeleteExpenseResponse,
        DeleteExpenseResponseTypedDict,
    )
    from .deletefileresponse import DeleteFileResponse, DeleteFileResponseTypedDict
    from .deletefolderresponse import (
        DeleteFolderResponse,
        DeleteFolderResponseTypedDict,
    )
    from .deletehriscompanyresponse import (
        DeleteHrisCompanyResponse,
        DeleteHrisCompanyResponseTypedDict,
    )
    from .deleteinvoiceresponse import (
        DeleteInvoiceResponse,
        DeleteInvoiceResponseTypedDict,
    )
    from .deletejournalentryresponse import (
        DeleteJournalEntryResponse,
        DeleteJournalEntryResponseTypedDict,
    )
    from .deleteleadresponse import DeleteLeadResponse, DeleteLeadResponseTypedDict
    from .deleteledgeraccountresponse import (
        DeleteLedgerAccountResponse,
        DeleteLedgerAccountResponseTypedDict,
    )
    from .deletemessageresponse import (
        DeleteMessageResponse,
        DeleteMessageResponseTypedDict,
    )
    from .deletenoteresponse import DeleteNoteResponse, DeleteNoteResponseTypedDict
    from .deleteopportunityresponse import (
        DeleteOpportunityResponse,
        DeleteOpportunityResponseTypedDict,
    )
    from .deletepaymentresponse import (
        DeletePaymentResponse,
        DeletePaymentResponseTypedDict,
    )
    from .deletepipelineresponse import (
        DeletePipelineResponse,
        DeletePipelineResponseTypedDict,
    )
    from .deletepurchaseorderresponse import (
        DeletePurchaseOrderResponse,
        DeletePurchaseOrderResponseTypedDict,
    )
    from .deletequoteresponse import DeleteQuoteResponse, DeleteQuoteResponseTypedDict
    from .deletesharedlinkresponse import (
        DeleteSharedLinkResponse,
        DeleteSharedLinkResponseTypedDict,
    )
    from .deletesubsidiaryresponse import (
        DeleteSubsidiaryResponse,
        DeleteSubsidiaryResponseTypedDict,
    )
    from .deletesupplierresponse import (
        DeleteSupplierResponse,
        DeleteSupplierResponseTypedDict,
    )
    from .deletetaxrateresponse import (
        DeleteTaxRateResponse,
        DeleteTaxRateResponseTypedDict,
    )
    from .deleteticketresponse import (
        DeleteTicketResponse,
        DeleteTicketResponseTypedDict,
    )
    from .deletetimeoffrequestresponse import (
        DeleteTimeOffRequestResponse,
        DeleteTimeOffRequestResponseTypedDict,
    )
    from .deletetrackingcategoryresponse import (
        DeleteTrackingCategoryResponse,
        DeleteTrackingCategoryResponseTypedDict,
    )
    from .deleteuploadsessionresponse import (
        DeleteUploadSessionResponse,
        DeleteUploadSessionResponseTypedDict,
    )
    from .deleteuserresponse import DeleteUserResponse, DeleteUserResponseTypedDict
    from .deletewebhookresponse import (
        DeleteWebhookResponse,
        DeleteWebhookResponseTypedDict,
    )
    from .department import Department, DepartmentTypedDict
    from .department_input import DepartmentInput, DepartmentInputTypedDict
    from .deprecatedlinkedsupplier import (
        DeprecatedLinkedSupplier,
        DeprecatedLinkedSupplierTypedDict,
    )
    from .deprecatedlinkedsupplier_input import (
        DeprecatedLinkedSupplierInput,
        DeprecatedLinkedSupplierInputTypedDict,
    )
    from .deprecatedlinkedtrackingcategory import (
        DeprecatedLinkedTrackingCategory,
        DeprecatedLinkedTrackingCategoryTypedDict,
    )
    from .drive import Drive, DriveTypedDict
    from .drive_input import DriveInput, DriveInputTypedDict
    from .drivegroup import DriveGroup, DriveGroupTypedDict
    from .drivegroup_input import DriveGroupInput, DriveGroupInputTypedDict
    from .drivegroupsfilter import DriveGroupsFilter, DriveGroupsFilterTypedDict
    from .drivesfilter import DrivesFilter, DrivesFilterTypedDict
    from .ecommerce_customersallop import (
        EcommerceCustomersAllGlobals,
        EcommerceCustomersAllGlobalsTypedDict,
        EcommerceCustomersAllRequest,
        EcommerceCustomersAllRequestTypedDict,
        EcommerceCustomersAllResponse,
        EcommerceCustomersAllResponseTypedDict,
    )
    from .ecommerce_customersoneop import (
        EcommerceCustomersOneGlobals,
        EcommerceCustomersOneGlobalsTypedDict,
        EcommerceCustomersOneRequest,
        EcommerceCustomersOneRequestTypedDict,
        EcommerceCustomersOneResponse,
        EcommerceCustomersOneResponseTypedDict,
    )
    from .ecommerce_ordersallop import (
        EcommerceOrdersAllGlobals,
        EcommerceOrdersAllGlobalsTypedDict,
        EcommerceOrdersAllRequest,
        EcommerceOrdersAllRequestTypedDict,
        EcommerceOrdersAllResponse,
        EcommerceOrdersAllResponseTypedDict,
    )
    from .ecommerce_ordersoneop import (
        EcommerceOrdersOneGlobals,
        EcommerceOrdersOneGlobalsTypedDict,
        EcommerceOrdersOneRequest,
        EcommerceOrdersOneRequestTypedDict,
        EcommerceOrdersOneResponse,
        EcommerceOrdersOneResponseTypedDict,
    )
    from .ecommerce_productsallop import (
        EcommerceProductsAllGlobals,
        EcommerceProductsAllGlobalsTypedDict,
        EcommerceProductsAllRequest,
        EcommerceProductsAllRequestTypedDict,
        EcommerceProductsAllResponse,
        EcommerceProductsAllResponseTypedDict,
    )
    from .ecommerce_productsoneop import (
        EcommerceProductsOneGlobals,
        EcommerceProductsOneGlobalsTypedDict,
        EcommerceProductsOneRequest,
        EcommerceProductsOneRequestTypedDict,
        EcommerceProductsOneResponse,
        EcommerceProductsOneResponseTypedDict,
    )
    from .ecommerce_storesoneop import (
        EcommerceStoresOneGlobals,
        EcommerceStoresOneGlobalsTypedDict,
        EcommerceStoresOneRequest,
        EcommerceStoresOneRequestTypedDict,
        EcommerceStoresOneResponse,
        EcommerceStoresOneResponseTypedDict,
    )
    from .ecommerceaddress import EcommerceAddress, EcommerceAddressTypedDict
    from .ecommercecustomer import (
        Addresses,
        AddressesTypedDict,
        CustomerStatus,
        EcommerceCustomer,
        EcommerceCustomerType,
        EcommerceCustomerTypedDict,
    )
    from .ecommercecustomersfilter import (
        EcommerceCustomersFilter,
        EcommerceCustomersFilterTypedDict,
    )
    from .ecommercediscount import EcommerceDiscount, EcommerceDiscountTypedDict
    from .ecommerceorder import (
        EcommerceOrder,
        EcommerceOrderPaymentStatus,
        EcommerceOrderTypedDict,
        FulfillmentStatus,
    )
    from .ecommerceorderlineitem import (
        EcommerceOrderLineItem,
        EcommerceOrderLineItemTypedDict,
        Options,
        OptionsTypedDict,
    )
    from .ecommerceorderrefund import (
        EcommerceOrderRefund,
        EcommerceOrderRefundTypedDict,
    )
    from .ecommerceordersfilter import (
        EcommerceOrdersFilter,
        EcommerceOrdersFilterTypedDict,
    )
    from .ecommerceorderstatus import EcommerceOrderStatus
    from .ecommerceproduct import (
        EcommerceProduct,
        EcommerceProductCategories,
        EcommerceProductCategoriesTypedDict,
        EcommerceProductImages,
        EcommerceProductImagesTypedDict,
        EcommerceProductOptions,
        EcommerceProductOptionsTypedDict,
        EcommerceProductTypedDict,
        EcommerceProductVariantsOptions,
        EcommerceProductVariantsOptionsTypedDict,
        Images,
        ImagesTypedDict,
        ProductStatus,
        Variants,
        VariantsTypedDict,
    )
    from .ecommerceproductsfilter import (
        EcommerceProductsFilter,
        EcommerceProductsFilterTypedDict,
    )
    from .ecommercestore import EcommerceStore, EcommerceStoreTypedDict
    from .email import Email, EmailType, EmailTypedDict
    from .employee import (
        Employee,
        EmployeeInput,
        EmployeeInputTypedDict,
        EmployeeTypedDict,
        EmploymentRole,
        EmploymentRoleTypedDict,
        EmploymentSubType,
        EmploymentType,
        LeavingReason,
        Manager,
        ManagerTypedDict,
        ProbationPeriod,
        ProbationPeriodTypedDict,
    )
    from .employeecompensation import (
        EmployeeCompensation,
        EmployeeCompensationInput,
        EmployeeCompensationInputTypedDict,
        EmployeeCompensationTypedDict,
        FlsaStatus,
    )
    from .employeejob import (
        EmployeeJob,
        EmployeeJobInput,
        EmployeeJobInputTypedDict,
        EmployeeJobStatus,
        EmployeeJobTypedDict,
    )
    from .employeepayroll import EmployeePayroll, EmployeePayrollTypedDict
    from .employeeschedules import EmployeeSchedules, EmployeeSchedulesTypedDict
    from .employeesfilter import (
        EmployeesFilter,
        EmployeesFilterEmploymentStatus,
        EmployeesFilterTypedDict,
    )
    from .employeesonefilter import EmployeesOneFilter, EmployeesOneFilterTypedDict
    from .employeessort import EmployeesSort, EmployeesSortBy, EmployeesSortTypedDict
    from .employmentstatus import EmploymentStatus
    from .expense import (
        Expense,
        ExpenseInput,
        ExpenseInputTypedDict,
        ExpensePaymentType,
        ExpenseType,
        ExpenseTypedDict,
    )
    from .expenselineitem import ExpenseLineItem, ExpenseLineItemTypedDict
    from .expenselineitem_input import (
        ExpenseLineItemInput,
        ExpenseLineItemInputTypedDict,
    )
    from .filesfilter import FilesFilter, FilesFilterTypedDict
    from .filessearch import FilesSearch, FilesSearchTypedDict
    from .filessort import FilesSort, FilesSortBy, FilesSortTypedDict
    from .filestorage_drivegroupsaddop import (
        FileStorageDriveGroupsAddGlobals,
        FileStorageDriveGroupsAddGlobalsTypedDict,
        FileStorageDriveGroupsAddRequest,
        FileStorageDriveGroupsAddRequestTypedDict,
        FileStorageDriveGroupsAddResponse,
        FileStorageDriveGroupsAddResponseTypedDict,
    )
    from .filestorage_drivegroupsallop import (
        FileStorageDriveGroupsAllGlobals,
        FileStorageDriveGroupsAllGlobalsTypedDict,
        FileStorageDriveGroupsAllRequest,
        FileStorageDriveGroupsAllRequestTypedDict,
        FileStorageDriveGroupsAllResponse,
        FileStorageDriveGroupsAllResponseTypedDict,
    )
    from .filestorage_drivegroupsdeleteop import (
        FileStorageDriveGroupsDeleteGlobals,
        FileStorageDriveGroupsDeleteGlobalsTypedDict,
        FileStorageDriveGroupsDeleteRequest,
        FileStorageDriveGroupsDeleteRequestTypedDict,
        FileStorageDriveGroupsDeleteResponse,
        FileStorageDriveGroupsDeleteResponseTypedDict,
    )
    from .filestorage_drivegroupsoneop import (
        FileStorageDriveGroupsOneGlobals,
        FileStorageDriveGroupsOneGlobalsTypedDict,
        FileStorageDriveGroupsOneRequest,
        FileStorageDriveGroupsOneRequestTypedDict,
        FileStorageDriveGroupsOneResponse,
        FileStorageDriveGroupsOneResponseTypedDict,
    )
    from .filestorage_drivegroupsupdateop import (
        FileStorageDriveGroupsUpdateGlobals,
        FileStorageDriveGroupsUpdateGlobalsTypedDict,
        FileStorageDriveGroupsUpdateRequest,
        FileStorageDriveGroupsUpdateRequestTypedDict,
        FileStorageDriveGroupsUpdateResponse,
        FileStorageDriveGroupsUpdateResponseTypedDict,
    )
    from .filestorage_drivesaddop import (
        FileStorageDrivesAddGlobals,
        FileStorageDrivesAddGlobalsTypedDict,
        FileStorageDrivesAddRequest,
        FileStorageDrivesAddRequestTypedDict,
        FileStorageDrivesAddResponse,
        FileStorageDrivesAddResponseTypedDict,
    )
    from .filestorage_drivesallop import (
        FileStorageDrivesAllGlobals,
        FileStorageDrivesAllGlobalsTypedDict,
        FileStorageDrivesAllRequest,
        FileStorageDrivesAllRequestTypedDict,
        FileStorageDrivesAllResponse,
        FileStorageDrivesAllResponseTypedDict,
    )
    from .filestorage_drivesdeleteop import (
        FileStorageDrivesDeleteGlobals,
        FileStorageDrivesDeleteGlobalsTypedDict,
        FileStorageDrivesDeleteRequest,
        FileStorageDrivesDeleteRequestTypedDict,
        FileStorageDrivesDeleteResponse,
        FileStorageDrivesDeleteResponseTypedDict,
    )
    from .filestorage_drivesoneop import (
        FileStorageDrivesOneGlobals,
        FileStorageDrivesOneGlobalsTypedDict,
        FileStorageDrivesOneRequest,
        FileStorageDrivesOneRequestTypedDict,
        FileStorageDrivesOneResponse,
        FileStorageDrivesOneResponseTypedDict,
    )
    from .filestorage_drivesupdateop import (
        FileStorageDrivesUpdateGlobals,
        FileStorageDrivesUpdateGlobalsTypedDict,
        FileStorageDrivesUpdateRequest,
        FileStorageDrivesUpdateRequestTypedDict,
        FileStorageDrivesUpdateResponse,
        FileStorageDrivesUpdateResponseTypedDict,
    )
    from .filestorage_filesallop import (
        FileStorageFilesAllGlobals,
        FileStorageFilesAllGlobalsTypedDict,
        FileStorageFilesAllRequest,
        FileStorageFilesAllRequestTypedDict,
        FileStorageFilesAllResponse,
        FileStorageFilesAllResponseTypedDict,
    )
    from .filestorage_filesdeleteop import (
        FileStorageFilesDeleteGlobals,
        FileStorageFilesDeleteGlobalsTypedDict,
        FileStorageFilesDeleteRequest,
        FileStorageFilesDeleteRequestTypedDict,
        FileStorageFilesDeleteResponse,
        FileStorageFilesDeleteResponseTypedDict,
    )
    from .filestorage_filesdownloadop import (
        FileStorageFilesDownloadGlobals,
        FileStorageFilesDownloadGlobalsTypedDict,
        FileStorageFilesDownloadRequest,
        FileStorageFilesDownloadRequestTypedDict,
        FileStorageFilesDownloadResponse,
        FileStorageFilesDownloadResponseTypedDict,
    )
    from .filestorage_filesexportop import (
        FileStorageFilesExportGlobals,
        FileStorageFilesExportGlobalsTypedDict,
        FileStorageFilesExportRequest,
        FileStorageFilesExportRequestTypedDict,
        FileStorageFilesExportResponse,
        FileStorageFilesExportResponseTypedDict,
    )
    from .filestorage_filesoneop import (
        FileStorageFilesOneGlobals,
        FileStorageFilesOneGlobalsTypedDict,
        FileStorageFilesOneRequest,
        FileStorageFilesOneRequestTypedDict,
        FileStorageFilesOneResponse,
        FileStorageFilesOneResponseTypedDict,
    )
    from .filestorage_filessearchop import (
        FileStorageFilesSearchGlobals,
        FileStorageFilesSearchGlobalsTypedDict,
        FileStorageFilesSearchRequest,
        FileStorageFilesSearchRequestTypedDict,
        FileStorageFilesSearchResponse,
        FileStorageFilesSearchResponseTypedDict,
    )
    from .filestorage_filesupdateop import (
        FileStorageFilesUpdateGlobals,
        FileStorageFilesUpdateGlobalsTypedDict,
        FileStorageFilesUpdateRequest,
        FileStorageFilesUpdateRequestTypedDict,
        FileStorageFilesUpdateResponse,
        FileStorageFilesUpdateResponseTypedDict,
    )
    from .filestorage_foldersaddop import (
        FileStorageFoldersAddGlobals,
        FileStorageFoldersAddGlobalsTypedDict,
        FileStorageFoldersAddRequest,
        FileStorageFoldersAddRequestTypedDict,
        FileStorageFoldersAddResponse,
        FileStorageFoldersAddResponseTypedDict,
    )
    from .filestorage_folderscopyop import (
        FileStorageFoldersCopyGlobals,
        FileStorageFoldersCopyGlobalsTypedDict,
        FileStorageFoldersCopyRequest,
        FileStorageFoldersCopyRequestTypedDict,
        FileStorageFoldersCopyResponse,
        FileStorageFoldersCopyResponseTypedDict,
    )
    from .filestorage_foldersdeleteop import (
        FileStorageFoldersDeleteGlobals,
        FileStorageFoldersDeleteGlobalsTypedDict,
        FileStorageFoldersDeleteRequest,
        FileStorageFoldersDeleteRequestTypedDict,
        FileStorageFoldersDeleteResponse,
        FileStorageFoldersDeleteResponseTypedDict,
    )
    from .filestorage_foldersoneop import (
        FileStorageFoldersOneGlobals,
        FileStorageFoldersOneGlobalsTypedDict,
        FileStorageFoldersOneRequest,
        FileStorageFoldersOneRequestTypedDict,
        FileStorageFoldersOneResponse,
        FileStorageFoldersOneResponseTypedDict,
    )
    from .filestorage_foldersupdateop import (
        FileStorageFoldersUpdateGlobals,
        FileStorageFoldersUpdateGlobalsTypedDict,
        FileStorageFoldersUpdateRequest,
        FileStorageFoldersUpdateRequestTypedDict,
        FileStorageFoldersUpdateResponse,
        FileStorageFoldersUpdateResponseTypedDict,
    )
    from .filestorage_sharedlinksaddop import (
        FileStorageSharedLinksAddGlobals,
        FileStorageSharedLinksAddGlobalsTypedDict,
        FileStorageSharedLinksAddRequest,
        FileStorageSharedLinksAddRequestTypedDict,
        FileStorageSharedLinksAddResponse,
        FileStorageSharedLinksAddResponseTypedDict,
    )
    from .filestorage_sharedlinksallop import (
        FileStorageSharedLinksAllGlobals,
        FileStorageSharedLinksAllGlobalsTypedDict,
        FileStorageSharedLinksAllRequest,
        FileStorageSharedLinksAllRequestTypedDict,
        FileStorageSharedLinksAllResponse,
        FileStorageSharedLinksAllResponseTypedDict,
    )
    from .filestorage_sharedlinksdeleteop import (
        FileStorageSharedLinksDeleteGlobals,
        FileStorageSharedLinksDeleteGlobalsTypedDict,
        FileStorageSharedLinksDeleteRequest,
        FileStorageSharedLinksDeleteRequestTypedDict,
        FileStorageSharedLinksDeleteResponse,
        FileStorageSharedLinksDeleteResponseTypedDict,
    )
    from .filestorage_sharedlinksoneop import (
        FileStorageSharedLinksOneGlobals,
        FileStorageSharedLinksOneGlobalsTypedDict,
        FileStorageSharedLinksOneRequest,
        FileStorageSharedLinksOneRequestTypedDict,
        FileStorageSharedLinksOneResponse,
        FileStorageSharedLinksOneResponseTypedDict,
    )
    from .filestorage_sharedlinksupdateop import (
        FileStorageSharedLinksUpdateGlobals,
        FileStorageSharedLinksUpdateGlobalsTypedDict,
        FileStorageSharedLinksUpdateRequest,
        FileStorageSharedLinksUpdateRequestTypedDict,
        FileStorageSharedLinksUpdateResponse,
        FileStorageSharedLinksUpdateResponseTypedDict,
    )
    from .filestorage_uploadsessionsaddop import (
        FILE_STORAGE_UPLOAD_SESSIONS_ADD_OP_SERVERS,
        FileStorageUploadSessionsAddGlobals,
        FileStorageUploadSessionsAddGlobalsTypedDict,
        FileStorageUploadSessionsAddRequest,
        FileStorageUploadSessionsAddRequestTypedDict,
        FileStorageUploadSessionsAddResponse,
        FileStorageUploadSessionsAddResponseTypedDict,
    )
    from .filestorage_uploadsessionsdeleteop import (
        FileStorageUploadSessionsDeleteGlobals,
        FileStorageUploadSessionsDeleteGlobalsTypedDict,
        FileStorageUploadSessionsDeleteRequest,
        FileStorageUploadSessionsDeleteRequestTypedDict,
        FileStorageUploadSessionsDeleteResponse,
        FileStorageUploadSessionsDeleteResponseTypedDict,
    )
    from .filestorage_uploadsessionsfinishop import (
        FILE_STORAGE_UPLOAD_SESSIONS_FINISH_OP_SERVERS,
        FileStorageUploadSessionsFinishGlobals,
        FileStorageUploadSessionsFinishGlobalsTypedDict,
        FileStorageUploadSessionsFinishRequest,
        FileStorageUploadSessionsFinishRequestBody,
        FileStorageUploadSessionsFinishRequestBodyTypedDict,
        FileStorageUploadSessionsFinishRequestTypedDict,
        FileStorageUploadSessionsFinishResponse,
        FileStorageUploadSessionsFinishResponseTypedDict,
    )
    from .filestorage_uploadsessionsoneop import (
        FILE_STORAGE_UPLOAD_SESSIONS_ONE_OP_SERVERS,
        FileStorageUploadSessionsOneGlobals,
        FileStorageUploadSessionsOneGlobalsTypedDict,
        FileStorageUploadSessionsOneRequest,
        FileStorageUploadSessionsOneRequestTypedDict,
        FileStorageUploadSessionsOneResponse,
        FileStorageUploadSessionsOneResponseTypedDict,
    )
    from .filestorage_uploadsessionsuploadop import (
        FILE_STORAGE_UPLOAD_SESSIONS_UPLOAD_OP_SERVERS,
        FileStorageUploadSessionsUploadGlobals,
        FileStorageUploadSessionsUploadGlobalsTypedDict,
        FileStorageUploadSessionsUploadRequest,
        FileStorageUploadSessionsUploadRequestTypedDict,
        FileStorageUploadSessionsUploadResponse,
        FileStorageUploadSessionsUploadResponseTypedDict,
    )
    from .filetype import FileType
    from .folder import Folder, FolderTypedDict
    from .formfield import FormField, FormFieldType, FormFieldTypedDict
    from .formfieldoption import FormFieldOption, FormFieldOptionTypedDict
    from .formfieldoptiongroup import (
        FormFieldOptionGroup,
        FormFieldOptionGroupOptionType,
        FormFieldOptionGroupTypedDict,
    )
    from .gender import Gender
    from .getaccountingdepartmentresponse import (
        GetAccountingDepartmentResponse,
        GetAccountingDepartmentResponseTypedDict,
    )
    from .getaccountingdepartmentsresponse import (
        GetAccountingDepartmentsResponse,
        GetAccountingDepartmentsResponseTypedDict,
    )
    from .getaccountinglocationresponse import (
        GetAccountingLocationResponse,
        GetAccountingLocationResponseTypedDict,
    )
    from .getaccountinglocationsresponse import (
        GetAccountingLocationsResponse,
        GetAccountingLocationsResponseTypedDict,
    )
    from .getactivitiesresponse import (
        GetActivitiesResponse,
        GetActivitiesResponseTypedDict,
    )
    from .getactivityresponse import GetActivityResponse, GetActivityResponseTypedDict
    from .getagedcreditorsresponse import (
        GetAgedCreditorsResponse,
        GetAgedCreditorsResponseTypedDict,
    )
    from .getageddebtorsresponse import (
        GetAgedDebtorsResponse,
        GetAgedDebtorsResponseTypedDict,
    )
    from .getapiresourcecoverageresponse import (
        GetAPIResourceCoverageResponse,
        GetAPIResourceCoverageResponseTypedDict,
    )
    from .getapiresourceresponse import (
        GetAPIResourceResponse,
        GetAPIResourceResponseTypedDict,
    )
    from .getapiresponse import GetAPIResponse, GetAPIResponseTypedDict
    from .getapisresponse import GetApisResponse, GetApisResponseTypedDict
    from .getapplicantresponse import (
        GetApplicantResponse,
        GetApplicantResponseTypedDict,
    )
    from .getapplicantsresponse import (
        GetApplicantsResponse,
        GetApplicantsResponseTypedDict,
    )
    from .getapplicationresponse import (
        GetApplicationResponse,
        GetApplicationResponseTypedDict,
    )
    from .getapplicationsresponse import (
        GetApplicationsResponse,
        GetApplicationsResponseTypedDict,
    )
    from .getattachmentresponse import (
        GetAttachmentResponse,
        GetAttachmentResponseTypedDict,
    )
    from .getattachmentsresponse import (
        GetAttachmentsResponse,
        GetAttachmentsResponseTypedDict,
    )
    from .getbalancesheetresponse import (
        GetBalanceSheetResponse,
        GetBalanceSheetResponseTypedDict,
    )
    from .getbankfeedaccountresponse import (
        GetBankFeedAccountResponse,
        GetBankFeedAccountResponseTypedDict,
    )
    from .getbankfeedaccountsresponse import (
        GetBankFeedAccountsResponse,
        GetBankFeedAccountsResponseTypedDict,
    )
    from .getbankfeedstatementresponse import (
        GetBankFeedStatementResponse,
        GetBankFeedStatementResponseTypedDict,
    )
    from .getbankfeedstatementsresponse import (
        GetBankFeedStatementsResponse,
        GetBankFeedStatementsResponseTypedDict,
    )
    from .getbillpaymentresponse import (
        GetBillPaymentResponse,
        GetBillPaymentResponseTypedDict,
    )
    from .getbillpaymentsresponse import (
        GetBillPaymentsResponse,
        GetBillPaymentsResponseTypedDict,
    )
    from .getbillresponse import GetBillResponse, GetBillResponseTypedDict
    from .getbillsresponse import GetBillsResponse, GetBillsResponseTypedDict
    from .getcategoriesresponse import (
        GetCategoriesResponse,
        GetCategoriesResponseTypedDict,
    )
    from .getcategoryresponse import GetCategoryResponse, GetCategoryResponseTypedDict
    from .getcollectionresponse import (
        GetCollectionResponse,
        GetCollectionResponseTypedDict,
    )
    from .getcollectionsresponse import (
        GetCollectionsResponse,
        GetCollectionsResponseTypedDict,
    )
    from .getcollectiontagsresponse import (
        GetCollectionTagsResponse,
        GetCollectionTagsResponseTypedDict,
    )
    from .getcollectionuserresponse import (
        GetCollectionUserResponse,
        GetCollectionUserResponseTypedDict,
    )
    from .getcollectionusersresponse import (
        GetCollectionUsersResponse,
        GetCollectionUsersResponseTypedDict,
    )
    from .getcommentresponse import GetCommentResponse, GetCommentResponseTypedDict
    from .getcommentsresponse import GetCommentsResponse, GetCommentsResponseTypedDict
    from .getcompaniesresponse import (
        GetCompaniesResponse,
        GetCompaniesResponseTypedDict,
    )
    from .getcompanyinforesponse import (
        GetCompanyInfoResponse,
        GetCompanyInfoResponseTypedDict,
    )
    from .getcompanyresponse import GetCompanyResponse, GetCompanyResponseTypedDict
    from .getconnectionresponse import (
        GetConnectionResponse,
        GetConnectionResponseTypedDict,
    )
    from .getconnectionsresponse import (
        GetConnectionsResponse,
        GetConnectionsResponseTypedDict,
    )
    from .getconnectorresourceresponse import (
        GetConnectorResourceResponse,
        GetConnectorResourceResponseTypedDict,
    )
    from .getconnectorresponse import (
        GetConnectorResponse,
        GetConnectorResponseTypedDict,
    )
    from .getconnectorsresponse import (
        GetConnectorsResponse,
        GetConnectorsResponseTypedDict,
    )
    from .getconsumerresponse import GetConsumerResponse, GetConsumerResponseTypedDict
    from .getconsumersresponse import (
        Data,
        DataTypedDict,
        GetConsumersResponse,
        GetConsumersResponseTypedDict,
    )
    from .getcontactresponse import GetContactResponse, GetContactResponseTypedDict
    from .getcontactsresponse import GetContactsResponse, GetContactsResponseTypedDict
    from .getcreditnoteresponse import (
        GetCreditNoteResponse,
        GetCreditNoteResponseTypedDict,
    )
    from .getcreditnotesresponse import (
        GetCreditNotesResponse,
        GetCreditNotesResponseTypedDict,
    )
    from .getcustomerresponse import GetCustomerResponse, GetCustomerResponseTypedDict
    from .getcustomersresponse import (
        GetCustomersResponse,
        GetCustomersResponseTypedDict,
    )
    from .getcustomfieldsresponse import (
        GetCustomFieldsResponse,
        GetCustomFieldsResponseTypedDict,
    )
    from .getcustommappingsresponse import (
        GetCustomMappingsResponse,
        GetCustomMappingsResponseTypedDict,
    )
    from .getcustomobjectresponse import (
        GetCustomObjectResponse,
        GetCustomObjectResponseTypedDict,
    )
    from .getcustomobjectschemaresponse import (
        GetCustomObjectSchemaResponse,
        GetCustomObjectSchemaResponseTypedDict,
    )
    from .getcustomobjectschemasresponse import (
        GetCustomObjectSchemasResponse,
        GetCustomObjectSchemasResponseTypedDict,
    )
    from .getcustomobjectsresponse import (
        GetCustomObjectsResponse,
        GetCustomObjectsResponseTypedDict,
    )
    from .getdepartmentresponse import (
        GetDepartmentResponse,
        GetDepartmentResponseTypedDict,
    )
    from .getdepartmentsresponse import (
        GetDepartmentsResponse,
        GetDepartmentsResponseTypedDict,
    )
    from .getdrivegroupresponse import (
        GetDriveGroupResponse,
        GetDriveGroupResponseTypedDict,
    )
    from .getdrivegroupsresponse import (
        GetDriveGroupsResponse,
        GetDriveGroupsResponseTypedDict,
    )
    from .getdriveresponse import GetDriveResponse, GetDriveResponseTypedDict
    from .getdrivesresponse import GetDrivesResponse, GetDrivesResponseTypedDict
    from .getecommercecustomerresponse import (
        GetEcommerceCustomerResponse,
        GetEcommerceCustomerResponseTypedDict,
    )
    from .getecommercecustomersresponse import (
        GetEcommerceCustomersResponse,
        GetEcommerceCustomersResponseTypedDict,
    )
    from .getecommerceorderresponse import (
        GetEcommerceOrderResponse,
        GetEcommerceOrderResponseTypedDict,
    )
    from .getecommerceordersresponse import (
        GetEcommerceOrdersResponse,
        GetEcommerceOrdersResponseTypedDict,
    )
    from .getemployeepayrollresponse import (
        GetEmployeePayrollResponse,
        GetEmployeePayrollResponseTypedDict,
    )
    from .getemployeepayrollsresponse import (
        GetEmployeePayrollsResponse,
        GetEmployeePayrollsResponseTypedDict,
    )
    from .getemployeeresponse import GetEmployeeResponse, GetEmployeeResponseTypedDict
    from .getemployeeschedulesresponse import (
        GetEmployeeSchedulesResponse,
        GetEmployeeSchedulesResponseTypedDict,
    )
    from .getemployeesresponse import (
        GetEmployeesResponse,
        GetEmployeesResponseTypedDict,
    )
    from .getexpenseresponse import GetExpenseResponse, GetExpenseResponseTypedDict
    from .getexpensesresponse import GetExpensesResponse, GetExpensesResponseTypedDict
    from .getfileresponse import GetFileResponse, GetFileResponseTypedDict
    from .getfilesresponse import GetFilesResponse, GetFilesResponseTypedDict
    from .getfolderresponse import GetFolderResponse, GetFolderResponseTypedDict
    from .gethriscompaniesresponse import (
        GetHrisCompaniesResponse,
        GetHrisCompaniesResponseTypedDict,
    )
    from .gethriscompanyresponse import (
        GetHrisCompanyResponse,
        GetHrisCompanyResponseTypedDict,
    )
    from .getinvoiceitemresponse import (
        GetInvoiceItemResponse,
        GetInvoiceItemResponseTypedDict,
    )
    from .getinvoiceitemsresponse import (
        GetInvoiceItemsResponse,
        GetInvoiceItemsResponseTypedDict,
    )
    from .getinvoiceresponse import GetInvoiceResponse, GetInvoiceResponseTypedDict
    from .getinvoicesresponse import GetInvoicesResponse, GetInvoicesResponseTypedDict
    from .getjobresponse import GetJobResponse, GetJobResponseTypedDict
    from .getjobsresponse import GetJobsResponse, GetJobsResponseTypedDict
    from .getjournalentriesresponse import (
        GetJournalEntriesResponse,
        GetJournalEntriesResponseTypedDict,
    )
    from .getjournalentryresponse import (
        GetJournalEntryResponse,
        GetJournalEntryResponseTypedDict,
    )
    from .getleadresponse import GetLeadResponse, GetLeadResponseTypedDict
    from .getleadsresponse import GetLeadsResponse, GetLeadsResponseTypedDict
    from .getledgeraccountresponse import (
        GetLedgerAccountResponse,
        GetLedgerAccountResponseTypedDict,
    )
    from .getledgeraccountsresponse import (
        GetLedgerAccountsResponse,
        GetLedgerAccountsResponseTypedDict,
    )
    from .getlogsresponse import GetLogsResponse, GetLogsResponseTypedDict
    from .getmessageresponse import GetMessageResponse, GetMessageResponseTypedDict
    from .getmessagesresponse import GetMessagesResponse, GetMessagesResponseTypedDict
    from .getnoteresponse import GetNoteResponse, GetNoteResponseTypedDict
    from .getnotesresponse import GetNotesResponse, GetNotesResponseTypedDict
    from .getopportunitiesresponse import (
        GetOpportunitiesResponse,
        GetOpportunitiesResponseTypedDict,
    )
    from .getopportunityresponse import (
        GetOpportunityResponse,
        GetOpportunityResponseTypedDict,
    )
    from .getpaymentresponse import GetPaymentResponse, GetPaymentResponseTypedDict
    from .getpaymentsresponse import GetPaymentsResponse, GetPaymentsResponseTypedDict
    from .getpayrollresponse import GetPayrollResponse, GetPayrollResponseTypedDict
    from .getpayrollsresponse import GetPayrollsResponse, GetPayrollsResponseTypedDict
    from .getpipelineresponse import GetPipelineResponse, GetPipelineResponseTypedDict
    from .getpipelinesresponse import (
        GetPipelinesResponse,
        GetPipelinesResponseTypedDict,
    )
    from .getproductresponse import GetProductResponse, GetProductResponseTypedDict
    from .getproductsresponse import GetProductsResponse, GetProductsResponseTypedDict
    from .getprofitandlossresponse import (
        GetProfitAndLossResponse,
        GetProfitAndLossResponseTypedDict,
    )
    from .getpurchaseorderresponse import (
        GetPurchaseOrderResponse,
        GetPurchaseOrderResponseTypedDict,
    )
    from .getpurchaseordersresponse import (
        GetPurchaseOrdersResponse,
        GetPurchaseOrdersResponseTypedDict,
    )
    from .getquoteresponse import GetQuoteResponse, GetQuoteResponseTypedDict
    from .getquotesresponse import GetQuotesResponse, GetQuotesResponseTypedDict
    from .getsharedlinkresponse import (
        GetSharedLinkResponse,
        GetSharedLinkResponseTypedDict,
    )
    from .getsharedlinksresponse import (
        GetSharedLinksResponse,
        GetSharedLinksResponseTypedDict,
    )
    from .getstoreresponse import GetStoreResponse, GetStoreResponseTypedDict
    from .getsubsidiariesresponse import (
        GetSubsidiariesResponse,
        GetSubsidiariesResponseTypedDict,
    )
    from .getsubsidiaryresponse import (
        GetSubsidiaryResponse,
        GetSubsidiaryResponseTypedDict,
    )
    from .getsupplierresponse import GetSupplierResponse, GetSupplierResponseTypedDict
    from .getsuppliersresponse import (
        GetSuppliersResponse,
        GetSuppliersResponseTypedDict,
    )
    from .gettaxrateresponse import GetTaxRateResponse, GetTaxRateResponseTypedDict
    from .gettaxratesresponse import GetTaxRatesResponse, GetTaxRatesResponseTypedDict
    from .getticketresponse import GetTicketResponse, GetTicketResponseTypedDict
    from .getticketsresponse import GetTicketsResponse, GetTicketsResponseTypedDict
    from .gettimeoffrequestresponse import (
        GetTimeOffRequestResponse,
        GetTimeOffRequestResponseTypedDict,
    )
    from .gettimeoffrequestsresponse import (
        GetTimeOffRequestsResponse,
        GetTimeOffRequestsResponseTypedDict,
    )
    from .gettrackingcategoriesresponse import (
        GetTrackingCategoriesResponse,
        GetTrackingCategoriesResponseTypedDict,
    )
    from .gettrackingcategoryresponse import (
        GetTrackingCategoryResponse,
        GetTrackingCategoryResponseTypedDict,
    )
    from .getuploadsessionresponse import (
        GetUploadSessionResponse,
        GetUploadSessionResponseTypedDict,
    )
    from .getuserresponse import GetUserResponse, GetUserResponseTypedDict
    from .getusersresponse import GetUsersResponse, GetUsersResponseTypedDict
    from .getwebhookeventlogsresponse import (
        GetWebhookEventLogsResponse,
        GetWebhookEventLogsResponseTypedDict,
    )
    from .getwebhookresponse import GetWebhookResponse, GetWebhookResponseTypedDict
    from .getwebhooksresponse import GetWebhooksResponse, GetWebhooksResponseTypedDict
    from .hris_companiesaddop import (
        HrisCompaniesAddGlobals,
        HrisCompaniesAddGlobalsTypedDict,
        HrisCompaniesAddRequest,
        HrisCompaniesAddRequestTypedDict,
        HrisCompaniesAddResponse,
        HrisCompaniesAddResponseTypedDict,
    )
    from .hris_companiesallop import (
        HrisCompaniesAllGlobals,
        HrisCompaniesAllGlobalsTypedDict,
        HrisCompaniesAllRequest,
        HrisCompaniesAllRequestTypedDict,
        HrisCompaniesAllResponse,
        HrisCompaniesAllResponseTypedDict,
    )
    from .hris_companiesdeleteop import (
        HrisCompaniesDeleteGlobals,
        HrisCompaniesDeleteGlobalsTypedDict,
        HrisCompaniesDeleteRequest,
        HrisCompaniesDeleteRequestTypedDict,
        HrisCompaniesDeleteResponse,
        HrisCompaniesDeleteResponseTypedDict,
    )
    from .hris_companiesoneop import (
        HrisCompaniesOneGlobals,
        HrisCompaniesOneGlobalsTypedDict,
        HrisCompaniesOneRequest,
        HrisCompaniesOneRequestTypedDict,
        HrisCompaniesOneResponse,
        HrisCompaniesOneResponseTypedDict,
    )
    from .hris_companiesupdateop import (
        HrisCompaniesUpdateGlobals,
        HrisCompaniesUpdateGlobalsTypedDict,
        HrisCompaniesUpdateRequest,
        HrisCompaniesUpdateRequestTypedDict,
        HrisCompaniesUpdateResponse,
        HrisCompaniesUpdateResponseTypedDict,
    )
    from .hris_departmentsaddop import (
        HrisDepartmentsAddGlobals,
        HrisDepartmentsAddGlobalsTypedDict,
        HrisDepartmentsAddRequest,
        HrisDepartmentsAddRequestTypedDict,
        HrisDepartmentsAddResponse,
        HrisDepartmentsAddResponseTypedDict,
    )
    from .hris_departmentsallop import (
        HrisDepartmentsAllGlobals,
        HrisDepartmentsAllGlobalsTypedDict,
        HrisDepartmentsAllRequest,
        HrisDepartmentsAllRequestTypedDict,
        HrisDepartmentsAllResponse,
        HrisDepartmentsAllResponseTypedDict,
    )
    from .hris_departmentsdeleteop import (
        HrisDepartmentsDeleteGlobals,
        HrisDepartmentsDeleteGlobalsTypedDict,
        HrisDepartmentsDeleteRequest,
        HrisDepartmentsDeleteRequestTypedDict,
        HrisDepartmentsDeleteResponse,
        HrisDepartmentsDeleteResponseTypedDict,
    )
    from .hris_departmentsoneop import (
        HrisDepartmentsOneGlobals,
        HrisDepartmentsOneGlobalsTypedDict,
        HrisDepartmentsOneRequest,
        HrisDepartmentsOneRequestTypedDict,
        HrisDepartmentsOneResponse,
        HrisDepartmentsOneResponseTypedDict,
    )
    from .hris_departmentsupdateop import (
        HrisDepartmentsUpdateGlobals,
        HrisDepartmentsUpdateGlobalsTypedDict,
        HrisDepartmentsUpdateRequest,
        HrisDepartmentsUpdateRequestTypedDict,
        HrisDepartmentsUpdateResponse,
        HrisDepartmentsUpdateResponseTypedDict,
    )
    from .hris_employeepayrollsallop import (
        HrisEmployeePayrollsAllGlobals,
        HrisEmployeePayrollsAllGlobalsTypedDict,
        HrisEmployeePayrollsAllRequest,
        HrisEmployeePayrollsAllRequestTypedDict,
        HrisEmployeePayrollsAllResponse,
        HrisEmployeePayrollsAllResponseTypedDict,
    )
    from .hris_employeepayrollsoneop import (
        HrisEmployeePayrollsOneGlobals,
        HrisEmployeePayrollsOneGlobalsTypedDict,
        HrisEmployeePayrollsOneRequest,
        HrisEmployeePayrollsOneRequestTypedDict,
        HrisEmployeePayrollsOneResponse,
        HrisEmployeePayrollsOneResponseTypedDict,
    )
    from .hris_employeesaddop import (
        HrisEmployeesAddGlobals,
        HrisEmployeesAddGlobalsTypedDict,
        HrisEmployeesAddRequest,
        HrisEmployeesAddRequestTypedDict,
        HrisEmployeesAddResponse,
        HrisEmployeesAddResponseTypedDict,
    )
    from .hris_employeesallop import (
        HrisEmployeesAllGlobals,
        HrisEmployeesAllGlobalsTypedDict,
        HrisEmployeesAllRequest,
        HrisEmployeesAllRequestTypedDict,
        HrisEmployeesAllResponse,
        HrisEmployeesAllResponseTypedDict,
    )
    from .hris_employeeschedulesallop import (
        HrisEmployeeSchedulesAllGlobals,
        HrisEmployeeSchedulesAllGlobalsTypedDict,
        HrisEmployeeSchedulesAllRequest,
        HrisEmployeeSchedulesAllRequestTypedDict,
        HrisEmployeeSchedulesAllResponse,
        HrisEmployeeSchedulesAllResponseTypedDict,
    )
    from .hris_employeesdeleteop import (
        HrisEmployeesDeleteGlobals,
        HrisEmployeesDeleteGlobalsTypedDict,
        HrisEmployeesDeleteRequest,
        HrisEmployeesDeleteRequestTypedDict,
        HrisEmployeesDeleteResponse,
        HrisEmployeesDeleteResponseTypedDict,
    )
    from .hris_employeesoneop import (
        HrisEmployeesOneGlobals,
        HrisEmployeesOneGlobalsTypedDict,
        HrisEmployeesOneRequest,
        HrisEmployeesOneRequestTypedDict,
        HrisEmployeesOneResponse,
        HrisEmployeesOneResponseTypedDict,
    )
    from .hris_employeesupdateop import (
        HrisEmployeesUpdateGlobals,
        HrisEmployeesUpdateGlobalsTypedDict,
        HrisEmployeesUpdateRequest,
        HrisEmployeesUpdateRequestTypedDict,
        HrisEmployeesUpdateResponse,
        HrisEmployeesUpdateResponseTypedDict,
    )
    from .hris_payrollsallop import (
        HrisPayrollsAllGlobals,
        HrisPayrollsAllGlobalsTypedDict,
        HrisPayrollsAllRequest,
        HrisPayrollsAllRequestTypedDict,
        HrisPayrollsAllResponse,
        HrisPayrollsAllResponseTypedDict,
    )
    from .hris_payrollsoneop import (
        HrisPayrollsOneGlobals,
        HrisPayrollsOneGlobalsTypedDict,
        HrisPayrollsOneRequest,
        HrisPayrollsOneRequestTypedDict,
        HrisPayrollsOneResponse,
        HrisPayrollsOneResponseTypedDict,
    )
    from .hris_timeoffrequestsaddop import (
        HrisTimeOffRequestsAddGlobals,
        HrisTimeOffRequestsAddGlobalsTypedDict,
        HrisTimeOffRequestsAddRequest,
        HrisTimeOffRequestsAddRequestTypedDict,
        HrisTimeOffRequestsAddResponse,
        HrisTimeOffRequestsAddResponseTypedDict,
    )
    from .hris_timeoffrequestsallop import (
        HrisTimeOffRequestsAllGlobals,
        HrisTimeOffRequestsAllGlobalsTypedDict,
        HrisTimeOffRequestsAllRequest,
        HrisTimeOffRequestsAllRequestTypedDict,
        HrisTimeOffRequestsAllResponse,
        HrisTimeOffRequestsAllResponseTypedDict,
    )
    from .hris_timeoffrequestsdeleteop import (
        HrisTimeOffRequestsDeleteGlobals,
        HrisTimeOffRequestsDeleteGlobalsTypedDict,
        HrisTimeOffRequestsDeleteRequest,
        HrisTimeOffRequestsDeleteRequestTypedDict,
        HrisTimeOffRequestsDeleteResponse,
        HrisTimeOffRequestsDeleteResponseTypedDict,
    )
    from .hris_timeoffrequestsoneop import (
        HrisTimeOffRequestsOneGlobals,
        HrisTimeOffRequestsOneGlobalsTypedDict,
        HrisTimeOffRequestsOneRequest,
        HrisTimeOffRequestsOneRequestTypedDict,
        HrisTimeOffRequestsOneResponse,
        HrisTimeOffRequestsOneResponseTypedDict,
    )
    from .hris_timeoffrequestsupdateop import (
        HrisTimeOffRequestsUpdateGlobals,
        HrisTimeOffRequestsUpdateGlobalsTypedDict,
        HrisTimeOffRequestsUpdateRequest,
        HrisTimeOffRequestsUpdateRequestTypedDict,
        HrisTimeOffRequestsUpdateResponse,
        HrisTimeOffRequestsUpdateResponseTypedDict,
    )
    from .hriscompany import (
        HrisCompany,
        HrisCompanyInput,
        HrisCompanyInputTypedDict,
        HrisCompanyStatus,
        HrisCompanyTypedDict,
    )
    from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
    from .integrationstate import IntegrationState
    from .invoice import (
        Invoice,
        InvoiceInput,
        InvoiceInputTypedDict,
        InvoiceStatus,
        InvoiceType,
        InvoiceTypedDict,
        PaymentAllocations,
        PaymentAllocationsTypedDict,
    )
    from .invoiceitem import (
        InvoiceItem,
        InvoiceItemInput,
        InvoiceItemInputTypedDict,
        InvoiceItemPurchaseDetails,
        InvoiceItemPurchaseDetailsTypedDict,
        InvoiceItemSalesDetails,
        InvoiceItemSalesDetailsTypedDict,
        InvoiceItemTypeType,
        InvoiceItemTypedDict,
        PurchaseDetails,
        PurchaseDetailsTypedDict,
        SalesDetails,
        SalesDetailsTypedDict,
    )
    from .invoiceitemfilter import (
        InvoiceItemFilter,
        InvoiceItemFilterInvoiceItemType,
        InvoiceItemFilterTypedDict,
    )
    from .invoiceitemsfilter import (
        InvoiceItemType,
        InvoiceItemsFilter,
        InvoiceItemsFilterTypedDict,
    )
    from .invoiceitemssort import (
        InvoiceItemsSort,
        InvoiceItemsSortBy,
        InvoiceItemsSortTypedDict,
    )
    from .invoicelineitem import (
        InvoiceLineItem,
        InvoiceLineItemInput,
        InvoiceLineItemInputTypedDict,
        InvoiceLineItemType,
        InvoiceLineItemTypedDict,
    )
    from .invoiceresponse import InvoiceResponse, InvoiceResponseTypedDict
    from .invoicesfilter import InvoicesFilter, InvoicesFilterTypedDict
    from .invoicessort import InvoicesSort, InvoicesSortBy, InvoicesSortTypedDict
    from .issuesfilter import IssuesFilter, IssuesFilterTypedDict
    from .issuetracking_collectionsallop import (
        IssueTrackingCollectionsAllGlobals,
        IssueTrackingCollectionsAllGlobalsTypedDict,
        IssueTrackingCollectionsAllRequest,
        IssueTrackingCollectionsAllRequestTypedDict,
        IssueTrackingCollectionsAllResponse,
        IssueTrackingCollectionsAllResponseTypedDict,
    )
    from .issuetracking_collectionsoneop import (
        IssueTrackingCollectionsOneGlobals,
        IssueTrackingCollectionsOneGlobalsTypedDict,
        IssueTrackingCollectionsOneRequest,
        IssueTrackingCollectionsOneRequestTypedDict,
        IssueTrackingCollectionsOneResponse,
        IssueTrackingCollectionsOneResponseTypedDict,
    )
    from .issuetracking_collectiontagsallop import (
        IssueTrackingCollectionTagsAllGlobals,
        IssueTrackingCollectionTagsAllGlobalsTypedDict,
        IssueTrackingCollectionTagsAllRequest,
        IssueTrackingCollectionTagsAllRequestTypedDict,
        IssueTrackingCollectionTagsAllResponse,
        IssueTrackingCollectionTagsAllResponseTypedDict,
    )
    from .issuetracking_collectionticketcommentsaddop import (
        IssueTrackingCollectionTicketCommentsAddGlobals,
        IssueTrackingCollectionTicketCommentsAddGlobalsTypedDict,
        IssueTrackingCollectionTicketCommentsAddRequest,
        IssueTrackingCollectionTicketCommentsAddRequestTypedDict,
        IssueTrackingCollectionTicketCommentsAddResponse,
        IssueTrackingCollectionTicketCommentsAddResponseTypedDict,
    )
    from .issuetracking_collectionticketcommentsallop import (
        IssueTrackingCollectionTicketCommentsAllGlobals,
        IssueTrackingCollectionTicketCommentsAllGlobalsTypedDict,
        IssueTrackingCollectionTicketCommentsAllRequest,
        IssueTrackingCollectionTicketCommentsAllRequestTypedDict,
        IssueTrackingCollectionTicketCommentsAllResponse,
        IssueTrackingCollectionTicketCommentsAllResponseTypedDict,
    )
    from .issuetracking_collectionticketcommentsdeleteop import (
        IssueTrackingCollectionTicketCommentsDeleteGlobals,
        IssueTrackingCollectionTicketCommentsDeleteGlobalsTypedDict,
        IssueTrackingCollectionTicketCommentsDeleteRequest,
        IssueTrackingCollectionTicketCommentsDeleteRequestTypedDict,
        IssueTrackingCollectionTicketCommentsDeleteResponse,
        IssueTrackingCollectionTicketCommentsDeleteResponseTypedDict,
    )
    from .issuetracking_collectionticketcommentsoneop import (
        IssueTrackingCollectionTicketCommentsOneGlobals,
        IssueTrackingCollectionTicketCommentsOneGlobalsTypedDict,
        IssueTrackingCollectionTicketCommentsOneRequest,
        IssueTrackingCollectionTicketCommentsOneRequestTypedDict,
        IssueTrackingCollectionTicketCommentsOneResponse,
        IssueTrackingCollectionTicketCommentsOneResponseTypedDict,
    )
    from .issuetracking_collectionticketcommentsupdateop import (
        IssueTrackingCollectionTicketCommentsUpdateGlobals,
        IssueTrackingCollectionTicketCommentsUpdateGlobalsTypedDict,
        IssueTrackingCollectionTicketCommentsUpdateRequest,
        IssueTrackingCollectionTicketCommentsUpdateRequestTypedDict,
        IssueTrackingCollectionTicketCommentsUpdateResponse,
        IssueTrackingCollectionTicketCommentsUpdateResponseTypedDict,
    )
    from .issuetracking_collectionticketsaddop import (
        IssueTrackingCollectionTicketsAddGlobals,
        IssueTrackingCollectionTicketsAddGlobalsTypedDict,
        IssueTrackingCollectionTicketsAddRequest,
        IssueTrackingCollectionTicketsAddRequestTypedDict,
        IssueTrackingCollectionTicketsAddResponse,
        IssueTrackingCollectionTicketsAddResponseTypedDict,
    )
    from .issuetracking_collectionticketsallop import (
        IssueTrackingCollectionTicketsAllGlobals,
        IssueTrackingCollectionTicketsAllGlobalsTypedDict,
        IssueTrackingCollectionTicketsAllRequest,
        IssueTrackingCollectionTicketsAllRequestTypedDict,
        IssueTrackingCollectionTicketsAllResponse,
        IssueTrackingCollectionTicketsAllResponseTypedDict,
    )
    from .issuetracking_collectionticketsdeleteop import (
        IssueTrackingCollectionTicketsDeleteGlobals,
        IssueTrackingCollectionTicketsDeleteGlobalsTypedDict,
        IssueTrackingCollectionTicketsDeleteRequest,
        IssueTrackingCollectionTicketsDeleteRequestTypedDict,
        IssueTrackingCollectionTicketsDeleteResponse,
        IssueTrackingCollectionTicketsDeleteResponseTypedDict,
    )
    from .issuetracking_collectionticketsoneop import (
        IssueTrackingCollectionTicketsOneGlobals,
        IssueTrackingCollectionTicketsOneGlobalsTypedDict,
        IssueTrackingCollectionTicketsOneRequest,
        IssueTrackingCollectionTicketsOneRequestTypedDict,
        IssueTrackingCollectionTicketsOneResponse,
        IssueTrackingCollectionTicketsOneResponseTypedDict,
    )
    from .issuetracking_collectionticketsupdateop import (
        IssueTrackingCollectionTicketsUpdateGlobals,
        IssueTrackingCollectionTicketsUpdateGlobalsTypedDict,
        IssueTrackingCollectionTicketsUpdateRequest,
        IssueTrackingCollectionTicketsUpdateRequestTypedDict,
        IssueTrackingCollectionTicketsUpdateResponse,
        IssueTrackingCollectionTicketsUpdateResponseTypedDict,
    )
    from .issuetracking_collectionusersallop import (
        IssueTrackingCollectionUsersAllGlobals,
        IssueTrackingCollectionUsersAllGlobalsTypedDict,
        IssueTrackingCollectionUsersAllRequest,
        IssueTrackingCollectionUsersAllRequestTypedDict,
        IssueTrackingCollectionUsersAllResponse,
        IssueTrackingCollectionUsersAllResponseTypedDict,
    )
    from .issuetracking_collectionusersoneop import (
        IssueTrackingCollectionUsersOneGlobals,
        IssueTrackingCollectionUsersOneGlobalsTypedDict,
        IssueTrackingCollectionUsersOneRequest,
        IssueTrackingCollectionUsersOneRequestTypedDict,
        IssueTrackingCollectionUsersOneResponse,
        IssueTrackingCollectionUsersOneResponseTypedDict,
    )
    from .job import (
        Blocks,
        BlocksTypedDict,
        Branch,
        BranchTypedDict,
        EmploymentTerms,
        Job,
        JobLinks,
        JobLinksTypedDict,
        JobType,
        JobTypedDict,
        Salary,
        SalaryTypedDict,
        Visibility,
    )
    from .jobstatus import JobStatus
    from .journalentriesfilter import (
        JournalEntriesFilter,
        JournalEntriesFilterTypedDict,
    )
    from .journalentriessort import (
        JournalEntriesSort,
        JournalEntriesSortBy,
        JournalEntriesSortTypedDict,
    )
    from .journalentry import (
        JournalEntry,
        JournalEntryInput,
        JournalEntryInputTypedDict,
        JournalEntryStatus,
        JournalEntryTypedDict,
    )
    from .journalentrylineitem import (
        JournalEntryLineItem,
        JournalEntryLineItemInput,
        JournalEntryLineItemInputTypedDict,
        JournalEntryLineItemType,
        JournalEntryLineItemTypedDict,
    )
    from .lead import Lead, LeadTypedDict
    from .lead_input import LeadInput, LeadInputTypedDict
    from .leadsfilter import LeadsFilter, LeadsFilterTypedDict
    from .leadssort import LeadsSort, LeadsSortBy, LeadsSortTypedDict
    from .ledgeraccount import (
        AccountStatus,
        CategoriesModel,
        CategoriesModelTypedDict,
        LedgerAccount,
        LedgerAccountClassification,
        LedgerAccountInput,
        LedgerAccountInputTypedDict,
        LedgerAccountSubsidiaries,
        LedgerAccountSubsidiariesTypedDict,
        LedgerAccountType,
        LedgerAccountTypedDict,
        ParentAccount,
        ParentAccountTypedDict,
        SubAccounts,
        SubAccountsTypedDict,
    )
    from .ledgeraccountsfilter import (
        Classification,
        LedgerAccountsFilter,
        LedgerAccountsFilterTypedDict,
    )
    from .ledgeraccountssort import (
        LedgerAccountsSort,
        LedgerAccountsSortBy,
        LedgerAccountsSortTypedDict,
    )
    from .linkedattachment import LinkedAttachment, LinkedAttachmentTypedDict
    from .linkedconnectorresource import (
        LinkedConnectorResource,
        LinkedConnectorResourceTypedDict,
    )
    from .linkedcustomer import LinkedCustomer, LinkedCustomerTypedDict
    from .linkedcustomer_input import LinkedCustomerInput, LinkedCustomerInputTypedDict
    from .linkedecommercecustomer import (
        LinkedEcommerceCustomer,
        LinkedEcommerceCustomerTypedDict,
    )
    from .linkedecommerceorder import (
        LinkedEcommerceOrder,
        LinkedEcommerceOrderTypedDict,
    )
    from .linkedfolder import LinkedFolder, LinkedFolderTypedDict
    from .linkedinvoiceitem import LinkedInvoiceItem, LinkedInvoiceItemTypedDict
    from .linkedledgeraccount import LinkedLedgerAccount, LinkedLedgerAccountTypedDict
    from .linkedledgeraccount_input import (
        LinkedLedgerAccountInput,
        LinkedLedgerAccountInputTypedDict,
    )
    from .linkedparentcustomer import (
        LinkedParentCustomer,
        LinkedParentCustomerTypedDict,
    )
    from .linkedpurchaseorder import LinkedPurchaseOrder, LinkedPurchaseOrderTypedDict
    from .linkedsupplier import LinkedSupplier, LinkedSupplierTypedDict
    from .linkedsupplier_input import LinkedSupplierInput, LinkedSupplierInputTypedDict
    from .linkedtaxdetail import LinkedTaxDetail, LinkedTaxDetailTypedDict
    from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
    from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
    from .linkedtaxstatusdetail import (
        LinkedTaxStatusDetail,
        LinkedTaxStatusDetailTypedDict,
    )
    from .linkedtrackingcategory import (
        LinkedTrackingCategory,
        LinkedTrackingCategoryTypedDict,
    )
    from .linkedworktag import LinkedWorktag, LinkedWorktagTypedDict
    from .links import Links, LinksTypedDict
    from .log import (
        Log,
        LogTypedDict,
        Operation,
        OperationTypedDict,
        Service,
        ServiceTypedDict,
        UnifiedAPI,
    )
    from .logsfilter import LogsFilter, LogsFilterTypedDict
    from .message import (
        Direction,
        Error,
        ErrorTypedDict,
        Message,
        MessageInput,
        MessageInputTypedDict,
        MessageStatus,
        MessageType,
        MessageTypedDict,
        Price,
        PriceTypedDict,
    )
    from .meta import Cursors, CursorsTypedDict, Meta, MetaTypedDict
    from .no_response_error import NoResponseError
    from .note import Note, NoteTypedDict
    from .note_input import NoteInput, NoteInputTypedDict
    from .notfoundresponse import (
        NotFoundResponse,
        NotFoundResponseData,
        NotFoundResponseDetail,
    )
    from .oauthgranttype import OAuthGrantType
    from .opportunitiesfilter import OpportunitiesFilter, OpportunitiesFilterTypedDict
    from .opportunitiessort import (
        OpportunitiesSort,
        OpportunitiesSortBy,
        OpportunitiesSortTypedDict,
    )
    from .opportunity import Opportunity, OpportunityTypedDict
    from .opportunity_input import OpportunityInput, OpportunityInputTypedDict
    from .orderssort import OrdersSort, OrdersSortBy, OrdersSortTypedDict
    from .outstandingbalancebycurrency import (
        OutstandingBalanceByCurrency,
        OutstandingBalanceByCurrencyTypedDict,
    )
    from .outstandingbalancebycustomer import (
        OutstandingBalanceByCustomer,
        OutstandingBalanceByCustomerTypedDict,
    )
    from .outstandingbalancebysupplier import (
        OutstandingBalanceBySupplier,
        OutstandingBalanceBySupplierTypedDict,
    )
    from .owner import Owner, OwnerTypedDict
    from .paginationcoverage import (
        PaginationCoverage,
        PaginationCoverageMode,
        PaginationCoverageTypedDict,
    )
    from .passthroughbody import (
        ExtendPaths,
        ExtendPathsTypedDict,
        PassThroughBody,
        PassThroughBodyTypedDict,
    )
    from .payment import Payment, PaymentTypedDict
    from .payment_input import PaymentInput, PaymentInputTypedDict
    from .paymentfrequency import PaymentFrequency
    from .paymentrequiredresponse import (
        PaymentRequiredResponse,
        PaymentRequiredResponseData,
    )
    from .paymentsfilter import PaymentsFilter, PaymentsFilterTypedDict
    from .paymentssort import PaymentsSort, PaymentsSortBy, PaymentsSortTypedDict
    from .paymentstatus import PaymentStatus
    from .paymenttype import PaymentType
    from .paymentunit import PaymentUnit
    from .payroll import Payroll, PayrollTypedDict
    from .payrollsfilter import PayrollsFilter, PayrollsFilterTypedDict
    from .payrolltotals import PayrollTotals, PayrollTotalsTypedDict
    from .person import Person, PersonTypedDict
    from .person_input import PersonInput, PersonInputTypedDict
    from .phonenumber import PhoneNumber, PhoneNumberType, PhoneNumberTypedDict
    from .pipeline import Pipeline, PipelineTypedDict, Stages, StagesTypedDict
    from .pipeline_input import (
        PipelineInput,
        PipelineInputTypedDict,
        PipelineStages,
        PipelineStagesTypedDict,
    )
    from .profitandloss import (
        CostOfGoodsSold,
        CostOfGoodsSoldTypedDict,
        ExpensesModel,
        ExpensesModelTypedDict,
        Income,
        IncomeTypedDict,
        OtherExpenses,
        OtherExpensesTypedDict,
        OtherIncome,
        OtherIncomeTypedDict,
        ProfitAndLoss,
        ProfitAndLossTypedDict,
        UncategorizedAccounts,
        UncategorizedAccountsTypedDict,
    )
    from .profitandlossfilter import ProfitAndLossFilter, ProfitAndLossFilterTypedDict
    from .profitandlossindicator import (
        ProfitAndLossIndicator,
        ProfitAndLossIndicatorTypedDict,
    )
    from .profitandlosstype import ProfitAndLossType
    from .purchaseorder import (
        PurchaseOrder,
        PurchaseOrderAmortizationType,
        PurchaseOrderInput,
        PurchaseOrderInputTypedDict,
        PurchaseOrderStatus,
        PurchaseOrderTypedDict,
    )
    from .purchaseordersfilter import (
        PurchaseOrdersFilter,
        PurchaseOrdersFilterTypedDict,
    )
    from .purchaseorderssort import (
        PurchaseOrdersSort,
        PurchaseOrdersSortBy,
        PurchaseOrdersSortTypedDict,
    )
    from .quote import (
        Quote,
        QuoteInput,
        QuoteInputTypedDict,
        QuoteStatus,
        QuoteTypedDict,
    )
    from .quotelineitem import (
        QuoteLineItem,
        QuoteLineItemInput,
        QuoteLineItemInputTypedDict,
        QuoteLineItemType,
        QuoteLineItemTypedDict,
    )
    from .rebilling import RebillStatus, Rebilling, RebillingTypedDict
    from .requestcountallocation import (
        RequestCountAllocation,
        RequestCountAllocationTypedDict,
    )
    from .resourcestatus import ResourceStatus
    from .responsevalidationerror import ResponseValidationError
    from .schedule import (
        EvenWeeks,
        EvenWeeksTypedDict,
        OddWeeks,
        OddWeeksTypedDict,
        Schedule,
        ScheduleTypedDict,
        WorkPattern,
        WorkPatternTypedDict,
    )
    from .schemasupport import SchemaSupport, SchemaSupportTypedDict
    from .security import Security, SecurityTypedDict
    from .session import (
        AllowActions,
        Session,
        SessionTypedDict,
        Settings,
        SettingsTypedDict,
        Theme,
        ThemeTypedDict,
    )
    from .sharedlink import (
        Scope,
        SharedLink,
        SharedLinkInput,
        SharedLinkInputTypedDict,
        SharedLinkTypedDict,
    )
    from .sharedlinktarget import SharedLinkTarget, SharedLinkTargetTypedDict
    from .simpleformfieldoption import (
        OptionType,
        SimpleFormFieldOption,
        SimpleFormFieldOptionTypedDict,
        SimpleFormFieldOptionValue,
        SimpleFormFieldOptionValueTypedDict,
        Value5,
        Value5TypedDict,
    )
    from .sms_messagesaddop import (
        SmsMessagesAddGlobals,
        SmsMessagesAddGlobalsTypedDict,
        SmsMessagesAddRequest,
        SmsMessagesAddRequestTypedDict,
        SmsMessagesAddResponse,
        SmsMessagesAddResponseTypedDict,
    )
    from .sms_messagesallop import (
        SmsMessagesAllGlobals,
        SmsMessagesAllGlobalsTypedDict,
        SmsMessagesAllRequest,
        SmsMessagesAllRequestTypedDict,
        SmsMessagesAllResponse,
        SmsMessagesAllResponseTypedDict,
    )
    from .sms_messagesdeleteop import (
        SmsMessagesDeleteGlobals,
        SmsMessagesDeleteGlobalsTypedDict,
        SmsMessagesDeleteRequest,
        SmsMessagesDeleteRequestTypedDict,
        SmsMessagesDeleteResponse,
        SmsMessagesDeleteResponseTypedDict,
    )
    from .sms_messagesoneop import (
        SmsMessagesOneGlobals,
        SmsMessagesOneGlobalsTypedDict,
        SmsMessagesOneRequest,
        SmsMessagesOneRequestTypedDict,
        SmsMessagesOneResponse,
        SmsMessagesOneResponseTypedDict,
    )
    from .sms_messagesupdateop import (
        SmsMessagesUpdateGlobals,
        SmsMessagesUpdateGlobalsTypedDict,
        SmsMessagesUpdateRequest,
        SmsMessagesUpdateRequestTypedDict,
        SmsMessagesUpdateResponse,
        SmsMessagesUpdateResponseTypedDict,
    )
    from .sociallink import SocialLink, SocialLinkTypedDict
    from .sortdirection import SortDirection
    from .status import Status
    from .subsidiary import (
        Subsidiary,
        SubsidiaryInput,
        SubsidiaryInputTypedDict,
        SubsidiaryStatus,
        SubsidiaryTypedDict,
    )
    from .subsidiaryreference import SubsidiaryReference, SubsidiaryReferenceTypedDict
    from .subsidiaryreference_input import (
        SubsidiaryReferenceInput,
        SubsidiaryReferenceInputTypedDict,
    )
    from .supplier import (
        Supplier,
        SupplierInput,
        SupplierInputTypedDict,
        SupplierStatus,
        SupplierTypedDict,
    )
    from .suppliersfilter import SuppliersFilter, SuppliersFilterTypedDict
    from .supplierssort import SuppliersSort, SuppliersSortBy, SuppliersSortTypedDict
    from .supportedproperty import SupportedProperty, SupportedPropertyTypedDict
    from .tax import Tax, TaxTypedDict
    from .taxrate import (
        Components,
        ComponentsTypedDict,
        SubsidiariesModel,
        SubsidiariesModelTypedDict,
        TaxRate,
        TaxRateInput,
        TaxRateInputTypedDict,
        TaxRateStatus,
        TaxRateTypedDict,
    )
    from .taxratesfilter import TaxRatesFilter, TaxRatesFilterTypedDict
    from .team import Team, TeamTypedDict
    from .ticket import (
        Priority,
        Ticket,
        TicketInput,
        TicketInputTypedDict,
        TicketTypedDict,
    )
    from .ticketssort import TicketsSort, TicketsSortBy, TicketsSortTypedDict
    from .timeoffrequest import (
        NotesModel,
        NotesModelTypedDict,
        RequestType,
        TimeOffRequest,
        TimeOffRequestInput,
        TimeOffRequestInputTypedDict,
        TimeOffRequestStatusStatus,
        TimeOffRequestTypedDict,
        Units,
    )
    from .timeoffrequestsfilter import (
        TimeOffRequestStatus,
        TimeOffRequestsFilter,
        TimeOffRequestsFilterTypedDict,
    )
    from .trackingcategory import (
        TrackingCategory,
        TrackingCategoryInput,
        TrackingCategoryInputTypedDict,
        TrackingCategoryStatus,
        TrackingCategorySubsidiaries,
        TrackingCategorySubsidiariesTypedDict,
        TrackingCategoryTypedDict,
    )
    from .trackingitem import TrackingItem, TrackingItemTypedDict
    from .unauthorizedresponse import (
        UnauthorizedResponse,
        UnauthorizedResponseData,
        UnauthorizedResponseDetail,
    )
    from .unexpectederrorresponse import (
        Detail,
        DetailTypedDict,
        UnexpectedErrorResponse,
        UnexpectedErrorResponseTypedDict,
    )
    from .unifiedapiid import UnifiedAPIID
    from .unifiedfile import (
        Permissions,
        PermissionsTypedDict,
        UnifiedFile,
        UnifiedFileTypedDict,
    )
    from .unifiedid import UnifiedID, UnifiedIDTypedDict
    from .unprocessableresponse import (
        UnprocessableResponse,
        UnprocessableResponseData,
        UnprocessableResponseDetail,
    )
    from .updateaccountingdepartmentresponse import (
        UpdateAccountingDepartmentResponse,
        UpdateAccountingDepartmentResponseTypedDict,
    )
    from .updateaccountinglocationresponse import (
        UpdateAccountingLocationResponse,
        UpdateAccountingLocationResponseTypedDict,
    )
    from .updateactivityresponse import (
        UpdateActivityResponse,
        UpdateActivityResponseTypedDict,
    )
    from .updateapplicantresponse import (
        UpdateApplicantResponse,
        UpdateApplicantResponseTypedDict,
    )
    from .updateapplicationresponse import (
        UpdateApplicationResponse,
        UpdateApplicationResponseTypedDict,
    )
    from .updatebankfeedaccountresponse import (
        UpdateBankFeedAccountResponse,
        UpdateBankFeedAccountResponseTypedDict,
    )
    from .updatebankfeedstatementresponse import (
        UpdateBankFeedStatementResponse,
        UpdateBankFeedStatementResponseTypedDict,
    )
    from .updatebillpaymentresponse import (
        UpdateBillPaymentResponse,
        UpdateBillPaymentResponseTypedDict,
    )
    from .updatebillresponse import UpdateBillResponse, UpdateBillResponseTypedDict
    from .updatecommentresponse import (
        UpdateCommentResponse,
        UpdateCommentResponseTypedDict,
    )
    from .updatecompanyresponse import (
        UpdateCompanyResponse,
        UpdateCompanyResponseTypedDict,
    )
    from .updateconnectionresponse import (
        UpdateConnectionResponse,
        UpdateConnectionResponseTypedDict,
    )
    from .updateconsumerrequest import (
        UpdateConsumerRequest,
        UpdateConsumerRequestTypedDict,
    )
    from .updateconsumerresponse import (
        UpdateConsumerResponse,
        UpdateConsumerResponseTypedDict,
    )
    from .updatecontactresponse import (
        UpdateContactResponse,
        UpdateContactResponseTypedDict,
    )
    from .updatecreditnoteresponse import (
        UpdateCreditNoteResponse,
        UpdateCreditNoteResponseTypedDict,
    )
    from .updatecustomerresponse import (
        UpdateCustomerResponse,
        UpdateCustomerResponseTypedDict,
    )
    from .updatecustomobjectresponse import (
        UpdateCustomObjectResponse,
        UpdateCustomObjectResponseTypedDict,
    )
    from .updatecustomobjectschemaresponse import (
        UpdateCustomObjectSchemaResponse,
        UpdateCustomObjectSchemaResponseTypedDict,
    )
    from .updatedepartmentresponse import (
        UpdateDepartmentResponse,
        UpdateDepartmentResponseTypedDict,
    )
    from .updatedrivegroupresponse import (
        UpdateDriveGroupResponse,
        UpdateDriveGroupResponseTypedDict,
    )
    from .updatedriveresponse import UpdateDriveResponse, UpdateDriveResponseTypedDict
    from .updateemployeeresponse import (
        UpdateEmployeeResponse,
        UpdateEmployeeResponseTypedDict,
    )
    from .updateexpenseresponse import (
        UpdateExpenseResponse,
        UpdateExpenseResponseTypedDict,
    )
    from .updatefilerequest import UpdateFileRequest, UpdateFileRequestTypedDict
    from .updatefileresponse import UpdateFileResponse, UpdateFileResponseTypedDict
    from .updatefolderrequest import UpdateFolderRequest, UpdateFolderRequestTypedDict
    from .updatefolderresponse import (
        UpdateFolderResponse,
        UpdateFolderResponseTypedDict,
    )
    from .updatehriscompanyresponse import (
        UpdateHrisCompanyResponse,
        UpdateHrisCompanyResponseTypedDict,
    )
    from .updateinvoiceitemsresponse import (
        UpdateInvoiceItemsResponse,
        UpdateInvoiceItemsResponseTypedDict,
    )
    from .updateinvoiceresponse import (
        UpdateInvoiceResponse,
        UpdateInvoiceResponseTypedDict,
    )
    from .updatejournalentryresponse import (
        UpdateJournalEntryResponse,
        UpdateJournalEntryResponseTypedDict,
    )
    from .updateleadresponse import UpdateLeadResponse, UpdateLeadResponseTypedDict
    from .updateledgeraccountresponse import (
        UpdateLedgerAccountResponse,
        UpdateLedgerAccountResponseTypedDict,
    )
    from .updatemessageresponse import (
        UpdateMessageResponse,
        UpdateMessageResponseTypedDict,
    )
    from .updatenoteresponse import UpdateNoteResponse, UpdateNoteResponseTypedDict
    from .updateopportunityresponse import (
        UpdateOpportunityResponse,
        UpdateOpportunityResponseTypedDict,
    )
    from .updatepaymentresponse import (
        UpdatePaymentResponse,
        UpdatePaymentResponseTypedDict,
    )
    from .updatepipelineresponse import (
        UpdatePipelineResponse,
        UpdatePipelineResponseTypedDict,
    )
    from .updatepurchaseorderresponse import (
        UpdatePurchaseOrderResponse,
        UpdatePurchaseOrderResponseTypedDict,
    )
    from .updatequoteresponse import UpdateQuoteResponse, UpdateQuoteResponseTypedDict
    from .updatesharedlinkresponse import (
        UpdateSharedLinkResponse,
        UpdateSharedLinkResponseTypedDict,
    )
    from .updatesubsidiaryresponse import (
        UpdateSubsidiaryResponse,
        UpdateSubsidiaryResponseTypedDict,
    )
    from .updatesupplierresponse import (
        UpdateSupplierResponse,
        UpdateSupplierResponseTypedDict,
    )
    from .updatetaxrateresponse import (
        UpdateTaxRateResponse,
        UpdateTaxRateResponseTypedDict,
    )
    from .updateticketresponse import (
        UpdateTicketResponse,
        UpdateTicketResponseTypedDict,
    )
    from .updatetimeoffrequestresponse import (
        UpdateTimeOffRequestResponse,
        UpdateTimeOffRequestResponseTypedDict,
    )
    from .updatetrackingcategoryresponse import (
        UpdateTrackingCategoryResponse,
        UpdateTrackingCategoryResponseTypedDict,
    )
    from .updateuploadsessionresponse import (
        UpdateUploadSessionResponse,
        UpdateUploadSessionResponseTypedDict,
    )
    from .updateuserresponse import UpdateUserResponse, UpdateUserResponseTypedDict
    from .updatewebhookrequest import (
        UpdateWebhookRequest,
        UpdateWebhookRequestTypedDict,
    )
    from .updatewebhookresponse import (
        UpdateWebhookResponse,
        UpdateWebhookResponseTypedDict,
    )
    from .uploadsession import UploadSession, UploadSessionTypedDict
    from .user import User, UserTypedDict
    from .user_input import UserInput, UserInputTypedDict
    from .validateconnectionstateresponse import (
        ValidateConnectionStateResponse,
        ValidateConnectionStateResponseData,
        ValidateConnectionStateResponseDataTypedDict,
        ValidateConnectionStateResponseTypedDict,
    )
    from .vault_connectioncustommappingsallop import (
        VaultConnectionCustomMappingsAllGlobals,
        VaultConnectionCustomMappingsAllGlobalsTypedDict,
        VaultConnectionCustomMappingsAllRequest,
        VaultConnectionCustomMappingsAllRequestTypedDict,
        VaultConnectionCustomMappingsAllResponse,
        VaultConnectionCustomMappingsAllResponseTypedDict,
    )
    from .vault_connectionsallop import (
        VaultConnectionsAllGlobals,
        VaultConnectionsAllGlobalsTypedDict,
        VaultConnectionsAllRequest,
        VaultConnectionsAllRequestTypedDict,
        VaultConnectionsAllResponse,
        VaultConnectionsAllResponseTypedDict,
    )
    from .vault_connectionsdeleteop import (
        VaultConnectionsDeleteGlobals,
        VaultConnectionsDeleteGlobalsTypedDict,
        VaultConnectionsDeleteRequest,
        VaultConnectionsDeleteRequestTypedDict,
        VaultConnectionsDeleteResponse,
        VaultConnectionsDeleteResponseTypedDict,
    )
    from .vault_connectionsettingsallop import (
        VaultConnectionSettingsAllGlobals,
        VaultConnectionSettingsAllGlobalsTypedDict,
        VaultConnectionSettingsAllRequest,
        VaultConnectionSettingsAllRequestTypedDict,
        VaultConnectionSettingsAllResponse,
        VaultConnectionSettingsAllResponseTypedDict,
    )
    from .vault_connectionsettingsupdateop import (
        VaultConnectionSettingsUpdateGlobals,
        VaultConnectionSettingsUpdateGlobalsTypedDict,
        VaultConnectionSettingsUpdateRequest,
        VaultConnectionSettingsUpdateRequestTypedDict,
        VaultConnectionSettingsUpdateResponse,
        VaultConnectionSettingsUpdateResponseTypedDict,
    )
    from .vault_connectionsimportop import (
        VaultConnectionsImportGlobals,
        VaultConnectionsImportGlobalsTypedDict,
        VaultConnectionsImportRequest,
        VaultConnectionsImportRequestTypedDict,
        VaultConnectionsImportResponse,
        VaultConnectionsImportResponseTypedDict,
    )
    from .vault_connectionsoneop import (
        VaultConnectionsOneGlobals,
        VaultConnectionsOneGlobalsTypedDict,
        VaultConnectionsOneRequest,
        VaultConnectionsOneRequestTypedDict,
        VaultConnectionsOneResponse,
        VaultConnectionsOneResponseTypedDict,
    )
    from .vault_connectionstokenop import (
        VaultConnectionsTokenGlobals,
        VaultConnectionsTokenGlobalsTypedDict,
        VaultConnectionsTokenRequest,
        VaultConnectionsTokenRequestBody,
        VaultConnectionsTokenRequestBodyTypedDict,
        VaultConnectionsTokenRequestTypedDict,
        VaultConnectionsTokenResponse,
        VaultConnectionsTokenResponseTypedDict,
    )
    from .vault_connectionsupdateop import (
        VaultConnectionsUpdateGlobals,
        VaultConnectionsUpdateGlobalsTypedDict,
        VaultConnectionsUpdateRequest,
        VaultConnectionsUpdateRequestTypedDict,
        VaultConnectionsUpdateResponse,
        VaultConnectionsUpdateResponseTypedDict,
    )
    from .vault_consumerrequestcountsallop import (
        VaultConsumerRequestCountsAllGlobals,
        VaultConsumerRequestCountsAllGlobalsTypedDict,
        VaultConsumerRequestCountsAllRequest,
        VaultConsumerRequestCountsAllRequestTypedDict,
        VaultConsumerRequestCountsAllResponse,
        VaultConsumerRequestCountsAllResponseTypedDict,
    )
    from .vault_consumersaddop import (
        VaultConsumersAddGlobals,
        VaultConsumersAddGlobalsTypedDict,
        VaultConsumersAddRequest,
        VaultConsumersAddRequestTypedDict,
        VaultConsumersAddResponse,
        VaultConsumersAddResponseTypedDict,
    )
    from .vault_consumersallop import (
        VaultConsumersAllGlobals,
        VaultConsumersAllGlobalsTypedDict,
        VaultConsumersAllRequest,
        VaultConsumersAllRequestTypedDict,
        VaultConsumersAllResponse,
        VaultConsumersAllResponseTypedDict,
    )
    from .vault_consumersdeleteop import (
        VaultConsumersDeleteGlobals,
        VaultConsumersDeleteGlobalsTypedDict,
        VaultConsumersDeleteRequest,
        VaultConsumersDeleteRequestTypedDict,
        VaultConsumersDeleteResponse,
        VaultConsumersDeleteResponseTypedDict,
    )
    from .vault_consumersoneop import (
        VaultConsumersOneGlobals,
        VaultConsumersOneGlobalsTypedDict,
        VaultConsumersOneRequest,
        VaultConsumersOneRequestTypedDict,
        VaultConsumersOneResponse,
        VaultConsumersOneResponseTypedDict,
    )
    from .vault_consumersupdateop import (
        VaultConsumersUpdateGlobals,
        VaultConsumersUpdateGlobalsTypedDict,
        VaultConsumersUpdateRequest,
        VaultConsumersUpdateRequestTypedDict,
        VaultConsumersUpdateResponse,
        VaultConsumersUpdateResponseTypedDict,
    )
    from .vault_createcallbackstateop import (
        VaultCreateCallbackStateGlobals,
        VaultCreateCallbackStateGlobalsTypedDict,
        VaultCreateCallbackStateRequest,
        VaultCreateCallbackStateRequestTypedDict,
        VaultCreateCallbackStateResponse,
        VaultCreateCallbackStateResponseTypedDict,
    )
    from .vault_customfieldsallop import (
        VaultCustomFieldsAllGlobals,
        VaultCustomFieldsAllGlobalsTypedDict,
        VaultCustomFieldsAllRequest,
        VaultCustomFieldsAllRequestTypedDict,
        VaultCustomFieldsAllResponse,
        VaultCustomFieldsAllResponseTypedDict,
    )
    from .vault_custommappingsallop import (
        VaultCustomMappingsAllGlobals,
        VaultCustomMappingsAllGlobalsTypedDict,
        VaultCustomMappingsAllRequest,
        VaultCustomMappingsAllRequestTypedDict,
        VaultCustomMappingsAllResponse,
        VaultCustomMappingsAllResponseTypedDict,
    )
    from .vault_logsallop import (
        VaultLogsAllGlobals,
        VaultLogsAllGlobalsTypedDict,
        VaultLogsAllRequest,
        VaultLogsAllRequestTypedDict,
        VaultLogsAllResponse,
        VaultLogsAllResponseTypedDict,
    )
    from .vault_sessionscreateop import (
        VaultSessionsCreateGlobals,
        VaultSessionsCreateGlobalsTypedDict,
        VaultSessionsCreateRequest,
        VaultSessionsCreateRequestTypedDict,
        VaultSessionsCreateResponse,
        VaultSessionsCreateResponseTypedDict,
    )
    from .vault_validateconnectionstateop import (
        VaultValidateConnectionStateGlobals,
        VaultValidateConnectionStateGlobalsTypedDict,
        VaultValidateConnectionStateRequest,
        VaultValidateConnectionStateRequestBody,
        VaultValidateConnectionStateRequestBodyTypedDict,
        VaultValidateConnectionStateRequestTypedDict,
        VaultValidateConnectionStateResponse,
        VaultValidateConnectionStateResponseTypedDict,
    )
    from .webhook import DisabledReason, Webhook, WebhookTypedDict
    from .webhook_eventlogsallop import (
        WebhookEventLogsAllGlobals,
        WebhookEventLogsAllGlobalsTypedDict,
        WebhookEventLogsAllRequest,
        WebhookEventLogsAllRequestTypedDict,
        WebhookEventLogsAllResponse,
        WebhookEventLogsAllResponseTypedDict,
    )
    from .webhook_webhooksaddop import (
        WebhookWebhooksAddGlobals,
        WebhookWebhooksAddGlobalsTypedDict,
        WebhookWebhooksAddRequest,
        WebhookWebhooksAddRequestTypedDict,
        WebhookWebhooksAddResponse,
        WebhookWebhooksAddResponseTypedDict,
    )
    from .webhook_webhooksallop import (
        WebhookWebhooksAllGlobals,
        WebhookWebhooksAllGlobalsTypedDict,
        WebhookWebhooksAllRequest,
        WebhookWebhooksAllRequestTypedDict,
        WebhookWebhooksAllResponse,
        WebhookWebhooksAllResponseTypedDict,
    )
    from .webhook_webhooksdeleteop import (
        WebhookWebhooksDeleteGlobals,
        WebhookWebhooksDeleteGlobalsTypedDict,
        WebhookWebhooksDeleteRequest,
        WebhookWebhooksDeleteRequestTypedDict,
        WebhookWebhooksDeleteResponse,
        WebhookWebhooksDeleteResponseTypedDict,
    )
    from .webhook_webhooksoneop import (
        WebhookWebhooksOneGlobals,
        WebhookWebhooksOneGlobalsTypedDict,
        WebhookWebhooksOneRequest,
        WebhookWebhooksOneRequestTypedDict,
        WebhookWebhooksOneResponse,
        WebhookWebhooksOneResponseTypedDict,
    )
    from .webhook_webhooksupdateop import (
        WebhookWebhooksUpdateGlobals,
        WebhookWebhooksUpdateGlobalsTypedDict,
        WebhookWebhooksUpdateRequest,
        WebhookWebhooksUpdateRequestTypedDict,
        WebhookWebhooksUpdateResponse,
        WebhookWebhooksUpdateResponseTypedDict,
    )
    from .webhookeventlog import (
        Attempts,
        AttemptsTypedDict,
        WebhookEventLog,
        WebhookEventLogService,
        WebhookEventLogServiceTypedDict,
        WebhookEventLogTypedDict,
    )
    from .webhookeventlogsfilter import (
        WebhookEventLogsFilter,
        WebhookEventLogsFilterService,
        WebhookEventLogsFilterServiceTypedDict,
        WebhookEventLogsFilterTypedDict,
    )
    from .webhookeventtype import WebhookEventType
    from .webhooksubscription import WebhookSubscription, WebhookSubscriptionTypedDict
    from .webhooksupport import (
        ManagedVia,
        Mode,
        RequestRate,
        RequestRateTypedDict,
        SubscriptionLevel,
        Unit,
        VirtualWebhooks,
        VirtualWebhooksTypedDict,
        WebhookSupport,
        WebhookSupportResources,
        WebhookSupportResourcesTypedDict,
        WebhookSupportTypedDict,
    )
    from .website import Website, WebsiteType, WebsiteTypedDict

__all__ = [
    "ACCOUNTING_ATTACHMENTS_UPLOAD_OP_SERVERS",
    "API",
    "APIError",
    "APIResource",
    "APIResourceCoverage",
    "APIResourceCoverageTypedDict",
    "APIResourceTypedDict",
    "APIStatus",
    "APIType",
    "APITypedDict",
    "AccountStatus",
    "AccountType",
    "AccountingAgedCreditorsOneGlobals",
    "AccountingAgedCreditorsOneGlobalsTypedDict",
    "AccountingAgedCreditorsOneRequest",
    "AccountingAgedCreditorsOneRequestTypedDict",
    "AccountingAgedCreditorsOneResponse",
    "AccountingAgedCreditorsOneResponseTypedDict",
    "AccountingAgedDebtorsOneGlobals",
    "AccountingAgedDebtorsOneGlobalsTypedDict",
    "AccountingAgedDebtorsOneRequest",
    "AccountingAgedDebtorsOneRequestTypedDict",
    "AccountingAgedDebtorsOneResponse",
    "AccountingAgedDebtorsOneResponseTypedDict",
    "AccountingAttachmentsAllGlobals",
    "AccountingAttachmentsAllGlobalsTypedDict",
    "AccountingAttachmentsAllRequest",
    "AccountingAttachmentsAllRequestTypedDict",
    "AccountingAttachmentsAllResponse",
    "AccountingAttachmentsAllResponseTypedDict",
    "AccountingAttachmentsDeleteGlobals",
    "AccountingAttachmentsDeleteGlobalsTypedDict",
    "AccountingAttachmentsDeleteRequest",
    "AccountingAttachmentsDeleteRequestTypedDict",
    "AccountingAttachmentsDeleteResponse",
    "AccountingAttachmentsDeleteResponseTypedDict",
    "AccountingAttachmentsDownloadGlobals",
    "AccountingAttachmentsDownloadGlobalsTypedDict",
    "AccountingAttachmentsDownloadRequest",
    "AccountingAttachmentsDownloadRequestTypedDict",
    "AccountingAttachmentsDownloadResponse",
    "AccountingAttachmentsDownloadResponseTypedDict",
    "AccountingAttachmentsOneGlobals",
    "AccountingAttachmentsOneGlobalsTypedDict",
    "AccountingAttachmentsOneRequest",
    "AccountingAttachmentsOneRequestTypedDict",
    "AccountingAttachmentsOneResponse",
    "AccountingAttachmentsOneResponseTypedDict",
    "AccountingAttachmentsUploadGlobals",
    "AccountingAttachmentsUploadGlobalsTypedDict",
    "AccountingAttachmentsUploadRequest",
    "AccountingAttachmentsUploadRequestTypedDict",
    "AccountingAttachmentsUploadResponse",
    "AccountingAttachmentsUploadResponseTypedDict",
    "AccountingBalanceSheetOneGlobals",
    "AccountingBalanceSheetOneGlobalsTypedDict",
    "AccountingBalanceSheetOneRequest",
    "AccountingBalanceSheetOneRequestTypedDict",
    "AccountingBalanceSheetOneResponse",
    "AccountingBalanceSheetOneResponseTypedDict",
    "AccountingBankFeedAccountsAddGlobals",
    "AccountingBankFeedAccountsAddGlobalsTypedDict",
    "AccountingBankFeedAccountsAddRequest",
    "AccountingBankFeedAccountsAddRequestTypedDict",
    "AccountingBankFeedAccountsAddResponse",
    "AccountingBankFeedAccountsAddResponseTypedDict",
    "AccountingBankFeedAccountsAllGlobals",
    "AccountingBankFeedAccountsAllGlobalsTypedDict",
    "AccountingBankFeedAccountsAllRequest",
    "AccountingBankFeedAccountsAllRequestTypedDict",
    "AccountingBankFeedAccountsAllResponse",
    "AccountingBankFeedAccountsAllResponseTypedDict",
    "AccountingBankFeedAccountsDeleteGlobals",
    "AccountingBankFeedAccountsDeleteGlobalsTypedDict",
    "AccountingBankFeedAccountsDeleteRequest",
    "AccountingBankFeedAccountsDeleteRequestTypedDict",
    "AccountingBankFeedAccountsDeleteResponse",
    "AccountingBankFeedAccountsDeleteResponseTypedDict",
    "AccountingBankFeedAccountsOneGlobals",
    "AccountingBankFeedAccountsOneGlobalsTypedDict",
    "AccountingBankFeedAccountsOneRequest",
    "AccountingBankFeedAccountsOneRequestTypedDict",
    "AccountingBankFeedAccountsOneResponse",
    "AccountingBankFeedAccountsOneResponseTypedDict",
    "AccountingBankFeedAccountsUpdateGlobals",
    "AccountingBankFeedAccountsUpdateGlobalsTypedDict",
    "AccountingBankFeedAccountsUpdateRequest",
    "AccountingBankFeedAccountsUpdateRequestTypedDict",
    "AccountingBankFeedAccountsUpdateResponse",
    "AccountingBankFeedAccountsUpdateResponseTypedDict",
    "AccountingBankFeedStatementsAddGlobals",
    "AccountingBankFeedStatementsAddGlobalsTypedDict",
    "AccountingBankFeedStatementsAddRequest",
    "AccountingBankFeedStatementsAddRequestTypedDict",
    "AccountingBankFeedStatementsAddResponse",
    "AccountingBankFeedStatementsAddResponseTypedDict",
    "AccountingBankFeedStatementsAllGlobals",
    "AccountingBankFeedStatementsAllGlobalsTypedDict",
    "AccountingBankFeedStatementsAllRequest",
    "AccountingBankFeedStatementsAllRequestTypedDict",
    "AccountingBankFeedStatementsAllResponse",
    "AccountingBankFeedStatementsAllResponseTypedDict",
    "AccountingBankFeedStatementsDeleteGlobals",
    "AccountingBankFeedStatementsDeleteGlobalsTypedDict",
    "AccountingBankFeedStatementsDeleteRequest",
    "AccountingBankFeedStatementsDeleteRequestTypedDict",
    "AccountingBankFeedStatementsDeleteResponse",
    "AccountingBankFeedStatementsDeleteResponseTypedDict",
    "AccountingBankFeedStatementsOneGlobals",
    "AccountingBankFeedStatementsOneGlobalsTypedDict",
    "AccountingBankFeedStatementsOneRequest",
    "AccountingBankFeedStatementsOneRequestTypedDict",
    "AccountingBankFeedStatementsOneResponse",
    "AccountingBankFeedStatementsOneResponseTypedDict",
    "AccountingBankFeedStatementsUpdateGlobals",
    "AccountingBankFeedStatementsUpdateGlobalsTypedDict",
    "AccountingBankFeedStatementsUpdateRequest",
    "AccountingBankFeedStatementsUpdateRequestTypedDict",
    "AccountingBankFeedStatementsUpdateResponse",
    "AccountingBankFeedStatementsUpdateResponseTypedDict",
    "AccountingBillPaymentsAddGlobals",
    "AccountingBillPaymentsAddGlobalsTypedDict",
    "AccountingBillPaymentsAddRequest",
    "AccountingBillPaymentsAddRequestTypedDict",
    "AccountingBillPaymentsAddResponse",
    "AccountingBillPaymentsAddResponseTypedDict",
    "AccountingBillPaymentsAllGlobals",
    "AccountingBillPaymentsAllGlobalsTypedDict",
    "AccountingBillPaymentsAllRequest",
    "AccountingBillPaymentsAllRequestTypedDict",
    "AccountingBillPaymentsAllResponse",
    "AccountingBillPaymentsAllResponseTypedDict",
    "AccountingBillPaymentsDeleteGlobals",
    "AccountingBillPaymentsDeleteGlobalsTypedDict",
    "AccountingBillPaymentsDeleteRequest",
    "AccountingBillPaymentsDeleteRequestTypedDict",
    "AccountingBillPaymentsDeleteResponse",
    "AccountingBillPaymentsDeleteResponseTypedDict",
    "AccountingBillPaymentsOneGlobals",
    "AccountingBillPaymentsOneGlobalsTypedDict",
    "AccountingBillPaymentsOneRequest",
    "AccountingBillPaymentsOneRequestTypedDict",
    "AccountingBillPaymentsOneResponse",
    "AccountingBillPaymentsOneResponseTypedDict",
    "AccountingBillPaymentsUpdateGlobals",
    "AccountingBillPaymentsUpdateGlobalsTypedDict",
    "AccountingBillPaymentsUpdateRequest",
    "AccountingBillPaymentsUpdateRequestTypedDict",
    "AccountingBillPaymentsUpdateResponse",
    "AccountingBillPaymentsUpdateResponseTypedDict",
    "AccountingBillsAddGlobals",
    "AccountingBillsAddGlobalsTypedDict",
    "AccountingBillsAddRequest",
    "AccountingBillsAddRequestTypedDict",
    "AccountingBillsAddResponse",
    "AccountingBillsAddResponseTypedDict",
    "AccountingBillsAllGlobals",
    "AccountingBillsAllGlobalsTypedDict",
    "AccountingBillsAllRequest",
    "AccountingBillsAllRequestTypedDict",
    "AccountingBillsAllResponse",
    "AccountingBillsAllResponseTypedDict",
    "AccountingBillsDeleteGlobals",
    "AccountingBillsDeleteGlobalsTypedDict",
    "AccountingBillsDeleteRequest",
    "AccountingBillsDeleteRequestTypedDict",
    "AccountingBillsDeleteResponse",
    "AccountingBillsDeleteResponseTypedDict",
    "AccountingBillsOneGlobals",
    "AccountingBillsOneGlobalsTypedDict",
    "AccountingBillsOneRequest",
    "AccountingBillsOneRequestTypedDict",
    "AccountingBillsOneResponse",
    "AccountingBillsOneResponseTypedDict",
    "AccountingBillsUpdateGlobals",
    "AccountingBillsUpdateGlobalsTypedDict",
    "AccountingBillsUpdateRequest",
    "AccountingBillsUpdateRequestTypedDict",
    "AccountingBillsUpdateResponse",
    "AccountingBillsUpdateResponseTypedDict",
    "AccountingCategoriesAllGlobals",
    "AccountingCategoriesAllGlobalsTypedDict",
    "AccountingCategoriesAllRequest",
    "AccountingCategoriesAllRequestTypedDict",
    "AccountingCategoriesAllResponse",
    "AccountingCategoriesAllResponseTypedDict",
    "AccountingCategoriesOneGlobals",
    "AccountingCategoriesOneGlobalsTypedDict",
    "AccountingCategoriesOneRequest",
    "AccountingCategoriesOneRequestTypedDict",
    "AccountingCategoriesOneResponse",
    "AccountingCategoriesOneResponseTypedDict",
    "AccountingCompanyInfoOneGlobals",
    "AccountingCompanyInfoOneGlobalsTypedDict",
    "AccountingCompanyInfoOneRequest",
    "AccountingCompanyInfoOneRequestTypedDict",
    "AccountingCompanyInfoOneResponse",
    "AccountingCompanyInfoOneResponseTypedDict",
    "AccountingCreditNotesAddGlobals",
    "AccountingCreditNotesAddGlobalsTypedDict",
    "AccountingCreditNotesAddRequest",
    "AccountingCreditNotesAddRequestTypedDict",
    "AccountingCreditNotesAddResponse",
    "AccountingCreditNotesAddResponseTypedDict",
    "AccountingCreditNotesAllGlobals",
    "AccountingCreditNotesAllGlobalsTypedDict",
    "AccountingCreditNotesAllRequest",
    "AccountingCreditNotesAllRequestTypedDict",
    "AccountingCreditNotesAllResponse",
    "AccountingCreditNotesAllResponseTypedDict",
    "AccountingCreditNotesDeleteGlobals",
    "AccountingCreditNotesDeleteGlobalsTypedDict",
    "AccountingCreditNotesDeleteRequest",
    "AccountingCreditNotesDeleteRequestTypedDict",
    "AccountingCreditNotesDeleteResponse",
    "AccountingCreditNotesDeleteResponseTypedDict",
    "AccountingCreditNotesOneGlobals",
    "AccountingCreditNotesOneGlobalsTypedDict",
    "AccountingCreditNotesOneRequest",
    "AccountingCreditNotesOneRequestTypedDict",
    "AccountingCreditNotesOneResponse",
    "AccountingCreditNotesOneResponseTypedDict",
    "AccountingCreditNotesUpdateGlobals",
    "AccountingCreditNotesUpdateGlobalsTypedDict",
    "AccountingCreditNotesUpdateRequest",
    "AccountingCreditNotesUpdateRequestTypedDict",
    "AccountingCreditNotesUpdateResponse",
    "AccountingCreditNotesUpdateResponseTypedDict",
    "AccountingCustomersAddGlobals",
    "AccountingCustomersAddGlobalsTypedDict",
    "AccountingCustomersAddRequest",
    "AccountingCustomersAddRequestTypedDict",
    "AccountingCustomersAddResponse",
    "AccountingCustomersAddResponseTypedDict",
    "AccountingCustomersAllGlobals",
    "AccountingCustomersAllGlobalsTypedDict",
    "AccountingCustomersAllRequest",
    "AccountingCustomersAllRequestTypedDict",
    "AccountingCustomersAllResponse",
    "AccountingCustomersAllResponseTypedDict",
    "AccountingCustomersDeleteGlobals",
    "AccountingCustomersDeleteGlobalsTypedDict",
    "AccountingCustomersDeleteRequest",
    "AccountingCustomersDeleteRequestTypedDict",
    "AccountingCustomersDeleteResponse",
    "AccountingCustomersDeleteResponseTypedDict",
    "AccountingCustomersOneGlobals",
    "AccountingCustomersOneGlobalsTypedDict",
    "AccountingCustomersOneRequest",
    "AccountingCustomersOneRequestTypedDict",
    "AccountingCustomersOneResponse",
    "AccountingCustomersOneResponseTypedDict",
    "AccountingCustomersUpdateGlobals",
    "AccountingCustomersUpdateGlobalsTypedDict",
    "AccountingCustomersUpdateRequest",
    "AccountingCustomersUpdateRequestTypedDict",
    "AccountingCustomersUpdateResponse",
    "AccountingCustomersUpdateResponseTypedDict",
    "AccountingDepartment",
    "AccountingDepartmentInput",
    "AccountingDepartmentInputTypedDict",
    "AccountingDepartmentTypedDict",
    "AccountingDepartmentsAddGlobals",
    "AccountingDepartmentsAddGlobalsTypedDict",
    "AccountingDepartmentsAddRequest",
    "AccountingDepartmentsAddRequestTypedDict",
    "AccountingDepartmentsAddResponse",
    "AccountingDepartmentsAddResponseTypedDict",
    "AccountingDepartmentsAllGlobals",
    "AccountingDepartmentsAllGlobalsTypedDict",
    "AccountingDepartmentsAllRequest",
    "AccountingDepartmentsAllRequestTypedDict",
    "AccountingDepartmentsAllResponse",
    "AccountingDepartmentsAllResponseTypedDict",
    "AccountingDepartmentsDeleteGlobals",
    "AccountingDepartmentsDeleteGlobalsTypedDict",
    "AccountingDepartmentsDeleteRequest",
    "AccountingDepartmentsDeleteRequestTypedDict",
    "AccountingDepartmentsDeleteResponse",
    "AccountingDepartmentsDeleteResponseTypedDict",
    "AccountingDepartmentsFilter",
    "AccountingDepartmentsFilterTypedDict",
    "AccountingDepartmentsOneGlobals",
    "AccountingDepartmentsOneGlobalsTypedDict",
    "AccountingDepartmentsOneRequest",
    "AccountingDepartmentsOneRequestTypedDict",
    "AccountingDepartmentsOneResponse",
    "AccountingDepartmentsOneResponseTypedDict",
    "AccountingDepartmentsUpdateGlobals",
    "AccountingDepartmentsUpdateGlobalsTypedDict",
    "AccountingDepartmentsUpdateRequest",
    "AccountingDepartmentsUpdateRequestTypedDict",
    "AccountingDepartmentsUpdateResponse",
    "AccountingDepartmentsUpdateResponseTypedDict",
    "AccountingExpensesAddGlobals",
    "AccountingExpensesAddGlobalsTypedDict",
    "AccountingExpensesAddRequest",
    "AccountingExpensesAddRequestTypedDict",
    "AccountingExpensesAddResponse",
    "AccountingExpensesAddResponseTypedDict",
    "AccountingExpensesAllGlobals",
    "AccountingExpensesAllGlobalsTypedDict",
    "AccountingExpensesAllRequest",
    "AccountingExpensesAllRequestTypedDict",
    "AccountingExpensesAllResponse",
    "AccountingExpensesAllResponseTypedDict",
    "AccountingExpensesDeleteGlobals",
    "AccountingExpensesDeleteGlobalsTypedDict",
    "AccountingExpensesDeleteRequest",
    "AccountingExpensesDeleteRequestTypedDict",
    "AccountingExpensesDeleteResponse",
    "AccountingExpensesDeleteResponseTypedDict",
    "AccountingExpensesOneGlobals",
    "AccountingExpensesOneGlobalsTypedDict",
    "AccountingExpensesOneRequest",
    "AccountingExpensesOneRequestTypedDict",
    "AccountingExpensesOneResponse",
    "AccountingExpensesOneResponseTypedDict",
    "AccountingExpensesUpdateGlobals",
    "AccountingExpensesUpdateGlobalsTypedDict",
    "AccountingExpensesUpdateRequest",
    "AccountingExpensesUpdateRequestTypedDict",
    "AccountingExpensesUpdateResponse",
    "AccountingExpensesUpdateResponseTypedDict",
    "AccountingInvoiceItemsAddGlobals",
    "AccountingInvoiceItemsAddGlobalsTypedDict",
    "AccountingInvoiceItemsAddRequest",
    "AccountingInvoiceItemsAddRequestTypedDict",
    "AccountingInvoiceItemsAddResponse",
    "AccountingInvoiceItemsAddResponseTypedDict",
    "AccountingInvoiceItemsAllGlobals",
    "AccountingInvoiceItemsAllGlobalsTypedDict",
    "AccountingInvoiceItemsAllRequest",
    "AccountingInvoiceItemsAllRequestTypedDict",
    "AccountingInvoiceItemsAllResponse",
    "AccountingInvoiceItemsAllResponseTypedDict",
    "AccountingInvoiceItemsDeleteGlobals",
    "AccountingInvoiceItemsDeleteGlobalsTypedDict",
    "AccountingInvoiceItemsDeleteRequest",
    "AccountingInvoiceItemsDeleteRequestTypedDict",
    "AccountingInvoiceItemsDeleteResponse",
    "AccountingInvoiceItemsDeleteResponseTypedDict",
    "AccountingInvoiceItemsOneGlobals",
    "AccountingInvoiceItemsOneGlobalsTypedDict",
    "AccountingInvoiceItemsOneRequest",
    "AccountingInvoiceItemsOneRequestTypedDict",
    "AccountingInvoiceItemsOneResponse",
    "AccountingInvoiceItemsOneResponseTypedDict",
    "AccountingInvoiceItemsUpdateGlobals",
    "AccountingInvoiceItemsUpdateGlobalsTypedDict",
    "AccountingInvoiceItemsUpdateRequest",
    "AccountingInvoiceItemsUpdateRequestTypedDict",
    "AccountingInvoiceItemsUpdateResponse",
    "AccountingInvoiceItemsUpdateResponseTypedDict",
    "AccountingInvoicesAddGlobals",
    "AccountingInvoicesAddGlobalsTypedDict",
    "AccountingInvoicesAddRequest",
    "AccountingInvoicesAddRequestTypedDict",
    "AccountingInvoicesAddResponse",
    "AccountingInvoicesAddResponseTypedDict",
    "AccountingInvoicesAllGlobals",
    "AccountingInvoicesAllGlobalsTypedDict",
    "AccountingInvoicesAllRequest",
    "AccountingInvoicesAllRequestTypedDict",
    "AccountingInvoicesAllResponse",
    "AccountingInvoicesAllResponseTypedDict",
    "AccountingInvoicesDeleteGlobals",
    "AccountingInvoicesDeleteGlobalsTypedDict",
    "AccountingInvoicesDeleteRequest",
    "AccountingInvoicesDeleteRequestTypedDict",
    "AccountingInvoicesDeleteResponse",
    "AccountingInvoicesDeleteResponseTypedDict",
    "AccountingInvoicesOneGlobals",
    "AccountingInvoicesOneGlobalsTypedDict",
    "AccountingInvoicesOneRequest",
    "AccountingInvoicesOneRequestTypedDict",
    "AccountingInvoicesOneResponse",
    "AccountingInvoicesOneResponseTypedDict",
    "AccountingInvoicesUpdateGlobals",
    "AccountingInvoicesUpdateGlobalsTypedDict",
    "AccountingInvoicesUpdateRequest",
    "AccountingInvoicesUpdateRequestTypedDict",
    "AccountingInvoicesUpdateResponse",
    "AccountingInvoicesUpdateResponseTypedDict",
    "AccountingJournalEntriesAddGlobals",
    "AccountingJournalEntriesAddGlobalsTypedDict",
    "AccountingJournalEntriesAddRequest",
    "AccountingJournalEntriesAddRequestTypedDict",
    "AccountingJournalEntriesAddResponse",
    "AccountingJournalEntriesAddResponseTypedDict",
    "AccountingJournalEntriesAllGlobals",
    "AccountingJournalEntriesAllGlobalsTypedDict",
    "AccountingJournalEntriesAllRequest",
    "AccountingJournalEntriesAllRequestTypedDict",
    "AccountingJournalEntriesAllResponse",
    "AccountingJournalEntriesAllResponseTypedDict",
    "AccountingJournalEntriesDeleteGlobals",
    "AccountingJournalEntriesDeleteGlobalsTypedDict",
    "AccountingJournalEntriesDeleteRequest",
    "AccountingJournalEntriesDeleteRequestTypedDict",
    "AccountingJournalEntriesDeleteResponse",
    "AccountingJournalEntriesDeleteResponseTypedDict",
    "AccountingJournalEntriesOneGlobals",
    "AccountingJournalEntriesOneGlobalsTypedDict",
    "AccountingJournalEntriesOneRequest",
    "AccountingJournalEntriesOneRequestTypedDict",
    "AccountingJournalEntriesOneResponse",
    "AccountingJournalEntriesOneResponseTypedDict",
    "AccountingJournalEntriesUpdateGlobals",
    "AccountingJournalEntriesUpdateGlobalsTypedDict",
    "AccountingJournalEntriesUpdateRequest",
    "AccountingJournalEntriesUpdateRequestTypedDict",
    "AccountingJournalEntriesUpdateResponse",
    "AccountingJournalEntriesUpdateResponseTypedDict",
    "AccountingLedgerAccountsAddGlobals",
    "AccountingLedgerAccountsAddGlobalsTypedDict",
    "AccountingLedgerAccountsAddRequest",
    "AccountingLedgerAccountsAddRequestTypedDict",
    "AccountingLedgerAccountsAddResponse",
    "AccountingLedgerAccountsAddResponseTypedDict",
    "AccountingLedgerAccountsAllGlobals",
    "AccountingLedgerAccountsAllGlobalsTypedDict",
    "AccountingLedgerAccountsAllRequest",
    "AccountingLedgerAccountsAllRequestTypedDict",
    "AccountingLedgerAccountsAllResponse",
    "AccountingLedgerAccountsAllResponseTypedDict",
    "AccountingLedgerAccountsDeleteGlobals",
    "AccountingLedgerAccountsDeleteGlobalsTypedDict",
    "AccountingLedgerAccountsDeleteRequest",
    "AccountingLedgerAccountsDeleteRequestTypedDict",
    "AccountingLedgerAccountsDeleteResponse",
    "AccountingLedgerAccountsDeleteResponseTypedDict",
    "AccountingLedgerAccountsOneGlobals",
    "AccountingLedgerAccountsOneGlobalsTypedDict",
    "AccountingLedgerAccountsOneRequest",
    "AccountingLedgerAccountsOneRequestTypedDict",
    "AccountingLedgerAccountsOneResponse",
    "AccountingLedgerAccountsOneResponseTypedDict",
    "AccountingLedgerAccountsUpdateGlobals",
    "AccountingLedgerAccountsUpdateGlobalsTypedDict",
    "AccountingLedgerAccountsUpdateRequest",
    "AccountingLedgerAccountsUpdateRequestTypedDict",
    "AccountingLedgerAccountsUpdateResponse",
    "AccountingLedgerAccountsUpdateResponseTypedDict",
    "AccountingLocation",
    "AccountingLocationInput",
    "AccountingLocationInputTypedDict",
    "AccountingLocationTypedDict",
    "AccountingLocationsAddGlobals",
    "AccountingLocationsAddGlobalsTypedDict",
    "AccountingLocationsAddRequest",
    "AccountingLocationsAddRequestTypedDict",
    "AccountingLocationsAddResponse",
    "AccountingLocationsAddResponseTypedDict",
    "AccountingLocationsAllGlobals",
    "AccountingLocationsAllGlobalsTypedDict",
    "AccountingLocationsAllRequest",
    "AccountingLocationsAllRequestTypedDict",
    "AccountingLocationsAllResponse",
    "AccountingLocationsAllResponseTypedDict",
    "AccountingLocationsDeleteGlobals",
    "AccountingLocationsDeleteGlobalsTypedDict",
    "AccountingLocationsDeleteRequest",
    "AccountingLocationsDeleteRequestTypedDict",
    "AccountingLocationsDeleteResponse",
    "AccountingLocationsDeleteResponseTypedDict",
    "AccountingLocationsFilter",
    "AccountingLocationsFilterTypedDict",
    "AccountingLocationsOneGlobals",
    "AccountingLocationsOneGlobalsTypedDict",
    "AccountingLocationsOneRequest",
    "AccountingLocationsOneRequestTypedDict",
    "AccountingLocationsOneResponse",
    "AccountingLocationsOneResponseTypedDict",
    "AccountingLocationsUpdateGlobals",
    "AccountingLocationsUpdateGlobalsTypedDict",
    "AccountingLocationsUpdateRequest",
    "AccountingLocationsUpdateRequestTypedDict",
    "AccountingLocationsUpdateResponse",
    "AccountingLocationsUpdateResponseTypedDict",
    "AccountingPaymentsAddGlobals",
    "AccountingPaymentsAddGlobalsTypedDict",
    "AccountingPaymentsAddRequest",
    "AccountingPaymentsAddRequestTypedDict",
    "AccountingPaymentsAddResponse",
    "AccountingPaymentsAddResponseTypedDict",
    "AccountingPaymentsAllGlobals",
    "AccountingPaymentsAllGlobalsTypedDict",
    "AccountingPaymentsAllRequest",
    "AccountingPaymentsAllRequestTypedDict",
    "AccountingPaymentsAllResponse",
    "AccountingPaymentsAllResponseTypedDict",
    "AccountingPaymentsDeleteGlobals",
    "AccountingPaymentsDeleteGlobalsTypedDict",
    "AccountingPaymentsDeleteRequest",
    "AccountingPaymentsDeleteRequestTypedDict",
    "AccountingPaymentsDeleteResponse",
    "AccountingPaymentsDeleteResponseTypedDict",
    "AccountingPaymentsOneGlobals",
    "AccountingPaymentsOneGlobalsTypedDict",
    "AccountingPaymentsOneRequest",
    "AccountingPaymentsOneRequestTypedDict",
    "AccountingPaymentsOneResponse",
    "AccountingPaymentsOneResponseTypedDict",
    "AccountingPaymentsUpdateGlobals",
    "AccountingPaymentsUpdateGlobalsTypedDict",
    "AccountingPaymentsUpdateRequest",
    "AccountingPaymentsUpdateRequestTypedDict",
    "AccountingPaymentsUpdateResponse",
    "AccountingPaymentsUpdateResponseTypedDict",
    "AccountingProfitAndLossOneGlobals",
    "AccountingProfitAndLossOneGlobalsTypedDict",
    "AccountingProfitAndLossOneRequest",
    "AccountingProfitAndLossOneRequestTypedDict",
    "AccountingProfitAndLossOneResponse",
    "AccountingProfitAndLossOneResponseTypedDict",
    "AccountingPurchaseOrdersAddGlobals",
    "AccountingPurchaseOrdersAddGlobalsTypedDict",
    "AccountingPurchaseOrdersAddRequest",
    "AccountingPurchaseOrdersAddRequestTypedDict",
    "AccountingPurchaseOrdersAddResponse",
    "AccountingPurchaseOrdersAddResponseTypedDict",
    "AccountingPurchaseOrdersAllGlobals",
    "AccountingPurchaseOrdersAllGlobalsTypedDict",
    "AccountingPurchaseOrdersAllRequest",
    "AccountingPurchaseOrdersAllRequestTypedDict",
    "AccountingPurchaseOrdersAllResponse",
    "AccountingPurchaseOrdersAllResponseTypedDict",
    "AccountingPurchaseOrdersDeleteGlobals",
    "AccountingPurchaseOrdersDeleteGlobalsTypedDict",
    "AccountingPurchaseOrdersDeleteRequest",
    "AccountingPurchaseOrdersDeleteRequestTypedDict",
    "AccountingPurchaseOrdersDeleteResponse",
    "AccountingPurchaseOrdersDeleteResponseTypedDict",
    "AccountingPurchaseOrdersOneGlobals",
    "AccountingPurchaseOrdersOneGlobalsTypedDict",
    "AccountingPurchaseOrdersOneRequest",
    "AccountingPurchaseOrdersOneRequestTypedDict",
    "AccountingPurchaseOrdersOneResponse",
    "AccountingPurchaseOrdersOneResponseTypedDict",
    "AccountingPurchaseOrdersUpdateGlobals",
    "AccountingPurchaseOrdersUpdateGlobalsTypedDict",
    "AccountingPurchaseOrdersUpdateRequest",
    "AccountingPurchaseOrdersUpdateRequestTypedDict",
    "AccountingPurchaseOrdersUpdateResponse",
    "AccountingPurchaseOrdersUpdateResponseTypedDict",
    "AccountingQuotesAddGlobals",
    "AccountingQuotesAddGlobalsTypedDict",
    "AccountingQuotesAddRequest",
    "AccountingQuotesAddRequestTypedDict",
    "AccountingQuotesAddResponse",
    "AccountingQuotesAddResponseTypedDict",
    "AccountingQuotesAllGlobals",
    "AccountingQuotesAllGlobalsTypedDict",
    "AccountingQuotesAllRequest",
    "AccountingQuotesAllRequestTypedDict",
    "AccountingQuotesAllResponse",
    "AccountingQuotesAllResponseTypedDict",
    "AccountingQuotesDeleteGlobals",
    "AccountingQuotesDeleteGlobalsTypedDict",
    "AccountingQuotesDeleteRequest",
    "AccountingQuotesDeleteRequestTypedDict",
    "AccountingQuotesDeleteResponse",
    "AccountingQuotesDeleteResponseTypedDict",
    "AccountingQuotesOneGlobals",
    "AccountingQuotesOneGlobalsTypedDict",
    "AccountingQuotesOneRequest",
    "AccountingQuotesOneRequestTypedDict",
    "AccountingQuotesOneResponse",
    "AccountingQuotesOneResponseTypedDict",
    "AccountingQuotesUpdateGlobals",
    "AccountingQuotesUpdateGlobalsTypedDict",
    "AccountingQuotesUpdateRequest",
    "AccountingQuotesUpdateRequestTypedDict",
    "AccountingQuotesUpdateResponse",
    "AccountingQuotesUpdateResponseTypedDict",
    "AccountingSubsidiariesAddGlobals",
    "AccountingSubsidiariesAddGlobalsTypedDict",
    "AccountingSubsidiariesAddRequest",
    "AccountingSubsidiariesAddRequestTypedDict",
    "AccountingSubsidiariesAddResponse",
    "AccountingSubsidiariesAddResponseTypedDict",
    "AccountingSubsidiariesAllGlobals",
    "AccountingSubsidiariesAllGlobalsTypedDict",
    "AccountingSubsidiariesAllRequest",
    "AccountingSubsidiariesAllRequestTypedDict",
    "AccountingSubsidiariesAllResponse",
    "AccountingSubsidiariesAllResponseTypedDict",
    "AccountingSubsidiariesDeleteGlobals",
    "AccountingSubsidiariesDeleteGlobalsTypedDict",
    "AccountingSubsidiariesDeleteRequest",
    "AccountingSubsidiariesDeleteRequestTypedDict",
    "AccountingSubsidiariesDeleteResponse",
    "AccountingSubsidiariesDeleteResponseTypedDict",
    "AccountingSubsidiariesOneGlobals",
    "AccountingSubsidiariesOneGlobalsTypedDict",
    "AccountingSubsidiariesOneRequest",
    "AccountingSubsidiariesOneRequestTypedDict",
    "AccountingSubsidiariesOneResponse",
    "AccountingSubsidiariesOneResponseTypedDict",
    "AccountingSubsidiariesUpdateGlobals",
    "AccountingSubsidiariesUpdateGlobalsTypedDict",
    "AccountingSubsidiariesUpdateRequest",
    "AccountingSubsidiariesUpdateRequestTypedDict",
    "AccountingSubsidiariesUpdateResponse",
    "AccountingSubsidiariesUpdateResponseTypedDict",
    "AccountingSuppliersAddGlobals",
    "AccountingSuppliersAddGlobalsTypedDict",
    "AccountingSuppliersAddRequest",
    "AccountingSuppliersAddRequestTypedDict",
    "AccountingSuppliersAddResponse",
    "AccountingSuppliersAddResponseTypedDict",
    "AccountingSuppliersAllGlobals",
    "AccountingSuppliersAllGlobalsTypedDict",
    "AccountingSuppliersAllRequest",
    "AccountingSuppliersAllRequestTypedDict",
    "AccountingSuppliersAllResponse",
    "AccountingSuppliersAllResponseTypedDict",
    "AccountingSuppliersDeleteGlobals",
    "AccountingSuppliersDeleteGlobalsTypedDict",
    "AccountingSuppliersDeleteRequest",
    "AccountingSuppliersDeleteRequestTypedDict",
    "AccountingSuppliersDeleteResponse",
    "AccountingSuppliersDeleteResponseTypedDict",
    "AccountingSuppliersOneGlobals",
    "AccountingSuppliersOneGlobalsTypedDict",
    "AccountingSuppliersOneRequest",
    "AccountingSuppliersOneRequestTypedDict",
    "AccountingSuppliersOneResponse",
    "AccountingSuppliersOneResponseTypedDict",
    "AccountingSuppliersUpdateGlobals",
    "AccountingSuppliersUpdateGlobalsTypedDict",
    "AccountingSuppliersUpdateRequest",
    "AccountingSuppliersUpdateRequestTypedDict",
    "AccountingSuppliersUpdateResponse",
    "AccountingSuppliersUpdateResponseTypedDict",
    "AccountingTaxRatesAddGlobals",
    "AccountingTaxRatesAddGlobalsTypedDict",
    "AccountingTaxRatesAddRequest",
    "AccountingTaxRatesAddRequestTypedDict",
    "AccountingTaxRatesAddResponse",
    "AccountingTaxRatesAddResponseTypedDict",
    "AccountingTaxRatesAllGlobals",
    "AccountingTaxRatesAllGlobalsTypedDict",
    "AccountingTaxRatesAllRequest",
    "AccountingTaxRatesAllRequestTypedDict",
    "AccountingTaxRatesAllResponse",
    "AccountingTaxRatesAllResponseTypedDict",
    "AccountingTaxRatesDeleteGlobals",
    "AccountingTaxRatesDeleteGlobalsTypedDict",
    "AccountingTaxRatesDeleteRequest",
    "AccountingTaxRatesDeleteRequestTypedDict",
    "AccountingTaxRatesDeleteResponse",
    "AccountingTaxRatesDeleteResponseTypedDict",
    "AccountingTaxRatesOneGlobals",
    "AccountingTaxRatesOneGlobalsTypedDict",
    "AccountingTaxRatesOneRequest",
    "AccountingTaxRatesOneRequestTypedDict",
    "AccountingTaxRatesOneResponse",
    "AccountingTaxRatesOneResponseTypedDict",
    "AccountingTaxRatesUpdateGlobals",
    "AccountingTaxRatesUpdateGlobalsTypedDict",
    "AccountingTaxRatesUpdateRequest",
    "AccountingTaxRatesUpdateRequestTypedDict",
    "AccountingTaxRatesUpdateResponse",
    "AccountingTaxRatesUpdateResponseTypedDict",
    "AccountingTrackingCategoriesAddGlobals",
    "AccountingTrackingCategoriesAddGlobalsTypedDict",
    "AccountingTrackingCategoriesAddRequest",
    "AccountingTrackingCategoriesAddRequestTypedDict",
    "AccountingTrackingCategoriesAddResponse",
    "AccountingTrackingCategoriesAddResponseTypedDict",
    "AccountingTrackingCategoriesAllGlobals",
    "AccountingTrackingCategoriesAllGlobalsTypedDict",
    "AccountingTrackingCategoriesAllRequest",
    "AccountingTrackingCategoriesAllRequestTypedDict",
    "AccountingTrackingCategoriesAllResponse",
    "AccountingTrackingCategoriesAllResponseTypedDict",
    "AccountingTrackingCategoriesDeleteGlobals",
    "AccountingTrackingCategoriesDeleteGlobalsTypedDict",
    "AccountingTrackingCategoriesDeleteRequest",
    "AccountingTrackingCategoriesDeleteRequestTypedDict",
    "AccountingTrackingCategoriesDeleteResponse",
    "AccountingTrackingCategoriesDeleteResponseTypedDict",
    "AccountingTrackingCategoriesOneGlobals",
    "AccountingTrackingCategoriesOneGlobalsTypedDict",
    "AccountingTrackingCategoriesOneRequest",
    "AccountingTrackingCategoriesOneRequestTypedDict",
    "AccountingTrackingCategoriesOneResponse",
    "AccountingTrackingCategoriesOneResponseTypedDict",
    "AccountingTrackingCategoriesUpdateGlobals",
    "AccountingTrackingCategoriesUpdateGlobalsTypedDict",
    "AccountingTrackingCategoriesUpdateRequest",
    "AccountingTrackingCategoriesUpdateRequestTypedDict",
    "AccountingTrackingCategoriesUpdateResponse",
    "AccountingTrackingCategoriesUpdateResponseTypedDict",
    "ActivitiesFilter",
    "ActivitiesFilterTypedDict",
    "ActivitiesSort",
    "ActivitiesSortBy",
    "ActivitiesSortTypedDict",
    "Activity",
    "ActivityAttendee",
    "ActivityAttendeeInput",
    "ActivityAttendeeInputTypedDict",
    "ActivityAttendeeStatus",
    "ActivityAttendeeTypedDict",
    "ActivityInput",
    "ActivityInputTypedDict",
    "ActivityType",
    "ActivityTypedDict",
    "Address",
    "AddressTypedDict",
    "Addresses",
    "AddressesTypedDict",
    "AgedCreditors",
    "AgedCreditorsTypedDict",
    "AgedDebtors",
    "AgedDebtorsTypedDict",
    "AgedReportFilter",
    "AgedReportFilterTypedDict",
    "Allocation",
    "AllocationInput",
    "AllocationInputTypedDict",
    "AllocationType",
    "AllocationTypedDict",
    "Allocations",
    "AllocationsTypedDict",
    "AllowActions",
    "AmortizationType",
    "ApideckError",
    "ApisFilter",
    "ApisFilterTypedDict",
    "Applicant",
    "ApplicantGender",
    "ApplicantInput",
    "ApplicantInputTypedDict",
    "ApplicantType",
    "ApplicantTypedDict",
    "ApplicantsFilter",
    "ApplicantsFilterTypedDict",
    "Application",
    "ApplicationInput",
    "ApplicationInputTypedDict",
    "ApplicationStatus",
    "ApplicationTypedDict",
    "Assignee",
    "AssigneeInput",
    "AssigneeInputTypedDict",
    "AssigneeTypedDict",
    "AtsApplicantsAddGlobals",
    "AtsApplicantsAddGlobalsTypedDict",
    "AtsApplicantsAddRequest",
    "AtsApplicantsAddRequestTypedDict",
    "AtsApplicantsAddResponse",
    "AtsApplicantsAddResponseTypedDict",
    "AtsApplicantsAllGlobals",
    "AtsApplicantsAllGlobalsTypedDict",
    "AtsApplicantsAllRequest",
    "AtsApplicantsAllRequestTypedDict",
    "AtsApplicantsAllResponse",
    "AtsApplicantsAllResponseTypedDict",
    "AtsApplicantsDeleteGlobals",
    "AtsApplicantsDeleteGlobalsTypedDict",
    "AtsApplicantsDeleteRequest",
    "AtsApplicantsDeleteRequestTypedDict",
    "AtsApplicantsDeleteResponse",
    "AtsApplicantsDeleteResponseTypedDict",
    "AtsApplicantsOneGlobals",
    "AtsApplicantsOneGlobalsTypedDict",
    "AtsApplicantsOneRequest",
    "AtsApplicantsOneRequestTypedDict",
    "AtsApplicantsOneResponse",
    "AtsApplicantsOneResponseTypedDict",
    "AtsApplicantsUpdateGlobals",
    "AtsApplicantsUpdateGlobalsTypedDict",
    "AtsApplicantsUpdateRequest",
    "AtsApplicantsUpdateRequestTypedDict",
    "AtsApplicantsUpdateResponse",
    "AtsApplicantsUpdateResponseTypedDict",
    "AtsApplicationsAddGlobals",
    "AtsApplicationsAddGlobalsTypedDict",
    "AtsApplicationsAddRequest",
    "AtsApplicationsAddRequestTypedDict",
    "AtsApplicationsAddResponse",
    "AtsApplicationsAddResponseTypedDict",
    "AtsApplicationsAllGlobals",
    "AtsApplicationsAllGlobalsTypedDict",
    "AtsApplicationsAllRequest",
    "AtsApplicationsAllRequestTypedDict",
    "AtsApplicationsAllResponse",
    "AtsApplicationsAllResponseTypedDict",
    "AtsApplicationsDeleteGlobals",
    "AtsApplicationsDeleteGlobalsTypedDict",
    "AtsApplicationsDeleteRequest",
    "AtsApplicationsDeleteRequestTypedDict",
    "AtsApplicationsDeleteResponse",
    "AtsApplicationsDeleteResponseTypedDict",
    "AtsApplicationsOneGlobals",
    "AtsApplicationsOneGlobalsTypedDict",
    "AtsApplicationsOneRequest",
    "AtsApplicationsOneRequestTypedDict",
    "AtsApplicationsOneResponse",
    "AtsApplicationsOneResponseTypedDict",
    "AtsApplicationsUpdateGlobals",
    "AtsApplicationsUpdateGlobalsTypedDict",
    "AtsApplicationsUpdateRequest",
    "AtsApplicationsUpdateRequestTypedDict",
    "AtsApplicationsUpdateResponse",
    "AtsApplicationsUpdateResponseTypedDict",
    "AtsJobsAllGlobals",
    "AtsJobsAllGlobalsTypedDict",
    "AtsJobsAllRequest",
    "AtsJobsAllRequestTypedDict",
    "AtsJobsAllResponse",
    "AtsJobsAllResponseTypedDict",
    "AtsJobsOneGlobals",
    "AtsJobsOneGlobalsTypedDict",
    "AtsJobsOneRequest",
    "AtsJobsOneRequestTypedDict",
    "AtsJobsOneResponse",
    "AtsJobsOneResponseTypedDict",
    "Attachment",
    "AttachmentReference",
    "AttachmentReferenceType",
    "AttachmentReferenceTypedDict",
    "AttachmentTypedDict",
    "Attempts",
    "AttemptsTypedDict",
    "Audience",
    "AuthType",
    "BadRequestResponse",
    "BadRequestResponseData",
    "BadRequestResponseDetail",
    "BalanceByPeriod",
    "BalanceByPeriodTypedDict",
    "BalanceByTransaction",
    "BalanceByTransactionTypedDict",
    "BalanceSheet",
    "BalanceSheetAssetsAccount",
    "BalanceSheetAssetsAccountTypedDict",
    "BalanceSheetEquityAccount",
    "BalanceSheetEquityAccountTypedDict",
    "BalanceSheetFilter",
    "BalanceSheetFilterTypedDict",
    "BalanceSheetLiabilitiesAccount",
    "BalanceSheetLiabilitiesAccountTypedDict",
    "BalanceSheetTypedDict",
    "BalanceSheetUncategorizedItemsAccount",
    "BalanceSheetUncategorizedItemsAccountTypedDict",
    "BankAccount",
    "BankAccount1",
    "BankAccount1AccountType",
    "BankAccount1TypedDict",
    "BankAccountType",
    "BankAccountTypedDict",
    "BankFeedAccount",
    "BankFeedAccountInput",
    "BankFeedAccountInputTypedDict",
    "BankFeedAccountTypedDict",
    "BankFeedStatement",
    "BankFeedStatementInput",
    "BankFeedStatementInputTypedDict",
    "BankFeedStatementTransactionType",
    "BankFeedStatementTypedDict",
    "Benefit",
    "BenefitTypedDict",
    "Bill",
    "BillInput",
    "BillInputTypedDict",
    "BillLineItem",
    "BillLineItemInput",
    "BillLineItemInputTypedDict",
    "BillLineItemType",
    "BillLineItemTypedDict",
    "BillPayment",
    "BillPaymentAllocationType",
    "BillPaymentAllocations",
    "BillPaymentAllocationsTypedDict",
    "BillPaymentInput",
    "BillPaymentInputTypedDict",
    "BillPaymentType",
    "BillPaymentTypedDict",
    "BillStatus",
    "BillTypedDict",
    "BillsFilter",
    "BillsFilterTypedDict",
    "BillsSort",
    "BillsSortTypedDict",
    "Blocks",
    "BlocksTypedDict",
    "Branch",
    "BranchTypedDict",
    "By",
    "CategoriesFilter",
    "CategoriesFilterType",
    "CategoriesFilterTypedDict",
    "CategoriesModel",
    "CategoriesModelTypedDict",
    "Category",
    "CategoryStatus",
    "CategoryType",
    "CategoryTypedDict",
    "Classification",
    "Collection",
    "CollectionTag",
    "CollectionTagInput",
    "CollectionTagInputTypedDict",
    "CollectionTagTypedDict",
    "CollectionTicketComment",
    "CollectionTicketCommentInput",
    "CollectionTicketCommentInputTypedDict",
    "CollectionTicketCommentTypedDict",
    "CollectionTypedDict",
    "CollectionUser",
    "CollectionUserTypedDict",
    "CollectionsSort",
    "CollectionsSortBy",
    "CollectionsSortTypedDict",
    "CommentsSort",
    "CommentsSortBy",
    "CommentsSortTypedDict",
    "CompaniesFilter",
    "CompaniesFilterTypedDict",
    "CompaniesSort",
    "CompaniesSortBy",
    "CompaniesSortTypedDict",
    "Company1",
    "Company1Input",
    "Company1InputTypedDict",
    "Company1TypedDict",
    "CompanyInfo",
    "CompanyInfoTypedDict",
    "CompanyRowType",
    "CompanyRowTypeTypedDict",
    "CompanyStatus",
    "Compensation",
    "CompensationTypedDict",
    "Components",
    "ComponentsTypedDict",
    "Configuration",
    "ConfigurationTypedDict",
    "Connection",
    "ConnectionConfiguration",
    "ConnectionConfigurationTypedDict",
    "ConnectionDefaults",
    "ConnectionDefaultsTypedDict",
    "ConnectionImportData",
    "ConnectionImportDataTypedDict",
    "ConnectionInput",
    "ConnectionInputTypedDict",
    "ConnectionState",
    "ConnectionStatus",
    "ConnectionTypedDict",
    "ConnectionValue",
    "ConnectionValue5",
    "ConnectionValue5TypedDict",
    "ConnectionValueTypedDict",
    "Connector",
    "ConnectorAPIResourceCoverageOneGlobals",
    "ConnectorAPIResourceCoverageOneGlobalsTypedDict",
    "ConnectorAPIResourceCoverageOneRequest",
    "ConnectorAPIResourceCoverageOneRequestTypedDict",
    "ConnectorAPIResourceCoverageOneResponse",
    "ConnectorAPIResourceCoverageOneResponseTypedDict",
    "ConnectorAPIResourcesOneGlobals",
    "ConnectorAPIResourcesOneGlobalsTypedDict",
    "ConnectorAPIResourcesOneRequest",
    "ConnectorAPIResourcesOneRequestTypedDict",
    "ConnectorAPIResourcesOneResponse",
    "ConnectorAPIResourcesOneResponseTypedDict",
    "ConnectorApisAllGlobals",
    "ConnectorApisAllGlobalsTypedDict",
    "ConnectorApisAllRequest",
    "ConnectorApisAllRequestTypedDict",
    "ConnectorApisAllResponse",
    "ConnectorApisAllResponseTypedDict",
    "ConnectorApisOneGlobals",
    "ConnectorApisOneGlobalsTypedDict",
    "ConnectorApisOneRequest",
    "ConnectorApisOneRequestTypedDict",
    "ConnectorApisOneResponse",
    "ConnectorApisOneResponseTypedDict",
    "ConnectorAuthType",
    "ConnectorConnectorDocsOneGlobals",
    "ConnectorConnectorDocsOneGlobalsTypedDict",
    "ConnectorConnectorDocsOneRequest",
    "ConnectorConnectorDocsOneRequestTypedDict",
    "ConnectorConnectorDocsOneResponse",
    "ConnectorConnectorDocsOneResponseTypedDict",
    "ConnectorConnectorResourcesOneGlobals",
    "ConnectorConnectorResourcesOneGlobalsTypedDict",
    "ConnectorConnectorResourcesOneRequest",
    "ConnectorConnectorResourcesOneRequestTypedDict",
    "ConnectorConnectorResourcesOneResponse",
    "ConnectorConnectorResourcesOneResponseTypedDict",
    "ConnectorConnectorsAllGlobals",
    "ConnectorConnectorsAllGlobalsTypedDict",
    "ConnectorConnectorsAllRequest",
    "ConnectorConnectorsAllRequestTypedDict",
    "ConnectorConnectorsAllResponse",
    "ConnectorConnectorsAllResponseTypedDict",
    "ConnectorConnectorsOneGlobals",
    "ConnectorConnectorsOneGlobalsTypedDict",
    "ConnectorConnectorsOneRequest",
    "ConnectorConnectorsOneRequestTypedDict",
    "ConnectorConnectorsOneResponse",
    "ConnectorConnectorsOneResponseTypedDict",
    "ConnectorDoc",
    "ConnectorDocTypedDict",
    "ConnectorEvent",
    "ConnectorEventTypedDict",
    "ConnectorOauthGrantType",
    "ConnectorOauthScopes",
    "ConnectorOauthScopesTypedDict",
    "ConnectorResource",
    "ConnectorResourceTypedDict",
    "ConnectorSetting",
    "ConnectorSettingType",
    "ConnectorSettingTypedDict",
    "ConnectorStatus",
    "ConnectorTypedDict",
    "ConnectorsFilter",
    "ConnectorsFilterTypedDict",
    "Consumer",
    "ConsumerConnection",
    "ConsumerConnectionTypedDict",
    "ConsumerMetadata",
    "ConsumerMetadataTypedDict",
    "ConsumerRequestCountsInDateRangeResponse",
    "ConsumerRequestCountsInDateRangeResponseData",
    "ConsumerRequestCountsInDateRangeResponseDataTypedDict",
    "ConsumerRequestCountsInDateRangeResponseTypedDict",
    "ConsumerTypedDict",
    "Contact",
    "ContactGender",
    "ContactInput",
    "ContactInputTypedDict",
    "ContactType",
    "ContactTypedDict",
    "ContactsFilter",
    "ContactsFilterTypedDict",
    "ContactsSort",
    "ContactsSortBy",
    "ContactsSortTypedDict",
    "CopyFolderRequest",
    "CopyFolderRequestTypedDict",
    "CostOfGoodsSold",
    "CostOfGoodsSoldTypedDict",
    "Coverage",
    "CoverageTypedDict",
    "CreateAccountingDepartmentResponse",
    "CreateAccountingDepartmentResponseTypedDict",
    "CreateAccountingLocationResponse",
    "CreateAccountingLocationResponseTypedDict",
    "CreateActivityResponse",
    "CreateActivityResponseTypedDict",
    "CreateApplicantResponse",
    "CreateApplicantResponseTypedDict",
    "CreateApplicationResponse",
    "CreateApplicationResponseTypedDict",
    "CreateAttachmentResponse",
    "CreateAttachmentResponseTypedDict",
    "CreateBankFeedAccountResponse",
    "CreateBankFeedAccountResponseTypedDict",
    "CreateBankFeedStatementResponse",
    "CreateBankFeedStatementResponseTypedDict",
    "CreateBillPaymentResponse",
    "CreateBillPaymentResponseTypedDict",
    "CreateBillResponse",
    "CreateBillResponseTypedDict",
    "CreateCallbackState",
    "CreateCallbackStateResponse",
    "CreateCallbackStateResponseData",
    "CreateCallbackStateResponseDataTypedDict",
    "CreateCallbackStateResponseTypedDict",
    "CreateCallbackStateTypedDict",
    "CreateCommentResponse",
    "CreateCommentResponseTypedDict",
    "CreateCompanyResponse",
    "CreateCompanyResponseTypedDict",
    "CreateConnectionResponse",
    "CreateConnectionResponseTypedDict",
    "CreateConsumerRequest",
    "CreateConsumerRequestTypedDict",
    "CreateConsumerResponse",
    "CreateConsumerResponseTypedDict",
    "CreateContactResponse",
    "CreateContactResponseTypedDict",
    "CreateCreditNoteResponse",
    "CreateCreditNoteResponseTypedDict",
    "CreateCustomObjectResponse",
    "CreateCustomObjectResponseTypedDict",
    "CreateCustomObjectSchemaResponse",
    "CreateCustomObjectSchemaResponseTypedDict",
    "CreateCustomerResponse",
    "CreateCustomerResponseTypedDict",
    "CreateDepartmentResponse",
    "CreateDepartmentResponseTypedDict",
    "CreateDriveGroupResponse",
    "CreateDriveGroupResponseTypedDict",
    "CreateDriveResponse",
    "CreateDriveResponseTypedDict",
    "CreateEmployeeResponse",
    "CreateEmployeeResponseTypedDict",
    "CreateExpenseResponse",
    "CreateExpenseResponseTypedDict",
    "CreateFolderRequest",
    "CreateFolderRequestTypedDict",
    "CreateFolderResponse",
    "CreateFolderResponseTypedDict",
    "CreateHrisCompanyResponse",
    "CreateHrisCompanyResponseTypedDict",
    "CreateInvoiceItemResponse",
    "CreateInvoiceItemResponseTypedDict",
    "CreateInvoiceResponse",
    "CreateInvoiceResponseTypedDict",
    "CreateJournalEntryResponse",
    "CreateJournalEntryResponseTypedDict",
    "CreateLeadResponse",
    "CreateLeadResponseTypedDict",
    "CreateLedgerAccountResponse",
    "CreateLedgerAccountResponseTypedDict",
    "CreateMessageResponse",
    "CreateMessageResponseTypedDict",
    "CreateNoteResponse",
    "CreateNoteResponseTypedDict",
    "CreateOpportunityResponse",
    "CreateOpportunityResponseTypedDict",
    "CreatePaymentResponse",
    "CreatePaymentResponseTypedDict",
    "CreatePipelineResponse",
    "CreatePipelineResponseTypedDict",
    "CreatePurchaseOrderResponse",
    "CreatePurchaseOrderResponseTypedDict",
    "CreateQuoteResponse",
    "CreateQuoteResponseTypedDict",
    "CreateSessionResponse",
    "CreateSessionResponseData",
    "CreateSessionResponseDataTypedDict",
    "CreateSessionResponseTypedDict",
    "CreateSharedLinkResponse",
    "CreateSharedLinkResponseTypedDict",
    "CreateSubsidiaryResponse",
    "CreateSubsidiaryResponseTypedDict",
    "CreateSupplierResponse",
    "CreateSupplierResponseTypedDict",
    "CreateTaxRateResponse",
    "CreateTaxRateResponseTypedDict",
    "CreateTicketResponse",
    "CreateTicketResponseTypedDict",
    "CreateTimeOffRequestResponse",
    "CreateTimeOffRequestResponseTypedDict",
    "CreateTrackingCategoryResponse",
    "CreateTrackingCategoryResponseTypedDict",
    "CreateUploadSessionRequest",
    "CreateUploadSessionRequestTypedDict",
    "CreateUploadSessionResponse",
    "CreateUploadSessionResponseTypedDict",
    "CreateUserResponse",
    "CreateUserResponseTypedDict",
    "CreateWebhookRequest",
    "CreateWebhookRequestTypedDict",
    "CreateWebhookResponse",
    "CreateWebhookResponseTypedDict",
    "Credentials",
    "CredentialsTypedDict",
    "CreditNote",
    "CreditNoteInput",
    "CreditNoteInputTypedDict",
    "CreditNoteStatus",
    "CreditNoteType",
    "CreditNoteTypedDict",
    "CreditNotesFilter",
    "CreditNotesFilterTypedDict",
    "CreditNotesSort",
    "CreditNotesSortBy",
    "CreditNotesSortTypedDict",
    "CreditOrDebit",
    "CrmActivitiesAddGlobals",
    "CrmActivitiesAddGlobalsTypedDict",
    "CrmActivitiesAddRequest",
    "CrmActivitiesAddRequestTypedDict",
    "CrmActivitiesAddResponse",
    "CrmActivitiesAddResponseTypedDict",
    "CrmActivitiesAllGlobals",
    "CrmActivitiesAllGlobalsTypedDict",
    "CrmActivitiesAllRequest",
    "CrmActivitiesAllRequestTypedDict",
    "CrmActivitiesAllResponse",
    "CrmActivitiesAllResponseTypedDict",
    "CrmActivitiesDeleteGlobals",
    "CrmActivitiesDeleteGlobalsTypedDict",
    "CrmActivitiesDeleteRequest",
    "CrmActivitiesDeleteRequestTypedDict",
    "CrmActivitiesDeleteResponse",
    "CrmActivitiesDeleteResponseTypedDict",
    "CrmActivitiesOneGlobals",
    "CrmActivitiesOneGlobalsTypedDict",
    "CrmActivitiesOneRequest",
    "CrmActivitiesOneRequestTypedDict",
    "CrmActivitiesOneResponse",
    "CrmActivitiesOneResponseTypedDict",
    "CrmActivitiesUpdateGlobals",
    "CrmActivitiesUpdateGlobalsTypedDict",
    "CrmActivitiesUpdateRequest",
    "CrmActivitiesUpdateRequestTypedDict",
    "CrmActivitiesUpdateResponse",
    "CrmActivitiesUpdateResponseTypedDict",
    "CrmCompaniesAddGlobals",
    "CrmCompaniesAddGlobalsTypedDict",
    "CrmCompaniesAddRequest",
    "CrmCompaniesAddRequestTypedDict",
    "CrmCompaniesAddResponse",
    "CrmCompaniesAddResponseTypedDict",
    "CrmCompaniesAllGlobals",
    "CrmCompaniesAllGlobalsTypedDict",
    "CrmCompaniesAllRequest",
    "CrmCompaniesAllRequestTypedDict",
    "CrmCompaniesAllResponse",
    "CrmCompaniesAllResponseTypedDict",
    "CrmCompaniesDeleteGlobals",
    "CrmCompaniesDeleteGlobalsTypedDict",
    "CrmCompaniesDeleteRequest",
    "CrmCompaniesDeleteRequestTypedDict",
    "CrmCompaniesDeleteResponse",
    "CrmCompaniesDeleteResponseTypedDict",
    "CrmCompaniesOneGlobals",
    "CrmCompaniesOneGlobalsTypedDict",
    "CrmCompaniesOneRequest",
    "CrmCompaniesOneRequestTypedDict",
    "CrmCompaniesOneResponse",
    "CrmCompaniesOneResponseTypedDict",
    "CrmCompaniesUpdateGlobals",
    "CrmCompaniesUpdateGlobalsTypedDict",
    "CrmCompaniesUpdateRequest",
    "CrmCompaniesUpdateRequestTypedDict",
    "CrmCompaniesUpdateResponse",
    "CrmCompaniesUpdateResponseTypedDict",
    "CrmContactsAddGlobals",
    "CrmContactsAddGlobalsTypedDict",
    "CrmContactsAddRequest",
    "CrmContactsAddRequestTypedDict",
    "CrmContactsAddResponse",
    "CrmContactsAddResponseTypedDict",
    "CrmContactsAllGlobals",
    "CrmContactsAllGlobalsTypedDict",
    "CrmContactsAllRequest",
    "CrmContactsAllRequestTypedDict",
    "CrmContactsAllResponse",
    "CrmContactsAllResponseTypedDict",
    "CrmContactsDeleteGlobals",
    "CrmContactsDeleteGlobalsTypedDict",
    "CrmContactsDeleteRequest",
    "CrmContactsDeleteRequestTypedDict",
    "CrmContactsDeleteResponse",
    "CrmContactsDeleteResponseTypedDict",
    "CrmContactsOneGlobals",
    "CrmContactsOneGlobalsTypedDict",
    "CrmContactsOneRequest",
    "CrmContactsOneRequestTypedDict",
    "CrmContactsOneResponse",
    "CrmContactsOneResponseTypedDict",
    "CrmContactsUpdateGlobals",
    "CrmContactsUpdateGlobalsTypedDict",
    "CrmContactsUpdateRequest",
    "CrmContactsUpdateRequestTypedDict",
    "CrmContactsUpdateResponse",
    "CrmContactsUpdateResponseTypedDict",
    "CrmCustomObjectSchemasAddGlobals",
    "CrmCustomObjectSchemasAddGlobalsTypedDict",
    "CrmCustomObjectSchemasAddRequest",
    "CrmCustomObjectSchemasAddRequestTypedDict",
    "CrmCustomObjectSchemasAddResponse",
    "CrmCustomObjectSchemasAddResponseTypedDict",
    "CrmCustomObjectSchemasAllGlobals",
    "CrmCustomObjectSchemasAllGlobalsTypedDict",
    "CrmCustomObjectSchemasAllRequest",
    "CrmCustomObjectSchemasAllRequestTypedDict",
    "CrmCustomObjectSchemasAllResponse",
    "CrmCustomObjectSchemasAllResponseTypedDict",
    "CrmCustomObjectSchemasDeleteGlobals",
    "CrmCustomObjectSchemasDeleteGlobalsTypedDict",
    "CrmCustomObjectSchemasDeleteRequest",
    "CrmCustomObjectSchemasDeleteRequestTypedDict",
    "CrmCustomObjectSchemasDeleteResponse",
    "CrmCustomObjectSchemasDeleteResponseTypedDict",
    "CrmCustomObjectSchemasOneGlobals",
    "CrmCustomObjectSchemasOneGlobalsTypedDict",
    "CrmCustomObjectSchemasOneRequest",
    "CrmCustomObjectSchemasOneRequestTypedDict",
    "CrmCustomObjectSchemasOneResponse",
    "CrmCustomObjectSchemasOneResponseTypedDict",
    "CrmCustomObjectSchemasUpdateGlobals",
    "CrmCustomObjectSchemasUpdateGlobalsTypedDict",
    "CrmCustomObjectSchemasUpdateRequest",
    "CrmCustomObjectSchemasUpdateRequestTypedDict",
    "CrmCustomObjectSchemasUpdateResponse",
    "CrmCustomObjectSchemasUpdateResponseTypedDict",
    "CrmCustomObjectsAddGlobals",
    "CrmCustomObjectsAddGlobalsTypedDict",
    "CrmCustomObjectsAddRequest",
    "CrmCustomObjectsAddRequestTypedDict",
    "CrmCustomObjectsAddResponse",
    "CrmCustomObjectsAddResponseTypedDict",
    "CrmCustomObjectsAllGlobals",
    "CrmCustomObjectsAllGlobalsTypedDict",
    "CrmCustomObjectsAllRequest",
    "CrmCustomObjectsAllRequestTypedDict",
    "CrmCustomObjectsAllResponse",
    "CrmCustomObjectsAllResponseTypedDict",
    "CrmCustomObjectsDeleteGlobals",
    "CrmCustomObjectsDeleteGlobalsTypedDict",
    "CrmCustomObjectsDeleteRequest",
    "CrmCustomObjectsDeleteRequestTypedDict",
    "CrmCustomObjectsDeleteResponse",
    "CrmCustomObjectsDeleteResponseTypedDict",
    "CrmCustomObjectsOneGlobals",
    "CrmCustomObjectsOneGlobalsTypedDict",
    "CrmCustomObjectsOneRequest",
    "CrmCustomObjectsOneRequestTypedDict",
    "CrmCustomObjectsOneResponse",
    "CrmCustomObjectsOneResponseTypedDict",
    "CrmCustomObjectsUpdateGlobals",
    "CrmCustomObjectsUpdateGlobalsTypedDict",
    "CrmCustomObjectsUpdateRequest",
    "CrmCustomObjectsUpdateRequestTypedDict",
    "CrmCustomObjectsUpdateResponse",
    "CrmCustomObjectsUpdateResponseTypedDict",
    "CrmLeadsAddGlobals",
    "CrmLeadsAddGlobalsTypedDict",
    "CrmLeadsAddRequest",
    "CrmLeadsAddRequestTypedDict",
    "CrmLeadsAddResponse",
    "CrmLeadsAddResponseTypedDict",
    "CrmLeadsAllGlobals",
    "CrmLeadsAllGlobalsTypedDict",
    "CrmLeadsAllRequest",
    "CrmLeadsAllRequestTypedDict",
    "CrmLeadsAllResponse",
    "CrmLeadsAllResponseTypedDict",
    "CrmLeadsDeleteGlobals",
    "CrmLeadsDeleteGlobalsTypedDict",
    "CrmLeadsDeleteRequest",
    "CrmLeadsDeleteRequestTypedDict",
    "CrmLeadsDeleteResponse",
    "CrmLeadsDeleteResponseTypedDict",
    "CrmLeadsOneGlobals",
    "CrmLeadsOneGlobalsTypedDict",
    "CrmLeadsOneRequest",
    "CrmLeadsOneRequestTypedDict",
    "CrmLeadsOneResponse",
    "CrmLeadsOneResponseTypedDict",
    "CrmLeadsUpdateGlobals",
    "CrmLeadsUpdateGlobalsTypedDict",
    "CrmLeadsUpdateRequest",
    "CrmLeadsUpdateRequestTypedDict",
    "CrmLeadsUpdateResponse",
    "CrmLeadsUpdateResponseTypedDict",
    "CrmNotesAddGlobals",
    "CrmNotesAddGlobalsTypedDict",
    "CrmNotesAddRequest",
    "CrmNotesAddRequestTypedDict",
    "CrmNotesAddResponse",
    "CrmNotesAddResponseTypedDict",
    "CrmNotesAllGlobals",
    "CrmNotesAllGlobalsTypedDict",
    "CrmNotesAllRequest",
    "CrmNotesAllRequestTypedDict",
    "CrmNotesAllResponse",
    "CrmNotesAllResponseTypedDict",
    "CrmNotesDeleteGlobals",
    "CrmNotesDeleteGlobalsTypedDict",
    "CrmNotesDeleteRequest",
    "CrmNotesDeleteRequestTypedDict",
    "CrmNotesDeleteResponse",
    "CrmNotesDeleteResponseTypedDict",
    "CrmNotesOneGlobals",
    "CrmNotesOneGlobalsTypedDict",
    "CrmNotesOneRequest",
    "CrmNotesOneRequestTypedDict",
    "CrmNotesOneResponse",
    "CrmNotesOneResponseTypedDict",
    "CrmNotesUpdateGlobals",
    "CrmNotesUpdateGlobalsTypedDict",
    "CrmNotesUpdateRequest",
    "CrmNotesUpdateRequestTypedDict",
    "CrmNotesUpdateResponse",
    "CrmNotesUpdateResponseTypedDict",
    "CrmOpportunitiesAddGlobals",
    "CrmOpportunitiesAddGlobalsTypedDict",
    "CrmOpportunitiesAddRequest",
    "CrmOpportunitiesAddRequestTypedDict",
    "CrmOpportunitiesAddResponse",
    "CrmOpportunitiesAddResponseTypedDict",
    "CrmOpportunitiesAllGlobals",
    "CrmOpportunitiesAllGlobalsTypedDict",
    "CrmOpportunitiesAllRequest",
    "CrmOpportunitiesAllRequestTypedDict",
    "CrmOpportunitiesAllResponse",
    "CrmOpportunitiesAllResponseTypedDict",
    "CrmOpportunitiesDeleteGlobals",
    "CrmOpportunitiesDeleteGlobalsTypedDict",
    "CrmOpportunitiesDeleteRequest",
    "CrmOpportunitiesDeleteRequestTypedDict",
    "CrmOpportunitiesDeleteResponse",
    "CrmOpportunitiesDeleteResponseTypedDict",
    "CrmOpportunitiesOneGlobals",
    "CrmOpportunitiesOneGlobalsTypedDict",
    "CrmOpportunitiesOneRequest",
    "CrmOpportunitiesOneRequestTypedDict",
    "CrmOpportunitiesOneResponse",
    "CrmOpportunitiesOneResponseTypedDict",
    "CrmOpportunitiesUpdateGlobals",
    "CrmOpportunitiesUpdateGlobalsTypedDict",
    "CrmOpportunitiesUpdateRequest",
    "CrmOpportunitiesUpdateRequestTypedDict",
    "CrmOpportunitiesUpdateResponse",
    "CrmOpportunitiesUpdateResponseTypedDict",
    "CrmPipelinesAddGlobals",
    "CrmPipelinesAddGlobalsTypedDict",
    "CrmPipelinesAddRequest",
    "CrmPipelinesAddRequestTypedDict",
    "CrmPipelinesAddResponse",
    "CrmPipelinesAddResponseTypedDict",
    "CrmPipelinesAllGlobals",
    "CrmPipelinesAllGlobalsTypedDict",
    "CrmPipelinesAllRequest",
    "CrmPipelinesAllRequestTypedDict",
    "CrmPipelinesAllResponse",
    "CrmPipelinesAllResponseTypedDict",
    "CrmPipelinesDeleteGlobals",
    "CrmPipelinesDeleteGlobalsTypedDict",
    "CrmPipelinesDeleteRequest",
    "CrmPipelinesDeleteRequestTypedDict",
    "CrmPipelinesDeleteResponse",
    "CrmPipelinesDeleteResponseTypedDict",
    "CrmPipelinesOneGlobals",
    "CrmPipelinesOneGlobalsTypedDict",
    "CrmPipelinesOneRequest",
    "CrmPipelinesOneRequestTypedDict",
    "CrmPipelinesOneResponse",
    "CrmPipelinesOneResponseTypedDict",
    "CrmPipelinesUpdateGlobals",
    "CrmPipelinesUpdateGlobalsTypedDict",
    "CrmPipelinesUpdateRequest",
    "CrmPipelinesUpdateRequestTypedDict",
    "CrmPipelinesUpdateResponse",
    "CrmPipelinesUpdateResponseTypedDict",
    "CrmUsersAddGlobals",
    "CrmUsersAddGlobalsTypedDict",
    "CrmUsersAddRequest",
    "CrmUsersAddRequestTypedDict",
    "CrmUsersAddResponse",
    "CrmUsersAddResponseTypedDict",
    "CrmUsersAllGlobals",
    "CrmUsersAllGlobalsTypedDict",
    "CrmUsersAllRequest",
    "CrmUsersAllRequestTypedDict",
    "CrmUsersAllResponse",
    "CrmUsersAllResponseTypedDict",
    "CrmUsersDeleteGlobals",
    "CrmUsersDeleteGlobalsTypedDict",
    "CrmUsersDeleteRequest",
    "CrmUsersDeleteRequestTypedDict",
    "CrmUsersDeleteResponse",
    "CrmUsersDeleteResponseTypedDict",
    "CrmUsersOneGlobals",
    "CrmUsersOneGlobalsTypedDict",
    "CrmUsersOneRequest",
    "CrmUsersOneRequestTypedDict",
    "CrmUsersOneResponse",
    "CrmUsersOneResponseTypedDict",
    "CrmUsersUpdateGlobals",
    "CrmUsersUpdateGlobalsTypedDict",
    "CrmUsersUpdateRequest",
    "CrmUsersUpdateRequestTypedDict",
    "CrmUsersUpdateResponse",
    "CrmUsersUpdateResponseTypedDict",
    "Currency",
    "Cursors",
    "CursorsTypedDict",
    "CustomField",
    "CustomFieldFinder",
    "CustomFieldFinderTypedDict",
    "CustomFieldTypedDict",
    "CustomMapping",
    "CustomMappingInput",
    "CustomMappingInputTypedDict",
    "CustomMappingTypedDict",
    "CustomObject",
    "CustomObjectFields",
    "CustomObjectFieldsTypedDict",
    "CustomObjectInput",
    "CustomObjectInputTypedDict",
    "CustomObjectSchema",
    "CustomObjectSchemaInput",
    "CustomObjectSchemaInputTypedDict",
    "CustomObjectSchemaOptions",
    "CustomObjectSchemaOptionsTypedDict",
    "CustomObjectSchemaType",
    "CustomObjectSchemaTypedDict",
    "CustomObjectTypedDict",
    "Customer",
    "CustomerInput",
    "CustomerInputTypedDict",
    "CustomerStatus",
    "CustomerStatusStatus",
    "CustomerTypedDict",
    "CustomersFilter",
    "CustomersFilterStatus",
    "CustomersFilterTypedDict",
    "CustomersSort",
    "CustomersSortBy",
    "CustomersSortTypedDict",
    "Data",
    "DataTypedDict",
    "Deduction",
    "DeductionTypedDict",
    "Defaults",
    "DefaultsTypedDict",
    "DeleteAccountingDepartmentResponse",
    "DeleteAccountingDepartmentResponseTypedDict",
    "DeleteAccountingLocationResponse",
    "DeleteAccountingLocationResponseTypedDict",
    "DeleteActivityResponse",
    "DeleteActivityResponseTypedDict",
    "DeleteApplicantResponse",
    "DeleteApplicantResponseTypedDict",
    "DeleteApplicationResponse",
    "DeleteApplicationResponseTypedDict",
    "DeleteAttachmentResponse",
    "DeleteAttachmentResponseTypedDict",
    "DeleteBankFeedAccountResponse",
    "DeleteBankFeedAccountResponseTypedDict",
    "DeleteBankFeedStatementResponse",
    "DeleteBankFeedStatementResponseTypedDict",
    "DeleteBillPaymentResponse",
    "DeleteBillPaymentResponseTypedDict",
    "DeleteBillResponse",
    "DeleteBillResponseTypedDict",
    "DeleteCommentResponse",
    "DeleteCommentResponseTypedDict",
    "DeleteCompanyResponse",
    "DeleteCompanyResponseTypedDict",
    "DeleteConsumerResponse",
    "DeleteConsumerResponseData",
    "DeleteConsumerResponseDataTypedDict",
    "DeleteConsumerResponseTypedDict",
    "DeleteContactResponse",
    "DeleteContactResponseTypedDict",
    "DeleteCreditNoteResponse",
    "DeleteCreditNoteResponseTypedDict",
    "DeleteCustomObjectResponse",
    "DeleteCustomObjectResponseTypedDict",
    "DeleteCustomObjectSchemaResponse",
    "DeleteCustomObjectSchemaResponseTypedDict",
    "DeleteCustomerResponse",
    "DeleteCustomerResponseTypedDict",
    "DeleteDepartmentResponse",
    "DeleteDepartmentResponseTypedDict",
    "DeleteDriveGroupResponse",
    "DeleteDriveGroupResponseTypedDict",
    "DeleteDriveResponse",
    "DeleteDriveResponseTypedDict",
    "DeleteEmployeeResponse",
    "DeleteEmployeeResponseTypedDict",
    "DeleteExpenseResponse",
    "DeleteExpenseResponseTypedDict",
    "DeleteFileResponse",
    "DeleteFileResponseTypedDict",
    "DeleteFolderResponse",
    "DeleteFolderResponseTypedDict",
    "DeleteHrisCompanyResponse",
    "DeleteHrisCompanyResponseTypedDict",
    "DeleteInvoiceResponse",
    "DeleteInvoiceResponseTypedDict",
    "DeleteJournalEntryResponse",
    "DeleteJournalEntryResponseTypedDict",
    "DeleteLeadResponse",
    "DeleteLeadResponseTypedDict",
    "DeleteLedgerAccountResponse",
    "DeleteLedgerAccountResponseTypedDict",
    "DeleteMessageResponse",
    "DeleteMessageResponseTypedDict",
    "DeleteNoteResponse",
    "DeleteNoteResponseTypedDict",
    "DeleteOpportunityResponse",
    "DeleteOpportunityResponseTypedDict",
    "DeletePaymentResponse",
    "DeletePaymentResponseTypedDict",
    "DeletePipelineResponse",
    "DeletePipelineResponseTypedDict",
    "DeletePurchaseOrderResponse",
    "DeletePurchaseOrderResponseTypedDict",
    "DeleteQuoteResponse",
    "DeleteQuoteResponseTypedDict",
    "DeleteSharedLinkResponse",
    "DeleteSharedLinkResponseTypedDict",
    "DeleteSubsidiaryResponse",
    "DeleteSubsidiaryResponseTypedDict",
    "DeleteSupplierResponse",
    "DeleteSupplierResponseTypedDict",
    "DeleteTaxRateResponse",
    "DeleteTaxRateResponseTypedDict",
    "DeleteTicketResponse",
    "DeleteTicketResponseTypedDict",
    "DeleteTimeOffRequestResponse",
    "DeleteTimeOffRequestResponseTypedDict",
    "DeleteTrackingCategoryResponse",
    "DeleteTrackingCategoryResponseTypedDict",
    "DeleteUploadSessionResponse",
    "DeleteUploadSessionResponseTypedDict",
    "DeleteUserResponse",
    "DeleteUserResponseTypedDict",
    "DeleteWebhookResponse",
    "DeleteWebhookResponseTypedDict",
    "Department",
    "DepartmentInput",
    "DepartmentInputTypedDict",
    "DepartmentStatus",
    "DepartmentTypedDict",
    "DeprecatedLinkedSupplier",
    "DeprecatedLinkedSupplierInput",
    "DeprecatedLinkedSupplierInputTypedDict",
    "DeprecatedLinkedSupplierTypedDict",
    "DeprecatedLinkedTrackingCategory",
    "DeprecatedLinkedTrackingCategoryTypedDict",
    "Detail",
    "DetailTypedDict",
    "Direction",
    "DisabledReason",
    "Drive",
    "DriveGroup",
    "DriveGroupInput",
    "DriveGroupInputTypedDict",
    "DriveGroupTypedDict",
    "DriveGroupsFilter",
    "DriveGroupsFilterTypedDict",
    "DriveInput",
    "DriveInputTypedDict",
    "DriveTypedDict",
    "DrivesFilter",
    "DrivesFilterTypedDict",
    "EcommerceAddress",
    "EcommerceAddressTypedDict",
    "EcommerceCustomer",
    "EcommerceCustomerType",
    "EcommerceCustomerTypedDict",
    "EcommerceCustomersAllGlobals",
    "EcommerceCustomersAllGlobalsTypedDict",
    "EcommerceCustomersAllRequest",
    "EcommerceCustomersAllRequestTypedDict",
    "EcommerceCustomersAllResponse",
    "EcommerceCustomersAllResponseTypedDict",
    "EcommerceCustomersFilter",
    "EcommerceCustomersFilterTypedDict",
    "EcommerceCustomersOneGlobals",
    "EcommerceCustomersOneGlobalsTypedDict",
    "EcommerceCustomersOneRequest",
    "EcommerceCustomersOneRequestTypedDict",
    "EcommerceCustomersOneResponse",
    "EcommerceCustomersOneResponseTypedDict",
    "EcommerceDiscount",
    "EcommerceDiscountTypedDict",
    "EcommerceOrder",
    "EcommerceOrderLineItem",
    "EcommerceOrderLineItemTypedDict",
    "EcommerceOrderPaymentStatus",
    "EcommerceOrderRefund",
    "EcommerceOrderRefundTypedDict",
    "EcommerceOrderStatus",
    "EcommerceOrderTypedDict",
    "EcommerceOrdersAllGlobals",
    "EcommerceOrdersAllGlobalsTypedDict",
    "EcommerceOrdersAllRequest",
    "EcommerceOrdersAllRequestTypedDict",
    "EcommerceOrdersAllResponse",
    "EcommerceOrdersAllResponseTypedDict",
    "EcommerceOrdersFilter",
    "EcommerceOrdersFilterTypedDict",
    "EcommerceOrdersOneGlobals",
    "EcommerceOrdersOneGlobalsTypedDict",
    "EcommerceOrdersOneRequest",
    "EcommerceOrdersOneRequestTypedDict",
    "EcommerceOrdersOneResponse",
    "EcommerceOrdersOneResponseTypedDict",
    "EcommerceProduct",
    "EcommerceProductCategories",
    "EcommerceProductCategoriesTypedDict",
    "EcommerceProductImages",
    "EcommerceProductImagesTypedDict",
    "EcommerceProductOptions",
    "EcommerceProductOptionsTypedDict",
    "EcommerceProductTypedDict",
    "EcommerceProductVariantsOptions",
    "EcommerceProductVariantsOptionsTypedDict",
    "EcommerceProductsAllGlobals",
    "EcommerceProductsAllGlobalsTypedDict",
    "EcommerceProductsAllRequest",
    "EcommerceProductsAllRequestTypedDict",
    "EcommerceProductsAllResponse",
    "EcommerceProductsAllResponseTypedDict",
    "EcommerceProductsFilter",
    "EcommerceProductsFilterTypedDict",
    "EcommerceProductsOneGlobals",
    "EcommerceProductsOneGlobalsTypedDict",
    "EcommerceProductsOneRequest",
    "EcommerceProductsOneRequestTypedDict",
    "EcommerceProductsOneResponse",
    "EcommerceProductsOneResponseTypedDict",
    "EcommerceStore",
    "EcommerceStoreTypedDict",
    "EcommerceStoresOneGlobals",
    "EcommerceStoresOneGlobalsTypedDict",
    "EcommerceStoresOneRequest",
    "EcommerceStoresOneRequestTypedDict",
    "EcommerceStoresOneResponse",
    "EcommerceStoresOneResponseTypedDict",
    "Email",
    "EmailType",
    "EmailTypedDict",
    "Employee",
    "EmployeeCompensation",
    "EmployeeCompensationInput",
    "EmployeeCompensationInputTypedDict",
    "EmployeeCompensationTypedDict",
    "EmployeeInput",
    "EmployeeInputTypedDict",
    "EmployeeJob",
    "EmployeeJobInput",
    "EmployeeJobInputTypedDict",
    "EmployeeJobStatus",
    "EmployeeJobTypedDict",
    "EmployeePayroll",
    "EmployeePayrollTypedDict",
    "EmployeeSchedules",
    "EmployeeSchedulesTypedDict",
    "EmployeeTypedDict",
    "EmployeesFilter",
    "EmployeesFilterEmploymentStatus",
    "EmployeesFilterTypedDict",
    "EmployeesOneFilter",
    "EmployeesOneFilterTypedDict",
    "EmployeesSort",
    "EmployeesSortBy",
    "EmployeesSortTypedDict",
    "EmploymentRole",
    "EmploymentRoleTypedDict",
    "EmploymentStatus",
    "EmploymentSubType",
    "EmploymentTerms",
    "EmploymentType",
    "Error",
    "ErrorTypedDict",
    "EvenWeeks",
    "EvenWeeksTypedDict",
    "EventSource",
    "Expense",
    "ExpenseInput",
    "ExpenseInputTypedDict",
    "ExpenseLineItem",
    "ExpenseLineItemInput",
    "ExpenseLineItemInputTypedDict",
    "ExpenseLineItemTypedDict",
    "ExpensePaymentType",
    "ExpenseType",
    "ExpenseTypedDict",
    "ExpensesModel",
    "ExpensesModelTypedDict",
    "ExtendPaths",
    "ExtendPathsTypedDict",
    "FILE_STORAGE_UPLOAD_SESSIONS_ADD_OP_SERVERS",
    "FILE_STORAGE_UPLOAD_SESSIONS_FINISH_OP_SERVERS",
    "FILE_STORAGE_UPLOAD_SESSIONS_ONE_OP_SERVERS",
    "FILE_STORAGE_UPLOAD_SESSIONS_UPLOAD_OP_SERVERS",
    "FeedStatus",
    "Fields",
    "FieldsTypedDict",
    "FileStorageDriveGroupsAddGlobals",
    "FileStorageDriveGroupsAddGlobalsTypedDict",
    "FileStorageDriveGroupsAddRequest",
    "FileStorageDriveGroupsAddRequestTypedDict",
    "FileStorageDriveGroupsAddResponse",
    "FileStorageDriveGroupsAddResponseTypedDict",
    "FileStorageDriveGroupsAllGlobals",
    "FileStorageDriveGroupsAllGlobalsTypedDict",
    "FileStorageDriveGroupsAllRequest",
    "FileStorageDriveGroupsAllRequestTypedDict",
    "FileStorageDriveGroupsAllResponse",
    "FileStorageDriveGroupsAllResponseTypedDict",
    "FileStorageDriveGroupsDeleteGlobals",
    "FileStorageDriveGroupsDeleteGlobalsTypedDict",
    "FileStorageDriveGroupsDeleteRequest",
    "FileStorageDriveGroupsDeleteRequestTypedDict",
    "FileStorageDriveGroupsDeleteResponse",
    "FileStorageDriveGroupsDeleteResponseTypedDict",
    "FileStorageDriveGroupsOneGlobals",
    "FileStorageDriveGroupsOneGlobalsTypedDict",
    "FileStorageDriveGroupsOneRequest",
    "FileStorageDriveGroupsOneRequestTypedDict",
    "FileStorageDriveGroupsOneResponse",
    "FileStorageDriveGroupsOneResponseTypedDict",
    "FileStorageDriveGroupsUpdateGlobals",
    "FileStorageDriveGroupsUpdateGlobalsTypedDict",
    "FileStorageDriveGroupsUpdateRequest",
    "FileStorageDriveGroupsUpdateRequestTypedDict",
    "FileStorageDriveGroupsUpdateResponse",
    "FileStorageDriveGroupsUpdateResponseTypedDict",
    "FileStorageDrivesAddGlobals",
    "FileStorageDrivesAddGlobalsTypedDict",
    "FileStorageDrivesAddRequest",
    "FileStorageDrivesAddRequestTypedDict",
    "FileStorageDrivesAddResponse",
    "FileStorageDrivesAddResponseTypedDict",
    "FileStorageDrivesAllGlobals",
    "FileStorageDrivesAllGlobalsTypedDict",
    "FileStorageDrivesAllRequest",
    "FileStorageDrivesAllRequestTypedDict",
    "FileStorageDrivesAllResponse",
    "FileStorageDrivesAllResponseTypedDict",
    "FileStorageDrivesDeleteGlobals",
    "FileStorageDrivesDeleteGlobalsTypedDict",
    "FileStorageDrivesDeleteRequest",
    "FileStorageDrivesDeleteRequestTypedDict",
    "FileStorageDrivesDeleteResponse",
    "FileStorageDrivesDeleteResponseTypedDict",
    "FileStorageDrivesOneGlobals",
    "FileStorageDrivesOneGlobalsTypedDict",
    "FileStorageDrivesOneRequest",
    "FileStorageDrivesOneRequestTypedDict",
    "FileStorageDrivesOneResponse",
    "FileStorageDrivesOneResponseTypedDict",
    "FileStorageDrivesUpdateGlobals",
    "FileStorageDrivesUpdateGlobalsTypedDict",
    "FileStorageDrivesUpdateRequest",
    "FileStorageDrivesUpdateRequestTypedDict",
    "FileStorageDrivesUpdateResponse",
    "FileStorageDrivesUpdateResponseTypedDict",
    "FileStorageFilesAllGlobals",
    "FileStorageFilesAllGlobalsTypedDict",
    "FileStorageFilesAllRequest",
    "FileStorageFilesAllRequestTypedDict",
    "FileStorageFilesAllResponse",
    "FileStorageFilesAllResponseTypedDict",
    "FileStorageFilesDeleteGlobals",
    "FileStorageFilesDeleteGlobalsTypedDict",
    "FileStorageFilesDeleteRequest",
    "FileStorageFilesDeleteRequestTypedDict",
    "FileStorageFilesDeleteResponse",
    "FileStorageFilesDeleteResponseTypedDict",
    "FileStorageFilesDownloadGlobals",
    "FileStorageFilesDownloadGlobalsTypedDict",
    "FileStorageFilesDownloadRequest",
    "FileStorageFilesDownloadRequestTypedDict",
    "FileStorageFilesDownloadResponse",
    "FileStorageFilesDownloadResponseTypedDict",
    "FileStorageFilesExportGlobals",
    "FileStorageFilesExportGlobalsTypedDict",
    "FileStorageFilesExportRequest",
    "FileStorageFilesExportRequestTypedDict",
    "FileStorageFilesExportResponse",
    "FileStorageFilesExportResponseTypedDict",
    "FileStorageFilesOneGlobals",
    "FileStorageFilesOneGlobalsTypedDict",
    "FileStorageFilesOneRequest",
    "FileStorageFilesOneRequestTypedDict",
    "FileStorageFilesOneResponse",
    "FileStorageFilesOneResponseTypedDict",
    "FileStorageFilesSearchGlobals",
    "FileStorageFilesSearchGlobalsTypedDict",
    "FileStorageFilesSearchRequest",
    "FileStorageFilesSearchRequestTypedDict",
    "FileStorageFilesSearchResponse",
    "FileStorageFilesSearchResponseTypedDict",
    "FileStorageFilesUpdateGlobals",
    "FileStorageFilesUpdateGlobalsTypedDict",
    "FileStorageFilesUpdateRequest",
    "FileStorageFilesUpdateRequestTypedDict",
    "FileStorageFilesUpdateResponse",
    "FileStorageFilesUpdateResponseTypedDict",
    "FileStorageFoldersAddGlobals",
    "FileStorageFoldersAddGlobalsTypedDict",
    "FileStorageFoldersAddRequest",
    "FileStorageFoldersAddRequestTypedDict",
    "FileStorageFoldersAddResponse",
    "FileStorageFoldersAddResponseTypedDict",
    "FileStorageFoldersCopyGlobals",
    "FileStorageFoldersCopyGlobalsTypedDict",
    "FileStorageFoldersCopyRequest",
    "FileStorageFoldersCopyRequestTypedDict",
    "FileStorageFoldersCopyResponse",
    "FileStorageFoldersCopyResponseTypedDict",
    "FileStorageFoldersDeleteGlobals",
    "FileStorageFoldersDeleteGlobalsTypedDict",
    "FileStorageFoldersDeleteRequest",
    "FileStorageFoldersDeleteRequestTypedDict",
    "FileStorageFoldersDeleteResponse",
    "FileStorageFoldersDeleteResponseTypedDict",
    "FileStorageFoldersOneGlobals",
    "FileStorageFoldersOneGlobalsTypedDict",
    "FileStorageFoldersOneRequest",
    "FileStorageFoldersOneRequestTypedDict",
    "FileStorageFoldersOneResponse",
    "FileStorageFoldersOneResponseTypedDict",
    "FileStorageFoldersUpdateGlobals",
    "FileStorageFoldersUpdateGlobalsTypedDict",
    "FileStorageFoldersUpdateRequest",
    "FileStorageFoldersUpdateRequestTypedDict",
    "FileStorageFoldersUpdateResponse",
    "FileStorageFoldersUpdateResponseTypedDict",
    "FileStorageSharedLinksAddGlobals",
    "FileStorageSharedLinksAddGlobalsTypedDict",
    "FileStorageSharedLinksAddRequest",
    "FileStorageSharedLinksAddRequestTypedDict",
    "FileStorageSharedLinksAddResponse",
    "FileStorageSharedLinksAddResponseTypedDict",
    "FileStorageSharedLinksAllGlobals",
    "FileStorageSharedLinksAllGlobalsTypedDict",
    "FileStorageSharedLinksAllRequest",
    "FileStorageSharedLinksAllRequestTypedDict",
    "FileStorageSharedLinksAllResponse",
    "FileStorageSharedLinksAllResponseTypedDict",
    "FileStorageSharedLinksDeleteGlobals",
    "FileStorageSharedLinksDeleteGlobalsTypedDict",
    "FileStorageSharedLinksDeleteRequest",
    "FileStorageSharedLinksDeleteRequestTypedDict",
    "FileStorageSharedLinksDeleteResponse",
    "FileStorageSharedLinksDeleteResponseTypedDict",
    "FileStorageSharedLinksOneGlobals",
    "FileStorageSharedLinksOneGlobalsTypedDict",
    "FileStorageSharedLinksOneRequest",
    "FileStorageSharedLinksOneRequestTypedDict",
    "FileStorageSharedLinksOneResponse",
    "FileStorageSharedLinksOneResponseTypedDict",
    "FileStorageSharedLinksUpdateGlobals",
    "FileStorageSharedLinksUpdateGlobalsTypedDict",
    "FileStorageSharedLinksUpdateRequest",
    "FileStorageSharedLinksUpdateRequestTypedDict",
    "FileStorageSharedLinksUpdateResponse",
    "FileStorageSharedLinksUpdateResponseTypedDict",
    "FileStorageUploadSessionsAddGlobals",
    "FileStorageUploadSessionsAddGlobalsTypedDict",
    "FileStorageUploadSessionsAddRequest",
    "FileStorageUploadSessionsAddRequestTypedDict",
    "FileStorageUploadSessionsAddResponse",
    "FileStorageUploadSessionsAddResponseTypedDict",
    "FileStorageUploadSessionsDeleteGlobals",
    "FileStorageUploadSessionsDeleteGlobalsTypedDict",
    "FileStorageUploadSessionsDeleteRequest",
    "FileStorageUploadSessionsDeleteRequestTypedDict",
    "FileStorageUploadSessionsDeleteResponse",
    "FileStorageUploadSessionsDeleteResponseTypedDict",
    "FileStorageUploadSessionsFinishGlobals",
    "FileStorageUploadSessionsFinishGlobalsTypedDict",
    "FileStorageUploadSessionsFinishRequest",
    "FileStorageUploadSessionsFinishRequestBody",
    "FileStorageUploadSessionsFinishRequestBodyTypedDict",
    "FileStorageUploadSessionsFinishRequestTypedDict",
    "FileStorageUploadSessionsFinishResponse",
    "FileStorageUploadSessionsFinishResponseTypedDict",
    "FileStorageUploadSessionsOneGlobals",
    "FileStorageUploadSessionsOneGlobalsTypedDict",
    "FileStorageUploadSessionsOneRequest",
    "FileStorageUploadSessionsOneRequestTypedDict",
    "FileStorageUploadSessionsOneResponse",
    "FileStorageUploadSessionsOneResponseTypedDict",
    "FileStorageUploadSessionsUploadGlobals",
    "FileStorageUploadSessionsUploadGlobalsTypedDict",
    "FileStorageUploadSessionsUploadRequest",
    "FileStorageUploadSessionsUploadRequestTypedDict",
    "FileStorageUploadSessionsUploadResponse",
    "FileStorageUploadSessionsUploadResponseTypedDict",
    "FileType",
    "FilesFilter",
    "FilesFilterTypedDict",
    "FilesSearch",
    "FilesSearchTypedDict",
    "FilesSort",
    "FilesSortBy",
    "FilesSortTypedDict",
    "Five",
    "FiveTypedDict",
    "FlsaStatus",
    "Folder",
    "FolderTypedDict",
    "FormField",
    "FormFieldOption",
    "FormFieldOptionGroup",
    "FormFieldOptionGroupOptionType",
    "FormFieldOptionGroupTypedDict",
    "FormFieldOptionTypedDict",
    "FormFieldType",
    "FormFieldTypedDict",
    "Format",
    "FulfillmentStatus",
    "Gender",
    "GetAPIResourceCoverageResponse",
    "GetAPIResourceCoverageResponseTypedDict",
    "GetAPIResourceResponse",
    "GetAPIResourceResponseTypedDict",
    "GetAPIResponse",
    "GetAPIResponseTypedDict",
    "GetAccountingDepartmentResponse",
    "GetAccountingDepartmentResponseTypedDict",
    "GetAccountingDepartmentsResponse",
    "GetAccountingDepartmentsResponseTypedDict",
    "GetAccountingLocationResponse",
    "GetAccountingLocationResponseTypedDict",
    "GetAccountingLocationsResponse",
    "GetAccountingLocationsResponseTypedDict",
    "GetActivitiesResponse",
    "GetActivitiesResponseTypedDict",
    "GetActivityResponse",
    "GetActivityResponseTypedDict",
    "GetAgedCreditorsResponse",
    "GetAgedCreditorsResponseTypedDict",
    "GetAgedDebtorsResponse",
    "GetAgedDebtorsResponseTypedDict",
    "GetApisResponse",
    "GetApisResponseTypedDict",
    "GetApplicantResponse",
    "GetApplicantResponseTypedDict",
    "GetApplicantsResponse",
    "GetApplicantsResponseTypedDict",
    "GetApplicationResponse",
    "GetApplicationResponseTypedDict",
    "GetApplicationsResponse",
    "GetApplicationsResponseTypedDict",
    "GetAttachmentResponse",
    "GetAttachmentResponseTypedDict",
    "GetAttachmentsResponse",
    "GetAttachmentsResponseTypedDict",
    "GetBalanceSheetResponse",
    "GetBalanceSheetResponseTypedDict",
    "GetBankFeedAccountResponse",
    "GetBankFeedAccountResponseTypedDict",
    "GetBankFeedAccountsResponse",
    "GetBankFeedAccountsResponseTypedDict",
    "GetBankFeedStatementResponse",
    "GetBankFeedStatementResponseTypedDict",
    "GetBankFeedStatementsResponse",
    "GetBankFeedStatementsResponseTypedDict",
    "GetBillPaymentResponse",
    "GetBillPaymentResponseTypedDict",
    "GetBillPaymentsResponse",
    "GetBillPaymentsResponseTypedDict",
    "GetBillResponse",
    "GetBillResponseTypedDict",
    "GetBillsResponse",
    "GetBillsResponseTypedDict",
    "GetCategoriesResponse",
    "GetCategoriesResponseTypedDict",
    "GetCategoryResponse",
    "GetCategoryResponseTypedDict",
    "GetCollectionResponse",
    "GetCollectionResponseTypedDict",
    "GetCollectionTagsResponse",
    "GetCollectionTagsResponseTypedDict",
    "GetCollectionUserResponse",
    "GetCollectionUserResponseTypedDict",
    "GetCollectionUsersResponse",
    "GetCollectionUsersResponseTypedDict",
    "GetCollectionsResponse",
    "GetCollectionsResponseTypedDict",
    "GetCommentResponse",
    "GetCommentResponseTypedDict",
    "GetCommentsResponse",
    "GetCommentsResponseTypedDict",
    "GetCompaniesResponse",
    "GetCompaniesResponseTypedDict",
    "GetCompanyInfoResponse",
    "GetCompanyInfoResponseTypedDict",
    "GetCompanyResponse",
    "GetCompanyResponseTypedDict",
    "GetConnectionResponse",
    "GetConnectionResponseTypedDict",
    "GetConnectionsResponse",
    "GetConnectionsResponseTypedDict",
    "GetConnectorResourceResponse",
    "GetConnectorResourceResponseTypedDict",
    "GetConnectorResponse",
    "GetConnectorResponseTypedDict",
    "GetConnectorsResponse",
    "GetConnectorsResponseTypedDict",
    "GetConsumerResponse",
    "GetConsumerResponseTypedDict",
    "GetConsumersResponse",
    "GetConsumersResponseTypedDict",
    "GetContactResponse",
    "GetContactResponseTypedDict",
    "GetContactsResponse",
    "GetContactsResponseTypedDict",
    "GetCreditNoteResponse",
    "GetCreditNoteResponseTypedDict",
    "GetCreditNotesResponse",
    "GetCreditNotesResponseTypedDict",
    "GetCustomFieldsResponse",
    "GetCustomFieldsResponseTypedDict",
    "GetCustomMappingsResponse",
    "GetCustomMappingsResponseTypedDict",
    "GetCustomObjectResponse",
    "GetCustomObjectResponseTypedDict",
    "GetCustomObjectSchemaResponse",
    "GetCustomObjectSchemaResponseTypedDict",
    "GetCustomObjectSchemasResponse",
    "GetCustomObjectSchemasResponseTypedDict",
    "GetCustomObjectsResponse",
    "GetCustomObjectsResponseTypedDict",
    "GetCustomerResponse",
    "GetCustomerResponseTypedDict",
    "GetCustomersResponse",
    "GetCustomersResponseTypedDict",
    "GetDepartmentResponse",
    "GetDepartmentResponseTypedDict",
    "GetDepartmentsResponse",
    "GetDepartmentsResponseTypedDict",
    "GetDriveGroupResponse",
    "GetDriveGroupResponseTypedDict",
    "GetDriveGroupsResponse",
    "GetDriveGroupsResponseTypedDict",
    "GetDriveResponse",
    "GetDriveResponseTypedDict",
    "GetDrivesResponse",
    "GetDrivesResponseTypedDict",
    "GetEcommerceCustomerResponse",
    "GetEcommerceCustomerResponseTypedDict",
    "GetEcommerceCustomersResponse",
    "GetEcommerceCustomersResponseTypedDict",
    "GetEcommerceOrderResponse",
    "GetEcommerceOrderResponseTypedDict",
    "GetEcommerceOrdersResponse",
    "GetEcommerceOrdersResponseTypedDict",
    "GetEmployeePayrollResponse",
    "GetEmployeePayrollResponseTypedDict",
    "GetEmployeePayrollsResponse",
    "GetEmployeePayrollsResponseTypedDict",
    "GetEmployeeResponse",
    "GetEmployeeResponseTypedDict",
    "GetEmployeeSchedulesResponse",
    "GetEmployeeSchedulesResponseTypedDict",
    "GetEmployeesResponse",
    "GetEmployeesResponseTypedDict",
    "GetExpenseResponse",
    "GetExpenseResponseTypedDict",
    "GetExpensesResponse",
    "GetExpensesResponseTypedDict",
    "GetFileResponse",
    "GetFileResponseTypedDict",
    "GetFilesResponse",
    "GetFilesResponseTypedDict",
    "GetFolderResponse",
    "GetFolderResponseTypedDict",
    "GetHrisCompaniesResponse",
    "GetHrisCompaniesResponseTypedDict",
    "GetHrisCompanyResponse",
    "GetHrisCompanyResponseTypedDict",
    "GetInvoiceItemResponse",
    "GetInvoiceItemResponseTypedDict",
    "GetInvoiceItemsResponse",
    "GetInvoiceItemsResponseTypedDict",
    "GetInvoiceResponse",
    "GetInvoiceResponseTypedDict",
    "GetInvoicesResponse",
    "GetInvoicesResponseTypedDict",
    "GetJobResponse",
    "GetJobResponseTypedDict",
    "GetJobsResponse",
    "GetJobsResponseTypedDict",
    "GetJournalEntriesResponse",
    "GetJournalEntriesResponseTypedDict",
    "GetJournalEntryResponse",
    "GetJournalEntryResponseTypedDict",
    "GetLeadResponse",
    "GetLeadResponseTypedDict",
    "GetLeadsResponse",
    "GetLeadsResponseTypedDict",
    "GetLedgerAccountResponse",
    "GetLedgerAccountResponseTypedDict",
    "GetLedgerAccountsResponse",
    "GetLedgerAccountsResponseTypedDict",
    "GetLogsResponse",
    "GetLogsResponseTypedDict",
    "GetMessageResponse",
    "GetMessageResponseTypedDict",
    "GetMessagesResponse",
    "GetMessagesResponseTypedDict",
    "GetNoteResponse",
    "GetNoteResponseTypedDict",
    "GetNotesResponse",
    "GetNotesResponseTypedDict",
    "GetOpportunitiesResponse",
    "GetOpportunitiesResponseTypedDict",
    "GetOpportunityResponse",
    "GetOpportunityResponseTypedDict",
    "GetPaymentResponse",
    "GetPaymentResponseTypedDict",
    "GetPaymentsResponse",
    "GetPaymentsResponseTypedDict",
    "GetPayrollResponse",
    "GetPayrollResponseTypedDict",
    "GetPayrollsResponse",
    "GetPayrollsResponseTypedDict",
    "GetPipelineResponse",
    "GetPipelineResponseTypedDict",
    "GetPipelinesResponse",
    "GetPipelinesResponseTypedDict",
    "GetProductResponse",
    "GetProductResponseTypedDict",
    "GetProductsResponse",
    "GetProductsResponseTypedDict",
    "GetProfitAndLossResponse",
    "GetProfitAndLossResponseTypedDict",
    "GetPurchaseOrderResponse",
    "GetPurchaseOrderResponseTypedDict",
    "GetPurchaseOrdersResponse",
    "GetPurchaseOrdersResponseTypedDict",
    "GetQuoteResponse",
    "GetQuoteResponseTypedDict",
    "GetQuotesResponse",
    "GetQuotesResponseTypedDict",
    "GetSharedLinkResponse",
    "GetSharedLinkResponseTypedDict",
    "GetSharedLinksResponse",
    "GetSharedLinksResponseTypedDict",
    "GetStoreResponse",
    "GetStoreResponseTypedDict",
    "GetSubsidiariesResponse",
    "GetSubsidiariesResponseTypedDict",
    "GetSubsidiaryResponse",
    "GetSubsidiaryResponseTypedDict",
    "GetSupplierResponse",
    "GetSupplierResponseTypedDict",
    "GetSuppliersResponse",
    "GetSuppliersResponseTypedDict",
    "GetTaxRateResponse",
    "GetTaxRateResponseTypedDict",
    "GetTaxRatesResponse",
    "GetTaxRatesResponseTypedDict",
    "GetTicketResponse",
    "GetTicketResponseTypedDict",
    "GetTicketsResponse",
    "GetTicketsResponseTypedDict",
    "GetTimeOffRequestResponse",
    "GetTimeOffRequestResponseTypedDict",
    "GetTimeOffRequestsResponse",
    "GetTimeOffRequestsResponseTypedDict",
    "GetTrackingCategoriesResponse",
    "GetTrackingCategoriesResponseTypedDict",
    "GetTrackingCategoryResponse",
    "GetTrackingCategoryResponseTypedDict",
    "GetUploadSessionResponse",
    "GetUploadSessionResponseTypedDict",
    "GetUserResponse",
    "GetUserResponseTypedDict",
    "GetUsersResponse",
    "GetUsersResponseTypedDict",
    "GetWebhookEventLogsResponse",
    "GetWebhookEventLogsResponseTypedDict",
    "GetWebhookResponse",
    "GetWebhookResponseTypedDict",
    "GetWebhooksResponse",
    "GetWebhooksResponseTypedDict",
    "HTTPMetadata",
    "HTTPMetadataTypedDict",
    "HrisCompaniesAddGlobals",
    "HrisCompaniesAddGlobalsTypedDict",
    "HrisCompaniesAddRequest",
    "HrisCompaniesAddRequestTypedDict",
    "HrisCompaniesAddResponse",
    "HrisCompaniesAddResponseTypedDict",
    "HrisCompaniesAllGlobals",
    "HrisCompaniesAllGlobalsTypedDict",
    "HrisCompaniesAllRequest",
    "HrisCompaniesAllRequestTypedDict",
    "HrisCompaniesAllResponse",
    "HrisCompaniesAllResponseTypedDict",
    "HrisCompaniesDeleteGlobals",
    "HrisCompaniesDeleteGlobalsTypedDict",
    "HrisCompaniesDeleteRequest",
    "HrisCompaniesDeleteRequestTypedDict",
    "HrisCompaniesDeleteResponse",
    "HrisCompaniesDeleteResponseTypedDict",
    "HrisCompaniesOneGlobals",
    "HrisCompaniesOneGlobalsTypedDict",
    "HrisCompaniesOneRequest",
    "HrisCompaniesOneRequestTypedDict",
    "HrisCompaniesOneResponse",
    "HrisCompaniesOneResponseTypedDict",
    "HrisCompaniesUpdateGlobals",
    "HrisCompaniesUpdateGlobalsTypedDict",
    "HrisCompaniesUpdateRequest",
    "HrisCompaniesUpdateRequestTypedDict",
    "HrisCompaniesUpdateResponse",
    "HrisCompaniesUpdateResponseTypedDict",
    "HrisCompany",
    "HrisCompanyInput",
    "HrisCompanyInputTypedDict",
    "HrisCompanyStatus",
    "HrisCompanyTypedDict",
    "HrisDepartmentsAddGlobals",
    "HrisDepartmentsAddGlobalsTypedDict",
    "HrisDepartmentsAddRequest",
    "HrisDepartmentsAddRequestTypedDict",
    "HrisDepartmentsAddResponse",
    "HrisDepartmentsAddResponseTypedDict",
    "HrisDepartmentsAllGlobals",
    "HrisDepartmentsAllGlobalsTypedDict",
    "HrisDepartmentsAllRequest",
    "HrisDepartmentsAllRequestTypedDict",
    "HrisDepartmentsAllResponse",
    "HrisDepartmentsAllResponseTypedDict",
    "HrisDepartmentsDeleteGlobals",
    "HrisDepartmentsDeleteGlobalsTypedDict",
    "HrisDepartmentsDeleteRequest",
    "HrisDepartmentsDeleteRequestTypedDict",
    "HrisDepartmentsDeleteResponse",
    "HrisDepartmentsDeleteResponseTypedDict",
    "HrisDepartmentsOneGlobals",
    "HrisDepartmentsOneGlobalsTypedDict",
    "HrisDepartmentsOneRequest",
    "HrisDepartmentsOneRequestTypedDict",
    "HrisDepartmentsOneResponse",
    "HrisDepartmentsOneResponseTypedDict",
    "HrisDepartmentsUpdateGlobals",
    "HrisDepartmentsUpdateGlobalsTypedDict",
    "HrisDepartmentsUpdateRequest",
    "HrisDepartmentsUpdateRequestTypedDict",
    "HrisDepartmentsUpdateResponse",
    "HrisDepartmentsUpdateResponseTypedDict",
    "HrisEmployeePayrollsAllGlobals",
    "HrisEmployeePayrollsAllGlobalsTypedDict",
    "HrisEmployeePayrollsAllRequest",
    "HrisEmployeePayrollsAllRequestTypedDict",
    "HrisEmployeePayrollsAllResponse",
    "HrisEmployeePayrollsAllResponseTypedDict",
    "HrisEmployeePayrollsOneGlobals",
    "HrisEmployeePayrollsOneGlobalsTypedDict",
    "HrisEmployeePayrollsOneRequest",
    "HrisEmployeePayrollsOneRequestTypedDict",
    "HrisEmployeePayrollsOneResponse",
    "HrisEmployeePayrollsOneResponseTypedDict",
    "HrisEmployeeSchedulesAllGlobals",
    "HrisEmployeeSchedulesAllGlobalsTypedDict",
    "HrisEmployeeSchedulesAllRequest",
    "HrisEmployeeSchedulesAllRequestTypedDict",
    "HrisEmployeeSchedulesAllResponse",
    "HrisEmployeeSchedulesAllResponseTypedDict",
    "HrisEmployeesAddGlobals",
    "HrisEmployeesAddGlobalsTypedDict",
    "HrisEmployeesAddRequest",
    "HrisEmployeesAddRequestTypedDict",
    "HrisEmployeesAddResponse",
    "HrisEmployeesAddResponseTypedDict",
    "HrisEmployeesAllGlobals",
    "HrisEmployeesAllGlobalsTypedDict",
    "HrisEmployeesAllRequest",
    "HrisEmployeesAllRequestTypedDict",
    "HrisEmployeesAllResponse",
    "HrisEmployeesAllResponseTypedDict",
    "HrisEmployeesDeleteGlobals",
    "HrisEmployeesDeleteGlobalsTypedDict",
    "HrisEmployeesDeleteRequest",
    "HrisEmployeesDeleteRequestTypedDict",
    "HrisEmployeesDeleteResponse",
    "HrisEmployeesDeleteResponseTypedDict",
    "HrisEmployeesOneGlobals",
    "HrisEmployeesOneGlobalsTypedDict",
    "HrisEmployeesOneRequest",
    "HrisEmployeesOneRequestTypedDict",
    "HrisEmployeesOneResponse",
    "HrisEmployeesOneResponseTypedDict",
    "HrisEmployeesUpdateGlobals",
    "HrisEmployeesUpdateGlobalsTypedDict",
    "HrisEmployeesUpdateRequest",
    "HrisEmployeesUpdateRequestTypedDict",
    "HrisEmployeesUpdateResponse",
    "HrisEmployeesUpdateResponseTypedDict",
    "HrisPayrollsAllGlobals",
    "HrisPayrollsAllGlobalsTypedDict",
    "HrisPayrollsAllRequest",
    "HrisPayrollsAllRequestTypedDict",
    "HrisPayrollsAllResponse",
    "HrisPayrollsAllResponseTypedDict",
    "HrisPayrollsOneGlobals",
    "HrisPayrollsOneGlobalsTypedDict",
    "HrisPayrollsOneRequest",
    "HrisPayrollsOneRequestTypedDict",
    "HrisPayrollsOneResponse",
    "HrisPayrollsOneResponseTypedDict",
    "HrisTimeOffRequestsAddGlobals",
    "HrisTimeOffRequestsAddGlobalsTypedDict",
    "HrisTimeOffRequestsAddRequest",
    "HrisTimeOffRequestsAddRequestTypedDict",
    "HrisTimeOffRequestsAddResponse",
    "HrisTimeOffRequestsAddResponseTypedDict",
    "HrisTimeOffRequestsAllGlobals",
    "HrisTimeOffRequestsAllGlobalsTypedDict",
    "HrisTimeOffRequestsAllRequest",
    "HrisTimeOffRequestsAllRequestTypedDict",
    "HrisTimeOffRequestsAllResponse",
    "HrisTimeOffRequestsAllResponseTypedDict",
    "HrisTimeOffRequestsDeleteGlobals",
    "HrisTimeOffRequestsDeleteGlobalsTypedDict",
    "HrisTimeOffRequestsDeleteRequest",
    "HrisTimeOffRequestsDeleteRequestTypedDict",
    "HrisTimeOffRequestsDeleteResponse",
    "HrisTimeOffRequestsDeleteResponseTypedDict",
    "HrisTimeOffRequestsOneGlobals",
    "HrisTimeOffRequestsOneGlobalsTypedDict",
    "HrisTimeOffRequestsOneRequest",
    "HrisTimeOffRequestsOneRequestTypedDict",
    "HrisTimeOffRequestsOneResponse",
    "HrisTimeOffRequestsOneResponseTypedDict",
    "HrisTimeOffRequestsUpdateGlobals",
    "HrisTimeOffRequestsUpdateGlobalsTypedDict",
    "HrisTimeOffRequestsUpdateRequest",
    "HrisTimeOffRequestsUpdateRequestTypedDict",
    "HrisTimeOffRequestsUpdateResponse",
    "HrisTimeOffRequestsUpdateResponseTypedDict",
    "Images",
    "ImagesTypedDict",
    "Income",
    "IncomeTypedDict",
    "IntegrationState",
    "Invoice",
    "InvoiceInput",
    "InvoiceInputTypedDict",
    "InvoiceItem",
    "InvoiceItemFilter",
    "InvoiceItemFilterInvoiceItemType",
    "InvoiceItemFilterTypedDict",
    "InvoiceItemInput",
    "InvoiceItemInputTypedDict",
    "InvoiceItemPurchaseDetails",
    "InvoiceItemPurchaseDetailsTypedDict",
    "InvoiceItemSalesDetails",
    "InvoiceItemSalesDetailsTypedDict",
    "InvoiceItemType",
    "InvoiceItemTypeType",
    "InvoiceItemTypedDict",
    "InvoiceItemsFilter",
    "InvoiceItemsFilterTypedDict",
    "InvoiceItemsSort",
    "InvoiceItemsSortBy",
    "InvoiceItemsSortTypedDict",
    "InvoiceLineItem",
    "InvoiceLineItemInput",
    "InvoiceLineItemInputTypedDict",
    "InvoiceLineItemType",
    "InvoiceLineItemTypedDict",
    "InvoiceResponse",
    "InvoiceResponseTypedDict",
    "InvoiceStatus",
    "InvoiceType",
    "InvoiceTypedDict",
    "InvoicesFilter",
    "InvoicesFilterTypedDict",
    "InvoicesSort",
    "InvoicesSortBy",
    "InvoicesSortTypedDict",
    "IssueTrackingCollectionTagsAllGlobals",
    "IssueTrackingCollectionTagsAllGlobalsTypedDict",
    "IssueTrackingCollectionTagsAllRequest",
    "IssueTrackingCollectionTagsAllRequestTypedDict",
    "IssueTrackingCollectionTagsAllResponse",
    "IssueTrackingCollectionTagsAllResponseTypedDict",
    "IssueTrackingCollectionTicketCommentsAddGlobals",
    "IssueTrackingCollectionTicketCommentsAddGlobalsTypedDict",
    "IssueTrackingCollectionTicketCommentsAddRequest",
    "IssueTrackingCollectionTicketCommentsAddRequestTypedDict",
    "IssueTrackingCollectionTicketCommentsAddResponse",
    "IssueTrackingCollectionTicketCommentsAddResponseTypedDict",
    "IssueTrackingCollectionTicketCommentsAllGlobals",
    "IssueTrackingCollectionTicketCommentsAllGlobalsTypedDict",
    "IssueTrackingCollectionTicketCommentsAllRequest",
    "IssueTrackingCollectionTicketCommentsAllRequestTypedDict",
    "IssueTrackingCollectionTicketCommentsAllResponse",
    "IssueTrackingCollectionTicketCommentsAllResponseTypedDict",
    "IssueTrackingCollectionTicketCommentsDeleteGlobals",
    "IssueTrackingCollectionTicketCommentsDeleteGlobalsTypedDict",
    "IssueTrackingCollectionTicketCommentsDeleteRequest",
    "IssueTrackingCollectionTicketCommentsDeleteRequestTypedDict",
    "IssueTrackingCollectionTicketCommentsDeleteResponse",
    "IssueTrackingCollectionTicketCommentsDeleteResponseTypedDict",
    "IssueTrackingCollectionTicketCommentsOneGlobals",
    "IssueTrackingCollectionTicketCommentsOneGlobalsTypedDict",
    "IssueTrackingCollectionTicketCommentsOneRequest",
    "IssueTrackingCollectionTicketCommentsOneRequestTypedDict",
    "IssueTrackingCollectionTicketCommentsOneResponse",
    "IssueTrackingCollectionTicketCommentsOneResponseTypedDict",
    "IssueTrackingCollectionTicketCommentsUpdateGlobals",
    "IssueTrackingCollectionTicketCommentsUpdateGlobalsTypedDict",
    "IssueTrackingCollectionTicketCommentsUpdateRequest",
    "IssueTrackingCollectionTicketCommentsUpdateRequestTypedDict",
    "IssueTrackingCollectionTicketCommentsUpdateResponse",
    "IssueTrackingCollectionTicketCommentsUpdateResponseTypedDict",
    "IssueTrackingCollectionTicketsAddGlobals",
    "IssueTrackingCollectionTicketsAddGlobalsTypedDict",
    "IssueTrackingCollectionTicketsAddRequest",
    "IssueTrackingCollectionTicketsAddRequestTypedDict",
    "IssueTrackingCollectionTicketsAddResponse",
    "IssueTrackingCollectionTicketsAddResponseTypedDict",
    "IssueTrackingCollectionTicketsAllGlobals",
    "IssueTrackingCollectionTicketsAllGlobalsTypedDict",
    "IssueTrackingCollectionTicketsAllRequest",
    "IssueTrackingCollectionTicketsAllRequestTypedDict",
    "IssueTrackingCollectionTicketsAllResponse",
    "IssueTrackingCollectionTicketsAllResponseTypedDict",
    "IssueTrackingCollectionTicketsDeleteGlobals",
    "IssueTrackingCollectionTicketsDeleteGlobalsTypedDict",
    "IssueTrackingCollectionTicketsDeleteRequest",
    "IssueTrackingCollectionTicketsDeleteRequestTypedDict",
    "IssueTrackingCollectionTicketsDeleteResponse",
    "IssueTrackingCollectionTicketsDeleteResponseTypedDict",
    "IssueTrackingCollectionTicketsOneGlobals",
    "IssueTrackingCollectionTicketsOneGlobalsTypedDict",
    "IssueTrackingCollectionTicketsOneRequest",
    "IssueTrackingCollectionTicketsOneRequestTypedDict",
    "IssueTrackingCollectionTicketsOneResponse",
    "IssueTrackingCollectionTicketsOneResponseTypedDict",
    "IssueTrackingCollectionTicketsUpdateGlobals",
    "IssueTrackingCollectionTicketsUpdateGlobalsTypedDict",
    "IssueTrackingCollectionTicketsUpdateRequest",
    "IssueTrackingCollectionTicketsUpdateRequestTypedDict",
    "IssueTrackingCollectionTicketsUpdateResponse",
    "IssueTrackingCollectionTicketsUpdateResponseTypedDict",
    "IssueTrackingCollectionUsersAllGlobals",
    "IssueTrackingCollectionUsersAllGlobalsTypedDict",
    "IssueTrackingCollectionUsersAllRequest",
    "IssueTrackingCollectionUsersAllRequestTypedDict",
    "IssueTrackingCollectionUsersAllResponse",
    "IssueTrackingCollectionUsersAllResponseTypedDict",
    "IssueTrackingCollectionUsersOneGlobals",
    "IssueTrackingCollectionUsersOneGlobalsTypedDict",
    "IssueTrackingCollectionUsersOneRequest",
    "IssueTrackingCollectionUsersOneRequestTypedDict",
    "IssueTrackingCollectionUsersOneResponse",
    "IssueTrackingCollectionUsersOneResponseTypedDict",
    "IssueTrackingCollectionsAllGlobals",
    "IssueTrackingCollectionsAllGlobalsTypedDict",
    "IssueTrackingCollectionsAllRequest",
    "IssueTrackingCollectionsAllRequestTypedDict",
    "IssueTrackingCollectionsAllResponse",
    "IssueTrackingCollectionsAllResponseTypedDict",
    "IssueTrackingCollectionsOneGlobals",
    "IssueTrackingCollectionsOneGlobalsTypedDict",
    "IssueTrackingCollectionsOneRequest",
    "IssueTrackingCollectionsOneRequestTypedDict",
    "IssueTrackingCollectionsOneResponse",
    "IssueTrackingCollectionsOneResponseTypedDict",
    "IssuesFilter",
    "IssuesFilterTypedDict",
    "Job",
    "JobLinks",
    "JobLinksTypedDict",
    "JobStatus",
    "JobType",
    "JobTypedDict",
    "JournalEntriesFilter",
    "JournalEntriesFilterTypedDict",
    "JournalEntriesSort",
    "JournalEntriesSortBy",
    "JournalEntriesSortTypedDict",
    "JournalEntry",
    "JournalEntryInput",
    "JournalEntryInputTypedDict",
    "JournalEntryLineItem",
    "JournalEntryLineItemInput",
    "JournalEntryLineItemInputTypedDict",
    "JournalEntryLineItemType",
    "JournalEntryLineItemTypedDict",
    "JournalEntryStatus",
    "JournalEntryTypedDict",
    "Lead",
    "LeadInput",
    "LeadInputTypedDict",
    "LeadTypedDict",
    "LeadsFilter",
    "LeadsFilterTypedDict",
    "LeadsSort",
    "LeadsSortBy",
    "LeadsSortTypedDict",
    "LeavingReason",
    "LedgerAccount",
    "LedgerAccountClassification",
    "LedgerAccountInput",
    "LedgerAccountInputTypedDict",
    "LedgerAccountSubsidiaries",
    "LedgerAccountSubsidiariesTypedDict",
    "LedgerAccountType",
    "LedgerAccountTypedDict",
    "LedgerAccountsFilter",
    "LedgerAccountsFilterTypedDict",
    "LedgerAccountsSort",
    "LedgerAccountsSortBy",
    "LedgerAccountsSortTypedDict",
    "LinkedAttachment",
    "LinkedAttachmentTypedDict",
    "LinkedConnectorResource",
    "LinkedConnectorResourceTypedDict",
    "LinkedCustomer",
    "LinkedCustomerInput",
    "LinkedCustomerInputTypedDict",
    "LinkedCustomerTypedDict",
    "LinkedEcommerceCustomer",
    "LinkedEcommerceCustomerTypedDict",
    "LinkedEcommerceOrder",
    "LinkedEcommerceOrderTypedDict",
    "LinkedFolder",
    "LinkedFolderTypedDict",
    "LinkedInvoiceItem",
    "LinkedInvoiceItemTypedDict",
    "LinkedLedgerAccount",
    "LinkedLedgerAccountInput",
    "LinkedLedgerAccountInputTypedDict",
    "LinkedLedgerAccountTypedDict",
    "LinkedParentCustomer",
    "LinkedParentCustomerTypedDict",
    "LinkedPurchaseOrder",
    "LinkedPurchaseOrderTypedDict",
    "LinkedResources",
    "LinkedResourcesTypedDict",
    "LinkedSupplier",
    "LinkedSupplierInput",
    "LinkedSupplierInputTypedDict",
    "LinkedSupplierTypedDict",
    "LinkedTaxDetail",
    "LinkedTaxDetailTypedDict",
    "LinkedTaxRate",
    "LinkedTaxRateInput",
    "LinkedTaxRateInputTypedDict",
    "LinkedTaxRateTypedDict",
    "LinkedTaxStatusDetail",
    "LinkedTaxStatusDetailTypedDict",
    "LinkedTrackingCategory",
    "LinkedTrackingCategoryTypedDict",
    "LinkedWorktag",
    "LinkedWorktagTypedDict",
    "Links",
    "LinksTypedDict",
    "LocationStatus",
    "Log",
    "LogTypedDict",
    "LogsFilter",
    "LogsFilterTypedDict",
    "ManagedVia",
    "Manager",
    "ManagerTypedDict",
    "Message",
    "MessageInput",
    "MessageInputTypedDict",
    "MessageStatus",
    "MessageType",
    "MessageTypedDict",
    "Meta",
    "MetaTypedDict",
    "Mode",
    "NoResponseError",
    "NotFoundResponse",
    "NotFoundResponseData",
    "NotFoundResponseDetail",
    "Note",
    "NoteInput",
    "NoteInputTypedDict",
    "NoteTypedDict",
    "NotesModel",
    "NotesModelTypedDict",
    "OAuthGrantType",
    "OauthCredentialsSource",
    "OauthScopes",
    "OauthScopesTypedDict",
    "OddWeeks",
    "OddWeeksTypedDict",
    "Operation",
    "OperationTypedDict",
    "OpportunitiesFilter",
    "OpportunitiesFilterTypedDict",
    "OpportunitiesSort",
    "OpportunitiesSortBy",
    "OpportunitiesSortTypedDict",
    "Opportunity",
    "OpportunityInput",
    "OpportunityInputTypedDict",
    "OpportunityTypedDict",
    "OptionType",
    "Options",
    "OptionsTypedDict",
    "OrdersSort",
    "OrdersSortBy",
    "OrdersSortTypedDict",
    "OtherExpenses",
    "OtherExpensesTypedDict",
    "OtherIncome",
    "OtherIncomeTypedDict",
    "OutstandingBalanceByCurrency",
    "OutstandingBalanceByCurrencyTypedDict",
    "OutstandingBalanceByCustomer",
    "OutstandingBalanceByCustomerTypedDict",
    "OutstandingBalanceBySupplier",
    "OutstandingBalanceBySupplierTypedDict",
    "Owner",
    "OwnerTypedDict",
    "PaginationCoverage",
    "PaginationCoverageMode",
    "PaginationCoverageTypedDict",
    "ParentAccount",
    "ParentAccountTypedDict",
    "PassThroughBody",
    "PassThroughBodyTypedDict",
    "Payment",
    "PaymentAllocations",
    "PaymentAllocationsTypedDict",
    "PaymentFrequency",
    "PaymentInput",
    "PaymentInputTypedDict",
    "PaymentRequiredResponse",
    "PaymentRequiredResponseData",
    "PaymentStatus",
    "PaymentType",
    "PaymentTypedDict",
    "PaymentUnit",
    "PaymentsFilter",
    "PaymentsFilterTypedDict",
    "PaymentsSort",
    "PaymentsSortBy",
    "PaymentsSortTypedDict",
    "Payroll",
    "PayrollTotals",
    "PayrollTotalsTypedDict",
    "PayrollTypedDict",
    "PayrollsFilter",
    "PayrollsFilterTypedDict",
    "PeriodType",
    "Permissions",
    "PermissionsTypedDict",
    "Person",
    "PersonInput",
    "PersonInputTypedDict",
    "PersonTypedDict",
    "PhoneNumber",
    "PhoneNumberType",
    "PhoneNumberTypedDict",
    "Pipeline",
    "PipelineInput",
    "PipelineInputTypedDict",
    "PipelineStages",
    "PipelineStagesTypedDict",
    "PipelineTypedDict",
    "Price",
    "PriceTypedDict",
    "Priority",
    "ProbationPeriod",
    "ProbationPeriodTypedDict",
    "ProductStatus",
    "ProfitAndLoss",
    "ProfitAndLossFilter",
    "ProfitAndLossFilterTypedDict",
    "ProfitAndLossIndicator",
    "ProfitAndLossIndicatorTypedDict",
    "ProfitAndLossType",
    "ProfitAndLossTypedDict",
    "PurchaseDetails",
    "PurchaseDetailsTypedDict",
    "PurchaseOrder",
    "PurchaseOrderAmortizationType",
    "PurchaseOrderInput",
    "PurchaseOrderInputTypedDict",
    "PurchaseOrderStatus",
    "PurchaseOrderTypedDict",
    "PurchaseOrdersFilter",
    "PurchaseOrdersFilterTypedDict",
    "PurchaseOrdersSort",
    "PurchaseOrdersSortBy",
    "PurchaseOrdersSortTypedDict",
    "Quote",
    "QuoteInput",
    "QuoteInputTypedDict",
    "QuoteLineItem",
    "QuoteLineItemInput",
    "QuoteLineItemInputTypedDict",
    "QuoteLineItemType",
    "QuoteLineItemTypedDict",
    "QuoteStatus",
    "QuoteTypedDict",
    "RebillStatus",
    "Rebilling",
    "RebillingTypedDict",
    "Reports",
    "ReportsTypedDict",
    "RequestCountAllocation",
    "RequestCountAllocationTypedDict",
    "RequestRate",
    "RequestRateTypedDict",
    "RequestType",
    "ResourceStatus",
    "Resources",
    "ResourcesTypedDict",
    "ResponseValidationError",
    "Salary",
    "SalaryTypedDict",
    "SalesDetails",
    "SalesDetailsTypedDict",
    "Schedule",
    "ScheduleTypedDict",
    "Schema",
    "SchemaSupport",
    "SchemaSupportTypedDict",
    "SchemaTypedDict",
    "Scope",
    "Security",
    "SecurityTypedDict",
    "Service",
    "ServiceTypedDict",
    "Session",
    "SessionTypedDict",
    "Settings",
    "SettingsTypedDict",
    "SharedLink",
    "SharedLinkInput",
    "SharedLinkInputTypedDict",
    "SharedLinkTarget",
    "SharedLinkTargetTypedDict",
    "SharedLinkTypedDict",
    "ShowAs",
    "SimpleFormFieldOption",
    "SimpleFormFieldOptionTypedDict",
    "SimpleFormFieldOptionValue",
    "SimpleFormFieldOptionValueTypedDict",
    "SmsMessagesAddGlobals",
    "SmsMessagesAddGlobalsTypedDict",
    "SmsMessagesAddRequest",
    "SmsMessagesAddRequestTypedDict",
    "SmsMessagesAddResponse",
    "SmsMessagesAddResponseTypedDict",
    "SmsMessagesAllGlobals",
    "SmsMessagesAllGlobalsTypedDict",
    "SmsMessagesAllRequest",
    "SmsMessagesAllRequestTypedDict",
    "SmsMessagesAllResponse",
    "SmsMessagesAllResponseTypedDict",
    "SmsMessagesDeleteGlobals",
    "SmsMessagesDeleteGlobalsTypedDict",
    "SmsMessagesDeleteRequest",
    "SmsMessagesDeleteRequestTypedDict",
    "SmsMessagesDeleteResponse",
    "SmsMessagesDeleteResponseTypedDict",
    "SmsMessagesOneGlobals",
    "SmsMessagesOneGlobalsTypedDict",
    "SmsMessagesOneRequest",
    "SmsMessagesOneRequestTypedDict",
    "SmsMessagesOneResponse",
    "SmsMessagesOneResponseTypedDict",
    "SmsMessagesUpdateGlobals",
    "SmsMessagesUpdateGlobalsTypedDict",
    "SmsMessagesUpdateRequest",
    "SmsMessagesUpdateRequestTypedDict",
    "SmsMessagesUpdateResponse",
    "SmsMessagesUpdateResponseTypedDict",
    "SocialLink",
    "SocialLinkTypedDict",
    "SocialLinks",
    "SocialLinksTypedDict",
    "SortDirection",
    "Stage",
    "StageTypedDict",
    "Stages",
    "StagesTypedDict",
    "StatementStatus",
    "Status",
    "SubAccounts",
    "SubAccountsTypedDict",
    "SubscriptionLevel",
    "SubsidiariesModel",
    "SubsidiariesModelTypedDict",
    "Subsidiary",
    "SubsidiaryInput",
    "SubsidiaryInputTypedDict",
    "SubsidiaryReference",
    "SubsidiaryReferenceInput",
    "SubsidiaryReferenceInputTypedDict",
    "SubsidiaryReferenceTypedDict",
    "SubsidiaryStatus",
    "SubsidiaryTypedDict",
    "Supplier",
    "SupplierInput",
    "SupplierInputTypedDict",
    "SupplierStatus",
    "SupplierTypedDict",
    "SuppliersFilter",
    "SuppliersFilterTypedDict",
    "SuppliersSort",
    "SuppliersSortBy",
    "SuppliersSortTypedDict",
    "SupportedProperty",
    "SupportedPropertyTypedDict",
    "TLSSupport",
    "TLSSupportTypedDict",
    "Target",
    "Tax",
    "TaxRate",
    "TaxRateInput",
    "TaxRateInputTypedDict",
    "TaxRateStatus",
    "TaxRateTypedDict",
    "TaxRatesFilter",
    "TaxRatesFilterTypedDict",
    "TaxTypedDict",
    "Team",
    "TeamTypedDict",
    "TheStartMonthOfFiscalYear",
    "Theme",
    "ThemeTypedDict",
    "Ticket",
    "TicketInput",
    "TicketInputTypedDict",
    "TicketTypedDict",
    "TicketsSort",
    "TicketsSortBy",
    "TicketsSortTypedDict",
    "TimeOffRequest",
    "TimeOffRequestInput",
    "TimeOffRequestInputTypedDict",
    "TimeOffRequestStatus",
    "TimeOffRequestStatusStatus",
    "TimeOffRequestTypedDict",
    "TimeOffRequestsFilter",
    "TimeOffRequestsFilterTypedDict",
    "TrackingCategory",
    "TrackingCategoryInput",
    "TrackingCategoryInputTypedDict",
    "TrackingCategoryStatus",
    "TrackingCategorySubsidiaries",
    "TrackingCategorySubsidiariesTypedDict",
    "TrackingCategoryTypedDict",
    "TrackingItem",
    "TrackingItemTypedDict",
    "TransactionType",
    "Transactions",
    "TransactionsTypedDict",
    "Type",
    "UnauthorizedResponse",
    "UnauthorizedResponseData",
    "UnauthorizedResponseDetail",
    "UncategorizedAccounts",
    "UncategorizedAccountsTypedDict",
    "UnexpectedErrorResponse",
    "UnexpectedErrorResponseTypedDict",
    "UnifiedAPI",
    "UnifiedAPIID",
    "UnifiedApis",
    "UnifiedApisTypedDict",
    "UnifiedFile",
    "UnifiedFileTypedDict",
    "UnifiedID",
    "UnifiedIDTypedDict",
    "Unit",
    "Units",
    "UnprocessableResponse",
    "UnprocessableResponseData",
    "UnprocessableResponseDetail",
    "UpdateAccountingDepartmentResponse",
    "UpdateAccountingDepartmentResponseTypedDict",
    "UpdateAccountingLocationResponse",
    "UpdateAccountingLocationResponseTypedDict",
    "UpdateActivityResponse",
    "UpdateActivityResponseTypedDict",
    "UpdateApplicantResponse",
    "UpdateApplicantResponseTypedDict",
    "UpdateApplicationResponse",
    "UpdateApplicationResponseTypedDict",
    "UpdateBankFeedAccountResponse",
    "UpdateBankFeedAccountResponseTypedDict",
    "UpdateBankFeedStatementResponse",
    "UpdateBankFeedStatementResponseTypedDict",
    "UpdateBillPaymentResponse",
    "UpdateBillPaymentResponseTypedDict",
    "UpdateBillResponse",
    "UpdateBillResponseTypedDict",
    "UpdateCommentResponse",
    "UpdateCommentResponseTypedDict",
    "UpdateCompanyResponse",
    "UpdateCompanyResponseTypedDict",
    "UpdateConnectionResponse",
    "UpdateConnectionResponseTypedDict",
    "UpdateConsumerRequest",
    "UpdateConsumerRequestTypedDict",
    "UpdateConsumerResponse",
    "UpdateConsumerResponseTypedDict",
    "UpdateContactResponse",
    "UpdateContactResponseTypedDict",
    "UpdateCreditNoteResponse",
    "UpdateCreditNoteResponseTypedDict",
    "UpdateCustomObjectResponse",
    "UpdateCustomObjectResponseTypedDict",
    "UpdateCustomObjectSchemaResponse",
    "UpdateCustomObjectSchemaResponseTypedDict",
    "UpdateCustomerResponse",
    "UpdateCustomerResponseTypedDict",
    "UpdateDepartmentResponse",
    "UpdateDepartmentResponseTypedDict",
    "UpdateDriveGroupResponse",
    "UpdateDriveGroupResponseTypedDict",
    "UpdateDriveResponse",
    "UpdateDriveResponseTypedDict",
    "UpdateEmployeeResponse",
    "UpdateEmployeeResponseTypedDict",
    "UpdateExpenseResponse",
    "UpdateExpenseResponseTypedDict",
    "UpdateFileRequest",
    "UpdateFileRequestTypedDict",
    "UpdateFileResponse",
    "UpdateFileResponseTypedDict",
    "UpdateFolderRequest",
    "UpdateFolderRequestTypedDict",
    "UpdateFolderResponse",
    "UpdateFolderResponseTypedDict",
    "UpdateHrisCompanyResponse",
    "UpdateHrisCompanyResponseTypedDict",
    "UpdateInvoiceItemsResponse",
    "UpdateInvoiceItemsResponseTypedDict",
    "UpdateInvoiceResponse",
    "UpdateInvoiceResponseTypedDict",
    "UpdateJournalEntryResponse",
    "UpdateJournalEntryResponseTypedDict",
    "UpdateLeadResponse",
    "UpdateLeadResponseTypedDict",
    "UpdateLedgerAccountResponse",
    "UpdateLedgerAccountResponseTypedDict",
    "UpdateMessageResponse",
    "UpdateMessageResponseTypedDict",
    "UpdateNoteResponse",
    "UpdateNoteResponseTypedDict",
    "UpdateOpportunityResponse",
    "UpdateOpportunityResponseTypedDict",
    "UpdatePaymentResponse",
    "UpdatePaymentResponseTypedDict",
    "UpdatePipelineResponse",
    "UpdatePipelineResponseTypedDict",
    "UpdatePurchaseOrderResponse",
    "UpdatePurchaseOrderResponseTypedDict",
    "UpdateQuoteResponse",
    "UpdateQuoteResponseTypedDict",
    "UpdateSharedLinkResponse",
    "UpdateSharedLinkResponseTypedDict",
    "UpdateSubsidiaryResponse",
    "UpdateSubsidiaryResponseTypedDict",
    "UpdateSupplierResponse",
    "UpdateSupplierResponseTypedDict",
    "UpdateTaxRateResponse",
    "UpdateTaxRateResponseTypedDict",
    "UpdateTicketResponse",
    "UpdateTicketResponseTypedDict",
    "UpdateTimeOffRequestResponse",
    "UpdateTimeOffRequestResponseTypedDict",
    "UpdateTrackingCategoryResponse",
    "UpdateTrackingCategoryResponseTypedDict",
    "UpdateUploadSessionResponse",
    "UpdateUploadSessionResponseTypedDict",
    "UpdateUserResponse",
    "UpdateUserResponseTypedDict",
    "UpdateWebhookRequest",
    "UpdateWebhookRequestTypedDict",
    "UpdateWebhookResponse",
    "UpdateWebhookResponseTypedDict",
    "UploadSession",
    "UploadSessionTypedDict",
    "User",
    "UserInput",
    "UserInputTypedDict",
    "UserTypedDict",
    "ValidateConnectionStateResponse",
    "ValidateConnectionStateResponseData",
    "ValidateConnectionStateResponseDataTypedDict",
    "ValidateConnectionStateResponseTypedDict",
    "Value",
    "Value5",
    "Value5TypedDict",
    "ValueTypedDict",
    "Variants",
    "VariantsTypedDict",
    "VaultConnectionCustomMappingsAllGlobals",
    "VaultConnectionCustomMappingsAllGlobalsTypedDict",
    "VaultConnectionCustomMappingsAllRequest",
    "VaultConnectionCustomMappingsAllRequestTypedDict",
    "VaultConnectionCustomMappingsAllResponse",
    "VaultConnectionCustomMappingsAllResponseTypedDict",
    "VaultConnectionSettingsAllGlobals",
    "VaultConnectionSettingsAllGlobalsTypedDict",
    "VaultConnectionSettingsAllRequest",
    "VaultConnectionSettingsAllRequestTypedDict",
    "VaultConnectionSettingsAllResponse",
    "VaultConnectionSettingsAllResponseTypedDict",
    "VaultConnectionSettingsUpdateGlobals",
    "VaultConnectionSettingsUpdateGlobalsTypedDict",
    "VaultConnectionSettingsUpdateRequest",
    "VaultConnectionSettingsUpdateRequestTypedDict",
    "VaultConnectionSettingsUpdateResponse",
    "VaultConnectionSettingsUpdateResponseTypedDict",
    "VaultConnectionsAllGlobals",
    "VaultConnectionsAllGlobalsTypedDict",
    "VaultConnectionsAllRequest",
    "VaultConnectionsAllRequestTypedDict",
    "VaultConnectionsAllResponse",
    "VaultConnectionsAllResponseTypedDict",
    "VaultConnectionsDeleteGlobals",
    "VaultConnectionsDeleteGlobalsTypedDict",
    "VaultConnectionsDeleteRequest",
    "VaultConnectionsDeleteRequestTypedDict",
    "VaultConnectionsDeleteResponse",
    "VaultConnectionsDeleteResponseTypedDict",
    "VaultConnectionsImportGlobals",
    "VaultConnectionsImportGlobalsTypedDict",
    "VaultConnectionsImportRequest",
    "VaultConnectionsImportRequestTypedDict",
    "VaultConnectionsImportResponse",
    "VaultConnectionsImportResponseTypedDict",
    "VaultConnectionsOneGlobals",
    "VaultConnectionsOneGlobalsTypedDict",
    "VaultConnectionsOneRequest",
    "VaultConnectionsOneRequestTypedDict",
    "VaultConnectionsOneResponse",
    "VaultConnectionsOneResponseTypedDict",
    "VaultConnectionsTokenGlobals",
    "VaultConnectionsTokenGlobalsTypedDict",
    "VaultConnectionsTokenRequest",
    "VaultConnectionsTokenRequestBody",
    "VaultConnectionsTokenRequestBodyTypedDict",
    "VaultConnectionsTokenRequestTypedDict",
    "VaultConnectionsTokenResponse",
    "VaultConnectionsTokenResponseTypedDict",
    "VaultConnectionsUpdateGlobals",
    "VaultConnectionsUpdateGlobalsTypedDict",
    "VaultConnectionsUpdateRequest",
    "VaultConnectionsUpdateRequestTypedDict",
    "VaultConnectionsUpdateResponse",
    "VaultConnectionsUpdateResponseTypedDict",
    "VaultConsumerRequestCountsAllGlobals",
    "VaultConsumerRequestCountsAllGlobalsTypedDict",
    "VaultConsumerRequestCountsAllRequest",
    "VaultConsumerRequestCountsAllRequestTypedDict",
    "VaultConsumerRequestCountsAllResponse",
    "VaultConsumerRequestCountsAllResponseTypedDict",
    "VaultConsumersAddGlobals",
    "VaultConsumersAddGlobalsTypedDict",
    "VaultConsumersAddRequest",
    "VaultConsumersAddRequestTypedDict",
    "VaultConsumersAddResponse",
    "VaultConsumersAddResponseTypedDict",
    "VaultConsumersAllGlobals",
    "VaultConsumersAllGlobalsTypedDict",
    "VaultConsumersAllRequest",
    "VaultConsumersAllRequestTypedDict",
    "VaultConsumersAllResponse",
    "VaultConsumersAllResponseTypedDict",
    "VaultConsumersDeleteGlobals",
    "VaultConsumersDeleteGlobalsTypedDict",
    "VaultConsumersDeleteRequest",
    "VaultConsumersDeleteRequestTypedDict",
    "VaultConsumersDeleteResponse",
    "VaultConsumersDeleteResponseTypedDict",
    "VaultConsumersOneGlobals",
    "VaultConsumersOneGlobalsTypedDict",
    "VaultConsumersOneRequest",
    "VaultConsumersOneRequestTypedDict",
    "VaultConsumersOneResponse",
    "VaultConsumersOneResponseTypedDict",
    "VaultConsumersUpdateGlobals",
    "VaultConsumersUpdateGlobalsTypedDict",
    "VaultConsumersUpdateRequest",
    "VaultConsumersUpdateRequestTypedDict",
    "VaultConsumersUpdateResponse",
    "VaultConsumersUpdateResponseTypedDict",
    "VaultCreateCallbackStateGlobals",
    "VaultCreateCallbackStateGlobalsTypedDict",
    "VaultCreateCallbackStateRequest",
    "VaultCreateCallbackStateRequestTypedDict",
    "VaultCreateCallbackStateResponse",
    "VaultCreateCallbackStateResponseTypedDict",
    "VaultCustomFieldsAllGlobals",
    "VaultCustomFieldsAllGlobalsTypedDict",
    "VaultCustomFieldsAllRequest",
    "VaultCustomFieldsAllRequestTypedDict",
    "VaultCustomFieldsAllResponse",
    "VaultCustomFieldsAllResponseTypedDict",
    "VaultCustomMappingsAllGlobals",
    "VaultCustomMappingsAllGlobalsTypedDict",
    "VaultCustomMappingsAllRequest",
    "VaultCustomMappingsAllRequestTypedDict",
    "VaultCustomMappingsAllResponse",
    "VaultCustomMappingsAllResponseTypedDict",
    "VaultLogsAllGlobals",
    "VaultLogsAllGlobalsTypedDict",
    "VaultLogsAllRequest",
    "VaultLogsAllRequestTypedDict",
    "VaultLogsAllResponse",
    "VaultLogsAllResponseTypedDict",
    "VaultSessionsCreateGlobals",
    "VaultSessionsCreateGlobalsTypedDict",
    "VaultSessionsCreateRequest",
    "VaultSessionsCreateRequestTypedDict",
    "VaultSessionsCreateResponse",
    "VaultSessionsCreateResponseTypedDict",
    "VaultValidateConnectionStateGlobals",
    "VaultValidateConnectionStateGlobalsTypedDict",
    "VaultValidateConnectionStateRequest",
    "VaultValidateConnectionStateRequestBody",
    "VaultValidateConnectionStateRequestBodyTypedDict",
    "VaultValidateConnectionStateRequestTypedDict",
    "VaultValidateConnectionStateResponse",
    "VaultValidateConnectionStateResponseTypedDict",
    "VirtualWebhooks",
    "VirtualWebhooksTypedDict",
    "Visibility",
    "Webhook",
    "WebhookEventLog",
    "WebhookEventLogService",
    "WebhookEventLogServiceTypedDict",
    "WebhookEventLogTypedDict",
    "WebhookEventLogsAllGlobals",
    "WebhookEventLogsAllGlobalsTypedDict",
    "WebhookEventLogsAllRequest",
    "WebhookEventLogsAllRequestTypedDict",
    "WebhookEventLogsAllResponse",
    "WebhookEventLogsAllResponseTypedDict",
    "WebhookEventLogsFilter",
    "WebhookEventLogsFilterService",
    "WebhookEventLogsFilterServiceTypedDict",
    "WebhookEventLogsFilterTypedDict",
    "WebhookEventType",
    "WebhookSubscription",
    "WebhookSubscriptionTypedDict",
    "WebhookSupport",
    "WebhookSupportResources",
    "WebhookSupportResourcesTypedDict",
    "WebhookSupportTypedDict",
    "WebhookTypedDict",
    "WebhookWebhooksAddGlobals",
    "WebhookWebhooksAddGlobalsTypedDict",
    "WebhookWebhooksAddRequest",
    "WebhookWebhooksAddRequestTypedDict",
    "WebhookWebhooksAddResponse",
    "WebhookWebhooksAddResponseTypedDict",
    "WebhookWebhooksAllGlobals",
    "WebhookWebhooksAllGlobalsTypedDict",
    "WebhookWebhooksAllRequest",
    "WebhookWebhooksAllRequestTypedDict",
    "WebhookWebhooksAllResponse",
    "WebhookWebhooksAllResponseTypedDict",
    "WebhookWebhooksDeleteGlobals",
    "WebhookWebhooksDeleteGlobalsTypedDict",
    "WebhookWebhooksDeleteRequest",
    "WebhookWebhooksDeleteRequestTypedDict",
    "WebhookWebhooksDeleteResponse",
    "WebhookWebhooksDeleteResponseTypedDict",
    "WebhookWebhooksOneGlobals",
    "WebhookWebhooksOneGlobalsTypedDict",
    "WebhookWebhooksOneRequest",
    "WebhookWebhooksOneRequestTypedDict",
    "WebhookWebhooksOneResponse",
    "WebhookWebhooksOneResponseTypedDict",
    "WebhookWebhooksUpdateGlobals",
    "WebhookWebhooksUpdateGlobalsTypedDict",
    "WebhookWebhooksUpdateRequest",
    "WebhookWebhooksUpdateRequestTypedDict",
    "WebhookWebhooksUpdateResponse",
    "WebhookWebhooksUpdateResponseTypedDict",
    "Website",
    "WebsiteType",
    "WebsiteTypedDict",
    "Websites",
    "WebsitesTypedDict",
    "WorkPattern",
    "WorkPatternTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "AccountingAgedCreditorsOneGlobals": ".accounting_agedcreditorsoneop",
    "AccountingAgedCreditorsOneGlobalsTypedDict": ".accounting_agedcreditorsoneop",
    "AccountingAgedCreditorsOneRequest": ".accounting_agedcreditorsoneop",
    "AccountingAgedCreditorsOneRequestTypedDict": ".accounting_agedcreditorsoneop",
    "AccountingAgedCreditorsOneResponse": ".accounting_agedcreditorsoneop",
    "AccountingAgedCreditorsOneResponseTypedDict": ".accounting_agedcreditorsoneop",
    "AccountingAgedDebtorsOneGlobals": ".accounting_ageddebtorsoneop",
    "AccountingAgedDebtorsOneGlobalsTypedDict": ".accounting_ageddebtorsoneop",
    "AccountingAgedDebtorsOneRequest": ".accounting_ageddebtorsoneop",
    "AccountingAgedDebtorsOneRequestTypedDict": ".accounting_ageddebtorsoneop",
    "AccountingAgedDebtorsOneResponse": ".accounting_ageddebtorsoneop",
    "AccountingAgedDebtorsOneResponseTypedDict": ".accounting_ageddebtorsoneop",
    "AccountingAttachmentsAllGlobals": ".accounting_attachmentsallop",
    "AccountingAttachmentsAllGlobalsTypedDict": ".accounting_attachmentsallop",
    "AccountingAttachmentsAllRequest": ".accounting_attachmentsallop",
    "AccountingAttachmentsAllRequestTypedDict": ".accounting_attachmentsallop",
    "AccountingAttachmentsAllResponse": ".accounting_attachmentsallop",
    "AccountingAttachmentsAllResponseTypedDict": ".accounting_attachmentsallop",
    "AccountingAttachmentsDeleteGlobals": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDeleteGlobalsTypedDict": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDeleteRequest": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDeleteRequestTypedDict": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDeleteResponse": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDeleteResponseTypedDict": ".accounting_attachmentsdeleteop",
    "AccountingAttachmentsDownloadGlobals": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsDownloadGlobalsTypedDict": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsDownloadRequest": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsDownloadRequestTypedDict": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsDownloadResponse": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsDownloadResponseTypedDict": ".accounting_attachmentsdownloadop",
    "AccountingAttachmentsOneGlobals": ".accounting_attachmentsoneop",
    "AccountingAttachmentsOneGlobalsTypedDict": ".accounting_attachmentsoneop",
    "AccountingAttachmentsOneRequest": ".accounting_attachmentsoneop",
    "AccountingAttachmentsOneRequestTypedDict": ".accounting_attachmentsoneop",
    "AccountingAttachmentsOneResponse": ".accounting_attachmentsoneop",
    "AccountingAttachmentsOneResponseTypedDict": ".accounting_attachmentsoneop",
    "ACCOUNTING_ATTACHMENTS_UPLOAD_OP_SERVERS": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadGlobals": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadGlobalsTypedDict": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadRequest": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadRequestTypedDict": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadResponse": ".accounting_attachmentsuploadop",
    "AccountingAttachmentsUploadResponseTypedDict": ".accounting_attachmentsuploadop",
    "AccountingBalanceSheetOneGlobals": ".accounting_balancesheetoneop",
    "AccountingBalanceSheetOneGlobalsTypedDict": ".accounting_balancesheetoneop",
    "AccountingBalanceSheetOneRequest": ".accounting_balancesheetoneop",
    "AccountingBalanceSheetOneRequestTypedDict": ".accounting_balancesheetoneop",
    "AccountingBalanceSheetOneResponse": ".accounting_balancesheetoneop",
    "AccountingBalanceSheetOneResponseTypedDict": ".accounting_balancesheetoneop",
    "AccountingBankFeedAccountsAddGlobals": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAddGlobalsTypedDict": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAddRequest": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAddRequestTypedDict": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAddResponse": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAddResponseTypedDict": ".accounting_bankfeedaccountsaddop",
    "AccountingBankFeedAccountsAllGlobals": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsAllGlobalsTypedDict": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsAllRequest": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsAllRequestTypedDict": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsAllResponse": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsAllResponseTypedDict": ".accounting_bankfeedaccountsallop",
    "AccountingBankFeedAccountsDeleteGlobals": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsDeleteGlobalsTypedDict": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsDeleteRequest": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsDeleteRequestTypedDict": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsDeleteResponse": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsDeleteResponseTypedDict": ".accounting_bankfeedaccountsdeleteop",
    "AccountingBankFeedAccountsOneGlobals": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsOneGlobalsTypedDict": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsOneRequest": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsOneRequestTypedDict": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsOneResponse": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsOneResponseTypedDict": ".accounting_bankfeedaccountsoneop",
    "AccountingBankFeedAccountsUpdateGlobals": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedAccountsUpdateGlobalsTypedDict": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedAccountsUpdateRequest": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedAccountsUpdateRequestTypedDict": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedAccountsUpdateResponse": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedAccountsUpdateResponseTypedDict": ".accounting_bankfeedaccountsupdateop",
    "AccountingBankFeedStatementsAddGlobals": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAddGlobalsTypedDict": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAddRequest": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAddRequestTypedDict": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAddResponse": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAddResponseTypedDict": ".accounting_bankfeedstatementsaddop",
    "AccountingBankFeedStatementsAllGlobals": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsAllGlobalsTypedDict": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsAllRequest": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsAllRequestTypedDict": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsAllResponse": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsAllResponseTypedDict": ".accounting_bankfeedstatementsallop",
    "AccountingBankFeedStatementsDeleteGlobals": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsDeleteGlobalsTypedDict": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsDeleteRequest": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsDeleteRequestTypedDict": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsDeleteResponse": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsDeleteResponseTypedDict": ".accounting_bankfeedstatementsdeleteop",
    "AccountingBankFeedStatementsOneGlobals": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsOneGlobalsTypedDict": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsOneRequest": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsOneRequestTypedDict": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsOneResponse": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsOneResponseTypedDict": ".accounting_bankfeedstatementsoneop",
    "AccountingBankFeedStatementsUpdateGlobals": ".accounting_bankfeedstatementsupdateop",
    "AccountingBankFeedStatementsUpdateGlobalsTypedDict": ".accounting_bankfeedstatementsupdateop",
    "AccountingBankFeedStatementsUpdateRequest": ".accounting_bankfeedstatementsupdateop",
    "AccountingBankFeedStatementsUpdateRequestTypedDict": ".accounting_bankfeedstatementsupdateop",
    "AccountingBankFeedStatementsUpdateResponse": ".accounting_bankfeedstatementsupdateop",
    "AccountingBankFeedStatementsUpdateResponseTypedDict": ".accounting_bankfeedstatementsupdateop",
    "AccountingBillPaymentsAddGlobals": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAddGlobalsTypedDict": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAddRequest": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAddRequestTypedDict": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAddResponse": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAddResponseTypedDict": ".accounting_billpaymentsaddop",
    "AccountingBillPaymentsAllGlobals": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsAllGlobalsTypedDict": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsAllRequest": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsAllRequestTypedDict": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsAllResponse": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsAllResponseTypedDict": ".accounting_billpaymentsallop",
    "AccountingBillPaymentsDeleteGlobals": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsDeleteGlobalsTypedDict": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsDeleteRequest": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsDeleteRequestTypedDict": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsDeleteResponse": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsDeleteResponseTypedDict": ".accounting_billpaymentsdeleteop",
    "AccountingBillPaymentsOneGlobals": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsOneGlobalsTypedDict": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsOneRequest": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsOneRequestTypedDict": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsOneResponse": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsOneResponseTypedDict": ".accounting_billpaymentsoneop",
    "AccountingBillPaymentsUpdateGlobals": ".accounting_billpaymentsupdateop",
    "AccountingBillPaymentsUpdateGlobalsTypedDict": ".accounting_billpaymentsupdateop",
    "AccountingBillPaymentsUpdateRequest": ".accounting_billpaymentsupdateop",
    "AccountingBillPaymentsUpdateRequestTypedDict": ".accounting_billpaymentsupdateop",
    "AccountingBillPaymentsUpdateResponse": ".accounting_billpaymentsupdateop",
    "AccountingBillPaymentsUpdateResponseTypedDict": ".accounting_billpaymentsupdateop",
    "AccountingBillsAddGlobals": ".accounting_billsaddop",
    "AccountingBillsAddGlobalsTypedDict": ".accounting_billsaddop",
    "AccountingBillsAddRequest": ".accounting_billsaddop",
    "AccountingBillsAddRequestTypedDict": ".accounting_billsaddop",
    "AccountingBillsAddResponse": ".accounting_billsaddop",
    "AccountingBillsAddResponseTypedDict": ".accounting_billsaddop",
    "AccountingBillsAllGlobals": ".accounting_billsallop",
    "AccountingBillsAllGlobalsTypedDict": ".accounting_billsallop",
    "AccountingBillsAllRequest": ".accounting_billsallop",
    "AccountingBillsAllRequestTypedDict": ".accounting_billsallop",
    "AccountingBillsAllResponse": ".accounting_billsallop",
    "AccountingBillsAllResponseTypedDict": ".accounting_billsallop",
    "AccountingBillsDeleteGlobals": ".accounting_billsdeleteop",
    "AccountingBillsDeleteGlobalsTypedDict": ".accounting_billsdeleteop",
    "AccountingBillsDeleteRequest": ".accounting_billsdeleteop",
    "AccountingBillsDeleteRequestTypedDict": ".accounting_billsdeleteop",
    "AccountingBillsDeleteResponse": ".accounting_billsdeleteop",
    "AccountingBillsDeleteResponseTypedDict": ".accounting_billsdeleteop",
    "AccountingBillsOneGlobals": ".accounting_billsoneop",
    "AccountingBillsOneGlobalsTypedDict": ".accounting_billsoneop",
    "AccountingBillsOneRequest": ".accounting_billsoneop",
    "AccountingBillsOneRequestTypedDict": ".accounting_billsoneop",
    "AccountingBillsOneResponse": ".accounting_billsoneop",
    "AccountingBillsOneResponseTypedDict": ".accounting_billsoneop",
    "AccountingBillsUpdateGlobals": ".accounting_billsupdateop",
    "AccountingBillsUpdateGlobalsTypedDict": ".accounting_billsupdateop",
    "AccountingBillsUpdateRequest": ".accounting_billsupdateop",
    "AccountingBillsUpdateRequestTypedDict": ".accounting_billsupdateop",
    "AccountingBillsUpdateResponse": ".accounting_billsupdateop",
    "AccountingBillsUpdateResponseTypedDict": ".accounting_billsupdateop",
    "AccountingCategoriesAllGlobals": ".accounting_categoriesallop",
    "AccountingCategoriesAllGlobalsTypedDict": ".accounting_categoriesallop",
    "AccountingCategoriesAllRequest": ".accounting_categoriesallop",
    "AccountingCategoriesAllRequestTypedDict": ".accounting_categoriesallop",
    "AccountingCategoriesAllResponse": ".accounting_categoriesallop",
    "AccountingCategoriesAllResponseTypedDict": ".accounting_categoriesallop",
    "AccountingCategoriesOneGlobals": ".accounting_categoriesoneop",
    "AccountingCategoriesOneGlobalsTypedDict": ".accounting_categoriesoneop",
    "AccountingCategoriesOneRequest": ".accounting_categoriesoneop",
    "AccountingCategoriesOneRequestTypedDict": ".accounting_categoriesoneop",
    "AccountingCategoriesOneResponse": ".accounting_categoriesoneop",
    "AccountingCategoriesOneResponseTypedDict": ".accounting_categoriesoneop",
    "AccountingCompanyInfoOneGlobals": ".accounting_companyinfooneop",
    "AccountingCompanyInfoOneGlobalsTypedDict": ".accounting_companyinfooneop",
    "AccountingCompanyInfoOneRequest": ".accounting_companyinfooneop",
    "AccountingCompanyInfoOneRequestTypedDict": ".accounting_companyinfooneop",
    "AccountingCompanyInfoOneResponse": ".accounting_companyinfooneop",
    "AccountingCompanyInfoOneResponseTypedDict": ".accounting_companyinfooneop",
    "AccountingCreditNotesAddGlobals": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAddGlobalsTypedDict": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAddRequest": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAddRequestTypedDict": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAddResponse": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAddResponseTypedDict": ".accounting_creditnotesaddop",
    "AccountingCreditNotesAllGlobals": ".accounting_creditnotesallop",
    "AccountingCreditNotesAllGlobalsTypedDict": ".accounting_creditnotesallop",
    "AccountingCreditNotesAllRequest": ".accounting_creditnotesallop",
    "AccountingCreditNotesAllRequestTypedDict": ".accounting_creditnotesallop",
    "AccountingCreditNotesAllResponse": ".accounting_creditnotesallop",
    "AccountingCreditNotesAllResponseTypedDict": ".accounting_creditnotesallop",
    "AccountingCreditNotesDeleteGlobals": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesDeleteGlobalsTypedDict": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesDeleteRequest": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesDeleteRequestTypedDict": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesDeleteResponse": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesDeleteResponseTypedDict": ".accounting_creditnotesdeleteop",
    "AccountingCreditNotesOneGlobals": ".accounting_creditnotesoneop",
    "AccountingCreditNotesOneGlobalsTypedDict": ".accounting_creditnotesoneop",
    "AccountingCreditNotesOneRequest": ".accounting_creditnotesoneop",
    "AccountingCreditNotesOneRequestTypedDict": ".accounting_creditnotesoneop",
    "AccountingCreditNotesOneResponse": ".accounting_creditnotesoneop",
    "AccountingCreditNotesOneResponseTypedDict": ".accounting_creditnotesoneop",
    "AccountingCreditNotesUpdateGlobals": ".accounting_creditnotesupdateop",
    "AccountingCreditNotesUpdateGlobalsTypedDict": ".accounting_creditnotesupdateop",
    "AccountingCreditNotesUpdateRequest": ".accounting_creditnotesupdateop",
    "AccountingCreditNotesUpdateRequestTypedDict": ".accounting_creditnotesupdateop",
    "AccountingCreditNotesUpdateResponse": ".accounting_creditnotesupdateop",
    "AccountingCreditNotesUpdateResponseTypedDict": ".accounting_creditnotesupdateop",
    "AccountingCustomersAddGlobals": ".accounting_customersaddop",
    "AccountingCustomersAddGlobalsTypedDict": ".accounting_customersaddop",
    "AccountingCustomersAddRequest": ".accounting_customersaddop",
    "AccountingCustomersAddRequestTypedDict": ".accounting_customersaddop",
    "AccountingCustomersAddResponse": ".accounting_customersaddop",
    "AccountingCustomersAddResponseTypedDict": ".accounting_customersaddop",
    "AccountingCustomersAllGlobals": ".accounting_customersallop",
    "AccountingCustomersAllGlobalsTypedDict": ".accounting_customersallop",
    "AccountingCustomersAllRequest": ".accounting_customersallop",
    "AccountingCustomersAllRequestTypedDict": ".accounting_customersallop",
    "AccountingCustomersAllResponse": ".accounting_customersallop",
    "AccountingCustomersAllResponseTypedDict": ".accounting_customersallop",
    "AccountingCustomersDeleteGlobals": ".accounting_customersdeleteop",
    "AccountingCustomersDeleteGlobalsTypedDict": ".accounting_customersdeleteop",
    "AccountingCustomersDeleteRequest": ".accounting_customersdeleteop",
    "AccountingCustomersDeleteRequestTypedDict": ".accounting_customersdeleteop",
    "AccountingCustomersDeleteResponse": ".accounting_customersdeleteop",
    "AccountingCustomersDeleteResponseTypedDict": ".accounting_customersdeleteop",
    "AccountingCustomersOneGlobals": ".accounting_customersoneop",
    "AccountingCustomersOneGlobalsTypedDict": ".accounting_customersoneop",
    "AccountingCustomersOneRequest": ".accounting_customersoneop",
    "AccountingCustomersOneRequestTypedDict": ".accounting_customersoneop",
    "AccountingCustomersOneResponse": ".accounting_customersoneop",
    "AccountingCustomersOneResponseTypedDict": ".accounting_customersoneop",
    "AccountingCustomersUpdateGlobals": ".accounting_customersupdateop",
    "AccountingCustomersUpdateGlobalsTypedDict": ".accounting_customersupdateop",
    "AccountingCustomersUpdateRequest": ".accounting_customersupdateop",
    "AccountingCustomersUpdateRequestTypedDict": ".accounting_customersupdateop",
    "AccountingCustomersUpdateResponse": ".accounting_customersupdateop",
    "AccountingCustomersUpdateResponseTypedDict": ".accounting_customersupdateop",
    "AccountingDepartmentsAddGlobals": ".accounting_departmentsaddop",
    "AccountingDepartmentsAddGlobalsTypedDict": ".accounting_departmentsaddop",
    "AccountingDepartmentsAddRequest": ".accounting_departmentsaddop",
    "AccountingDepartmentsAddRequestTypedDict": ".accounting_departmentsaddop",
    "AccountingDepartmentsAddResponse": ".accounting_departmentsaddop",
    "AccountingDepartmentsAddResponseTypedDict": ".accounting_departmentsaddop",
    "AccountingDepartmentsAllGlobals": ".accounting_departmentsallop",
    "AccountingDepartmentsAllGlobalsTypedDict": ".accounting_departmentsallop",
    "AccountingDepartmentsAllRequest": ".accounting_departmentsallop",
    "AccountingDepartmentsAllRequestTypedDict": ".accounting_departmentsallop",
    "AccountingDepartmentsAllResponse": ".accounting_departmentsallop",
    "AccountingDepartmentsAllResponseTypedDict": ".accounting_departmentsallop",
    "AccountingDepartmentsDeleteGlobals": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsDeleteGlobalsTypedDict": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsDeleteRequest": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsDeleteRequestTypedDict": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsDeleteResponse": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsDeleteResponseTypedDict": ".accounting_departmentsdeleteop",
    "AccountingDepartmentsOneGlobals": ".accounting_departmentsoneop",
    "AccountingDepartmentsOneGlobalsTypedDict": ".accounting_departmentsoneop",
    "AccountingDepartmentsOneRequest": ".accounting_departmentsoneop",
    "AccountingDepartmentsOneRequestTypedDict": ".accounting_departmentsoneop",
    "AccountingDepartmentsOneResponse": ".accounting_departmentsoneop",
    "AccountingDepartmentsOneResponseTypedDict": ".accounting_departmentsoneop",
    "AccountingDepartmentsUpdateGlobals": ".accounting_departmentsupdateop",
    "AccountingDepartmentsUpdateGlobalsTypedDict": ".accounting_departmentsupdateop",
    "AccountingDepartmentsUpdateRequest": ".accounting_departmentsupdateop",
    "AccountingDepartmentsUpdateRequestTypedDict": ".accounting_departmentsupdateop",
    "AccountingDepartmentsUpdateResponse": ".accounting_departmentsupdateop",
    "AccountingDepartmentsUpdateResponseTypedDict": ".accounting_departmentsupdateop",
    "AccountingExpensesAddGlobals": ".accounting_expensesaddop",
    "AccountingExpensesAddGlobalsTypedDict": ".accounting_expensesaddop",
    "AccountingExpensesAddRequest": ".accounting_expensesaddop",
    "AccountingExpensesAddRequestTypedDict": ".accounting_expensesaddop",
    "AccountingExpensesAddResponse": ".accounting_expensesaddop",
    "AccountingExpensesAddResponseTypedDict": ".accounting_expensesaddop",
    "AccountingExpensesAllGlobals": ".accounting_expensesallop",
    "AccountingExpensesAllGlobalsTypedDict": ".accounting_expensesallop",
    "AccountingExpensesAllRequest": ".accounting_expensesallop",
    "AccountingExpensesAllRequestTypedDict": ".accounting_expensesallop",
    "AccountingExpensesAllResponse": ".accounting_expensesallop",
    "AccountingExpensesAllResponseTypedDict": ".accounting_expensesallop",
    "AccountingExpensesDeleteGlobals": ".accounting_expensesdeleteop",
    "AccountingExpensesDeleteGlobalsTypedDict": ".accounting_expensesdeleteop",
    "AccountingExpensesDeleteRequest": ".accounting_expensesdeleteop",
    "AccountingExpensesDeleteRequestTypedDict": ".accounting_expensesdeleteop",
    "AccountingExpensesDeleteResponse": ".accounting_expensesdeleteop",
    "AccountingExpensesDeleteResponseTypedDict": ".accounting_expensesdeleteop",
    "AccountingExpensesOneGlobals": ".accounting_expensesoneop",
    "AccountingExpensesOneGlobalsTypedDict": ".accounting_expensesoneop",
    "AccountingExpensesOneRequest": ".accounting_expensesoneop",
    "AccountingExpensesOneRequestTypedDict": ".accounting_expensesoneop",
    "AccountingExpensesOneResponse": ".accounting_expensesoneop",
    "AccountingExpensesOneResponseTypedDict": ".accounting_expensesoneop",
    "AccountingExpensesUpdateGlobals": ".accounting_expensesupdateop",
    "AccountingExpensesUpdateGlobalsTypedDict": ".accounting_expensesupdateop",
    "AccountingExpensesUpdateRequest": ".accounting_expensesupdateop",
    "AccountingExpensesUpdateRequestTypedDict": ".accounting_expensesupdateop",
    "AccountingExpensesUpdateResponse": ".accounting_expensesupdateop",
    "AccountingExpensesUpdateResponseTypedDict": ".accounting_expensesupdateop",
    "AccountingInvoiceItemsAddGlobals": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAddGlobalsTypedDict": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAddRequest": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAddRequestTypedDict": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAddResponse": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAddResponseTypedDict": ".accounting_invoiceitemsaddop",
    "AccountingInvoiceItemsAllGlobals": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsAllGlobalsTypedDict": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsAllRequest": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsAllRequestTypedDict": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsAllResponse": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsAllResponseTypedDict": ".accounting_invoiceitemsallop",
    "AccountingInvoiceItemsDeleteGlobals": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsDeleteGlobalsTypedDict": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsDeleteRequest": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsDeleteRequestTypedDict": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsDeleteResponse": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsDeleteResponseTypedDict": ".accounting_invoiceitemsdeleteop",
    "AccountingInvoiceItemsOneGlobals": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsOneGlobalsTypedDict": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsOneRequest": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsOneRequestTypedDict": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsOneResponse": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsOneResponseTypedDict": ".accounting_invoiceitemsoneop",
    "AccountingInvoiceItemsUpdateGlobals": ".accounting_invoiceitemsupdateop",
    "AccountingInvoiceItemsUpdateGlobalsTypedDict": ".accounting_invoiceitemsupdateop",
    "AccountingInvoiceItemsUpdateRequest": ".accounting_invoiceitemsupdateop",
    "AccountingInvoiceItemsUpdateRequestTypedDict": ".accounting_invoiceitemsupdateop",
    "AccountingInvoiceItemsUpdateResponse": ".accounting_invoiceitemsupdateop",
    "AccountingInvoiceItemsUpdateResponseTypedDict": ".accounting_invoiceitemsupdateop",
    "AccountingInvoicesAddGlobals": ".accounting_invoicesaddop",
    "AccountingInvoicesAddGlobalsTypedDict": ".accounting_invoicesaddop",
    "AccountingInvoicesAddRequest": ".accounting_invoicesaddop",
    "AccountingInvoicesAddRequestTypedDict": ".accounting_invoicesaddop",
    "AccountingInvoicesAddResponse": ".accounting_invoicesaddop",
    "AccountingInvoicesAddResponseTypedDict": ".accounting_invoicesaddop",
    "AccountingInvoicesAllGlobals": ".accounting_invoicesallop",
    "AccountingInvoicesAllGlobalsTypedDict": ".accounting_invoicesallop",
    "AccountingInvoicesAllRequest": ".accounting_invoicesallop",
    "AccountingInvoicesAllRequestTypedDict": ".accounting_invoicesallop",
    "AccountingInvoicesAllResponse": ".accounting_invoicesallop",
    "AccountingInvoicesAllResponseTypedDict": ".accounting_invoicesallop",
    "AccountingInvoicesDeleteGlobals": ".accounting_invoicesdeleteop",
    "AccountingInvoicesDeleteGlobalsTypedDict": ".accounting_invoicesdeleteop",
    "AccountingInvoicesDeleteRequest": ".accounting_invoicesdeleteop",
    "AccountingInvoicesDeleteRequestTypedDict": ".accounting_invoicesdeleteop",
    "AccountingInvoicesDeleteResponse": ".accounting_invoicesdeleteop",
    "AccountingInvoicesDeleteResponseTypedDict": ".accounting_invoicesdeleteop",
    "AccountingInvoicesOneGlobals": ".accounting_invoicesoneop",
    "AccountingInvoicesOneGlobalsTypedDict": ".accounting_invoicesoneop",
    "AccountingInvoicesOneRequest": ".accounting_invoicesoneop",
    "AccountingInvoicesOneRequestTypedDict": ".accounting_invoicesoneop",
    "AccountingInvoicesOneResponse": ".accounting_invoicesoneop",
    "AccountingInvoicesOneResponseTypedDict": ".accounting_invoicesoneop",
    "AccountingInvoicesUpdateGlobals": ".accounting_invoicesupdateop",
    "AccountingInvoicesUpdateGlobalsTypedDict": ".accounting_invoicesupdateop",
    "AccountingInvoicesUpdateRequest": ".accounting_invoicesupdateop",
    "AccountingInvoicesUpdateRequestTypedDict": ".accounting_invoicesupdateop",
    "AccountingInvoicesUpdateResponse": ".accounting_invoicesupdateop",
    "AccountingInvoicesUpdateResponseTypedDict": ".accounting_invoicesupdateop",
    "AccountingJournalEntriesAddGlobals": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAddGlobalsTypedDict": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAddRequest": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAddRequestTypedDict": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAddResponse": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAddResponseTypedDict": ".accounting_journalentriesaddop",
    "AccountingJournalEntriesAllGlobals": ".accounting_journalentriesallop",
    "AccountingJournalEntriesAllGlobalsTypedDict": ".accounting_journalentriesallop",
    "AccountingJournalEntriesAllRequest": ".accounting_journalentriesallop",
    "AccountingJournalEntriesAllRequestTypedDict": ".accounting_journalentriesallop",
    "AccountingJournalEntriesAllResponse": ".accounting_journalentriesallop",
    "AccountingJournalEntriesAllResponseTypedDict": ".accounting_journalentriesallop",
    "AccountingJournalEntriesDeleteGlobals": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesDeleteGlobalsTypedDict": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesDeleteRequest": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesDeleteRequestTypedDict": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesDeleteResponse": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesDeleteResponseTypedDict": ".accounting_journalentriesdeleteop",
    "AccountingJournalEntriesOneGlobals": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesOneGlobalsTypedDict": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesOneRequest": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesOneRequestTypedDict": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesOneResponse": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesOneResponseTypedDict": ".accounting_journalentriesoneop",
    "AccountingJournalEntriesUpdateGlobals": ".accounting_journalentriesupdateop",
    "AccountingJournalEntriesUpdateGlobalsTypedDict": ".accounting_journalentriesupdateop",
    "AccountingJournalEntriesUpdateRequest": ".accounting_journalentriesupdateop",
    "AccountingJournalEntriesUpdateRequestTypedDict": ".accounting_journalentriesupdateop",
    "AccountingJournalEntriesUpdateResponse": ".accounting_journalentriesupdateop",
    "AccountingJournalEntriesUpdateResponseTypedDict": ".accounting_journalentriesupdateop",
    "AccountingLedgerAccountsAddGlobals": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAddGlobalsTypedDict": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAddRequest": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAddRequestTypedDict": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAddResponse": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAddResponseTypedDict": ".accounting_ledgeraccountsaddop",
    "AccountingLedgerAccountsAllGlobals": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsAllGlobalsTypedDict": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsAllRequest": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsAllRequestTypedDict": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsAllResponse": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsAllResponseTypedDict": ".accounting_ledgeraccountsallop",
    "AccountingLedgerAccountsDeleteGlobals": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsDeleteGlobalsTypedDict": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsDeleteRequest": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsDeleteRequestTypedDict": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsDeleteResponse": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsDeleteResponseTypedDict": ".accounting_ledgeraccountsdeleteop",
    "AccountingLedgerAccountsOneGlobals": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsOneGlobalsTypedDict": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsOneRequest": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsOneRequestTypedDict": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsOneResponse": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsOneResponseTypedDict": ".accounting_ledgeraccountsoneop",
    "AccountingLedgerAccountsUpdateGlobals": ".accounting_ledgeraccountsupdateop",
    "AccountingLedgerAccountsUpdateGlobalsTypedDict": ".accounting_ledgeraccountsupdateop",
    "AccountingLedgerAccountsUpdateRequest": ".accounting_ledgeraccountsupdateop",
    "AccountingLedgerAccountsUpdateRequestTypedDict": ".accounting_ledgeraccountsupdateop",
    "AccountingLedgerAccountsUpdateResponse": ".accounting_ledgeraccountsupdateop",
    "AccountingLedgerAccountsUpdateResponseTypedDict": ".accounting_ledgeraccountsupdateop",
    "AccountingLocationsAddGlobals": ".accounting_locationsaddop",
    "AccountingLocationsAddGlobalsTypedDict": ".accounting_locationsaddop",
    "AccountingLocationsAddRequest": ".accounting_locationsaddop",
    "AccountingLocationsAddRequestTypedDict": ".accounting_locationsaddop",
    "AccountingLocationsAddResponse": ".accounting_locationsaddop",
    "AccountingLocationsAddResponseTypedDict": ".accounting_locationsaddop",
    "AccountingLocationsAllGlobals": ".accounting_locationsallop",
    "AccountingLocationsAllGlobalsTypedDict": ".accounting_locationsallop",
    "AccountingLocationsAllRequest": ".accounting_locationsallop",
    "AccountingLocationsAllRequestTypedDict": ".accounting_locationsallop",
    "AccountingLocationsAllResponse": ".accounting_locationsallop",
    "AccountingLocationsAllResponseTypedDict": ".accounting_locationsallop",
    "AccountingLocationsDeleteGlobals": ".accounting_locationsdeleteop",
    "AccountingLocationsDeleteGlobalsTypedDict": ".accounting_locationsdeleteop",
    "AccountingLocationsDeleteRequest": ".accounting_locationsdeleteop",
    "AccountingLocationsDeleteRequestTypedDict": ".accounting_locationsdeleteop",
    "AccountingLocationsDeleteResponse": ".accounting_locationsdeleteop",
    "AccountingLocationsDeleteResponseTypedDict": ".accounting_locationsdeleteop",
    "AccountingLocationsOneGlobals": ".accounting_locationsoneop",
    "AccountingLocationsOneGlobalsTypedDict": ".accounting_locationsoneop",
    "AccountingLocationsOneRequest": ".accounting_locationsoneop",
    "AccountingLocationsOneRequestTypedDict": ".accounting_locationsoneop",
    "AccountingLocationsOneResponse": ".accounting_locationsoneop",
    "AccountingLocationsOneResponseTypedDict": ".accounting_locationsoneop",
    "AccountingLocationsUpdateGlobals": ".accounting_locationsupdateop",
    "AccountingLocationsUpdateGlobalsTypedDict": ".accounting_locationsupdateop",
    "AccountingLocationsUpdateRequest": ".accounting_locationsupdateop",
    "AccountingLocationsUpdateRequestTypedDict": ".accounting_locationsupdateop",
    "AccountingLocationsUpdateResponse": ".accounting_locationsupdateop",
    "AccountingLocationsUpdateResponseTypedDict": ".accounting_locationsupdateop",
    "AccountingPaymentsAddGlobals": ".accounting_paymentsaddop",
    "AccountingPaymentsAddGlobalsTypedDict": ".accounting_paymentsaddop",
    "AccountingPaymentsAddRequest": ".accounting_paymentsaddop",
    "AccountingPaymentsAddRequestTypedDict": ".accounting_paymentsaddop",
    "AccountingPaymentsAddResponse": ".accounting_paymentsaddop",
    "AccountingPaymentsAddResponseTypedDict": ".accounting_paymentsaddop",
    "AccountingPaymentsAllGlobals": ".accounting_paymentsallop",
    "AccountingPaymentsAllGlobalsTypedDict": ".accounting_paymentsallop",
    "AccountingPaymentsAllRequest": ".accounting_paymentsallop",
    "AccountingPaymentsAllRequestTypedDict": ".accounting_paymentsallop",
    "AccountingPaymentsAllResponse": ".accounting_paymentsallop",
    "AccountingPaymentsAllResponseTypedDict": ".accounting_paymentsallop",
    "AccountingPaymentsDeleteGlobals": ".accounting_paymentsdeleteop",
    "AccountingPaymentsDeleteGlobalsTypedDict": ".accounting_paymentsdeleteop",
    "AccountingPaymentsDeleteRequest": ".accounting_paymentsdeleteop",
    "AccountingPaymentsDeleteRequestTypedDict": ".accounting_paymentsdeleteop",
    "AccountingPaymentsDeleteResponse": ".accounting_paymentsdeleteop",
    "AccountingPaymentsDeleteResponseTypedDict": ".accounting_paymentsdeleteop",
    "AccountingPaymentsOneGlobals": ".accounting_paymentsoneop",
    "AccountingPaymentsOneGlobalsTypedDict": ".accounting_paymentsoneop",
    "AccountingPaymentsOneRequest": ".accounting_paymentsoneop",
    "AccountingPaymentsOneRequestTypedDict": ".accounting_paymentsoneop",
    "AccountingPaymentsOneResponse": ".accounting_paymentsoneop",
    "AccountingPaymentsOneResponseTypedDict": ".accounting_paymentsoneop",
    "AccountingPaymentsUpdateGlobals": ".accounting_paymentsupdateop",
    "AccountingPaymentsUpdateGlobalsTypedDict": ".accounting_paymentsupdateop",
    "AccountingPaymentsUpdateRequest": ".accounting_paymentsupdateop",
    "AccountingPaymentsUpdateRequestTypedDict": ".accounting_paymentsupdateop",
    "AccountingPaymentsUpdateResponse": ".accounting_paymentsupdateop",
    "AccountingPaymentsUpdateResponseTypedDict": ".accounting_paymentsupdateop",
    "AccountingProfitAndLossOneGlobals": ".accounting_profitandlossoneop",
    "AccountingProfitAndLossOneGlobalsTypedDict": ".accounting_profitandlossoneop",
    "AccountingProfitAndLossOneRequest": ".accounting_profitandlossoneop",
    "AccountingProfitAndLossOneRequestTypedDict": ".accounting_profitandlossoneop",
    "AccountingProfitAndLossOneResponse": ".accounting_profitandlossoneop",
    "AccountingProfitAndLossOneResponseTypedDict": ".accounting_profitandlossoneop",
    "AccountingPurchaseOrdersAddGlobals": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAddGlobalsTypedDict": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAddRequest": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAddRequestTypedDict": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAddResponse": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAddResponseTypedDict": ".accounting_purchaseordersaddop",
    "AccountingPurchaseOrdersAllGlobals": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersAllGlobalsTypedDict": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersAllRequest": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersAllRequestTypedDict": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersAllResponse": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersAllResponseTypedDict": ".accounting_purchaseordersallop",
    "AccountingPurchaseOrdersDeleteGlobals": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersDeleteGlobalsTypedDict": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersDeleteRequest": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersDeleteRequestTypedDict": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersDeleteResponse": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersDeleteResponseTypedDict": ".accounting_purchaseordersdeleteop",
    "AccountingPurchaseOrdersOneGlobals": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersOneGlobalsTypedDict": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersOneRequest": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersOneRequestTypedDict": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersOneResponse": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersOneResponseTypedDict": ".accounting_purchaseordersoneop",
    "AccountingPurchaseOrdersUpdateGlobals": ".accounting_purchaseordersupdateop",
    "AccountingPurchaseOrdersUpdateGlobalsTypedDict": ".accounting_purchaseordersupdateop",
    "AccountingPurchaseOrdersUpdateRequest": ".accounting_purchaseordersupdateop",
    "AccountingPurchaseOrdersUpdateRequestTypedDict": ".accounting_purchaseordersupdateop",
    "AccountingPurchaseOrdersUpdateResponse": ".accounting_purchaseordersupdateop",
    "AccountingPurchaseOrdersUpdateResponseTypedDict": ".accounting_purchaseordersupdateop",
    "AccountingQuotesAddGlobals": ".accounting_quotesaddop",
    "AccountingQuotesAddGlobalsTypedDict": ".accounting_quotesaddop",
    "AccountingQuotesAddRequest": ".accounting_quotesaddop",
    "AccountingQuotesAddRequestTypedDict": ".accounting_quotesaddop",
    "AccountingQuotesAddResponse": ".accounting_quotesaddop",
    "AccountingQuotesAddResponseTypedDict": ".accounting_quotesaddop",
    "AccountingQuotesAllGlobals": ".accounting_quotesallop",
    "AccountingQuotesAllGlobalsTypedDict": ".accounting_quotesallop",
    "AccountingQuotesAllRequest": ".accounting_quotesallop",
    "AccountingQuotesAllRequestTypedDict": ".accounting_quotesallop",
    "AccountingQuotesAllResponse": ".accounting_quotesallop",
    "AccountingQuotesAllResponseTypedDict": ".accounting_quotesallop",
    "AccountingQuotesDeleteGlobals": ".accounting_quotesdeleteop",
    "AccountingQuotesDeleteGlobalsTypedDict": ".accounting_quotesdeleteop",
    "AccountingQuotesDeleteRequest": ".accounting_quotesdeleteop",
    "AccountingQuotesDeleteRequestTypedDict": ".accounting_quotesdeleteop",
    "AccountingQuotesDeleteResponse": ".accounting_quotesdeleteop",
    "AccountingQuotesDeleteResponseTypedDict": ".accounting_quotesdeleteop",
    "AccountingQuotesOneGlobals": ".accounting_quotesoneop",
    "AccountingQuotesOneGlobalsTypedDict": ".accounting_quotesoneop",
    "AccountingQuotesOneRequest": ".accounting_quotesoneop",
    "AccountingQuotesOneRequestTypedDict": ".accounting_quotesoneop",
    "AccountingQuotesOneResponse": ".accounting_quotesoneop",
    "AccountingQuotesOneResponseTypedDict": ".accounting_quotesoneop",
    "AccountingQuotesUpdateGlobals": ".accounting_quotesupdateop",
    "AccountingQuotesUpdateGlobalsTypedDict": ".accounting_quotesupdateop",
    "AccountingQuotesUpdateRequest": ".accounting_quotesupdateop",
    "AccountingQuotesUpdateRequestTypedDict": ".accounting_quotesupdateop",
    "AccountingQuotesUpdateResponse": ".accounting_quotesupdateop",
    "AccountingQuotesUpdateResponseTypedDict": ".accounting_quotesupdateop",
    "AccountingSubsidiariesAddGlobals": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAddGlobalsTypedDict": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAddRequest": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAddRequestTypedDict": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAddResponse": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAddResponseTypedDict": ".accounting_subsidiariesaddop",
    "AccountingSubsidiariesAllGlobals": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesAllGlobalsTypedDict": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesAllRequest": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesAllRequestTypedDict": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesAllResponse": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesAllResponseTypedDict": ".accounting_subsidiariesallop",
    "AccountingSubsidiariesDeleteGlobals": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesDeleteGlobalsTypedDict": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesDeleteRequest": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesDeleteRequestTypedDict": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesDeleteResponse": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesDeleteResponseTypedDict": ".accounting_subsidiariesdeleteop",
    "AccountingSubsidiariesOneGlobals": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesOneGlobalsTypedDict": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesOneRequest": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesOneRequestTypedDict": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesOneResponse": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesOneResponseTypedDict": ".accounting_subsidiariesoneop",
    "AccountingSubsidiariesUpdateGlobals": ".accounting_subsidiariesupdateop",
    "AccountingSubsidiariesUpdateGlobalsTypedDict": ".accounting_subsidiariesupdateop",
    "AccountingSubsidiariesUpdateRequest": ".accounting_subsidiariesupdateop",
    "AccountingSubsidiariesUpdateRequestTypedDict": ".accounting_subsidiariesupdateop",
    "AccountingSubsidiariesUpdateResponse": ".accounting_subsidiariesupdateop",
    "AccountingSubsidiariesUpdateResponseTypedDict": ".accounting_subsidiariesupdateop",
    "AccountingSuppliersAddGlobals": ".accounting_suppliersaddop",
    "AccountingSuppliersAddGlobalsTypedDict": ".accounting_suppliersaddop",
    "AccountingSuppliersAddRequest": ".accounting_suppliersaddop",
    "AccountingSuppliersAddRequestTypedDict": ".accounting_suppliersaddop",
    "AccountingSuppliersAddResponse": ".accounting_suppliersaddop",
    "AccountingSuppliersAddResponseTypedDict": ".accounting_suppliersaddop",
    "AccountingSuppliersAllGlobals": ".accounting_suppliersallop",
    "AccountingSuppliersAllGlobalsTypedDict": ".accounting_suppliersallop",
    "AccountingSuppliersAllRequest": ".accounting_suppliersallop",
    "AccountingSuppliersAllRequestTypedDict": ".accounting_suppliersallop",
    "AccountingSuppliersAllResponse": ".accounting_suppliersallop",
    "AccountingSuppliersAllResponseTypedDict": ".accounting_suppliersallop",
    "AccountingSuppliersDeleteGlobals": ".accounting_suppliersdeleteop",
    "AccountingSuppliersDeleteGlobalsTypedDict": ".accounting_suppliersdeleteop",
    "AccountingSuppliersDeleteRequest": ".accounting_suppliersdeleteop",
    "AccountingSuppliersDeleteRequestTypedDict": ".accounting_suppliersdeleteop",
    "AccountingSuppliersDeleteResponse": ".accounting_suppliersdeleteop",
    "AccountingSuppliersDeleteResponseTypedDict": ".accounting_suppliersdeleteop",
    "AccountingSuppliersOneGlobals": ".accounting_suppliersoneop",
    "AccountingSuppliersOneGlobalsTypedDict": ".accounting_suppliersoneop",
    "AccountingSuppliersOneRequest": ".accounting_suppliersoneop",
    "AccountingSuppliersOneRequestTypedDict": ".accounting_suppliersoneop",
    "AccountingSuppliersOneResponse": ".accounting_suppliersoneop",
    "AccountingSuppliersOneResponseTypedDict": ".accounting_suppliersoneop",
    "AccountingSuppliersUpdateGlobals": ".accounting_suppliersupdateop",
    "AccountingSuppliersUpdateGlobalsTypedDict": ".accounting_suppliersupdateop",
    "AccountingSuppliersUpdateRequest": ".accounting_suppliersupdateop",
    "AccountingSuppliersUpdateRequestTypedDict": ".accounting_suppliersupdateop",
    "AccountingSuppliersUpdateResponse": ".accounting_suppliersupdateop",
    "AccountingSuppliersUpdateResponseTypedDict": ".accounting_suppliersupdateop",
    "AccountingTaxRatesAddGlobals": ".accounting_taxratesaddop",
    "AccountingTaxRatesAddGlobalsTypedDict": ".accounting_taxratesaddop",
    "AccountingTaxRatesAddRequest": ".accounting_taxratesaddop",
    "AccountingTaxRatesAddRequestTypedDict": ".accounting_taxratesaddop",
    "AccountingTaxRatesAddResponse": ".accounting_taxratesaddop",
    "AccountingTaxRatesAddResponseTypedDict": ".accounting_taxratesaddop",
    "AccountingTaxRatesAllGlobals": ".accounting_taxratesallop",
    "AccountingTaxRatesAllGlobalsTypedDict": ".accounting_taxratesallop",
    "AccountingTaxRatesAllRequest": ".accounting_taxratesallop",
    "AccountingTaxRatesAllRequestTypedDict": ".accounting_taxratesallop",
    "AccountingTaxRatesAllResponse": ".accounting_taxratesallop",
    "AccountingTaxRatesAllResponseTypedDict": ".accounting_taxratesallop",
    "AccountingTaxRatesDeleteGlobals": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesDeleteGlobalsTypedDict": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesDeleteRequest": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesDeleteRequestTypedDict": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesDeleteResponse": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesDeleteResponseTypedDict": ".accounting_taxratesdeleteop",
    "AccountingTaxRatesOneGlobals": ".accounting_taxratesoneop",
    "AccountingTaxRatesOneGlobalsTypedDict": ".accounting_taxratesoneop",
    "AccountingTaxRatesOneRequest": ".accounting_taxratesoneop",
    "AccountingTaxRatesOneRequestTypedDict": ".accounting_taxratesoneop",
    "AccountingTaxRatesOneResponse": ".accounting_taxratesoneop",
    "AccountingTaxRatesOneResponseTypedDict": ".accounting_taxratesoneop",
    "AccountingTaxRatesUpdateGlobals": ".accounting_taxratesupdateop",
    "AccountingTaxRatesUpdateGlobalsTypedDict": ".accounting_taxratesupdateop",
    "AccountingTaxRatesUpdateRequest": ".accounting_taxratesupdateop",
    "AccountingTaxRatesUpdateRequestTypedDict": ".accounting_taxratesupdateop",
    "AccountingTaxRatesUpdateResponse": ".accounting_taxratesupdateop",
    "AccountingTaxRatesUpdateResponseTypedDict": ".accounting_taxratesupdateop",
    "AccountingTrackingCategoriesAddGlobals": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAddGlobalsTypedDict": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAddRequest": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAddRequestTypedDict": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAddResponse": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAddResponseTypedDict": ".accounting_trackingcategoriesaddop",
    "AccountingTrackingCategoriesAllGlobals": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesAllGlobalsTypedDict": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesAllRequest": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesAllRequestTypedDict": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesAllResponse": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesAllResponseTypedDict": ".accounting_trackingcategoriesallop",
    "AccountingTrackingCategoriesDeleteGlobals": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesDeleteGlobalsTypedDict": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesDeleteRequest": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesDeleteRequestTypedDict": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesDeleteResponse": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesDeleteResponseTypedDict": ".accounting_trackingcategoriesdeleteop",
    "AccountingTrackingCategoriesOneGlobals": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesOneGlobalsTypedDict": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesOneRequest": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesOneRequestTypedDict": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesOneResponse": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesOneResponseTypedDict": ".accounting_trackingcategoriesoneop",
    "AccountingTrackingCategoriesUpdateGlobals": ".accounting_trackingcategoriesupdateop",
    "AccountingTrackingCategoriesUpdateGlobalsTypedDict": ".accounting_trackingcategoriesupdateop",
    "AccountingTrackingCategoriesUpdateRequest": ".accounting_trackingcategoriesupdateop",
    "AccountingTrackingCategoriesUpdateRequestTypedDict": ".accounting_trackingcategoriesupdateop",
    "AccountingTrackingCategoriesUpdateResponse": ".accounting_trackingcategoriesupdateop",
    "AccountingTrackingCategoriesUpdateResponseTypedDict": ".accounting_trackingcategoriesupdateop",
    "AccountingDepartment": ".accountingdepartment",
    "AccountingDepartmentInput": ".accountingdepartment",
    "AccountingDepartmentInputTypedDict": ".accountingdepartment",
    "AccountingDepartmentTypedDict": ".accountingdepartment",
    "DepartmentStatus": ".accountingdepartment",
    "AccountingDepartmentsFilter": ".accountingdepartmentsfilter",
    "AccountingDepartmentsFilterTypedDict": ".accountingdepartmentsfilter",
    "AccountingLocation": ".accountinglocation",
    "AccountingLocationInput": ".accountinglocation",
    "AccountingLocationInputTypedDict": ".accountinglocation",
    "AccountingLocationTypedDict": ".accountinglocation",
    "LocationStatus": ".accountinglocation",
    "AccountingLocationsFilter": ".accountinglocationsfilter",
    "AccountingLocationsFilterTypedDict": ".accountinglocationsfilter",
    "ActivitiesFilter": ".activitiesfilter",
    "ActivitiesFilterTypedDict": ".activitiesfilter",
    "ActivitiesSort": ".activitiessort",
    "ActivitiesSortBy": ".activitiessort",
    "ActivitiesSortTypedDict": ".activitiessort",
    "Activity": ".activity",
    "ActivityInput": ".activity",
    "ActivityInputTypedDict": ".activity",
    "ActivityType": ".activity",
    "ActivityTypedDict": ".activity",
    "ShowAs": ".activity",
    "ActivityAttendee": ".activityattendee",
    "ActivityAttendeeInput": ".activityattendee",
    "ActivityAttendeeInputTypedDict": ".activityattendee",
    "ActivityAttendeeStatus": ".activityattendee",
    "ActivityAttendeeTypedDict": ".activityattendee",
    "Address": ".address",
    "AddressTypedDict": ".address",
    "Type": ".address",
    "AgedCreditors": ".agedcreditors",
    "AgedCreditorsTypedDict": ".agedcreditors",
    "AgedDebtors": ".ageddebtors",
    "AgedDebtorsTypedDict": ".ageddebtors",
    "AgedReportFilter": ".agedreportfilter",
    "AgedReportFilterTypedDict": ".agedreportfilter",
    "Allocation": ".allocation",
    "AllocationInput": ".allocation",
    "AllocationInputTypedDict": ".allocation",
    "AllocationType": ".allocation",
    "AllocationTypedDict": ".allocation",
    "API": ".api",
    "APIType": ".api",
    "APITypedDict": ".api",
    "Resources": ".api",
    "ResourcesTypedDict": ".api",
    "ApideckError": ".apideckerror",
    "APIError": ".apierror",
    "APIResource": ".apiresource",
    "APIResourceTypedDict": ".apiresource",
    "LinkedResources": ".apiresource",
    "LinkedResourcesTypedDict": ".apiresource",
    "Schema": ".apiresource",
    "SchemaTypedDict": ".apiresource",
    "APIResourceCoverage": ".apiresourcecoverage",
    "APIResourceCoverageTypedDict": ".apiresourcecoverage",
    "Coverage": ".apiresourcecoverage",
    "CoverageTypedDict": ".apiresourcecoverage",
    "ApisFilter": ".apisfilter",
    "ApisFilterTypedDict": ".apisfilter",
    "APIStatus": ".apistatus",
    "Applicant": ".applicant",
    "ApplicantGender": ".applicant",
    "ApplicantInput": ".applicant",
    "ApplicantInputTypedDict": ".applicant",
    "ApplicantType": ".applicant",
    "ApplicantTypedDict": ".applicant",
    "SocialLinks": ".applicant",
    "SocialLinksTypedDict": ".applicant",
    "Websites": ".applicant",
    "WebsitesTypedDict": ".applicant",
    "ApplicantsFilter": ".applicantsfilter",
    "ApplicantsFilterTypedDict": ".applicantsfilter",
    "Application": ".application",
    "ApplicationInput": ".application",
    "ApplicationInputTypedDict": ".application",
    "ApplicationStatus": ".application",
    "ApplicationTypedDict": ".application",
    "Stage": ".application",
    "StageTypedDict": ".application",
    "Assignee": ".assignee",
    "AssigneeTypedDict": ".assignee",
    "AssigneeInput": ".assignee_input",
    "AssigneeInputTypedDict": ".assignee_input",
    "AtsApplicantsAddGlobals": ".ats_applicantsaddop",
    "AtsApplicantsAddGlobalsTypedDict": ".ats_applicantsaddop",
    "AtsApplicantsAddRequest": ".ats_applicantsaddop",
    "AtsApplicantsAddRequestTypedDict": ".ats_applicantsaddop",
    "AtsApplicantsAddResponse": ".ats_applicantsaddop",
    "AtsApplicantsAddResponseTypedDict": ".ats_applicantsaddop",
    "AtsApplicantsAllGlobals": ".ats_applicantsallop",
    "AtsApplicantsAllGlobalsTypedDict": ".ats_applicantsallop",
    "AtsApplicantsAllRequest": ".ats_applicantsallop",
    "AtsApplicantsAllRequestTypedDict": ".ats_applicantsallop",
    "AtsApplicantsAllResponse": ".ats_applicantsallop",
    "AtsApplicantsAllResponseTypedDict": ".ats_applicantsallop",
    "AtsApplicantsDeleteGlobals": ".ats_applicantsdeleteop",
    "AtsApplicantsDeleteGlobalsTypedDict": ".ats_applicantsdeleteop",
    "AtsApplicantsDeleteRequest": ".ats_applicantsdeleteop",
    "AtsApplicantsDeleteRequestTypedDict": ".ats_applicantsdeleteop",
    "AtsApplicantsDeleteResponse": ".ats_applicantsdeleteop",
    "AtsApplicantsDeleteResponseTypedDict": ".ats_applicantsdeleteop",
    "AtsApplicantsOneGlobals": ".ats_applicantsoneop",
    "AtsApplicantsOneGlobalsTypedDict": ".ats_applicantsoneop",
    "AtsApplicantsOneRequest": ".ats_applicantsoneop",
    "AtsApplicantsOneRequestTypedDict": ".ats_applicantsoneop",
    "AtsApplicantsOneResponse": ".ats_applicantsoneop",
    "AtsApplicantsOneResponseTypedDict": ".ats_applicantsoneop",
    "AtsApplicantsUpdateGlobals": ".ats_applicantsupdateop",
    "AtsApplicantsUpdateGlobalsTypedDict": ".ats_applicantsupdateop",
    "AtsApplicantsUpdateRequest": ".ats_applicantsupdateop",
    "AtsApplicantsUpdateRequestTypedDict": ".ats_applicantsupdateop",
    "AtsApplicantsUpdateResponse": ".ats_applicantsupdateop",
    "AtsApplicantsUpdateResponseTypedDict": ".ats_applicantsupdateop",
    "AtsApplicationsAddGlobals": ".ats_applicationsaddop",
    "AtsApplicationsAddGlobalsTypedDict": ".ats_applicationsaddop",
    "AtsApplicationsAddRequest": ".ats_applicationsaddop",
    "AtsApplicationsAddRequestTypedDict": ".ats_applicationsaddop",
    "AtsApplicationsAddResponse": ".ats_applicationsaddop",
    "AtsApplicationsAddResponseTypedDict": ".ats_applicationsaddop",
    "AtsApplicationsAllGlobals": ".ats_applicationsallop",
    "AtsApplicationsAllGlobalsTypedDict": ".ats_applicationsallop",
    "AtsApplicationsAllRequest": ".ats_applicationsallop",
    "AtsApplicationsAllRequestTypedDict": ".ats_applicationsallop",
    "AtsApplicationsAllResponse": ".ats_applicationsallop",
    "AtsApplicationsAllResponseTypedDict": ".ats_applicationsallop",
    "AtsApplicationsDeleteGlobals": ".ats_applicationsdeleteop",
    "AtsApplicationsDeleteGlobalsTypedDict": ".ats_applicationsdeleteop",
    "AtsApplicationsDeleteRequest": ".ats_applicationsdeleteop",
    "AtsApplicationsDeleteRequestTypedDict": ".ats_applicationsdeleteop",
    "AtsApplicationsDeleteResponse": ".ats_applicationsdeleteop",
    "AtsApplicationsDeleteResponseTypedDict": ".ats_applicationsdeleteop",
    "AtsApplicationsOneGlobals": ".ats_applicationsoneop",
    "AtsApplicationsOneGlobalsTypedDict": ".ats_applicationsoneop",
    "AtsApplicationsOneRequest": ".ats_applicationsoneop",
    "AtsApplicationsOneRequestTypedDict": ".ats_applicationsoneop",
    "AtsApplicationsOneResponse": ".ats_applicationsoneop",
    "AtsApplicationsOneResponseTypedDict": ".ats_applicationsoneop",
    "AtsApplicationsUpdateGlobals": ".ats_applicationsupdateop",
    "AtsApplicationsUpdateGlobalsTypedDict": ".ats_applicationsupdateop",
    "AtsApplicationsUpdateRequest": ".ats_applicationsupdateop",
    "AtsApplicationsUpdateRequestTypedDict": ".ats_applicationsupdateop",
    "AtsApplicationsUpdateResponse": ".ats_applicationsupdateop",
    "AtsApplicationsUpdateResponseTypedDict": ".ats_applicationsupdateop",
    "AtsJobsAllGlobals": ".ats_jobsallop",
    "AtsJobsAllGlobalsTypedDict": ".ats_jobsallop",
    "AtsJobsAllRequest": ".ats_jobsallop",
    "AtsJobsAllRequestTypedDict": ".ats_jobsallop",
    "AtsJobsAllResponse": ".ats_jobsallop",
    "AtsJobsAllResponseTypedDict": ".ats_jobsallop",
    "AtsJobsOneGlobals": ".ats_jobsoneop",
    "AtsJobsOneGlobalsTypedDict": ".ats_jobsoneop",
    "AtsJobsOneRequest": ".ats_jobsoneop",
    "AtsJobsOneRequestTypedDict": ".ats_jobsoneop",
    "AtsJobsOneResponse": ".ats_jobsoneop",
    "AtsJobsOneResponseTypedDict": ".ats_jobsoneop",
    "Attachment": ".attachment",
    "AttachmentTypedDict": ".attachment",
    "AttachmentReference": ".attachmentreference",
    "AttachmentReferenceTypedDict": ".attachmentreference",
    "AttachmentReferenceType": ".attachmentreferencetype",
    "AuthType": ".authtype",
    "BadRequestResponse": ".badrequestresponse",
    "BadRequestResponseData": ".badrequestresponse",
    "BadRequestResponseDetail": ".badrequestresponse",
    "BalanceByPeriod": ".balancebyperiod",
    "BalanceByPeriodTypedDict": ".balancebyperiod",
    "BalanceByTransaction": ".balancebytransaction",
    "BalanceByTransactionTypedDict": ".balancebytransaction",
    "TransactionType": ".balancebytransaction",
    "BalanceSheet": ".balancesheet",
    "BalanceSheetAssetsAccount": ".balancesheet",
    "BalanceSheetAssetsAccountTypedDict": ".balancesheet",
    "BalanceSheetEquityAccount": ".balancesheet",
    "BalanceSheetEquityAccountTypedDict": ".balancesheet",
    "BalanceSheetLiabilitiesAccount": ".balancesheet",
    "BalanceSheetLiabilitiesAccountTypedDict": ".balancesheet",
    "BalanceSheetTypedDict": ".balancesheet",
    "BalanceSheetUncategorizedItemsAccount": ".balancesheet",
    "BalanceSheetUncategorizedItemsAccountTypedDict": ".balancesheet",
    "Reports": ".balancesheet",
    "ReportsTypedDict": ".balancesheet",
    "BalanceSheetFilter": ".balancesheetfilter",
    "BalanceSheetFilterTypedDict": ".balancesheetfilter",
    "PeriodType": ".balancesheetfilter",
    "AccountType": ".bankaccount",
    "BankAccount": ".bankaccount",
    "BankAccountTypedDict": ".bankaccount",
    "BankAccount1": ".bankaccount1",
    "BankAccount1AccountType": ".bankaccount1",
    "BankAccount1TypedDict": ".bankaccount1",
    "BankAccountType": ".bankfeedaccount",
    "BankFeedAccount": ".bankfeedaccount",
    "BankFeedAccountInput": ".bankfeedaccount",
    "BankFeedAccountInputTypedDict": ".bankfeedaccount",
    "BankFeedAccountTypedDict": ".bankfeedaccount",
    "FeedStatus": ".bankfeedaccount",
    "BankFeedStatement": ".bankfeedstatement",
    "BankFeedStatementInput": ".bankfeedstatement",
    "BankFeedStatementInputTypedDict": ".bankfeedstatement",
    "BankFeedStatementTransactionType": ".bankfeedstatement",
    "BankFeedStatementTypedDict": ".bankfeedstatement",
    "StatementStatus": ".bankfeedstatement",
    "Transactions": ".bankfeedstatement",
    "TransactionsTypedDict": ".bankfeedstatement",
    "Benefit": ".benefit",
    "BenefitTypedDict": ".benefit",
    "AmortizationType": ".bill",
    "Bill": ".bill",
    "BillInput": ".bill",
    "BillInputTypedDict": ".bill",
    "BillStatus": ".bill",
    "BillTypedDict": ".bill",
    "BillLineItem": ".billlineitem",
    "BillLineItemInput": ".billlineitem",
    "BillLineItemInputTypedDict": ".billlineitem",
    "BillLineItemType": ".billlineitem",
    "BillLineItemTypedDict": ".billlineitem",
    "Allocations": ".billpayment",
    "AllocationsTypedDict": ".billpayment",
    "BillPayment": ".billpayment",
    "BillPaymentAllocationType": ".billpayment",
    "BillPaymentAllocations": ".billpayment",
    "BillPaymentAllocationsTypedDict": ".billpayment",
    "BillPaymentInput": ".billpayment",
    "BillPaymentInputTypedDict": ".billpayment",
    "BillPaymentType": ".billpayment",
    "BillPaymentTypedDict": ".billpayment",
    "BillsFilter": ".billsfilter",
    "BillsFilterTypedDict": ".billsfilter",
    "BillsSort": ".billssort",
    "BillsSortTypedDict": ".billssort",
    "By": ".billssort",
    "CategoriesFilter": ".categoriesfilter",
    "CategoriesFilterType": ".categoriesfilter",
    "CategoriesFilterTypedDict": ".categoriesfilter",
    "Category": ".category",
    "CategoryStatus": ".category",
    "CategoryType": ".category",
    "CategoryTypedDict": ".category",
    "Collection": ".collection",
    "CollectionTypedDict": ".collection",
    "CollectionsSort": ".collectionssort",
    "CollectionsSortBy": ".collectionssort",
    "CollectionsSortTypedDict": ".collectionssort",
    "CollectionTag": ".collectiontag",
    "CollectionTagTypedDict": ".collectiontag",
    "CollectionTagInput": ".collectiontag_input",
    "CollectionTagInputTypedDict": ".collectiontag_input",
    "CollectionTicketComment": ".collectionticketcomment",
    "CollectionTicketCommentTypedDict": ".collectionticketcomment",
    "CollectionTicketCommentInput": ".collectionticketcomment_input",
    "CollectionTicketCommentInputTypedDict": ".collectionticketcomment_input",
    "CollectionUser": ".collectionuser",
    "CollectionUserTypedDict": ".collectionuser",
    "CommentsSort": ".commentssort",
    "CommentsSortBy": ".commentssort",
    "CommentsSortTypedDict": ".commentssort",
    "CompaniesFilter": ".companiesfilter",
    "CompaniesFilterTypedDict": ".companiesfilter",
    "CompaniesSort": ".companiessort",
    "CompaniesSortBy": ".companiessort",
    "CompaniesSortTypedDict": ".companiessort",
    "Company1": ".company1",
    "Company1Input": ".company1",
    "Company1InputTypedDict": ".company1",
    "Company1TypedDict": ".company1",
    "CompanyRowType": ".company1",
    "CompanyRowTypeTypedDict": ".company1",
    "CompanyInfo": ".companyinfo",
    "CompanyInfoTypedDict": ".companyinfo",
    "CompanyStatus": ".companyinfo",
    "TheStartMonthOfFiscalYear": ".companyinfo",
    "Compensation": ".compensation",
    "CompensationTypedDict": ".compensation",
    "Configuration": ".connection",
    "ConfigurationTypedDict": ".connection",
    "Connection": ".connection",
    "ConnectionConfiguration": ".connection",
    "ConnectionConfigurationTypedDict": ".connection",
    "ConnectionDefaults": ".connection",
    "ConnectionDefaultsTypedDict": ".connection",
    "ConnectionInput": ".connection",
    "ConnectionInputTypedDict": ".connection",
    "ConnectionStatus": ".connection",
    "ConnectionTypedDict": ".connection",
    "ConnectionValue": ".connection",
    "ConnectionValue5": ".connection",
    "ConnectionValue5TypedDict": ".connection",
    "ConnectionValueTypedDict": ".connection",
    "Defaults": ".connection",
    "DefaultsTypedDict": ".connection",
    "Target": ".connection",
    "ConnectionImportData": ".connectionimportdata",
    "ConnectionImportDataTypedDict": ".connectionimportdata",
    "Credentials": ".connectionimportdata",
    "CredentialsTypedDict": ".connectionimportdata",
    "ConnectionState": ".connectionstate",
    "Connector": ".connector",
    "ConnectorAuthType": ".connector",
    "ConnectorOauthGrantType": ".connector",
    "ConnectorOauthScopes": ".connector",
    "ConnectorOauthScopesTypedDict": ".connector",
    "ConnectorTypedDict": ".connector",
    "OauthCredentialsSource": ".connector",
    "OauthScopes": ".connector",
    "OauthScopesTypedDict": ".connector",
    "TLSSupport": ".connector",
    "TLSSupportTypedDict": ".connector",
    "UnifiedApis": ".connector",
    "UnifiedApisTypedDict": ".connector",
    "ConnectorAPIResourceCoverageOneGlobals": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourceCoverageOneGlobalsTypedDict": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourceCoverageOneRequest": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourceCoverageOneRequestTypedDict": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourceCoverageOneResponse": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourceCoverageOneResponseTypedDict": ".connector_apiresourcecoverageoneop",
    "ConnectorAPIResourcesOneGlobals": ".connector_apiresourcesoneop",
    "ConnectorAPIResourcesOneGlobalsTypedDict": ".connector_apiresourcesoneop",
    "ConnectorAPIResourcesOneRequest": ".connector_apiresourcesoneop",
    "ConnectorAPIResourcesOneRequestTypedDict": ".connector_apiresourcesoneop",
    "ConnectorAPIResourcesOneResponse": ".connector_apiresourcesoneop",
    "ConnectorAPIResourcesOneResponseTypedDict": ".connector_apiresourcesoneop",
    "ConnectorApisAllGlobals": ".connector_apisallop",
    "ConnectorApisAllGlobalsTypedDict": ".connector_apisallop",
    "ConnectorApisAllRequest": ".connector_apisallop",
    "ConnectorApisAllRequestTypedDict": ".connector_apisallop",
    "ConnectorApisAllResponse": ".connector_apisallop",
    "ConnectorApisAllResponseTypedDict": ".connector_apisallop",
    "ConnectorApisOneGlobals": ".connector_apisoneop",
    "ConnectorApisOneGlobalsTypedDict": ".connector_apisoneop",
    "ConnectorApisOneRequest": ".connector_apisoneop",
    "ConnectorApisOneRequestTypedDict": ".connector_apisoneop",
    "ConnectorApisOneResponse": ".connector_apisoneop",
    "ConnectorApisOneResponseTypedDict": ".connector_apisoneop",
    "ConnectorConnectorDocsOneGlobals": ".connector_connectordocsoneop",
    "ConnectorConnectorDocsOneGlobalsTypedDict": ".connector_connectordocsoneop",
    "ConnectorConnectorDocsOneRequest": ".connector_connectordocsoneop",
    "ConnectorConnectorDocsOneRequestTypedDict": ".connector_connectordocsoneop",
    "ConnectorConnectorDocsOneResponse": ".connector_connectordocsoneop",
    "ConnectorConnectorDocsOneResponseTypedDict": ".connector_connectordocsoneop",
    "ConnectorConnectorResourcesOneGlobals": ".connector_connectorresourcesoneop",
    "ConnectorConnectorResourcesOneGlobalsTypedDict": ".connector_connectorresourcesoneop",
    "ConnectorConnectorResourcesOneRequest": ".connector_connectorresourcesoneop",
    "ConnectorConnectorResourcesOneRequestTypedDict": ".connector_connectorresourcesoneop",
    "ConnectorConnectorResourcesOneResponse": ".connector_connectorresourcesoneop",
    "ConnectorConnectorResourcesOneResponseTypedDict": ".connector_connectorresourcesoneop",
    "ConnectorConnectorsAllGlobals": ".connector_connectorsallop",
    "ConnectorConnectorsAllGlobalsTypedDict": ".connector_connectorsallop",
    "ConnectorConnectorsAllRequest": ".connector_connectorsallop",
    "ConnectorConnectorsAllRequestTypedDict": ".connector_connectorsallop",
    "ConnectorConnectorsAllResponse": ".connector_connectorsallop",
    "ConnectorConnectorsAllResponseTypedDict": ".connector_connectorsallop",
    "ConnectorConnectorsOneGlobals": ".connector_connectorsoneop",
    "ConnectorConnectorsOneGlobalsTypedDict": ".connector_connectorsoneop",
    "ConnectorConnectorsOneRequest": ".connector_connectorsoneop",
    "ConnectorConnectorsOneRequestTypedDict": ".connector_connectorsoneop",
    "ConnectorConnectorsOneResponse": ".connector_connectorsoneop",
    "ConnectorConnectorsOneResponseTypedDict": ".connector_connectorsoneop",
    "Audience": ".connectordoc",
    "ConnectorDoc": ".connectordoc",
    "ConnectorDocTypedDict": ".connectordoc",
    "Format": ".connectordoc",
    "ConnectorEvent": ".connectorevent",
    "ConnectorEventTypedDict": ".connectorevent",
    "EventSource": ".connectorevent",
    "ConnectorResource": ".connectorresource",
    "ConnectorResourceTypedDict": ".connectorresource",
    "ConnectorSetting": ".connectorsetting",
    "ConnectorSettingType": ".connectorsetting",
    "ConnectorSettingTypedDict": ".connectorsetting",
    "ConnectorsFilter": ".connectorsfilter",
    "ConnectorsFilterTypedDict": ".connectorsfilter",
    "ConnectorStatus": ".connectorstatus",
    "Consumer": ".consumer",
    "ConsumerTypedDict": ".consumer",
    "ConsumerConnection": ".consumerconnection",
    "ConsumerConnectionTypedDict": ".consumerconnection",
    "ConsumerMetadata": ".consumermetadata",
    "ConsumerMetadataTypedDict": ".consumermetadata",
    "ConsumerRequestCountsInDateRangeResponse": ".consumerrequestcountsindaterangeresponse",
    "ConsumerRequestCountsInDateRangeResponseData": ".consumerrequestcountsindaterangeresponse",
    "ConsumerRequestCountsInDateRangeResponseDataTypedDict": ".consumerrequestcountsindaterangeresponse",
    "ConsumerRequestCountsInDateRangeResponseTypedDict": ".consumerrequestcountsindaterangeresponse",
    "Contact": ".contact",
    "ContactGender": ".contact",
    "ContactInput": ".contact",
    "ContactInputTypedDict": ".contact",
    "ContactType": ".contact",
    "ContactTypedDict": ".contact",
    "ContactsFilter": ".contactsfilter",
    "ContactsFilterTypedDict": ".contactsfilter",
    "ContactsSort": ".contactssort",
    "ContactsSortBy": ".contactssort",
    "ContactsSortTypedDict": ".contactssort",
    "CopyFolderRequest": ".copyfolderrequest",
    "CopyFolderRequestTypedDict": ".copyfolderrequest",
    "CreateAccountingDepartmentResponse": ".createaccountingdepartmentresponse",
    "CreateAccountingDepartmentResponseTypedDict": ".createaccountingdepartmentresponse",
    "CreateAccountingLocationResponse": ".createaccountinglocationresponse",
    "CreateAccountingLocationResponseTypedDict": ".createaccountinglocationresponse",
    "CreateActivityResponse": ".createactivityresponse",
    "CreateActivityResponseTypedDict": ".createactivityresponse",
    "CreateApplicantResponse": ".createapplicantresponse",
    "CreateApplicantResponseTypedDict": ".createapplicantresponse",
    "CreateApplicationResponse": ".createapplicationresponse",
    "CreateApplicationResponseTypedDict": ".createapplicationresponse",
    "CreateAttachmentResponse": ".createattachmentresponse",
    "CreateAttachmentResponseTypedDict": ".createattachmentresponse",
    "CreateBankFeedAccountResponse": ".createbankfeedaccountresponse",
    "CreateBankFeedAccountResponseTypedDict": ".createbankfeedaccountresponse",
    "CreateBankFeedStatementResponse": ".createbankfeedstatementresponse",
    "CreateBankFeedStatementResponseTypedDict": ".createbankfeedstatementresponse",
    "CreateBillPaymentResponse": ".createbillpaymentresponse",
    "CreateBillPaymentResponseTypedDict": ".createbillpaymentresponse",
    "CreateBillResponse": ".createbillresponse",
    "CreateBillResponseTypedDict": ".createbillresponse",
    "CreateCallbackState": ".createcallbackstate",
    "CreateCallbackStateTypedDict": ".createcallbackstate",
    "CreateCallbackStateResponse": ".createcallbackstateresponse",
    "CreateCallbackStateResponseData": ".createcallbackstateresponse",
    "CreateCallbackStateResponseDataTypedDict": ".createcallbackstateresponse",
    "CreateCallbackStateResponseTypedDict": ".createcallbackstateresponse",
    "CreateCommentResponse": ".createcommentresponse",
    "CreateCommentResponseTypedDict": ".createcommentresponse",
    "CreateCompanyResponse": ".createcompanyresponse",
    "CreateCompanyResponseTypedDict": ".createcompanyresponse",
    "CreateConnectionResponse": ".createconnectionresponse",
    "CreateConnectionResponseTypedDict": ".createconnectionresponse",
    "CreateConsumerRequest": ".createconsumerrequest",
    "CreateConsumerRequestTypedDict": ".createconsumerrequest",
    "CreateConsumerResponse": ".createconsumerresponse",
    "CreateConsumerResponseTypedDict": ".createconsumerresponse",
    "CreateContactResponse": ".createcontactresponse",
    "CreateContactResponseTypedDict": ".createcontactresponse",
    "CreateCreditNoteResponse": ".createcreditnoteresponse",
    "CreateCreditNoteResponseTypedDict": ".createcreditnoteresponse",
    "CreateCustomerResponse": ".createcustomerresponse",
    "CreateCustomerResponseTypedDict": ".createcustomerresponse",
    "CreateCustomObjectResponse": ".createcustomobjectresponse",
    "CreateCustomObjectResponseTypedDict": ".createcustomobjectresponse",
    "CreateCustomObjectSchemaResponse": ".createcustomobjectschemaresponse",
    "CreateCustomObjectSchemaResponseTypedDict": ".createcustomobjectschemaresponse",
    "CreateDepartmentResponse": ".createdepartmentresponse",
    "CreateDepartmentResponseTypedDict": ".createdepartmentresponse",
    "CreateDriveGroupResponse": ".createdrivegroupresponse",
    "CreateDriveGroupResponseTypedDict": ".createdrivegroupresponse",
    "CreateDriveResponse": ".createdriveresponse",
    "CreateDriveResponseTypedDict": ".createdriveresponse",
    "CreateEmployeeResponse": ".createemployeeresponse",
    "CreateEmployeeResponseTypedDict": ".createemployeeresponse",
    "CreateExpenseResponse": ".createexpenseresponse",
    "CreateExpenseResponseTypedDict": ".createexpenseresponse",
    "CreateFolderRequest": ".createfolderrequest",
    "CreateFolderRequestTypedDict": ".createfolderrequest",
    "CreateFolderResponse": ".createfolderresponse",
    "CreateFolderResponseTypedDict": ".createfolderresponse",
    "CreateHrisCompanyResponse": ".createhriscompanyresponse",
    "CreateHrisCompanyResponseTypedDict": ".createhriscompanyresponse",
    "CreateInvoiceItemResponse": ".createinvoiceitemresponse",
    "CreateInvoiceItemResponseTypedDict": ".createinvoiceitemresponse",
    "CreateInvoiceResponse": ".createinvoiceresponse",
    "CreateInvoiceResponseTypedDict": ".createinvoiceresponse",
    "CreateJournalEntryResponse": ".createjournalentryresponse",
    "CreateJournalEntryResponseTypedDict": ".createjournalentryresponse",
    "CreateLeadResponse": ".createleadresponse",
    "CreateLeadResponseTypedDict": ".createleadresponse",
    "CreateLedgerAccountResponse": ".createledgeraccountresponse",
    "CreateLedgerAccountResponseTypedDict": ".createledgeraccountresponse",
    "CreateMessageResponse": ".createmessageresponse",
    "CreateMessageResponseTypedDict": ".createmessageresponse",
    "CreateNoteResponse": ".createnoteresponse",
    "CreateNoteResponseTypedDict": ".createnoteresponse",
    "CreateOpportunityResponse": ".createopportunityresponse",
    "CreateOpportunityResponseTypedDict": ".createopportunityresponse",
    "CreatePaymentResponse": ".createpaymentresponse",
    "CreatePaymentResponseTypedDict": ".createpaymentresponse",
    "CreatePipelineResponse": ".createpipelineresponse",
    "CreatePipelineResponseTypedDict": ".createpipelineresponse",
    "CreatePurchaseOrderResponse": ".createpurchaseorderresponse",
    "CreatePurchaseOrderResponseTypedDict": ".createpurchaseorderresponse",
    "CreateQuoteResponse": ".createquoteresponse",
    "CreateQuoteResponseTypedDict": ".createquoteresponse",
    "CreateSessionResponse": ".createsessionresponse",
    "CreateSessionResponseData": ".createsessionresponse",
    "CreateSessionResponseDataTypedDict": ".createsessionresponse",
    "CreateSessionResponseTypedDict": ".createsessionresponse",
    "CreateSharedLinkResponse": ".createsharedlinkresponse",
    "CreateSharedLinkResponseTypedDict": ".createsharedlinkresponse",
    "CreateSubsidiaryResponse": ".createsubsidiaryresponse",
    "CreateSubsidiaryResponseTypedDict": ".createsubsidiaryresponse",
    "CreateSupplierResponse": ".createsupplierresponse",
    "CreateSupplierResponseTypedDict": ".createsupplierresponse",
    "CreateTaxRateResponse": ".createtaxrateresponse",
    "CreateTaxRateResponseTypedDict": ".createtaxrateresponse",
    "CreateTicketResponse": ".createticketresponse",
    "CreateTicketResponseTypedDict": ".createticketresponse",
    "CreateTimeOffRequestResponse": ".createtimeoffrequestresponse",
    "CreateTimeOffRequestResponseTypedDict": ".createtimeoffrequestresponse",
    "CreateTrackingCategoryResponse": ".createtrackingcategoryresponse",
    "CreateTrackingCategoryResponseTypedDict": ".createtrackingcategoryresponse",
    "CreateUploadSessionRequest": ".createuploadsessionrequest",
    "CreateUploadSessionRequestTypedDict": ".createuploadsessionrequest",
    "CreateUploadSessionResponse": ".createuploadsessionresponse",
    "CreateUploadSessionResponseTypedDict": ".createuploadsessionresponse",
    "CreateUserResponse": ".createuserresponse",
    "CreateUserResponseTypedDict": ".createuserresponse",
    "CreateWebhookRequest": ".createwebhookrequest",
    "CreateWebhookRequestTypedDict": ".createwebhookrequest",
    "CreateWebhookResponse": ".createwebhookresponse",
    "CreateWebhookResponseTypedDict": ".createwebhookresponse",
    "CreditNote": ".creditnote",
    "CreditNoteInput": ".creditnote",
    "CreditNoteInputTypedDict": ".creditnote",
    "CreditNoteStatus": ".creditnote",
    "CreditNoteType": ".creditnote",
    "CreditNoteTypedDict": ".creditnote",
    "CreditNotesFilter": ".creditnotesfilter",
    "CreditNotesFilterTypedDict": ".creditnotesfilter",
    "CreditNotesSort": ".creditnotessort",
    "CreditNotesSortBy": ".creditnotessort",
    "CreditNotesSortTypedDict": ".creditnotessort",
    "CreditOrDebit": ".creditordebit",
    "CrmActivitiesAddGlobals": ".crm_activitiesaddop",
    "CrmActivitiesAddGlobalsTypedDict": ".crm_activitiesaddop",
    "CrmActivitiesAddRequest": ".crm_activitiesaddop",
    "CrmActivitiesAddRequestTypedDict": ".crm_activitiesaddop",
    "CrmActivitiesAddResponse": ".crm_activitiesaddop",
    "CrmActivitiesAddResponseTypedDict": ".crm_activitiesaddop",
    "CrmActivitiesAllGlobals": ".crm_activitiesallop",
    "CrmActivitiesAllGlobalsTypedDict": ".crm_activitiesallop",
    "CrmActivitiesAllRequest": ".crm_activitiesallop",
    "CrmActivitiesAllRequestTypedDict": ".crm_activitiesallop",
    "CrmActivitiesAllResponse": ".crm_activitiesallop",
    "CrmActivitiesAllResponseTypedDict": ".crm_activitiesallop",
    "CrmActivitiesDeleteGlobals": ".crm_activitiesdeleteop",
    "CrmActivitiesDeleteGlobalsTypedDict": ".crm_activitiesdeleteop",
    "CrmActivitiesDeleteRequest": ".crm_activitiesdeleteop",
    "CrmActivitiesDeleteRequestTypedDict": ".crm_activitiesdeleteop",
    "CrmActivitiesDeleteResponse": ".crm_activitiesdeleteop",
    "CrmActivitiesDeleteResponseTypedDict": ".crm_activitiesdeleteop",
    "CrmActivitiesOneGlobals": ".crm_activitiesoneop",
    "CrmActivitiesOneGlobalsTypedDict": ".crm_activitiesoneop",
    "CrmActivitiesOneRequest": ".crm_activitiesoneop",
    "CrmActivitiesOneRequestTypedDict": ".crm_activitiesoneop",
    "CrmActivitiesOneResponse": ".crm_activitiesoneop",
    "CrmActivitiesOneResponseTypedDict": ".crm_activitiesoneop",
    "CrmActivitiesUpdateGlobals": ".crm_activitiesupdateop",
    "CrmActivitiesUpdateGlobalsTypedDict": ".crm_activitiesupdateop",
    "CrmActivitiesUpdateRequest": ".crm_activitiesupdateop",
    "CrmActivitiesUpdateRequestTypedDict": ".crm_activitiesupdateop",
    "CrmActivitiesUpdateResponse": ".crm_activitiesupdateop",
    "CrmActivitiesUpdateResponseTypedDict": ".crm_activitiesupdateop",
    "CrmCompaniesAddGlobals": ".crm_companiesaddop",
    "CrmCompaniesAddGlobalsTypedDict": ".crm_companiesaddop",
    "CrmCompaniesAddRequest": ".crm_companiesaddop",
    "CrmCompaniesAddRequestTypedDict": ".crm_companiesaddop",
    "CrmCompaniesAddResponse": ".crm_companiesaddop",
    "CrmCompaniesAddResponseTypedDict": ".crm_companiesaddop",
    "CrmCompaniesAllGlobals": ".crm_companiesallop",
    "CrmCompaniesAllGlobalsTypedDict": ".crm_companiesallop",
    "CrmCompaniesAllRequest": ".crm_companiesallop",
    "CrmCompaniesAllRequestTypedDict": ".crm_companiesallop",
    "CrmCompaniesAllResponse": ".crm_companiesallop",
    "CrmCompaniesAllResponseTypedDict": ".crm_companiesallop",
    "CrmCompaniesDeleteGlobals": ".crm_companiesdeleteop",
    "CrmCompaniesDeleteGlobalsTypedDict": ".crm_companiesdeleteop",
    "CrmCompaniesDeleteRequest": ".crm_companiesdeleteop",
    "CrmCompaniesDeleteRequestTypedDict": ".crm_companiesdeleteop",
    "CrmCompaniesDeleteResponse": ".crm_companiesdeleteop",
    "CrmCompaniesDeleteResponseTypedDict": ".crm_companiesdeleteop",
    "CrmCompaniesOneGlobals": ".crm_companiesoneop",
    "CrmCompaniesOneGlobalsTypedDict": ".crm_companiesoneop",
    "CrmCompaniesOneRequest": ".crm_companiesoneop",
    "CrmCompaniesOneRequestTypedDict": ".crm_companiesoneop",
    "CrmCompaniesOneResponse": ".crm_companiesoneop",
    "CrmCompaniesOneResponseTypedDict": ".crm_companiesoneop",
    "CrmCompaniesUpdateGlobals": ".crm_companiesupdateop",
    "CrmCompaniesUpdateGlobalsTypedDict": ".crm_companiesupdateop",
    "CrmCompaniesUpdateRequest": ".crm_companiesupdateop",
    "CrmCompaniesUpdateRequestTypedDict": ".crm_companiesupdateop",
    "CrmCompaniesUpdateResponse": ".crm_companiesupdateop",
    "CrmCompaniesUpdateResponseTypedDict": ".crm_companiesupdateop",
    "CrmContactsAddGlobals": ".crm_contactsaddop",
    "CrmContactsAddGlobalsTypedDict": ".crm_contactsaddop",
    "CrmContactsAddRequest": ".crm_contactsaddop",
    "CrmContactsAddRequestTypedDict": ".crm_contactsaddop",
    "CrmContactsAddResponse": ".crm_contactsaddop",
    "CrmContactsAddResponseTypedDict": ".crm_contactsaddop",
    "CrmContactsAllGlobals": ".crm_contactsallop",
    "CrmContactsAllGlobalsTypedDict": ".crm_contactsallop",
    "CrmContactsAllRequest": ".crm_contactsallop",
    "CrmContactsAllRequestTypedDict": ".crm_contactsallop",
    "CrmContactsAllResponse": ".crm_contactsallop",
    "CrmContactsAllResponseTypedDict": ".crm_contactsallop",
    "CrmContactsDeleteGlobals": ".crm_contactsdeleteop",
    "CrmContactsDeleteGlobalsTypedDict": ".crm_contactsdeleteop",
    "CrmContactsDeleteRequest": ".crm_contactsdeleteop",
    "CrmContactsDeleteRequestTypedDict": ".crm_contactsdeleteop",
    "CrmContactsDeleteResponse": ".crm_contactsdeleteop",
    "CrmContactsDeleteResponseTypedDict": ".crm_contactsdeleteop",
    "CrmContactsOneGlobals": ".crm_contactsoneop",
    "CrmContactsOneGlobalsTypedDict": ".crm_contactsoneop",
    "CrmContactsOneRequest": ".crm_contactsoneop",
    "CrmContactsOneRequestTypedDict": ".crm_contactsoneop",
    "CrmContactsOneResponse": ".crm_contactsoneop",
    "CrmContactsOneResponseTypedDict": ".crm_contactsoneop",
    "CrmContactsUpdateGlobals": ".crm_contactsupdateop",
    "CrmContactsUpdateGlobalsTypedDict": ".crm_contactsupdateop",
    "CrmContactsUpdateRequest": ".crm_contactsupdateop",
    "CrmContactsUpdateRequestTypedDict": ".crm_contactsupdateop",
    "CrmContactsUpdateResponse": ".crm_contactsupdateop",
    "CrmContactsUpdateResponseTypedDict": ".crm_contactsupdateop",
    "CrmCustomObjectsAddGlobals": ".crm_customobjectsaddop",
    "CrmCustomObjectsAddGlobalsTypedDict": ".crm_customobjectsaddop",
    "CrmCustomObjectsAddRequest": ".crm_customobjectsaddop",
    "CrmCustomObjectsAddRequestTypedDict": ".crm_customobjectsaddop",
    "CrmCustomObjectsAddResponse": ".crm_customobjectsaddop",
    "CrmCustomObjectsAddResponseTypedDict": ".crm_customobjectsaddop",
    "CrmCustomObjectsAllGlobals": ".crm_customobjectsallop",
    "CrmCustomObjectsAllGlobalsTypedDict": ".crm_customobjectsallop",
    "CrmCustomObjectsAllRequest": ".crm_customobjectsallop",
    "CrmCustomObjectsAllRequestTypedDict": ".crm_customobjectsallop",
    "CrmCustomObjectsAllResponse": ".crm_customobjectsallop",
    "CrmCustomObjectsAllResponseTypedDict": ".crm_customobjectsallop",
    "CrmCustomObjectSchemasAddGlobals": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAddGlobalsTypedDict": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAddRequest": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAddRequestTypedDict": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAddResponse": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAddResponseTypedDict": ".crm_customobjectschemasaddop",
    "CrmCustomObjectSchemasAllGlobals": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasAllGlobalsTypedDict": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasAllRequest": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasAllRequestTypedDict": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasAllResponse": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasAllResponseTypedDict": ".crm_customobjectschemasallop",
    "CrmCustomObjectSchemasDeleteGlobals": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasDeleteGlobalsTypedDict": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasDeleteRequest": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasDeleteRequestTypedDict": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasDeleteResponse": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasDeleteResponseTypedDict": ".crm_customobjectschemasdeleteop",
    "CrmCustomObjectSchemasOneGlobals": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasOneGlobalsTypedDict": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasOneRequest": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasOneRequestTypedDict": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasOneResponse": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasOneResponseTypedDict": ".crm_customobjectschemasoneop",
    "CrmCustomObjectSchemasUpdateGlobals": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectSchemasUpdateGlobalsTypedDict": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectSchemasUpdateRequest": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectSchemasUpdateRequestTypedDict": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectSchemasUpdateResponse": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectSchemasUpdateResponseTypedDict": ".crm_customobjectschemasupdateop",
    "CrmCustomObjectsDeleteGlobals": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsDeleteGlobalsTypedDict": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsDeleteRequest": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsDeleteRequestTypedDict": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsDeleteResponse": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsDeleteResponseTypedDict": ".crm_customobjectsdeleteop",
    "CrmCustomObjectsOneGlobals": ".crm_customobjectsoneop",
    "CrmCustomObjectsOneGlobalsTypedDict": ".crm_customobjectsoneop",
    "CrmCustomObjectsOneRequest": ".crm_customobjectsoneop",
    "CrmCustomObjectsOneRequestTypedDict": ".crm_customobjectsoneop",
    "CrmCustomObjectsOneResponse": ".crm_customobjectsoneop",
    "CrmCustomObjectsOneResponseTypedDict": ".crm_customobjectsoneop",
    "CrmCustomObjectsUpdateGlobals": ".crm_customobjectsupdateop",
    "CrmCustomObjectsUpdateGlobalsTypedDict": ".crm_customobjectsupdateop",
    "CrmCustomObjectsUpdateRequest": ".crm_customobjectsupdateop",
    "CrmCustomObjectsUpdateRequestTypedDict": ".crm_customobjectsupdateop",
    "CrmCustomObjectsUpdateResponse": ".crm_customobjectsupdateop",
    "CrmCustomObjectsUpdateResponseTypedDict": ".crm_customobjectsupdateop",
    "CrmLeadsAddGlobals": ".crm_leadsaddop",
    "CrmLeadsAddGlobalsTypedDict": ".crm_leadsaddop",
    "CrmLeadsAddRequest": ".crm_leadsaddop",
    "CrmLeadsAddRequestTypedDict": ".crm_leadsaddop",
    "CrmLeadsAddResponse": ".crm_leadsaddop",
    "CrmLeadsAddResponseTypedDict": ".crm_leadsaddop",
    "CrmLeadsAllGlobals": ".crm_leadsallop",
    "CrmLeadsAllGlobalsTypedDict": ".crm_leadsallop",
    "CrmLeadsAllRequest": ".crm_leadsallop",
    "CrmLeadsAllRequestTypedDict": ".crm_leadsallop",
    "CrmLeadsAllResponse": ".crm_leadsallop",
    "CrmLeadsAllResponseTypedDict": ".crm_leadsallop",
    "CrmLeadsDeleteGlobals": ".crm_leadsdeleteop",
    "CrmLeadsDeleteGlobalsTypedDict": ".crm_leadsdeleteop",
    "CrmLeadsDeleteRequest": ".crm_leadsdeleteop",
    "CrmLeadsDeleteRequestTypedDict": ".crm_leadsdeleteop",
    "CrmLeadsDeleteResponse": ".crm_leadsdeleteop",
    "CrmLeadsDeleteResponseTypedDict": ".crm_leadsdeleteop",
    "CrmLeadsOneGlobals": ".crm_leadsoneop",
    "CrmLeadsOneGlobalsTypedDict": ".crm_leadsoneop",
    "CrmLeadsOneRequest": ".crm_leadsoneop",
    "CrmLeadsOneRequestTypedDict": ".crm_leadsoneop",
    "CrmLeadsOneResponse": ".crm_leadsoneop",
    "CrmLeadsOneResponseTypedDict": ".crm_leadsoneop",
    "CrmLeadsUpdateGlobals": ".crm_leadsupdateop",
    "CrmLeadsUpdateGlobalsTypedDict": ".crm_leadsupdateop",
    "CrmLeadsUpdateRequest": ".crm_leadsupdateop",
    "CrmLeadsUpdateRequestTypedDict": ".crm_leadsupdateop",
    "CrmLeadsUpdateResponse": ".crm_leadsupdateop",
    "CrmLeadsUpdateResponseTypedDict": ".crm_leadsupdateop",
    "CrmNotesAddGlobals": ".crm_notesaddop",
    "CrmNotesAddGlobalsTypedDict": ".crm_notesaddop",
    "CrmNotesAddRequest": ".crm_notesaddop",
    "CrmNotesAddRequestTypedDict": ".crm_notesaddop",
    "CrmNotesAddResponse": ".crm_notesaddop",
    "CrmNotesAddResponseTypedDict": ".crm_notesaddop",
    "CrmNotesAllGlobals": ".crm_notesallop",
    "CrmNotesAllGlobalsTypedDict": ".crm_notesallop",
    "CrmNotesAllRequest": ".crm_notesallop",
    "CrmNotesAllRequestTypedDict": ".crm_notesallop",
    "CrmNotesAllResponse": ".crm_notesallop",
    "CrmNotesAllResponseTypedDict": ".crm_notesallop",
    "CrmNotesDeleteGlobals": ".crm_notesdeleteop",
    "CrmNotesDeleteGlobalsTypedDict": ".crm_notesdeleteop",
    "CrmNotesDeleteRequest": ".crm_notesdeleteop",
    "CrmNotesDeleteRequestTypedDict": ".crm_notesdeleteop",
    "CrmNotesDeleteResponse": ".crm_notesdeleteop",
    "CrmNotesDeleteResponseTypedDict": ".crm_notesdeleteop",
    "CrmNotesOneGlobals": ".crm_notesoneop",
    "CrmNotesOneGlobalsTypedDict": ".crm_notesoneop",
    "CrmNotesOneRequest": ".crm_notesoneop",
    "CrmNotesOneRequestTypedDict": ".crm_notesoneop",
    "CrmNotesOneResponse": ".crm_notesoneop",
    "CrmNotesOneResponseTypedDict": ".crm_notesoneop",
    "CrmNotesUpdateGlobals": ".crm_notesupdateop",
    "CrmNotesUpdateGlobalsTypedDict": ".crm_notesupdateop",
    "CrmNotesUpdateRequest": ".crm_notesupdateop",
    "CrmNotesUpdateRequestTypedDict": ".crm_notesupdateop",
    "CrmNotesUpdateResponse": ".crm_notesupdateop",
    "CrmNotesUpdateResponseTypedDict": ".crm_notesupdateop",
    "CrmOpportunitiesAddGlobals": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAddGlobalsTypedDict": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAddRequest": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAddRequestTypedDict": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAddResponse": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAddResponseTypedDict": ".crm_opportunitiesaddop",
    "CrmOpportunitiesAllGlobals": ".crm_opportunitiesallop",
    "CrmOpportunitiesAllGlobalsTypedDict": ".crm_opportunitiesallop",
    "CrmOpportunitiesAllRequest": ".crm_opportunitiesallop",
    "CrmOpportunitiesAllRequestTypedDict": ".crm_opportunitiesallop",
    "CrmOpportunitiesAllResponse": ".crm_opportunitiesallop",
    "CrmOpportunitiesAllResponseTypedDict": ".crm_opportunitiesallop",
    "CrmOpportunitiesDeleteGlobals": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesDeleteGlobalsTypedDict": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesDeleteRequest": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesDeleteRequestTypedDict": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesDeleteResponse": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesDeleteResponseTypedDict": ".crm_opportunitiesdeleteop",
    "CrmOpportunitiesOneGlobals": ".crm_opportunitiesoneop",
    "CrmOpportunitiesOneGlobalsTypedDict": ".crm_opportunitiesoneop",
    "CrmOpportunitiesOneRequest": ".crm_opportunitiesoneop",
    "CrmOpportunitiesOneRequestTypedDict": ".crm_opportunitiesoneop",
    "CrmOpportunitiesOneResponse": ".crm_opportunitiesoneop",
    "CrmOpportunitiesOneResponseTypedDict": ".crm_opportunitiesoneop",
    "CrmOpportunitiesUpdateGlobals": ".crm_opportunitiesupdateop",
    "CrmOpportunitiesUpdateGlobalsTypedDict": ".crm_opportunitiesupdateop",
    "CrmOpportunitiesUpdateRequest": ".crm_opportunitiesupdateop",
    "CrmOpportunitiesUpdateRequestTypedDict": ".crm_opportunitiesupdateop",
    "CrmOpportunitiesUpdateResponse": ".crm_opportunitiesupdateop",
    "CrmOpportunitiesUpdateResponseTypedDict": ".crm_opportunitiesupdateop",
    "CrmPipelinesAddGlobals": ".crm_pipelinesaddop",
    "CrmPipelinesAddGlobalsTypedDict": ".crm_pipelinesaddop",
    "CrmPipelinesAddRequest": ".crm_pipelinesaddop",
    "CrmPipelinesAddRequestTypedDict": ".crm_pipelinesaddop",
    "CrmPipelinesAddResponse": ".crm_pipelinesaddop",
    "CrmPipelinesAddResponseTypedDict": ".crm_pipelinesaddop",
    "CrmPipelinesAllGlobals": ".crm_pipelinesallop",
    "CrmPipelinesAllGlobalsTypedDict": ".crm_pipelinesallop",
    "CrmPipelinesAllRequest": ".crm_pipelinesallop",
    "CrmPipelinesAllRequestTypedDict": ".crm_pipelinesallop",
    "CrmPipelinesAllResponse": ".crm_pipelinesallop",
    "CrmPipelinesAllResponseTypedDict": ".crm_pipelinesallop",
    "CrmPipelinesDeleteGlobals": ".crm_pipelinesdeleteop",
    "CrmPipelinesDeleteGlobalsTypedDict": ".crm_pipelinesdeleteop",
    "CrmPipelinesDeleteRequest": ".crm_pipelinesdeleteop",
    "CrmPipelinesDeleteRequestTypedDict": ".crm_pipelinesdeleteop",
    "CrmPipelinesDeleteResponse": ".crm_pipelinesdeleteop",
    "CrmPipelinesDeleteResponseTypedDict": ".crm_pipelinesdeleteop",
    "CrmPipelinesOneGlobals": ".crm_pipelinesoneop",
    "CrmPipelinesOneGlobalsTypedDict": ".crm_pipelinesoneop",
    "CrmPipelinesOneRequest": ".crm_pipelinesoneop",
    "CrmPipelinesOneRequestTypedDict": ".crm_pipelinesoneop",
    "CrmPipelinesOneResponse": ".crm_pipelinesoneop",
    "CrmPipelinesOneResponseTypedDict": ".crm_pipelinesoneop",
    "CrmPipelinesUpdateGlobals": ".crm_pipelinesupdateop",
    "CrmPipelinesUpdateGlobalsTypedDict": ".crm_pipelinesupdateop",
    "CrmPipelinesUpdateRequest": ".crm_pipelinesupdateop",
    "CrmPipelinesUpdateRequestTypedDict": ".crm_pipelinesupdateop",
    "CrmPipelinesUpdateResponse": ".crm_pipelinesupdateop",
    "CrmPipelinesUpdateResponseTypedDict": ".crm_pipelinesupdateop",
    "CrmUsersAddGlobals": ".crm_usersaddop",
    "CrmUsersAddGlobalsTypedDict": ".crm_usersaddop",
    "CrmUsersAddRequest": ".crm_usersaddop",
    "CrmUsersAddRequestTypedDict": ".crm_usersaddop",
    "CrmUsersAddResponse": ".crm_usersaddop",
    "CrmUsersAddResponseTypedDict": ".crm_usersaddop",
    "CrmUsersAllGlobals": ".crm_usersallop",
    "CrmUsersAllGlobalsTypedDict": ".crm_usersallop",
    "CrmUsersAllRequest": ".crm_usersallop",
    "CrmUsersAllRequestTypedDict": ".crm_usersallop",
    "CrmUsersAllResponse": ".crm_usersallop",
    "CrmUsersAllResponseTypedDict": ".crm_usersallop",
    "CrmUsersDeleteGlobals": ".crm_usersdeleteop",
    "CrmUsersDeleteGlobalsTypedDict": ".crm_usersdeleteop",
    "CrmUsersDeleteRequest": ".crm_usersdeleteop",
    "CrmUsersDeleteRequestTypedDict": ".crm_usersdeleteop",
    "CrmUsersDeleteResponse": ".crm_usersdeleteop",
    "CrmUsersDeleteResponseTypedDict": ".crm_usersdeleteop",
    "CrmUsersOneGlobals": ".crm_usersoneop",
    "CrmUsersOneGlobalsTypedDict": ".crm_usersoneop",
    "CrmUsersOneRequest": ".crm_usersoneop",
    "CrmUsersOneRequestTypedDict": ".crm_usersoneop",
    "CrmUsersOneResponse": ".crm_usersoneop",
    "CrmUsersOneResponseTypedDict": ".crm_usersoneop",
    "CrmUsersUpdateGlobals": ".crm_usersupdateop",
    "CrmUsersUpdateGlobalsTypedDict": ".crm_usersupdateop",
    "CrmUsersUpdateRequest": ".crm_usersupdateop",
    "CrmUsersUpdateRequestTypedDict": ".crm_usersupdateop",
    "CrmUsersUpdateResponse": ".crm_usersupdateop",
    "CrmUsersUpdateResponseTypedDict": ".crm_usersupdateop",
    "Currency": ".currency",
    "Customer": ".customer",
    "CustomerInput": ".customer",
    "CustomerInputTypedDict": ".customer",
    "CustomerStatusStatus": ".customer",
    "CustomerTypedDict": ".customer",
    "CustomersFilter": ".customersfilter",
    "CustomersFilterStatus": ".customersfilter",
    "CustomersFilterTypedDict": ".customersfilter",
    "CustomersSort": ".customerssort",
    "CustomersSortBy": ".customerssort",
    "CustomersSortTypedDict": ".customerssort",
    "CustomField": ".customfield",
    "CustomFieldTypedDict": ".customfield",
    "Five": ".customfield",
    "FiveTypedDict": ".customfield",
    "Value": ".customfield",
    "ValueTypedDict": ".customfield",
    "CustomFieldFinder": ".customfieldfinder",
    "CustomFieldFinderTypedDict": ".customfieldfinder",
    "CustomMapping": ".custommapping",
    "CustomMappingTypedDict": ".custommapping",
    "CustomMappingInput": ".custommapping_input",
    "CustomMappingInputTypedDict": ".custommapping_input",
    "CustomObject": ".customobject",
    "CustomObjectFields": ".customobject",
    "CustomObjectFieldsTypedDict": ".customobject",
    "CustomObjectInput": ".customobject",
    "CustomObjectInputTypedDict": ".customobject",
    "CustomObjectTypedDict": ".customobject",
    "CustomObjectSchema": ".customobjectschema",
    "CustomObjectSchemaInput": ".customobjectschema",
    "CustomObjectSchemaInputTypedDict": ".customobjectschema",
    "CustomObjectSchemaOptions": ".customobjectschema",
    "CustomObjectSchemaOptionsTypedDict": ".customobjectschema",
    "CustomObjectSchemaType": ".customobjectschema",
    "CustomObjectSchemaTypedDict": ".customobjectschema",
    "Fields": ".customobjectschema",
    "FieldsTypedDict": ".customobjectschema",
    "Deduction": ".deduction",
    "DeductionTypedDict": ".deduction",
    "DeleteAccountingDepartmentResponse": ".deleteaccountingdepartmentresponse",
    "DeleteAccountingDepartmentResponseTypedDict": ".deleteaccountingdepartmentresponse",
    "DeleteAccountingLocationResponse": ".deleteaccountinglocationresponse",
    "DeleteAccountingLocationResponseTypedDict": ".deleteaccountinglocationresponse",
    "DeleteActivityResponse": ".deleteactivityresponse",
    "DeleteActivityResponseTypedDict": ".deleteactivityresponse",
    "DeleteApplicantResponse": ".deleteapplicantresponse",
    "DeleteApplicantResponseTypedDict": ".deleteapplicantresponse",
    "DeleteApplicationResponse": ".deleteapplicationresponse",
    "DeleteApplicationResponseTypedDict": ".deleteapplicationresponse",
    "DeleteAttachmentResponse": ".deleteattachmentresponse",
    "DeleteAttachmentResponseTypedDict": ".deleteattachmentresponse",
    "DeleteBankFeedAccountResponse": ".deletebankfeedaccountresponse",
    "DeleteBankFeedAccountResponseTypedDict": ".deletebankfeedaccountresponse",
    "DeleteBankFeedStatementResponse": ".deletebankfeedstatementresponse",
    "DeleteBankFeedStatementResponseTypedDict": ".deletebankfeedstatementresponse",
    "DeleteBillPaymentResponse": ".deletebillpaymentresponse",
    "DeleteBillPaymentResponseTypedDict": ".deletebillpaymentresponse",
    "DeleteBillResponse": ".deletebillresponse",
    "DeleteBillResponseTypedDict": ".deletebillresponse",
    "DeleteCommentResponse": ".deletecommentresponse",
    "DeleteCommentResponseTypedDict": ".deletecommentresponse",
    "DeleteCompanyResponse": ".deletecompanyresponse",
    "DeleteCompanyResponseTypedDict": ".deletecompanyresponse",
    "DeleteConsumerResponse": ".deleteconsumerresponse",
    "DeleteConsumerResponseData": ".deleteconsumerresponse",
    "DeleteConsumerResponseDataTypedDict": ".deleteconsumerresponse",
    "DeleteConsumerResponseTypedDict": ".deleteconsumerresponse",
    "DeleteContactResponse": ".deletecontactresponse",
    "DeleteContactResponseTypedDict": ".deletecontactresponse",
    "DeleteCreditNoteResponse": ".deletecreditnoteresponse",
    "DeleteCreditNoteResponseTypedDict": ".deletecreditnoteresponse",
    "DeleteCustomerResponse": ".deletecustomerresponse",
    "DeleteCustomerResponseTypedDict": ".deletecustomerresponse",
    "DeleteCustomObjectResponse": ".deletecustomobjectresponse",
    "DeleteCustomObjectResponseTypedDict": ".deletecustomobjectresponse",
    "DeleteCustomObjectSchemaResponse": ".deletecustomobjectschemaresponse",
    "DeleteCustomObjectSchemaResponseTypedDict": ".deletecustomobjectschemaresponse",
    "DeleteDepartmentResponse": ".deletedepartmentresponse",
    "DeleteDepartmentResponseTypedDict": ".deletedepartmentresponse",
    "DeleteDriveGroupResponse": ".deletedrivegroupresponse",
    "DeleteDriveGroupResponseTypedDict": ".deletedrivegroupresponse",
    "DeleteDriveResponse": ".deletedriveresponse",
    "DeleteDriveResponseTypedDict": ".deletedriveresponse",
    "DeleteEmployeeResponse": ".deleteemployeeresponse",
    "DeleteEmployeeResponseTypedDict": ".deleteemployeeresponse",
    "DeleteExpenseResponse": ".deleteexpenseresponse",
    "DeleteExpenseResponseTypedDict": ".deleteexpenseresponse",
    "DeleteFileResponse": ".deletefileresponse",
    "DeleteFileResponseTypedDict": ".deletefileresponse",
    "DeleteFolderResponse": ".deletefolderresponse",
    "DeleteFolderResponseTypedDict": ".deletefolderresponse",
    "DeleteHrisCompanyResponse": ".deletehriscompanyresponse",
    "DeleteHrisCompanyResponseTypedDict": ".deletehriscompanyresponse",
    "DeleteInvoiceResponse": ".deleteinvoiceresponse",
    "DeleteInvoiceResponseTypedDict": ".deleteinvoiceresponse",
    "DeleteJournalEntryResponse": ".deletejournalentryresponse",
    "DeleteJournalEntryResponseTypedDict": ".deletejournalentryresponse",
    "DeleteLeadResponse": ".deleteleadresponse",
    "DeleteLeadResponseTypedDict": ".deleteleadresponse",
    "DeleteLedgerAccountResponse": ".deleteledgeraccountresponse",
    "DeleteLedgerAccountResponseTypedDict": ".deleteledgeraccountresponse",
    "DeleteMessageResponse": ".deletemessageresponse",
    "DeleteMessageResponseTypedDict": ".deletemessageresponse",
    "DeleteNoteResponse": ".deletenoteresponse",
    "DeleteNoteResponseTypedDict": ".deletenoteresponse",
    "DeleteOpportunityResponse": ".deleteopportunityresponse",
    "DeleteOpportunityResponseTypedDict": ".deleteopportunityresponse",
    "DeletePaymentResponse": ".deletepaymentresponse",
    "DeletePaymentResponseTypedDict": ".deletepaymentresponse",
    "DeletePipelineResponse": ".deletepipelineresponse",
    "DeletePipelineResponseTypedDict": ".deletepipelineresponse",
    "DeletePurchaseOrderResponse": ".deletepurchaseorderresponse",
    "DeletePurchaseOrderResponseTypedDict": ".deletepurchaseorderresponse",
    "DeleteQuoteResponse": ".deletequoteresponse",
    "DeleteQuoteResponseTypedDict": ".deletequoteresponse",
    "DeleteSharedLinkResponse": ".deletesharedlinkresponse",
    "DeleteSharedLinkResponseTypedDict": ".deletesharedlinkresponse",
    "DeleteSubsidiaryResponse": ".deletesubsidiaryresponse",
    "DeleteSubsidiaryResponseTypedDict": ".deletesubsidiaryresponse",
    "DeleteSupplierResponse": ".deletesupplierresponse",
    "DeleteSupplierResponseTypedDict": ".deletesupplierresponse",
    "DeleteTaxRateResponse": ".deletetaxrateresponse",
    "DeleteTaxRateResponseTypedDict": ".deletetaxrateresponse",
    "DeleteTicketResponse": ".deleteticketresponse",
    "DeleteTicketResponseTypedDict": ".deleteticketresponse",
    "DeleteTimeOffRequestResponse": ".deletetimeoffrequestresponse",
    "DeleteTimeOffRequestResponseTypedDict": ".deletetimeoffrequestresponse",
    "DeleteTrackingCategoryResponse": ".deletetrackingcategoryresponse",
    "DeleteTrackingCategoryResponseTypedDict": ".deletetrackingcategoryresponse",
    "DeleteUploadSessionResponse": ".deleteuploadsessionresponse",
    "DeleteUploadSessionResponseTypedDict": ".deleteuploadsessionresponse",
    "DeleteUserResponse": ".deleteuserresponse",
    "DeleteUserResponseTypedDict": ".deleteuserresponse",
    "DeleteWebhookResponse": ".deletewebhookresponse",
    "DeleteWebhookResponseTypedDict": ".deletewebhookresponse",
    "Department": ".department",
    "DepartmentTypedDict": ".department",
    "DepartmentInput": ".department_input",
    "DepartmentInputTypedDict": ".department_input",
    "DeprecatedLinkedSupplier": ".deprecatedlinkedsupplier",
    "DeprecatedLinkedSupplierTypedDict": ".deprecatedlinkedsupplier",
    "DeprecatedLinkedSupplierInput": ".deprecatedlinkedsupplier_input",
    "DeprecatedLinkedSupplierInputTypedDict": ".deprecatedlinkedsupplier_input",
    "DeprecatedLinkedTrackingCategory": ".deprecatedlinkedtrackingcategory",
    "DeprecatedLinkedTrackingCategoryTypedDict": ".deprecatedlinkedtrackingcategory",
    "Drive": ".drive",
    "DriveTypedDict": ".drive",
    "DriveInput": ".drive_input",
    "DriveInputTypedDict": ".drive_input",
    "DriveGroup": ".drivegroup",
    "DriveGroupTypedDict": ".drivegroup",
    "DriveGroupInput": ".drivegroup_input",
    "DriveGroupInputTypedDict": ".drivegroup_input",
    "DriveGroupsFilter": ".drivegroupsfilter",
    "DriveGroupsFilterTypedDict": ".drivegroupsfilter",
    "DrivesFilter": ".drivesfilter",
    "DrivesFilterTypedDict": ".drivesfilter",
    "EcommerceCustomersAllGlobals": ".ecommerce_customersallop",
    "EcommerceCustomersAllGlobalsTypedDict": ".ecommerce_customersallop",
    "EcommerceCustomersAllRequest": ".ecommerce_customersallop",
    "EcommerceCustomersAllRequestTypedDict": ".ecommerce_customersallop",
    "EcommerceCustomersAllResponse": ".ecommerce_customersallop",
    "EcommerceCustomersAllResponseTypedDict": ".ecommerce_customersallop",
    "EcommerceCustomersOneGlobals": ".ecommerce_customersoneop",
    "EcommerceCustomersOneGlobalsTypedDict": ".ecommerce_customersoneop",
    "EcommerceCustomersOneRequest": ".ecommerce_customersoneop",
    "EcommerceCustomersOneRequestTypedDict": ".ecommerce_customersoneop",
    "EcommerceCustomersOneResponse": ".ecommerce_customersoneop",
    "EcommerceCustomersOneResponseTypedDict": ".ecommerce_customersoneop",
    "EcommerceOrdersAllGlobals": ".ecommerce_ordersallop",
    "EcommerceOrdersAllGlobalsTypedDict": ".ecommerce_ordersallop",
    "EcommerceOrdersAllRequest": ".ecommerce_ordersallop",
    "EcommerceOrdersAllRequestTypedDict": ".ecommerce_ordersallop",
    "EcommerceOrdersAllResponse": ".ecommerce_ordersallop",
    "EcommerceOrdersAllResponseTypedDict": ".ecommerce_ordersallop",
    "EcommerceOrdersOneGlobals": ".ecommerce_ordersoneop",
    "EcommerceOrdersOneGlobalsTypedDict": ".ecommerce_ordersoneop",
    "EcommerceOrdersOneRequest": ".ecommerce_ordersoneop",
    "EcommerceOrdersOneRequestTypedDict": ".ecommerce_ordersoneop",
    "EcommerceOrdersOneResponse": ".ecommerce_ordersoneop",
    "EcommerceOrdersOneResponseTypedDict": ".ecommerce_ordersoneop",
    "EcommerceProductsAllGlobals": ".ecommerce_productsallop",
    "EcommerceProductsAllGlobalsTypedDict": ".ecommerce_productsallop",
    "EcommerceProductsAllRequest": ".ecommerce_productsallop",
    "EcommerceProductsAllRequestTypedDict": ".ecommerce_productsallop",
    "EcommerceProductsAllResponse": ".ecommerce_productsallop",
    "EcommerceProductsAllResponseTypedDict": ".ecommerce_productsallop",
    "EcommerceProductsOneGlobals": ".ecommerce_productsoneop",
    "EcommerceProductsOneGlobalsTypedDict": ".ecommerce_productsoneop",
    "EcommerceProductsOneRequest": ".ecommerce_productsoneop",
    "EcommerceProductsOneRequestTypedDict": ".ecommerce_productsoneop",
    "EcommerceProductsOneResponse": ".ecommerce_productsoneop",
    "EcommerceProductsOneResponseTypedDict": ".ecommerce_productsoneop",
    "EcommerceStoresOneGlobals": ".ecommerce_storesoneop",
    "EcommerceStoresOneGlobalsTypedDict": ".ecommerce_storesoneop",
    "EcommerceStoresOneRequest": ".ecommerce_storesoneop",
    "EcommerceStoresOneRequestTypedDict": ".ecommerce_storesoneop",
    "EcommerceStoresOneResponse": ".ecommerce_storesoneop",
    "EcommerceStoresOneResponseTypedDict": ".ecommerce_storesoneop",
    "EcommerceAddress": ".ecommerceaddress",
    "EcommerceAddressTypedDict": ".ecommerceaddress",
    "Addresses": ".ecommercecustomer",
    "AddressesTypedDict": ".ecommercecustomer",
    "CustomerStatus": ".ecommercecustomer",
    "EcommerceCustomer": ".ecommercecustomer",
    "EcommerceCustomerType": ".ecommercecustomer",
    "EcommerceCustomerTypedDict": ".ecommercecustomer",
    "EcommerceCustomersFilter": ".ecommercecustomersfilter",
    "EcommerceCustomersFilterTypedDict": ".ecommercecustomersfilter",
    "EcommerceDiscount": ".ecommercediscount",
    "EcommerceDiscountTypedDict": ".ecommercediscount",
    "EcommerceOrder": ".ecommerceorder",
    "EcommerceOrderPaymentStatus": ".ecommerceorder",
    "EcommerceOrderTypedDict": ".ecommerceorder",
    "FulfillmentStatus": ".ecommerceorder",
    "EcommerceOrderLineItem": ".ecommerceorderlineitem",
    "EcommerceOrderLineItemTypedDict": ".ecommerceorderlineitem",
    "Options": ".ecommerceorderlineitem",
    "OptionsTypedDict": ".ecommerceorderlineitem",
    "EcommerceOrderRefund": ".ecommerceorderrefund",
    "EcommerceOrderRefundTypedDict": ".ecommerceorderrefund",
    "EcommerceOrdersFilter": ".ecommerceordersfilter",
    "EcommerceOrdersFilterTypedDict": ".ecommerceordersfilter",
    "EcommerceOrderStatus": ".ecommerceorderstatus",
    "EcommerceProduct": ".ecommerceproduct",
    "EcommerceProductCategories": ".ecommerceproduct",
    "EcommerceProductCategoriesTypedDict": ".ecommerceproduct",
    "EcommerceProductImages": ".ecommerceproduct",
    "EcommerceProductImagesTypedDict": ".ecommerceproduct",
    "EcommerceProductOptions": ".ecommerceproduct",
    "EcommerceProductOptionsTypedDict": ".ecommerceproduct",
    "EcommerceProductTypedDict": ".ecommerceproduct",
    "EcommerceProductVariantsOptions": ".ecommerceproduct",
    "EcommerceProductVariantsOptionsTypedDict": ".ecommerceproduct",
    "Images": ".ecommerceproduct",
    "ImagesTypedDict": ".ecommerceproduct",
    "ProductStatus": ".ecommerceproduct",
    "Variants": ".ecommerceproduct",
    "VariantsTypedDict": ".ecommerceproduct",
    "EcommerceProductsFilter": ".ecommerceproductsfilter",
    "EcommerceProductsFilterTypedDict": ".ecommerceproductsfilter",
    "EcommerceStore": ".ecommercestore",
    "EcommerceStoreTypedDict": ".ecommercestore",
    "Email": ".email",
    "EmailType": ".email",
    "EmailTypedDict": ".email",
    "Employee": ".employee",
    "EmployeeInput": ".employee",
    "EmployeeInputTypedDict": ".employee",
    "EmployeeTypedDict": ".employee",
    "EmploymentRole": ".employee",
    "EmploymentRoleTypedDict": ".employee",
    "EmploymentSubType": ".employee",
    "EmploymentType": ".employee",
    "LeavingReason": ".employee",
    "Manager": ".employee",
    "ManagerTypedDict": ".employee",
    "ProbationPeriod": ".employee",
    "ProbationPeriodTypedDict": ".employee",
    "EmployeeCompensation": ".employeecompensation",
    "EmployeeCompensationInput": ".employeecompensation",
    "EmployeeCompensationInputTypedDict": ".employeecompensation",
    "EmployeeCompensationTypedDict": ".employeecompensation",
    "FlsaStatus": ".employeecompensation",
    "EmployeeJob": ".employeejob",
    "EmployeeJobInput": ".employeejob",
    "EmployeeJobInputTypedDict": ".employeejob",
    "EmployeeJobStatus": ".employeejob",
    "EmployeeJobTypedDict": ".employeejob",
    "EmployeePayroll": ".employeepayroll",
    "EmployeePayrollTypedDict": ".employeepayroll",
    "EmployeeSchedules": ".employeeschedules",
    "EmployeeSchedulesTypedDict": ".employeeschedules",
    "EmployeesFilter": ".employeesfilter",
    "EmployeesFilterEmploymentStatus": ".employeesfilter",
    "EmployeesFilterTypedDict": ".employeesfilter",
    "EmployeesOneFilter": ".employeesonefilter",
    "EmployeesOneFilterTypedDict": ".employeesonefilter",
    "EmployeesSort": ".employeessort",
    "EmployeesSortBy": ".employeessort",
    "EmployeesSortTypedDict": ".employeessort",
    "EmploymentStatus": ".employmentstatus",
    "Expense": ".expense",
    "ExpenseInput": ".expense",
    "ExpenseInputTypedDict": ".expense",
    "ExpensePaymentType": ".expense",
    "ExpenseType": ".expense",
    "ExpenseTypedDict": ".expense",
    "ExpenseLineItem": ".expenselineitem",
    "ExpenseLineItemTypedDict": ".expenselineitem",
    "ExpenseLineItemInput": ".expenselineitem_input",
    "ExpenseLineItemInputTypedDict": ".expenselineitem_input",
    "FilesFilter": ".filesfilter",
    "FilesFilterTypedDict": ".filesfilter",
    "FilesSearch": ".filessearch",
    "FilesSearchTypedDict": ".filessearch",
    "FilesSort": ".filessort",
    "FilesSortBy": ".filessort",
    "FilesSortTypedDict": ".filessort",
    "FileStorageDriveGroupsAddGlobals": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAddGlobalsTypedDict": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAddRequest": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAddRequestTypedDict": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAddResponse": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAddResponseTypedDict": ".filestorage_drivegroupsaddop",
    "FileStorageDriveGroupsAllGlobals": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsAllGlobalsTypedDict": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsAllRequest": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsAllRequestTypedDict": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsAllResponse": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsAllResponseTypedDict": ".filestorage_drivegroupsallop",
    "FileStorageDriveGroupsDeleteGlobals": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsDeleteGlobalsTypedDict": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsDeleteRequest": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsDeleteRequestTypedDict": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsDeleteResponse": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsDeleteResponseTypedDict": ".filestorage_drivegroupsdeleteop",
    "FileStorageDriveGroupsOneGlobals": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsOneGlobalsTypedDict": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsOneRequest": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsOneRequestTypedDict": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsOneResponse": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsOneResponseTypedDict": ".filestorage_drivegroupsoneop",
    "FileStorageDriveGroupsUpdateGlobals": ".filestorage_drivegroupsupdateop",
    "FileStorageDriveGroupsUpdateGlobalsTypedDict": ".filestorage_drivegroupsupdateop",
    "FileStorageDriveGroupsUpdateRequest": ".filestorage_drivegroupsupdateop",
    "FileStorageDriveGroupsUpdateRequestTypedDict": ".filestorage_drivegroupsupdateop",
    "FileStorageDriveGroupsUpdateResponse": ".filestorage_drivegroupsupdateop",
    "FileStorageDriveGroupsUpdateResponseTypedDict": ".filestorage_drivegroupsupdateop",
    "FileStorageDrivesAddGlobals": ".filestorage_drivesaddop",
    "FileStorageDrivesAddGlobalsTypedDict": ".filestorage_drivesaddop",
    "FileStorageDrivesAddRequest": ".filestorage_drivesaddop",
    "FileStorageDrivesAddRequestTypedDict": ".filestorage_drivesaddop",
    "FileStorageDrivesAddResponse": ".filestorage_drivesaddop",
    "FileStorageDrivesAddResponseTypedDict": ".filestorage_drivesaddop",
    "FileStorageDrivesAllGlobals": ".filestorage_drivesallop",
    "FileStorageDrivesAllGlobalsTypedDict": ".filestorage_drivesallop",
    "FileStorageDrivesAllRequest": ".filestorage_drivesallop",
    "FileStorageDrivesAllRequestTypedDict": ".filestorage_drivesallop",
    "FileStorageDrivesAllResponse": ".filestorage_drivesallop",
    "FileStorageDrivesAllResponseTypedDict": ".filestorage_drivesallop",
    "FileStorageDrivesDeleteGlobals": ".filestorage_drivesdeleteop",
    "FileStorageDrivesDeleteGlobalsTypedDict": ".filestorage_drivesdeleteop",
    "FileStorageDrivesDeleteRequest": ".filestorage_drivesdeleteop",
    "FileStorageDrivesDeleteRequestTypedDict": ".filestorage_drivesdeleteop",
    "FileStorageDrivesDeleteResponse": ".filestorage_drivesdeleteop",
    "FileStorageDrivesDeleteResponseTypedDict": ".filestorage_drivesdeleteop",
    "FileStorageDrivesOneGlobals": ".filestorage_drivesoneop",
    "FileStorageDrivesOneGlobalsTypedDict": ".filestorage_drivesoneop",
    "FileStorageDrivesOneRequest": ".filestorage_drivesoneop",
    "FileStorageDrivesOneRequestTypedDict": ".filestorage_drivesoneop",
    "FileStorageDrivesOneResponse": ".filestorage_drivesoneop",
    "FileStorageDrivesOneResponseTypedDict": ".filestorage_drivesoneop",
    "FileStorageDrivesUpdateGlobals": ".filestorage_drivesupdateop",
    "FileStorageDrivesUpdateGlobalsTypedDict": ".filestorage_drivesupdateop",
    "FileStorageDrivesUpdateRequest": ".filestorage_drivesupdateop",
    "FileStorageDrivesUpdateRequestTypedDict": ".filestorage_drivesupdateop",
    "FileStorageDrivesUpdateResponse": ".filestorage_drivesupdateop",
    "FileStorageDrivesUpdateResponseTypedDict": ".filestorage_drivesupdateop",
    "FileStorageFilesAllGlobals": ".filestorage_filesallop",
    "FileStorageFilesAllGlobalsTypedDict": ".filestorage_filesallop",
    "FileStorageFilesAllRequest": ".filestorage_filesallop",
    "FileStorageFilesAllRequestTypedDict": ".filestorage_filesallop",
    "FileStorageFilesAllResponse": ".filestorage_filesallop",
    "FileStorageFilesAllResponseTypedDict": ".filestorage_filesallop",
    "FileStorageFilesDeleteGlobals": ".filestorage_filesdeleteop",
    "FileStorageFilesDeleteGlobalsTypedDict": ".filestorage_filesdeleteop",
    "FileStorageFilesDeleteRequest": ".filestorage_filesdeleteop",
    "FileStorageFilesDeleteRequestTypedDict": ".filestorage_filesdeleteop",
    "FileStorageFilesDeleteResponse": ".filestorage_filesdeleteop",
    "FileStorageFilesDeleteResponseTypedDict": ".filestorage_filesdeleteop",
    "FileStorageFilesDownloadGlobals": ".filestorage_filesdownloadop",
    "FileStorageFilesDownloadGlobalsTypedDict": ".filestorage_filesdownloadop",
    "FileStorageFilesDownloadRequest": ".filestorage_filesdownloadop",
    "FileStorageFilesDownloadRequestTypedDict": ".filestorage_filesdownloadop",
    "FileStorageFilesDownloadResponse": ".filestorage_filesdownloadop",
    "FileStorageFilesDownloadResponseTypedDict": ".filestorage_filesdownloadop",
    "FileStorageFilesExportGlobals": ".filestorage_filesexportop",
    "FileStorageFilesExportGlobalsTypedDict": ".filestorage_filesexportop",
    "FileStorageFilesExportRequest": ".filestorage_filesexportop",
    "FileStorageFilesExportRequestTypedDict": ".filestorage_filesexportop",
    "FileStorageFilesExportResponse": ".filestorage_filesexportop",
    "FileStorageFilesExportResponseTypedDict": ".filestorage_filesexportop",
    "FileStorageFilesOneGlobals": ".filestorage_filesoneop",
    "FileStorageFilesOneGlobalsTypedDict": ".filestorage_filesoneop",
    "FileStorageFilesOneRequest": ".filestorage_filesoneop",
    "FileStorageFilesOneRequestTypedDict": ".filestorage_filesoneop",
    "FileStorageFilesOneResponse": ".filestorage_filesoneop",
    "FileStorageFilesOneResponseTypedDict": ".filestorage_filesoneop",
    "FileStorageFilesSearchGlobals": ".filestorage_filessearchop",
    "FileStorageFilesSearchGlobalsTypedDict": ".filestorage_filessearchop",
    "FileStorageFilesSearchRequest": ".filestorage_filessearchop",
    "FileStorageFilesSearchRequestTypedDict": ".filestorage_filessearchop",
    "FileStorageFilesSearchResponse": ".filestorage_filessearchop",
    "FileStorageFilesSearchResponseTypedDict": ".filestorage_filessearchop",
    "FileStorageFilesUpdateGlobals": ".filestorage_filesupdateop",
    "FileStorageFilesUpdateGlobalsTypedDict": ".filestorage_filesupdateop",
    "FileStorageFilesUpdateRequest": ".filestorage_filesupdateop",
    "FileStorageFilesUpdateRequestTypedDict": ".filestorage_filesupdateop",
    "FileStorageFilesUpdateResponse": ".filestorage_filesupdateop",
    "FileStorageFilesUpdateResponseTypedDict": ".filestorage_filesupdateop",
    "FileStorageFoldersAddGlobals": ".filestorage_foldersaddop",
    "FileStorageFoldersAddGlobalsTypedDict": ".filestorage_foldersaddop",
    "FileStorageFoldersAddRequest": ".filestorage_foldersaddop",
    "FileStorageFoldersAddRequestTypedDict": ".filestorage_foldersaddop",
    "FileStorageFoldersAddResponse": ".filestorage_foldersaddop",
    "FileStorageFoldersAddResponseTypedDict": ".filestorage_foldersaddop",
    "FileStorageFoldersCopyGlobals": ".filestorage_folderscopyop",
    "FileStorageFoldersCopyGlobalsTypedDict": ".filestorage_folderscopyop",
    "FileStorageFoldersCopyRequest": ".filestorage_folderscopyop",
    "FileStorageFoldersCopyRequestTypedDict": ".filestorage_folderscopyop",
    "FileStorageFoldersCopyResponse": ".filestorage_folderscopyop",
    "FileStorageFoldersCopyResponseTypedDict": ".filestorage_folderscopyop",
    "FileStorageFoldersDeleteGlobals": ".filestorage_foldersdeleteop",
    "FileStorageFoldersDeleteGlobalsTypedDict": ".filestorage_foldersdeleteop",
    "FileStorageFoldersDeleteRequest": ".filestorage_foldersdeleteop",
    "FileStorageFoldersDeleteRequestTypedDict": ".filestorage_foldersdeleteop",
    "FileStorageFoldersDeleteResponse": ".filestorage_foldersdeleteop",
    "FileStorageFoldersDeleteResponseTypedDict": ".filestorage_foldersdeleteop",
    "FileStorageFoldersOneGlobals": ".filestorage_foldersoneop",
    "FileStorageFoldersOneGlobalsTypedDict": ".filestorage_foldersoneop",
    "FileStorageFoldersOneRequest": ".filestorage_foldersoneop",
    "FileStorageFoldersOneRequestTypedDict": ".filestorage_foldersoneop",
    "FileStorageFoldersOneResponse": ".filestorage_foldersoneop",
    "FileStorageFoldersOneResponseTypedDict": ".filestorage_foldersoneop",
    "FileStorageFoldersUpdateGlobals": ".filestorage_foldersupdateop",
    "FileStorageFoldersUpdateGlobalsTypedDict": ".filestorage_foldersupdateop",
    "FileStorageFoldersUpdateRequest": ".filestorage_foldersupdateop",
    "FileStorageFoldersUpdateRequestTypedDict": ".filestorage_foldersupdateop",
    "FileStorageFoldersUpdateResponse": ".filestorage_foldersupdateop",
    "FileStorageFoldersUpdateResponseTypedDict": ".filestorage_foldersupdateop",
    "FileStorageSharedLinksAddGlobals": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAddGlobalsTypedDict": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAddRequest": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAddRequestTypedDict": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAddResponse": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAddResponseTypedDict": ".filestorage_sharedlinksaddop",
    "FileStorageSharedLinksAllGlobals": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksAllGlobalsTypedDict": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksAllRequest": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksAllRequestTypedDict": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksAllResponse": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksAllResponseTypedDict": ".filestorage_sharedlinksallop",
    "FileStorageSharedLinksDeleteGlobals": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksDeleteGlobalsTypedDict": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksDeleteRequest": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksDeleteRequestTypedDict": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksDeleteResponse": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksDeleteResponseTypedDict": ".filestorage_sharedlinksdeleteop",
    "FileStorageSharedLinksOneGlobals": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksOneGlobalsTypedDict": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksOneRequest": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksOneRequestTypedDict": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksOneResponse": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksOneResponseTypedDict": ".filestorage_sharedlinksoneop",
    "FileStorageSharedLinksUpdateGlobals": ".filestorage_sharedlinksupdateop",
    "FileStorageSharedLinksUpdateGlobalsTypedDict": ".filestorage_sharedlinksupdateop",
    "FileStorageSharedLinksUpdateRequest": ".filestorage_sharedlinksupdateop",
    "FileStorageSharedLinksUpdateRequestTypedDict": ".filestorage_sharedlinksupdateop",
    "FileStorageSharedLinksUpdateResponse": ".filestorage_sharedlinksupdateop",
    "FileStorageSharedLinksUpdateResponseTypedDict": ".filestorage_sharedlinksupdateop",
    "FILE_STORAGE_UPLOAD_SESSIONS_ADD_OP_SERVERS": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddGlobals": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddGlobalsTypedDict": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddRequest": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddRequestTypedDict": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddResponse": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsAddResponseTypedDict": ".filestorage_uploadsessionsaddop",
    "FileStorageUploadSessionsDeleteGlobals": ".filestorage_uploadsessionsdeleteop",
    "FileStorageUploadSessionsDeleteGlobalsTypedDict": ".filestorage_uploadsessionsdeleteop",
    "FileStorageUploadSessionsDeleteRequest": ".filestorage_uploadsessionsdeleteop",
    "FileStorageUploadSessionsDeleteRequestTypedDict": ".filestorage_uploadsessionsdeleteop",
    "FileStorageUploadSessionsDeleteResponse": ".filestorage_uploadsessionsdeleteop",
    "FileStorageUploadSessionsDeleteResponseTypedDict": ".filestorage_uploadsessionsdeleteop",
    "FILE_STORAGE_UPLOAD_SESSIONS_FINISH_OP_SERVERS": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishGlobals": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishGlobalsTypedDict": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishRequest": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishRequestBody": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishRequestBodyTypedDict": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishRequestTypedDict": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishResponse": ".filestorage_uploadsessionsfinishop",
    "FileStorageUploadSessionsFinishResponseTypedDict": ".filestorage_uploadsessionsfinishop",
    "FILE_STORAGE_UPLOAD_SESSIONS_ONE_OP_SERVERS": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneGlobals": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneGlobalsTypedDict": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneRequest": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneRequestTypedDict": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneResponse": ".filestorage_uploadsessionsoneop",
    "FileStorageUploadSessionsOneResponseTypedDict": ".filestorage_uploadsessionsoneop",
    "FILE_STORAGE_UPLOAD_SESSIONS_UPLOAD_OP_SERVERS": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadGlobals": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadGlobalsTypedDict": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadRequest": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadRequestTypedDict": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadResponse": ".filestorage_uploadsessionsuploadop",
    "FileStorageUploadSessionsUploadResponseTypedDict": ".filestorage_uploadsessionsuploadop",
    "FileType": ".filetype",
    "Folder": ".folder",
    "FolderTypedDict": ".folder",
    "FormField": ".formfield",
    "FormFieldType": ".formfield",
    "FormFieldTypedDict": ".formfield",
    "FormFieldOption": ".formfieldoption",
    "FormFieldOptionTypedDict": ".formfieldoption",
    "FormFieldOptionGroup": ".formfieldoptiongroup",
    "FormFieldOptionGroupOptionType": ".formfieldoptiongroup",
    "FormFieldOptionGroupTypedDict": ".formfieldoptiongroup",
    "Gender": ".gender",
    "GetAccountingDepartmentResponse": ".getaccountingdepartmentresponse",
    "GetAccountingDepartmentResponseTypedDict": ".getaccountingdepartmentresponse",
    "GetAccountingDepartmentsResponse": ".getaccountingdepartmentsresponse",
    "GetAccountingDepartmentsResponseTypedDict": ".getaccountingdepartmentsresponse",
    "GetAccountingLocationResponse": ".getaccountinglocationresponse",
    "GetAccountingLocationResponseTypedDict": ".getaccountinglocationresponse",
    "GetAccountingLocationsResponse": ".getaccountinglocationsresponse",
    "GetAccountingLocationsResponseTypedDict": ".getaccountinglocationsresponse",
    "GetActivitiesResponse": ".getactivitiesresponse",
    "GetActivitiesResponseTypedDict": ".getactivitiesresponse",
    "GetActivityResponse": ".getactivityresponse",
    "GetActivityResponseTypedDict": ".getactivityresponse",
    "GetAgedCreditorsResponse": ".getagedcreditorsresponse",
    "GetAgedCreditorsResponseTypedDict": ".getagedcreditorsresponse",
    "GetAgedDebtorsResponse": ".getageddebtorsresponse",
    "GetAgedDebtorsResponseTypedDict": ".getageddebtorsresponse",
    "GetAPIResourceCoverageResponse": ".getapiresourcecoverageresponse",
    "GetAPIResourceCoverageResponseTypedDict": ".getapiresourcecoverageresponse",
    "GetAPIResourceResponse": ".getapiresourceresponse",
    "GetAPIResourceResponseTypedDict": ".getapiresourceresponse",
    "GetAPIResponse": ".getapiresponse",
    "GetAPIResponseTypedDict": ".getapiresponse",
    "GetApisResponse": ".getapisresponse",
    "GetApisResponseTypedDict": ".getapisresponse",
    "GetApplicantResponse": ".getapplicantresponse",
    "GetApplicantResponseTypedDict": ".getapplicantresponse",
    "GetApplicantsResponse": ".getapplicantsresponse",
    "GetApplicantsResponseTypedDict": ".getapplicantsresponse",
    "GetApplicationResponse": ".getapplicationresponse",
    "GetApplicationResponseTypedDict": ".getapplicationresponse",
    "GetApplicationsResponse": ".getapplicationsresponse",
    "GetApplicationsResponseTypedDict": ".getapplicationsresponse",
    "GetAttachmentResponse": ".getattachmentresponse",
    "GetAttachmentResponseTypedDict": ".getattachmentresponse",
    "GetAttachmentsResponse": ".getattachmentsresponse",
    "GetAttachmentsResponseTypedDict": ".getattachmentsresponse",
    "GetBalanceSheetResponse": ".getbalancesheetresponse",
    "GetBalanceSheetResponseTypedDict": ".getbalancesheetresponse",
    "GetBankFeedAccountResponse": ".getbankfeedaccountresponse",
    "GetBankFeedAccountResponseTypedDict": ".getbankfeedaccountresponse",
    "GetBankFeedAccountsResponse": ".getbankfeedaccountsresponse",
    "GetBankFeedAccountsResponseTypedDict": ".getbankfeedaccountsresponse",
    "GetBankFeedStatementResponse": ".getbankfeedstatementresponse",
    "GetBankFeedStatementResponseTypedDict": ".getbankfeedstatementresponse",
    "GetBankFeedStatementsResponse": ".getbankfeedstatementsresponse",
    "GetBankFeedStatementsResponseTypedDict": ".getbankfeedstatementsresponse",
    "GetBillPaymentResponse": ".getbillpaymentresponse",
    "GetBillPaymentResponseTypedDict": ".getbillpaymentresponse",
    "GetBillPaymentsResponse": ".getbillpaymentsresponse",
    "GetBillPaymentsResponseTypedDict": ".getbillpaymentsresponse",
    "GetBillResponse": ".getbillresponse",
    "GetBillResponseTypedDict": ".getbillresponse",
    "GetBillsResponse": ".getbillsresponse",
    "GetBillsResponseTypedDict": ".getbillsresponse",
    "GetCategoriesResponse": ".getcategoriesresponse",
    "GetCategoriesResponseTypedDict": ".getcategoriesresponse",
    "GetCategoryResponse": ".getcategoryresponse",
    "GetCategoryResponseTypedDict": ".getcategoryresponse",
    "GetCollectionResponse": ".getcollectionresponse",
    "GetCollectionResponseTypedDict": ".getcollectionresponse",
    "GetCollectionsResponse": ".getcollectionsresponse",
    "GetCollectionsResponseTypedDict": ".getcollectionsresponse",
    "GetCollectionTagsResponse": ".getcollectiontagsresponse",
    "GetCollectionTagsResponseTypedDict": ".getcollectiontagsresponse",
    "GetCollectionUserResponse": ".getcollectionuserresponse",
    "GetCollectionUserResponseTypedDict": ".getcollectionuserresponse",
    "GetCollectionUsersResponse": ".getcollectionusersresponse",
    "GetCollectionUsersResponseTypedDict": ".getcollectionusersresponse",
    "GetCommentResponse": ".getcommentresponse",
    "GetCommentResponseTypedDict": ".getcommentresponse",
    "GetCommentsResponse": ".getcommentsresponse",
    "GetCommentsResponseTypedDict": ".getcommentsresponse",
    "GetCompaniesResponse": ".getcompaniesresponse",
    "GetCompaniesResponseTypedDict": ".getcompaniesresponse",
    "GetCompanyInfoResponse": ".getcompanyinforesponse",
    "GetCompanyInfoResponseTypedDict": ".getcompanyinforesponse",
    "GetCompanyResponse": ".getcompanyresponse",
    "GetCompanyResponseTypedDict": ".getcompanyresponse",
    "GetConnectionResponse": ".getconnectionresponse",
    "GetConnectionResponseTypedDict": ".getconnectionresponse",
    "GetConnectionsResponse": ".getconnectionsresponse",
    "GetConnectionsResponseTypedDict": ".getconnectionsresponse",
    "GetConnectorResourceResponse": ".getconnectorresourceresponse",
    "GetConnectorResourceResponseTypedDict": ".getconnectorresourceresponse",
    "GetConnectorResponse": ".getconnectorresponse",
    "GetConnectorResponseTypedDict": ".getconnectorresponse",
    "GetConnectorsResponse": ".getconnectorsresponse",
    "GetConnectorsResponseTypedDict": ".getconnectorsresponse",
    "GetConsumerResponse": ".getconsumerresponse",
    "GetConsumerResponseTypedDict": ".getconsumerresponse",
    "Data": ".getconsumersresponse",
    "DataTypedDict": ".getconsumersresponse",
    "GetConsumersResponse": ".getconsumersresponse",
    "GetConsumersResponseTypedDict": ".getconsumersresponse",
    "GetContactResponse": ".getcontactresponse",
    "GetContactResponseTypedDict": ".getcontactresponse",
    "GetContactsResponse": ".getcontactsresponse",
    "GetContactsResponseTypedDict": ".getcontactsresponse",
    "GetCreditNoteResponse": ".getcreditnoteresponse",
    "GetCreditNoteResponseTypedDict": ".getcreditnoteresponse",
    "GetCreditNotesResponse": ".getcreditnotesresponse",
    "GetCreditNotesResponseTypedDict": ".getcreditnotesresponse",
    "GetCustomerResponse": ".getcustomerresponse",
    "GetCustomerResponseTypedDict": ".getcustomerresponse",
    "GetCustomersResponse": ".getcustomersresponse",
    "GetCustomersResponseTypedDict": ".getcustomersresponse",
    "GetCustomFieldsResponse": ".getcustomfieldsresponse",
    "GetCustomFieldsResponseTypedDict": ".getcustomfieldsresponse",
    "GetCustomMappingsResponse": ".getcustommappingsresponse",
    "GetCustomMappingsResponseTypedDict": ".getcustommappingsresponse",
    "GetCustomObjectResponse": ".getcustomobjectresponse",
    "GetCustomObjectResponseTypedDict": ".getcustomobjectresponse",
    "GetCustomObjectSchemaResponse": ".getcustomobjectschemaresponse",
    "GetCustomObjectSchemaResponseTypedDict": ".getcustomobjectschemaresponse",
    "GetCustomObjectSchemasResponse": ".getcustomobjectschemasresponse",
    "GetCustomObjectSchemasResponseTypedDict": ".getcustomobjectschemasresponse",
    "GetCustomObjectsResponse": ".getcustomobjectsresponse",
    "GetCustomObjectsResponseTypedDict": ".getcustomobjectsresponse",
    "GetDepartmentResponse": ".getdepartmentresponse",
    "GetDepartmentResponseTypedDict": ".getdepartmentresponse",
    "GetDepartmentsResponse": ".getdepartmentsresponse",
    "GetDepartmentsResponseTypedDict": ".getdepartmentsresponse",
    "GetDriveGroupResponse": ".getdrivegroupresponse",
    "GetDriveGroupResponseTypedDict": ".getdrivegroupresponse",
    "GetDriveGroupsResponse": ".getdrivegroupsresponse",
    "GetDriveGroupsResponseTypedDict": ".getdrivegroupsresponse",
    "GetDriveResponse": ".getdriveresponse",
    "GetDriveResponseTypedDict": ".getdriveresponse",
    "GetDrivesResponse": ".getdrivesresponse",
    "GetDrivesResponseTypedDict": ".getdrivesresponse",
    "GetEcommerceCustomerResponse": ".getecommercecustomerresponse",
    "GetEcommerceCustomerResponseTypedDict": ".getecommercecustomerresponse",
    "GetEcommerceCustomersResponse": ".getecommercecustomersresponse",
    "GetEcommerceCustomersResponseTypedDict": ".getecommercecustomersresponse",
    "GetEcommerceOrderResponse": ".getecommerceorderresponse",
    "GetEcommerceOrderResponseTypedDict": ".getecommerceorderresponse",
    "GetEcommerceOrdersResponse": ".getecommerceordersresponse",
    "GetEcommerceOrdersResponseTypedDict": ".getecommerceordersresponse",
    "GetEmployeePayrollResponse": ".getemployeepayrollresponse",
    "GetEmployeePayrollResponseTypedDict": ".getemployeepayrollresponse",
    "GetEmployeePayrollsResponse": ".getemployeepayrollsresponse",
    "GetEmployeePayrollsResponseTypedDict": ".getemployeepayrollsresponse",
    "GetEmployeeResponse": ".getemployeeresponse",
    "GetEmployeeResponseTypedDict": ".getemployeeresponse",
    "GetEmployeeSchedulesResponse": ".getemployeeschedulesresponse",
    "GetEmployeeSchedulesResponseTypedDict": ".getemployeeschedulesresponse",
    "GetEmployeesResponse": ".getemployeesresponse",
    "GetEmployeesResponseTypedDict": ".getemployeesresponse",
    "GetExpenseResponse": ".getexpenseresponse",
    "GetExpenseResponseTypedDict": ".getexpenseresponse",
    "GetExpensesResponse": ".getexpensesresponse",
    "GetExpensesResponseTypedDict": ".getexpensesresponse",
    "GetFileResponse": ".getfileresponse",
    "GetFileResponseTypedDict": ".getfileresponse",
    "GetFilesResponse": ".getfilesresponse",
    "GetFilesResponseTypedDict": ".getfilesresponse",
    "GetFolderResponse": ".getfolderresponse",
    "GetFolderResponseTypedDict": ".getfolderresponse",
    "GetHrisCompaniesResponse": ".gethriscompaniesresponse",
    "GetHrisCompaniesResponseTypedDict": ".gethriscompaniesresponse",
    "GetHrisCompanyResponse": ".gethriscompanyresponse",
    "GetHrisCompanyResponseTypedDict": ".gethriscompanyresponse",
    "GetInvoiceItemResponse": ".getinvoiceitemresponse",
    "GetInvoiceItemResponseTypedDict": ".getinvoiceitemresponse",
    "GetInvoiceItemsResponse": ".getinvoiceitemsresponse",
    "GetInvoiceItemsResponseTypedDict": ".getinvoiceitemsresponse",
    "GetInvoiceResponse": ".getinvoiceresponse",
    "GetInvoiceResponseTypedDict": ".getinvoiceresponse",
    "GetInvoicesResponse": ".getinvoicesresponse",
    "GetInvoicesResponseTypedDict": ".getinvoicesresponse",
    "GetJobResponse": ".getjobresponse",
    "GetJobResponseTypedDict": ".getjobresponse",
    "GetJobsResponse": ".getjobsresponse",
    "GetJobsResponseTypedDict": ".getjobsresponse",
    "GetJournalEntriesResponse": ".getjournalentriesresponse",
    "GetJournalEntriesResponseTypedDict": ".getjournalentriesresponse",
    "GetJournalEntryResponse": ".getjournalentryresponse",
    "GetJournalEntryResponseTypedDict": ".getjournalentryresponse",
    "GetLeadResponse": ".getleadresponse",
    "GetLeadResponseTypedDict": ".getleadresponse",
    "GetLeadsResponse": ".getleadsresponse",
    "GetLeadsResponseTypedDict": ".getleadsresponse",
    "GetLedgerAccountResponse": ".getledgeraccountresponse",
    "GetLedgerAccountResponseTypedDict": ".getledgeraccountresponse",
    "GetLedgerAccountsResponse": ".getledgeraccountsresponse",
    "GetLedgerAccountsResponseTypedDict": ".getledgeraccountsresponse",
    "GetLogsResponse": ".getlogsresponse",
    "GetLogsResponseTypedDict": ".getlogsresponse",
    "GetMessageResponse": ".getmessageresponse",
    "GetMessageResponseTypedDict": ".getmessageresponse",
    "GetMessagesResponse": ".getmessagesresponse",
    "GetMessagesResponseTypedDict": ".getmessagesresponse",
    "GetNoteResponse": ".getnoteresponse",
    "GetNoteResponseTypedDict": ".getnoteresponse",
    "GetNotesResponse": ".getnotesresponse",
    "GetNotesResponseTypedDict": ".getnotesresponse",
    "GetOpportunitiesResponse": ".getopportunitiesresponse",
    "GetOpportunitiesResponseTypedDict": ".getopportunitiesresponse",
    "GetOpportunityResponse": ".getopportunityresponse",
    "GetOpportunityResponseTypedDict": ".getopportunityresponse",
    "GetPaymentResponse": ".getpaymentresponse",
    "GetPaymentResponseTypedDict": ".getpaymentresponse",
    "GetPaymentsResponse": ".getpaymentsresponse",
    "GetPaymentsResponseTypedDict": ".getpaymentsresponse",
    "GetPayrollResponse": ".getpayrollresponse",
    "GetPayrollResponseTypedDict": ".getpayrollresponse",
    "GetPayrollsResponse": ".getpayrollsresponse",
    "GetPayrollsResponseTypedDict": ".getpayrollsresponse",
    "GetPipelineResponse": ".getpipelineresponse",
    "GetPipelineResponseTypedDict": ".getpipelineresponse",
    "GetPipelinesResponse": ".getpipelinesresponse",
    "GetPipelinesResponseTypedDict": ".getpipelinesresponse",
    "GetProductResponse": ".getproductresponse",
    "GetProductResponseTypedDict": ".getproductresponse",
    "GetProductsResponse": ".getproductsresponse",
    "GetProductsResponseTypedDict": ".getproductsresponse",
    "GetProfitAndLossResponse": ".getprofitandlossresponse",
    "GetProfitAndLossResponseTypedDict": ".getprofitandlossresponse",
    "GetPurchaseOrderResponse": ".getpurchaseorderresponse",
    "GetPurchaseOrderResponseTypedDict": ".getpurchaseorderresponse",
    "GetPurchaseOrdersResponse": ".getpurchaseordersresponse",
    "GetPurchaseOrdersResponseTypedDict": ".getpurchaseordersresponse",
    "GetQuoteResponse": ".getquoteresponse",
    "GetQuoteResponseTypedDict": ".getquoteresponse",
    "GetQuotesResponse": ".getquotesresponse",
    "GetQuotesResponseTypedDict": ".getquotesresponse",
    "GetSharedLinkResponse": ".getsharedlinkresponse",
    "GetSharedLinkResponseTypedDict": ".getsharedlinkresponse",
    "GetSharedLinksResponse": ".getsharedlinksresponse",
    "GetSharedLinksResponseTypedDict": ".getsharedlinksresponse",
    "GetStoreResponse": ".getstoreresponse",
    "GetStoreResponseTypedDict": ".getstoreresponse",
    "GetSubsidiariesResponse": ".getsubsidiariesresponse",
    "GetSubsidiariesResponseTypedDict": ".getsubsidiariesresponse",
    "GetSubsidiaryResponse": ".getsubsidiaryresponse",
    "GetSubsidiaryResponseTypedDict": ".getsubsidiaryresponse",
    "GetSupplierResponse": ".getsupplierresponse",
    "GetSupplierResponseTypedDict": ".getsupplierresponse",
    "GetSuppliersResponse": ".getsuppliersresponse",
    "GetSuppliersResponseTypedDict": ".getsuppliersresponse",
    "GetTaxRateResponse": ".gettaxrateresponse",
    "GetTaxRateResponseTypedDict": ".gettaxrateresponse",
    "GetTaxRatesResponse": ".gettaxratesresponse",
    "GetTaxRatesResponseTypedDict": ".gettaxratesresponse",
    "GetTicketResponse": ".getticketresponse",
    "GetTicketResponseTypedDict": ".getticketresponse",
    "GetTicketsResponse": ".getticketsresponse",
    "GetTicketsResponseTypedDict": ".getticketsresponse",
    "GetTimeOffRequestResponse": ".gettimeoffrequestresponse",
    "GetTimeOffRequestResponseTypedDict": ".gettimeoffrequestresponse",
    "GetTimeOffRequestsResponse": ".gettimeoffrequestsresponse",
    "GetTimeOffRequestsResponseTypedDict": ".gettimeoffrequestsresponse",
    "GetTrackingCategoriesResponse": ".gettrackingcategoriesresponse",
    "GetTrackingCategoriesResponseTypedDict": ".gettrackingcategoriesresponse",
    "GetTrackingCategoryResponse": ".gettrackingcategoryresponse",
    "GetTrackingCategoryResponseTypedDict": ".gettrackingcategoryresponse",
    "GetUploadSessionResponse": ".getuploadsessionresponse",
    "GetUploadSessionResponseTypedDict": ".getuploadsessionresponse",
    "GetUserResponse": ".getuserresponse",
    "GetUserResponseTypedDict": ".getuserresponse",
    "GetUsersResponse": ".getusersresponse",
    "GetUsersResponseTypedDict": ".getusersresponse",
    "GetWebhookEventLogsResponse": ".getwebhookeventlogsresponse",
    "GetWebhookEventLogsResponseTypedDict": ".getwebhookeventlogsresponse",
    "GetWebhookResponse": ".getwebhookresponse",
    "GetWebhookResponseTypedDict": ".getwebhookresponse",
    "GetWebhooksResponse": ".getwebhooksresponse",
    "GetWebhooksResponseTypedDict": ".getwebhooksresponse",
    "HrisCompaniesAddGlobals": ".hris_companiesaddop",
    "HrisCompaniesAddGlobalsTypedDict": ".hris_companiesaddop",
    "HrisCompaniesAddRequest": ".hris_companiesaddop",
    "HrisCompaniesAddRequestTypedDict": ".hris_companiesaddop",
    "HrisCompaniesAddResponse": ".hris_companiesaddop",
    "HrisCompaniesAddResponseTypedDict": ".hris_companiesaddop",
    "HrisCompaniesAllGlobals": ".hris_companiesallop",
    "HrisCompaniesAllGlobalsTypedDict": ".hris_companiesallop",
    "HrisCompaniesAllRequest": ".hris_companiesallop",
    "HrisCompaniesAllRequestTypedDict": ".hris_companiesallop",
    "HrisCompaniesAllResponse": ".hris_companiesallop",
    "HrisCompaniesAllResponseTypedDict": ".hris_companiesallop",
    "HrisCompaniesDeleteGlobals": ".hris_companiesdeleteop",
    "HrisCompaniesDeleteGlobalsTypedDict": ".hris_companiesdeleteop",
    "HrisCompaniesDeleteRequest": ".hris_companiesdeleteop",
    "HrisCompaniesDeleteRequestTypedDict": ".hris_companiesdeleteop",
    "HrisCompaniesDeleteResponse": ".hris_companiesdeleteop",
    "HrisCompaniesDeleteResponseTypedDict": ".hris_companiesdeleteop",
    "HrisCompaniesOneGlobals": ".hris_companiesoneop",
    "HrisCompaniesOneGlobalsTypedDict": ".hris_companiesoneop",
    "HrisCompaniesOneRequest": ".hris_companiesoneop",
    "HrisCompaniesOneRequestTypedDict": ".hris_companiesoneop",
    "HrisCompaniesOneResponse": ".hris_companiesoneop",
    "HrisCompaniesOneResponseTypedDict": ".hris_companiesoneop",
    "HrisCompaniesUpdateGlobals": ".hris_companiesupdateop",
    "HrisCompaniesUpdateGlobalsTypedDict": ".hris_companiesupdateop",
    "HrisCompaniesUpdateRequest": ".hris_companiesupdateop",
    "HrisCompaniesUpdateRequestTypedDict": ".hris_companiesupdateop",
    "HrisCompaniesUpdateResponse": ".hris_companiesupdateop",
    "HrisCompaniesUpdateResponseTypedDict": ".hris_companiesupdateop",
    "HrisDepartmentsAddGlobals": ".hris_departmentsaddop",
    "HrisDepartmentsAddGlobalsTypedDict": ".hris_departmentsaddop",
    "HrisDepartmentsAddRequest": ".hris_departmentsaddop",
    "HrisDepartmentsAddRequestTypedDict": ".hris_departmentsaddop",
    "HrisDepartmentsAddResponse": ".hris_departmentsaddop",
    "HrisDepartmentsAddResponseTypedDict": ".hris_departmentsaddop",
    "HrisDepartmentsAllGlobals": ".hris_departmentsallop",
    "HrisDepartmentsAllGlobalsTypedDict": ".hris_departmentsallop",
    "HrisDepartmentsAllRequest": ".hris_departmentsallop",
    "HrisDepartmentsAllRequestTypedDict": ".hris_departmentsallop",
    "HrisDepartmentsAllResponse": ".hris_departmentsallop",
    "HrisDepartmentsAllResponseTypedDict": ".hris_departmentsallop",
    "HrisDepartmentsDeleteGlobals": ".hris_departmentsdeleteop",
    "HrisDepartmentsDeleteGlobalsTypedDict": ".hris_departmentsdeleteop",
    "HrisDepartmentsDeleteRequest": ".hris_departmentsdeleteop",
    "HrisDepartmentsDeleteRequestTypedDict": ".hris_departmentsdeleteop",
    "HrisDepartmentsDeleteResponse": ".hris_departmentsdeleteop",
    "HrisDepartmentsDeleteResponseTypedDict": ".hris_departmentsdeleteop",
    "HrisDepartmentsOneGlobals": ".hris_departmentsoneop",
    "HrisDepartmentsOneGlobalsTypedDict": ".hris_departmentsoneop",
    "HrisDepartmentsOneRequest": ".hris_departmentsoneop",
    "HrisDepartmentsOneRequestTypedDict": ".hris_departmentsoneop",
    "HrisDepartmentsOneResponse": ".hris_departmentsoneop",
    "HrisDepartmentsOneResponseTypedDict": ".hris_departmentsoneop",
    "HrisDepartmentsUpdateGlobals": ".hris_departmentsupdateop",
    "HrisDepartmentsUpdateGlobalsTypedDict": ".hris_departmentsupdateop",
    "HrisDepartmentsUpdateRequest": ".hris_departmentsupdateop",
    "HrisDepartmentsUpdateRequestTypedDict": ".hris_departmentsupdateop",
    "HrisDepartmentsUpdateResponse": ".hris_departmentsupdateop",
    "HrisDepartmentsUpdateResponseTypedDict": ".hris_departmentsupdateop",
    "HrisEmployeePayrollsAllGlobals": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsAllGlobalsTypedDict": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsAllRequest": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsAllRequestTypedDict": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsAllResponse": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsAllResponseTypedDict": ".hris_employeepayrollsallop",
    "HrisEmployeePayrollsOneGlobals": ".hris_employeepayrollsoneop",
    "HrisEmployeePayrollsOneGlobalsTypedDict": ".hris_employeepayrollsoneop",
    "HrisEmployeePayrollsOneRequest": ".hris_employeepayrollsoneop",
    "HrisEmployeePayrollsOneRequestTypedDict": ".hris_employeepayrollsoneop",
    "HrisEmployeePayrollsOneResponse": ".hris_employeepayrollsoneop",
    "HrisEmployeePayrollsOneResponseTypedDict": ".hris_employeepayrollsoneop",
    "HrisEmployeesAddGlobals": ".hris_employeesaddop",
    "HrisEmployeesAddGlobalsTypedDict": ".hris_employeesaddop",
    "HrisEmployeesAddRequest": ".hris_employeesaddop",
    "HrisEmployeesAddRequestTypedDict": ".hris_employeesaddop",
    "HrisEmployeesAddResponse": ".hris_employeesaddop",
    "HrisEmployeesAddResponseTypedDict": ".hris_employeesaddop",
    "HrisEmployeesAllGlobals": ".hris_employeesallop",
    "HrisEmployeesAllGlobalsTypedDict": ".hris_employeesallop",
    "HrisEmployeesAllRequest": ".hris_employeesallop",
    "HrisEmployeesAllRequestTypedDict": ".hris_employeesallop",
    "HrisEmployeesAllResponse": ".hris_employeesallop",
    "HrisEmployeesAllResponseTypedDict": ".hris_employeesallop",
    "HrisEmployeeSchedulesAllGlobals": ".hris_employeeschedulesallop",
    "HrisEmployeeSchedulesAllGlobalsTypedDict": ".hris_employeeschedulesallop",
    "HrisEmployeeSchedulesAllRequest": ".hris_employeeschedulesallop",
    "HrisEmployeeSchedulesAllRequestTypedDict": ".hris_employeeschedulesallop",
    "HrisEmployeeSchedulesAllResponse": ".hris_employeeschedulesallop",
    "HrisEmployeeSchedulesAllResponseTypedDict": ".hris_employeeschedulesallop",
    "HrisEmployeesDeleteGlobals": ".hris_employeesdeleteop",
    "HrisEmployeesDeleteGlobalsTypedDict": ".hris_employeesdeleteop",
    "HrisEmployeesDeleteRequest": ".hris_employeesdeleteop",
    "HrisEmployeesDeleteRequestTypedDict": ".hris_employeesdeleteop",
    "HrisEmployeesDeleteResponse": ".hris_employeesdeleteop",
    "HrisEmployeesDeleteResponseTypedDict": ".hris_employeesdeleteop",
    "HrisEmployeesOneGlobals": ".hris_employeesoneop",
    "HrisEmployeesOneGlobalsTypedDict": ".hris_employeesoneop",
    "HrisEmployeesOneRequest": ".hris_employeesoneop",
    "HrisEmployeesOneRequestTypedDict": ".hris_employeesoneop",
    "HrisEmployeesOneResponse": ".hris_employeesoneop",
    "HrisEmployeesOneResponseTypedDict": ".hris_employeesoneop",
    "HrisEmployeesUpdateGlobals": ".hris_employeesupdateop",
    "HrisEmployeesUpdateGlobalsTypedDict": ".hris_employeesupdateop",
    "HrisEmployeesUpdateRequest": ".hris_employeesupdateop",
    "HrisEmployeesUpdateRequestTypedDict": ".hris_employeesupdateop",
    "HrisEmployeesUpdateResponse": ".hris_employeesupdateop",
    "HrisEmployeesUpdateResponseTypedDict": ".hris_employeesupdateop",
    "HrisPayrollsAllGlobals": ".hris_payrollsallop",
    "HrisPayrollsAllGlobalsTypedDict": ".hris_payrollsallop",
    "HrisPayrollsAllRequest": ".hris_payrollsallop",
    "HrisPayrollsAllRequestTypedDict": ".hris_payrollsallop",
    "HrisPayrollsAllResponse": ".hris_payrollsallop",
    "HrisPayrollsAllResponseTypedDict": ".hris_payrollsallop",
    "HrisPayrollsOneGlobals": ".hris_payrollsoneop",
    "HrisPayrollsOneGlobalsTypedDict": ".hris_payrollsoneop",
    "HrisPayrollsOneRequest": ".hris_payrollsoneop",
    "HrisPayrollsOneRequestTypedDict": ".hris_payrollsoneop",
    "HrisPayrollsOneResponse": ".hris_payrollsoneop",
    "HrisPayrollsOneResponseTypedDict": ".hris_payrollsoneop",
    "HrisTimeOffRequestsAddGlobals": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAddGlobalsTypedDict": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAddRequest": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAddRequestTypedDict": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAddResponse": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAddResponseTypedDict": ".hris_timeoffrequestsaddop",
    "HrisTimeOffRequestsAllGlobals": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsAllGlobalsTypedDict": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsAllRequest": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsAllRequestTypedDict": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsAllResponse": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsAllResponseTypedDict": ".hris_timeoffrequestsallop",
    "HrisTimeOffRequestsDeleteGlobals": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsDeleteGlobalsTypedDict": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsDeleteRequest": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsDeleteRequestTypedDict": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsDeleteResponse": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsDeleteResponseTypedDict": ".hris_timeoffrequestsdeleteop",
    "HrisTimeOffRequestsOneGlobals": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsOneGlobalsTypedDict": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsOneRequest": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsOneRequestTypedDict": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsOneResponse": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsOneResponseTypedDict": ".hris_timeoffrequestsoneop",
    "HrisTimeOffRequestsUpdateGlobals": ".hris_timeoffrequestsupdateop",
    "HrisTimeOffRequestsUpdateGlobalsTypedDict": ".hris_timeoffrequestsupdateop",
    "HrisTimeOffRequestsUpdateRequest": ".hris_timeoffrequestsupdateop",
    "HrisTimeOffRequestsUpdateRequestTypedDict": ".hris_timeoffrequestsupdateop",
    "HrisTimeOffRequestsUpdateResponse": ".hris_timeoffrequestsupdateop",
    "HrisTimeOffRequestsUpdateResponseTypedDict": ".hris_timeoffrequestsupdateop",
    "HrisCompany": ".hriscompany",
    "HrisCompanyInput": ".hriscompany",
    "HrisCompanyInputTypedDict": ".hriscompany",
    "HrisCompanyStatus": ".hriscompany",
    "HrisCompanyTypedDict": ".hriscompany",
    "HTTPMetadata": ".httpmetadata",
    "HTTPMetadataTypedDict": ".httpmetadata",
    "IntegrationState": ".integrationstate",
    "Invoice": ".invoice",
    "InvoiceInput": ".invoice",
    "InvoiceInputTypedDict": ".invoice",
    "InvoiceStatus": ".invoice",
    "InvoiceType": ".invoice",
    "InvoiceTypedDict": ".invoice",
    "PaymentAllocations": ".invoice",
    "PaymentAllocationsTypedDict": ".invoice",
    "InvoiceItem": ".invoiceitem",
    "InvoiceItemInput": ".invoiceitem",
    "InvoiceItemInputTypedDict": ".invoiceitem",
    "InvoiceItemPurchaseDetails": ".invoiceitem",
    "InvoiceItemPurchaseDetailsTypedDict": ".invoiceitem",
    "InvoiceItemSalesDetails": ".invoiceitem",
    "InvoiceItemSalesDetailsTypedDict": ".invoiceitem",
    "InvoiceItemTypeType": ".invoiceitem",
    "InvoiceItemTypedDict": ".invoiceitem",
    "PurchaseDetails": ".invoiceitem",
    "PurchaseDetailsTypedDict": ".invoiceitem",
    "SalesDetails": ".invoiceitem",
    "SalesDetailsTypedDict": ".invoiceitem",
    "InvoiceItemFilter": ".invoiceitemfilter",
    "InvoiceItemFilterInvoiceItemType": ".invoiceitemfilter",
    "InvoiceItemFilterTypedDict": ".invoiceitemfilter",
    "InvoiceItemType": ".invoiceitemsfilter",
    "InvoiceItemsFilter": ".invoiceitemsfilter",
    "InvoiceItemsFilterTypedDict": ".invoiceitemsfilter",
    "InvoiceItemsSort": ".invoiceitemssort",
    "InvoiceItemsSortBy": ".invoiceitemssort",
    "InvoiceItemsSortTypedDict": ".invoiceitemssort",
    "InvoiceLineItem": ".invoicelineitem",
    "InvoiceLineItemInput": ".invoicelineitem",
    "InvoiceLineItemInputTypedDict": ".invoicelineitem",
    "InvoiceLineItemType": ".invoicelineitem",
    "InvoiceLineItemTypedDict": ".invoicelineitem",
    "InvoiceResponse": ".invoiceresponse",
    "InvoiceResponseTypedDict": ".invoiceresponse",
    "InvoicesFilter": ".invoicesfilter",
    "InvoicesFilterTypedDict": ".invoicesfilter",
    "InvoicesSort": ".invoicessort",
    "InvoicesSortBy": ".invoicessort",
    "InvoicesSortTypedDict": ".invoicessort",
    "IssuesFilter": ".issuesfilter",
    "IssuesFilterTypedDict": ".issuesfilter",
    "IssueTrackingCollectionsAllGlobals": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsAllGlobalsTypedDict": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsAllRequest": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsAllRequestTypedDict": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsAllResponse": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsAllResponseTypedDict": ".issuetracking_collectionsallop",
    "IssueTrackingCollectionsOneGlobals": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionsOneGlobalsTypedDict": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionsOneRequest": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionsOneRequestTypedDict": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionsOneResponse": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionsOneResponseTypedDict": ".issuetracking_collectionsoneop",
    "IssueTrackingCollectionTagsAllGlobals": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTagsAllGlobalsTypedDict": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTagsAllRequest": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTagsAllRequestTypedDict": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTagsAllResponse": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTagsAllResponseTypedDict": ".issuetracking_collectiontagsallop",
    "IssueTrackingCollectionTicketCommentsAddGlobals": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAddGlobalsTypedDict": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAddRequest": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAddRequestTypedDict": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAddResponse": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAddResponseTypedDict": ".issuetracking_collectionticketcommentsaddop",
    "IssueTrackingCollectionTicketCommentsAllGlobals": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsAllGlobalsTypedDict": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsAllRequest": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsAllRequestTypedDict": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsAllResponse": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsAllResponseTypedDict": ".issuetracking_collectionticketcommentsallop",
    "IssueTrackingCollectionTicketCommentsDeleteGlobals": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsDeleteGlobalsTypedDict": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsDeleteRequest": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsDeleteRequestTypedDict": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsDeleteResponse": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsDeleteResponseTypedDict": ".issuetracking_collectionticketcommentsdeleteop",
    "IssueTrackingCollectionTicketCommentsOneGlobals": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsOneGlobalsTypedDict": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsOneRequest": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsOneRequestTypedDict": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsOneResponse": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsOneResponseTypedDict": ".issuetracking_collectionticketcommentsoneop",
    "IssueTrackingCollectionTicketCommentsUpdateGlobals": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketCommentsUpdateGlobalsTypedDict": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketCommentsUpdateRequest": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketCommentsUpdateRequestTypedDict": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketCommentsUpdateResponse": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketCommentsUpdateResponseTypedDict": ".issuetracking_collectionticketcommentsupdateop",
    "IssueTrackingCollectionTicketsAddGlobals": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAddGlobalsTypedDict": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAddRequest": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAddRequestTypedDict": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAddResponse": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAddResponseTypedDict": ".issuetracking_collectionticketsaddop",
    "IssueTrackingCollectionTicketsAllGlobals": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsAllGlobalsTypedDict": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsAllRequest": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsAllRequestTypedDict": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsAllResponse": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsAllResponseTypedDict": ".issuetracking_collectionticketsallop",
    "IssueTrackingCollectionTicketsDeleteGlobals": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsDeleteGlobalsTypedDict": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsDeleteRequest": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsDeleteRequestTypedDict": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsDeleteResponse": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsDeleteResponseTypedDict": ".issuetracking_collectionticketsdeleteop",
    "IssueTrackingCollectionTicketsOneGlobals": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsOneGlobalsTypedDict": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsOneRequest": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsOneRequestTypedDict": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsOneResponse": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsOneResponseTypedDict": ".issuetracking_collectionticketsoneop",
    "IssueTrackingCollectionTicketsUpdateGlobals": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionTicketsUpdateGlobalsTypedDict": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionTicketsUpdateRequest": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionTicketsUpdateRequestTypedDict": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionTicketsUpdateResponse": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionTicketsUpdateResponseTypedDict": ".issuetracking_collectionticketsupdateop",
    "IssueTrackingCollectionUsersAllGlobals": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersAllGlobalsTypedDict": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersAllRequest": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersAllRequestTypedDict": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersAllResponse": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersAllResponseTypedDict": ".issuetracking_collectionusersallop",
    "IssueTrackingCollectionUsersOneGlobals": ".issuetracking_collectionusersoneop",
    "IssueTrackingCollectionUsersOneGlobalsTypedDict": ".issuetracking_collectionusersoneop",
    "IssueTrackingCollectionUsersOneRequest": ".issuetracking_collectionusersoneop",
    "IssueTrackingCollectionUsersOneRequestTypedDict": ".issuetracking_collectionusersoneop",
    "IssueTrackingCollectionUsersOneResponse": ".issuetracking_collectionusersoneop",
    "IssueTrackingCollectionUsersOneResponseTypedDict": ".issuetracking_collectionusersoneop",
    "Blocks": ".job",
    "BlocksTypedDict": ".job",
    "Branch": ".job",
    "BranchTypedDict": ".job",
    "EmploymentTerms": ".job",
    "Job": ".job",
    "JobLinks": ".job",
    "JobLinksTypedDict": ".job",
    "JobType": ".job",
    "JobTypedDict": ".job",
    "Salary": ".job",
    "SalaryTypedDict": ".job",
    "Visibility": ".job",
    "JobStatus": ".jobstatus",
    "JournalEntriesFilter": ".journalentriesfilter",
    "JournalEntriesFilterTypedDict": ".journalentriesfilter",
    "JournalEntriesSort": ".journalentriessort",
    "JournalEntriesSortBy": ".journalentriessort",
    "JournalEntriesSortTypedDict": ".journalentriessort",
    "JournalEntry": ".journalentry",
    "JournalEntryInput": ".journalentry",
    "JournalEntryInputTypedDict": ".journalentry",
    "JournalEntryStatus": ".journalentry",
    "JournalEntryTypedDict": ".journalentry",
    "JournalEntryLineItem": ".journalentrylineitem",
    "JournalEntryLineItemInput": ".journalentrylineitem",
    "JournalEntryLineItemInputTypedDict": ".journalentrylineitem",
    "JournalEntryLineItemType": ".journalentrylineitem",
    "JournalEntryLineItemTypedDict": ".journalentrylineitem",
    "Lead": ".lead",
    "LeadTypedDict": ".lead",
    "LeadInput": ".lead_input",
    "LeadInputTypedDict": ".lead_input",
    "LeadsFilter": ".leadsfilter",
    "LeadsFilterTypedDict": ".leadsfilter",
    "LeadsSort": ".leadssort",
    "LeadsSortBy": ".leadssort",
    "LeadsSortTypedDict": ".leadssort",
    "AccountStatus": ".ledgeraccount",
    "CategoriesModel": ".ledgeraccount",
    "CategoriesModelTypedDict": ".ledgeraccount",
    "LedgerAccount": ".ledgeraccount",
    "LedgerAccountClassification": ".ledgeraccount",
    "LedgerAccountInput": ".ledgeraccount",
    "LedgerAccountInputTypedDict": ".ledgeraccount",
    "LedgerAccountSubsidiaries": ".ledgeraccount",
    "LedgerAccountSubsidiariesTypedDict": ".ledgeraccount",
    "LedgerAccountType": ".ledgeraccount",
    "LedgerAccountTypedDict": ".ledgeraccount",
    "ParentAccount": ".ledgeraccount",
    "ParentAccountTypedDict": ".ledgeraccount",
    "SubAccounts": ".ledgeraccount",
    "SubAccountsTypedDict": ".ledgeraccount",
    "Classification": ".ledgeraccountsfilter",
    "LedgerAccountsFilter": ".ledgeraccountsfilter",
    "LedgerAccountsFilterTypedDict": ".ledgeraccountsfilter",
    "LedgerAccountsSort": ".ledgeraccountssort",
    "LedgerAccountsSortBy": ".ledgeraccountssort",
    "LedgerAccountsSortTypedDict": ".ledgeraccountssort",
    "LinkedAttachment": ".linkedattachment",
    "LinkedAttachmentTypedDict": ".linkedattachment",
    "LinkedConnectorResource": ".linkedconnectorresource",
    "LinkedConnectorResourceTypedDict": ".linkedconnectorresource",
    "LinkedCustomer": ".linkedcustomer",
    "LinkedCustomerTypedDict": ".linkedcustomer",
    "LinkedCustomerInput": ".linkedcustomer_input",
    "LinkedCustomerInputTypedDict": ".linkedcustomer_input",
    "LinkedEcommerceCustomer": ".linkedecommercecustomer",
    "LinkedEcommerceCustomerTypedDict": ".linkedecommercecustomer",
    "LinkedEcommerceOrder": ".linkedecommerceorder",
    "LinkedEcommerceOrderTypedDict": ".linkedecommerceorder",
    "LinkedFolder": ".linkedfolder",
    "LinkedFolderTypedDict": ".linkedfolder",
    "LinkedInvoiceItem": ".linkedinvoiceitem",
    "LinkedInvoiceItemTypedDict": ".linkedinvoiceitem",
    "LinkedLedgerAccount": ".linkedledgeraccount",
    "LinkedLedgerAccountTypedDict": ".linkedledgeraccount",
    "LinkedLedgerAccountInput": ".linkedledgeraccount_input",
    "LinkedLedgerAccountInputTypedDict": ".linkedledgeraccount_input",
    "LinkedParentCustomer": ".linkedparentcustomer",
    "LinkedParentCustomerTypedDict": ".linkedparentcustomer",
    "LinkedPurchaseOrder": ".linkedpurchaseorder",
    "LinkedPurchaseOrderTypedDict": ".linkedpurchaseorder",
    "LinkedSupplier": ".linkedsupplier",
    "LinkedSupplierTypedDict": ".linkedsupplier",
    "LinkedSupplierInput": ".linkedsupplier_input",
    "LinkedSupplierInputTypedDict": ".linkedsupplier_input",
    "LinkedTaxDetail": ".linkedtaxdetail",
    "LinkedTaxDetailTypedDict": ".linkedtaxdetail",
    "LinkedTaxRate": ".linkedtaxrate",
    "LinkedTaxRateTypedDict": ".linkedtaxrate",
    "LinkedTaxRateInput": ".linkedtaxrate_input",
    "LinkedTaxRateInputTypedDict": ".linkedtaxrate_input",
    "LinkedTaxStatusDetail": ".linkedtaxstatusdetail",
    "LinkedTaxStatusDetailTypedDict": ".linkedtaxstatusdetail",
    "LinkedTrackingCategory": ".linkedtrackingcategory",
    "LinkedTrackingCategoryTypedDict": ".linkedtrackingcategory",
    "LinkedWorktag": ".linkedworktag",
    "LinkedWorktagTypedDict": ".linkedworktag",
    "Links": ".links",
    "LinksTypedDict": ".links",
    "Log": ".log",
    "LogTypedDict": ".log",
    "Operation": ".log",
    "OperationTypedDict": ".log",
    "Service": ".log",
    "ServiceTypedDict": ".log",
    "UnifiedAPI": ".log",
    "LogsFilter": ".logsfilter",
    "LogsFilterTypedDict": ".logsfilter",
    "Direction": ".message",
    "Error": ".message",
    "ErrorTypedDict": ".message",
    "Message": ".message",
    "MessageInput": ".message",
    "MessageInputTypedDict": ".message",
    "MessageStatus": ".message",
    "MessageType": ".message",
    "MessageTypedDict": ".message",
    "Price": ".message",
    "PriceTypedDict": ".message",
    "Cursors": ".meta",
    "CursorsTypedDict": ".meta",
    "Meta": ".meta",
    "MetaTypedDict": ".meta",
    "NoResponseError": ".no_response_error",
    "Note": ".note",
    "NoteTypedDict": ".note",
    "NoteInput": ".note_input",
    "NoteInputTypedDict": ".note_input",
    "NotFoundResponse": ".notfoundresponse",
    "NotFoundResponseData": ".notfoundresponse",
    "NotFoundResponseDetail": ".notfoundresponse",
    "OAuthGrantType": ".oauthgranttype",
    "OpportunitiesFilter": ".opportunitiesfilter",
    "OpportunitiesFilterTypedDict": ".opportunitiesfilter",
    "OpportunitiesSort": ".opportunitiessort",
    "OpportunitiesSortBy": ".opportunitiessort",
    "OpportunitiesSortTypedDict": ".opportunitiessort",
    "Opportunity": ".opportunity",
    "OpportunityTypedDict": ".opportunity",
    "OpportunityInput": ".opportunity_input",
    "OpportunityInputTypedDict": ".opportunity_input",
    "OrdersSort": ".orderssort",
    "OrdersSortBy": ".orderssort",
    "OrdersSortTypedDict": ".orderssort",
    "OutstandingBalanceByCurrency": ".outstandingbalancebycurrency",
    "OutstandingBalanceByCurrencyTypedDict": ".outstandingbalancebycurrency",
    "OutstandingBalanceByCustomer": ".outstandingbalancebycustomer",
    "OutstandingBalanceByCustomerTypedDict": ".outstandingbalancebycustomer",
    "OutstandingBalanceBySupplier": ".outstandingbalancebysupplier",
    "OutstandingBalanceBySupplierTypedDict": ".outstandingbalancebysupplier",
    "Owner": ".owner",
    "OwnerTypedDict": ".owner",
    "PaginationCoverage": ".paginationcoverage",
    "PaginationCoverageMode": ".paginationcoverage",
    "PaginationCoverageTypedDict": ".paginationcoverage",
    "ExtendPaths": ".passthroughbody",
    "ExtendPathsTypedDict": ".passthroughbody",
    "PassThroughBody": ".passthroughbody",
    "PassThroughBodyTypedDict": ".passthroughbody",
    "Payment": ".payment",
    "PaymentTypedDict": ".payment",
    "PaymentInput": ".payment_input",
    "PaymentInputTypedDict": ".payment_input",
    "PaymentFrequency": ".paymentfrequency",
    "PaymentRequiredResponse": ".paymentrequiredresponse",
    "PaymentRequiredResponseData": ".paymentrequiredresponse",
    "PaymentsFilter": ".paymentsfilter",
    "PaymentsFilterTypedDict": ".paymentsfilter",
    "PaymentsSort": ".paymentssort",
    "PaymentsSortBy": ".paymentssort",
    "PaymentsSortTypedDict": ".paymentssort",
    "PaymentStatus": ".paymentstatus",
    "PaymentType": ".paymenttype",
    "PaymentUnit": ".paymentunit",
    "Payroll": ".payroll",
    "PayrollTypedDict": ".payroll",
    "PayrollsFilter": ".payrollsfilter",
    "PayrollsFilterTypedDict": ".payrollsfilter",
    "PayrollTotals": ".payrolltotals",
    "PayrollTotalsTypedDict": ".payrolltotals",
    "Person": ".person",
    "PersonTypedDict": ".person",
    "PersonInput": ".person_input",
    "PersonInputTypedDict": ".person_input",
    "PhoneNumber": ".phonenumber",
    "PhoneNumberType": ".phonenumber",
    "PhoneNumberTypedDict": ".phonenumber",
    "Pipeline": ".pipeline",
    "PipelineTypedDict": ".pipeline",
    "Stages": ".pipeline",
    "StagesTypedDict": ".pipeline",
    "PipelineInput": ".pipeline_input",
    "PipelineInputTypedDict": ".pipeline_input",
    "PipelineStages": ".pipeline_input",
    "PipelineStagesTypedDict": ".pipeline_input",
    "CostOfGoodsSold": ".profitandloss",
    "CostOfGoodsSoldTypedDict": ".profitandloss",
    "ExpensesModel": ".profitandloss",
    "ExpensesModelTypedDict": ".profitandloss",
    "Income": ".profitandloss",
    "IncomeTypedDict": ".profitandloss",
    "OtherExpenses": ".profitandloss",
    "OtherExpensesTypedDict": ".profitandloss",
    "OtherIncome": ".profitandloss",
    "OtherIncomeTypedDict": ".profitandloss",
    "ProfitAndLoss": ".profitandloss",
    "ProfitAndLossTypedDict": ".profitandloss",
    "UncategorizedAccounts": ".profitandloss",
    "UncategorizedAccountsTypedDict": ".profitandloss",
    "ProfitAndLossFilter": ".profitandlossfilter",
    "ProfitAndLossFilterTypedDict": ".profitandlossfilter",
    "ProfitAndLossIndicator": ".profitandlossindicator",
    "ProfitAndLossIndicatorTypedDict": ".profitandlossindicator",
    "ProfitAndLossType": ".profitandlosstype",
    "PurchaseOrder": ".purchaseorder",
    "PurchaseOrderAmortizationType": ".purchaseorder",
    "PurchaseOrderInput": ".purchaseorder",
    "PurchaseOrderInputTypedDict": ".purchaseorder",
    "PurchaseOrderStatus": ".purchaseorder",
    "PurchaseOrderTypedDict": ".purchaseorder",
    "PurchaseOrdersFilter": ".purchaseordersfilter",
    "PurchaseOrdersFilterTypedDict": ".purchaseordersfilter",
    "PurchaseOrdersSort": ".purchaseorderssort",
    "PurchaseOrdersSortBy": ".purchaseorderssort",
    "PurchaseOrdersSortTypedDict": ".purchaseorderssort",
    "Quote": ".quote",
    "QuoteInput": ".quote",
    "QuoteInputTypedDict": ".quote",
    "QuoteStatus": ".quote",
    "QuoteTypedDict": ".quote",
    "QuoteLineItem": ".quotelineitem",
    "QuoteLineItemInput": ".quotelineitem",
    "QuoteLineItemInputTypedDict": ".quotelineitem",
    "QuoteLineItemType": ".quotelineitem",
    "QuoteLineItemTypedDict": ".quotelineitem",
    "RebillStatus": ".rebilling",
    "Rebilling": ".rebilling",
    "RebillingTypedDict": ".rebilling",
    "RequestCountAllocation": ".requestcountallocation",
    "RequestCountAllocationTypedDict": ".requestcountallocation",
    "ResourceStatus": ".resourcestatus",
    "ResponseValidationError": ".responsevalidationerror",
    "EvenWeeks": ".schedule",
    "EvenWeeksTypedDict": ".schedule",
    "OddWeeks": ".schedule",
    "OddWeeksTypedDict": ".schedule",
    "Schedule": ".schedule",
    "ScheduleTypedDict": ".schedule",
    "WorkPattern": ".schedule",
    "WorkPatternTypedDict": ".schedule",
    "SchemaSupport": ".schemasupport",
    "SchemaSupportTypedDict": ".schemasupport",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "AllowActions": ".session",
    "Session": ".session",
    "SessionTypedDict": ".session",
    "Settings": ".session",
    "SettingsTypedDict": ".session",
    "Theme": ".session",
    "ThemeTypedDict": ".session",
    "Scope": ".sharedlink",
    "SharedLink": ".sharedlink",
    "SharedLinkInput": ".sharedlink",
    "SharedLinkInputTypedDict": ".sharedlink",
    "SharedLinkTypedDict": ".sharedlink",
    "SharedLinkTarget": ".sharedlinktarget",
    "SharedLinkTargetTypedDict": ".sharedlinktarget",
    "OptionType": ".simpleformfieldoption",
    "SimpleFormFieldOption": ".simpleformfieldoption",
    "SimpleFormFieldOptionTypedDict": ".simpleformfieldoption",
    "SimpleFormFieldOptionValue": ".simpleformfieldoption",
    "SimpleFormFieldOptionValueTypedDict": ".simpleformfieldoption",
    "Value5": ".simpleformfieldoption",
    "Value5TypedDict": ".simpleformfieldoption",
    "SmsMessagesAddGlobals": ".sms_messagesaddop",
    "SmsMessagesAddGlobalsTypedDict": ".sms_messagesaddop",
    "SmsMessagesAddRequest": ".sms_messagesaddop",
    "SmsMessagesAddRequestTypedDict": ".sms_messagesaddop",
    "SmsMessagesAddResponse": ".sms_messagesaddop",
    "SmsMessagesAddResponseTypedDict": ".sms_messagesaddop",
    "SmsMessagesAllGlobals": ".sms_messagesallop",
    "SmsMessagesAllGlobalsTypedDict": ".sms_messagesallop",
    "SmsMessagesAllRequest": ".sms_messagesallop",
    "SmsMessagesAllRequestTypedDict": ".sms_messagesallop",
    "SmsMessagesAllResponse": ".sms_messagesallop",
    "SmsMessagesAllResponseTypedDict": ".sms_messagesallop",
    "SmsMessagesDeleteGlobals": ".sms_messagesdeleteop",
    "SmsMessagesDeleteGlobalsTypedDict": ".sms_messagesdeleteop",
    "SmsMessagesDeleteRequest": ".sms_messagesdeleteop",
    "SmsMessagesDeleteRequestTypedDict": ".sms_messagesdeleteop",
    "SmsMessagesDeleteResponse": ".sms_messagesdeleteop",
    "SmsMessagesDeleteResponseTypedDict": ".sms_messagesdeleteop",
    "SmsMessagesOneGlobals": ".sms_messagesoneop",
    "SmsMessagesOneGlobalsTypedDict": ".sms_messagesoneop",
    "SmsMessagesOneRequest": ".sms_messagesoneop",
    "SmsMessagesOneRequestTypedDict": ".sms_messagesoneop",
    "SmsMessagesOneResponse": ".sms_messagesoneop",
    "SmsMessagesOneResponseTypedDict": ".sms_messagesoneop",
    "SmsMessagesUpdateGlobals": ".sms_messagesupdateop",
    "SmsMessagesUpdateGlobalsTypedDict": ".sms_messagesupdateop",
    "SmsMessagesUpdateRequest": ".sms_messagesupdateop",
    "SmsMessagesUpdateRequestTypedDict": ".sms_messagesupdateop",
    "SmsMessagesUpdateResponse": ".sms_messagesupdateop",
    "SmsMessagesUpdateResponseTypedDict": ".sms_messagesupdateop",
    "SocialLink": ".sociallink",
    "SocialLinkTypedDict": ".sociallink",
    "SortDirection": ".sortdirection",
    "Status": ".status",
    "Subsidiary": ".subsidiary",
    "SubsidiaryInput": ".subsidiary",
    "SubsidiaryInputTypedDict": ".subsidiary",
    "SubsidiaryStatus": ".subsidiary",
    "SubsidiaryTypedDict": ".subsidiary",
    "SubsidiaryReference": ".subsidiaryreference",
    "SubsidiaryReferenceTypedDict": ".subsidiaryreference",
    "SubsidiaryReferenceInput": ".subsidiaryreference_input",
    "SubsidiaryReferenceInputTypedDict": ".subsidiaryreference_input",
    "Supplier": ".supplier",
    "SupplierInput": ".supplier",
    "SupplierInputTypedDict": ".supplier",
    "SupplierStatus": ".supplier",
    "SupplierTypedDict": ".supplier",
    "SuppliersFilter": ".suppliersfilter",
    "SuppliersFilterTypedDict": ".suppliersfilter",
    "SuppliersSort": ".supplierssort",
    "SuppliersSortBy": ".supplierssort",
    "SuppliersSortTypedDict": ".supplierssort",
    "SupportedProperty": ".supportedproperty",
    "SupportedPropertyTypedDict": ".supportedproperty",
    "Tax": ".tax",
    "TaxTypedDict": ".tax",
    "Components": ".taxrate",
    "ComponentsTypedDict": ".taxrate",
    "SubsidiariesModel": ".taxrate",
    "SubsidiariesModelTypedDict": ".taxrate",
    "TaxRate": ".taxrate",
    "TaxRateInput": ".taxrate",
    "TaxRateInputTypedDict": ".taxrate",
    "TaxRateStatus": ".taxrate",
    "TaxRateTypedDict": ".taxrate",
    "TaxRatesFilter": ".taxratesfilter",
    "TaxRatesFilterTypedDict": ".taxratesfilter",
    "Team": ".team",
    "TeamTypedDict": ".team",
    "Priority": ".ticket",
    "Ticket": ".ticket",
    "TicketInput": ".ticket",
    "TicketInputTypedDict": ".ticket",
    "TicketTypedDict": ".ticket",
    "TicketsSort": ".ticketssort",
    "TicketsSortBy": ".ticketssort",
    "TicketsSortTypedDict": ".ticketssort",
    "NotesModel": ".timeoffrequest",
    "NotesModelTypedDict": ".timeoffrequest",
    "RequestType": ".timeoffrequest",
    "TimeOffRequest": ".timeoffrequest",
    "TimeOffRequestInput": ".timeoffrequest",
    "TimeOffRequestInputTypedDict": ".timeoffrequest",
    "TimeOffRequestStatusStatus": ".timeoffrequest",
    "TimeOffRequestTypedDict": ".timeoffrequest",
    "Units": ".timeoffrequest",
    "TimeOffRequestStatus": ".timeoffrequestsfilter",
    "TimeOffRequestsFilter": ".timeoffrequestsfilter",
    "TimeOffRequestsFilterTypedDict": ".timeoffrequestsfilter",
    "TrackingCategory": ".trackingcategory",
    "TrackingCategoryInput": ".trackingcategory",
    "TrackingCategoryInputTypedDict": ".trackingcategory",
    "TrackingCategoryStatus": ".trackingcategory",
    "TrackingCategorySubsidiaries": ".trackingcategory",
    "TrackingCategorySubsidiariesTypedDict": ".trackingcategory",
    "TrackingCategoryTypedDict": ".trackingcategory",
    "TrackingItem": ".trackingitem",
    "TrackingItemTypedDict": ".trackingitem",
    "UnauthorizedResponse": ".unauthorizedresponse",
    "UnauthorizedResponseData": ".unauthorizedresponse",
    "UnauthorizedResponseDetail": ".unauthorizedresponse",
    "Detail": ".unexpectederrorresponse",
    "DetailTypedDict": ".unexpectederrorresponse",
    "UnexpectedErrorResponse": ".unexpectederrorresponse",
    "UnexpectedErrorResponseTypedDict": ".unexpectederrorresponse",
    "UnifiedAPIID": ".unifiedapiid",
    "Permissions": ".unifiedfile",
    "PermissionsTypedDict": ".unifiedfile",
    "UnifiedFile": ".unifiedfile",
    "UnifiedFileTypedDict": ".unifiedfile",
    "UnifiedID": ".unifiedid",
    "UnifiedIDTypedDict": ".unifiedid",
    "UnprocessableResponse": ".unprocessableresponse",
    "UnprocessableResponseData": ".unprocessableresponse",
    "UnprocessableResponseDetail": ".unprocessableresponse",
    "UpdateAccountingDepartmentResponse": ".updateaccountingdepartmentresponse",
    "UpdateAccountingDepartmentResponseTypedDict": ".updateaccountingdepartmentresponse",
    "UpdateAccountingLocationResponse": ".updateaccountinglocationresponse",
    "UpdateAccountingLocationResponseTypedDict": ".updateaccountinglocationresponse",
    "UpdateActivityResponse": ".updateactivityresponse",
    "UpdateActivityResponseTypedDict": ".updateactivityresponse",
    "UpdateApplicantResponse": ".updateapplicantresponse",
    "UpdateApplicantResponseTypedDict": ".updateapplicantresponse",
    "UpdateApplicationResponse": ".updateapplicationresponse",
    "UpdateApplicationResponseTypedDict": ".updateapplicationresponse",
    "UpdateBankFeedAccountResponse": ".updatebankfeedaccountresponse",
    "UpdateBankFeedAccountResponseTypedDict": ".updatebankfeedaccountresponse",
    "UpdateBankFeedStatementResponse": ".updatebankfeedstatementresponse",
    "UpdateBankFeedStatementResponseTypedDict": ".updatebankfeedstatementresponse",
    "UpdateBillPaymentResponse": ".updatebillpaymentresponse",
    "UpdateBillPaymentResponseTypedDict": ".updatebillpaymentresponse",
    "UpdateBillResponse": ".updatebillresponse",
    "UpdateBillResponseTypedDict": ".updatebillresponse",
    "UpdateCommentResponse": ".updatecommentresponse",
    "UpdateCommentResponseTypedDict": ".updatecommentresponse",
    "UpdateCompanyResponse": ".updatecompanyresponse",
    "UpdateCompanyResponseTypedDict": ".updatecompanyresponse",
    "UpdateConnectionResponse": ".updateconnectionresponse",
    "UpdateConnectionResponseTypedDict": ".updateconnectionresponse",
    "UpdateConsumerRequest": ".updateconsumerrequest",
    "UpdateConsumerRequestTypedDict": ".updateconsumerrequest",
    "UpdateConsumerResponse": ".updateconsumerresponse",
    "UpdateConsumerResponseTypedDict": ".updateconsumerresponse",
    "UpdateContactResponse": ".updatecontactresponse",
    "UpdateContactResponseTypedDict": ".updatecontactresponse",
    "UpdateCreditNoteResponse": ".updatecreditnoteresponse",
    "UpdateCreditNoteResponseTypedDict": ".updatecreditnoteresponse",
    "UpdateCustomerResponse": ".updatecustomerresponse",
    "UpdateCustomerResponseTypedDict": ".updatecustomerresponse",
    "UpdateCustomObjectResponse": ".updatecustomobjectresponse",
    "UpdateCustomObjectResponseTypedDict": ".updatecustomobjectresponse",
    "UpdateCustomObjectSchemaResponse": ".updatecustomobjectschemaresponse",
    "UpdateCustomObjectSchemaResponseTypedDict": ".updatecustomobjectschemaresponse",
    "UpdateDepartmentResponse": ".updatedepartmentresponse",
    "UpdateDepartmentResponseTypedDict": ".updatedepartmentresponse",
    "UpdateDriveGroupResponse": ".updatedrivegroupresponse",
    "UpdateDriveGroupResponseTypedDict": ".updatedrivegroupresponse",
    "UpdateDriveResponse": ".updatedriveresponse",
    "UpdateDriveResponseTypedDict": ".updatedriveresponse",
    "UpdateEmployeeResponse": ".updateemployeeresponse",
    "UpdateEmployeeResponseTypedDict": ".updateemployeeresponse",
    "UpdateExpenseResponse": ".updateexpenseresponse",
    "UpdateExpenseResponseTypedDict": ".updateexpenseresponse",
    "UpdateFileRequest": ".updatefilerequest",
    "UpdateFileRequestTypedDict": ".updatefilerequest",
    "UpdateFileResponse": ".updatefileresponse",
    "UpdateFileResponseTypedDict": ".updatefileresponse",
    "UpdateFolderRequest": ".updatefolderrequest",
    "UpdateFolderRequestTypedDict": ".updatefolderrequest",
    "UpdateFolderResponse": ".updatefolderresponse",
    "UpdateFolderResponseTypedDict": ".updatefolderresponse",
    "UpdateHrisCompanyResponse": ".updatehriscompanyresponse",
    "UpdateHrisCompanyResponseTypedDict": ".updatehriscompanyresponse",
    "UpdateInvoiceItemsResponse": ".updateinvoiceitemsresponse",
    "UpdateInvoiceItemsResponseTypedDict": ".updateinvoiceitemsresponse",
    "UpdateInvoiceResponse": ".updateinvoiceresponse",
    "UpdateInvoiceResponseTypedDict": ".updateinvoiceresponse",
    "UpdateJournalEntryResponse": ".updatejournalentryresponse",
    "UpdateJournalEntryResponseTypedDict": ".updatejournalentryresponse",
    "UpdateLeadResponse": ".updateleadresponse",
    "UpdateLeadResponseTypedDict": ".updateleadresponse",
    "UpdateLedgerAccountResponse": ".updateledgeraccountresponse",
    "UpdateLedgerAccountResponseTypedDict": ".updateledgeraccountresponse",
    "UpdateMessageResponse": ".updatemessageresponse",
    "UpdateMessageResponseTypedDict": ".updatemessageresponse",
    "UpdateNoteResponse": ".updatenoteresponse",
    "UpdateNoteResponseTypedDict": ".updatenoteresponse",
    "UpdateOpportunityResponse": ".updateopportunityresponse",
    "UpdateOpportunityResponseTypedDict": ".updateopportunityresponse",
    "UpdatePaymentResponse": ".updatepaymentresponse",
    "UpdatePaymentResponseTypedDict": ".updatepaymentresponse",
    "UpdatePipelineResponse": ".updatepipelineresponse",
    "UpdatePipelineResponseTypedDict": ".updatepipelineresponse",
    "UpdatePurchaseOrderResponse": ".updatepurchaseorderresponse",
    "UpdatePurchaseOrderResponseTypedDict": ".updatepurchaseorderresponse",
    "UpdateQuoteResponse": ".updatequoteresponse",
    "UpdateQuoteResponseTypedDict": ".updatequoteresponse",
    "UpdateSharedLinkResponse": ".updatesharedlinkresponse",
    "UpdateSharedLinkResponseTypedDict": ".updatesharedlinkresponse",
    "UpdateSubsidiaryResponse": ".updatesubsidiaryresponse",
    "UpdateSubsidiaryResponseTypedDict": ".updatesubsidiaryresponse",
    "UpdateSupplierResponse": ".updatesupplierresponse",
    "UpdateSupplierResponseTypedDict": ".updatesupplierresponse",
    "UpdateTaxRateResponse": ".updatetaxrateresponse",
    "UpdateTaxRateResponseTypedDict": ".updatetaxrateresponse",
    "UpdateTicketResponse": ".updateticketresponse",
    "UpdateTicketResponseTypedDict": ".updateticketresponse",
    "UpdateTimeOffRequestResponse": ".updatetimeoffrequestresponse",
    "UpdateTimeOffRequestResponseTypedDict": ".updatetimeoffrequestresponse",
    "UpdateTrackingCategoryResponse": ".updatetrackingcategoryresponse",
    "UpdateTrackingCategoryResponseTypedDict": ".updatetrackingcategoryresponse",
    "UpdateUploadSessionResponse": ".updateuploadsessionresponse",
    "UpdateUploadSessionResponseTypedDict": ".updateuploadsessionresponse",
    "UpdateUserResponse": ".updateuserresponse",
    "UpdateUserResponseTypedDict": ".updateuserresponse",
    "UpdateWebhookRequest": ".updatewebhookrequest",
    "UpdateWebhookRequestTypedDict": ".updatewebhookrequest",
    "UpdateWebhookResponse": ".updatewebhookresponse",
    "UpdateWebhookResponseTypedDict": ".updatewebhookresponse",
    "UploadSession": ".uploadsession",
    "UploadSessionTypedDict": ".uploadsession",
    "User": ".user",
    "UserTypedDict": ".user",
    "UserInput": ".user_input",
    "UserInputTypedDict": ".user_input",
    "ValidateConnectionStateResponse": ".validateconnectionstateresponse",
    "ValidateConnectionStateResponseData": ".validateconnectionstateresponse",
    "ValidateConnectionStateResponseDataTypedDict": ".validateconnectionstateresponse",
    "ValidateConnectionStateResponseTypedDict": ".validateconnectionstateresponse",
    "VaultConnectionCustomMappingsAllGlobals": ".vault_connectioncustommappingsallop",
    "VaultConnectionCustomMappingsAllGlobalsTypedDict": ".vault_connectioncustommappingsallop",
    "VaultConnectionCustomMappingsAllRequest": ".vault_connectioncustommappingsallop",
    "VaultConnectionCustomMappingsAllRequestTypedDict": ".vault_connectioncustommappingsallop",
    "VaultConnectionCustomMappingsAllResponse": ".vault_connectioncustommappingsallop",
    "VaultConnectionCustomMappingsAllResponseTypedDict": ".vault_connectioncustommappingsallop",
    "VaultConnectionsAllGlobals": ".vault_connectionsallop",
    "VaultConnectionsAllGlobalsTypedDict": ".vault_connectionsallop",
    "VaultConnectionsAllRequest": ".vault_connectionsallop",
    "VaultConnectionsAllRequestTypedDict": ".vault_connectionsallop",
    "VaultConnectionsAllResponse": ".vault_connectionsallop",
    "VaultConnectionsAllResponseTypedDict": ".vault_connectionsallop",
    "VaultConnectionsDeleteGlobals": ".vault_connectionsdeleteop",
    "VaultConnectionsDeleteGlobalsTypedDict": ".vault_connectionsdeleteop",
    "VaultConnectionsDeleteRequest": ".vault_connectionsdeleteop",
    "VaultConnectionsDeleteRequestTypedDict": ".vault_connectionsdeleteop",
    "VaultConnectionsDeleteResponse": ".vault_connectionsdeleteop",
    "VaultConnectionsDeleteResponseTypedDict": ".vault_connectionsdeleteop",
    "VaultConnectionSettingsAllGlobals": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsAllGlobalsTypedDict": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsAllRequest": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsAllRequestTypedDict": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsAllResponse": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsAllResponseTypedDict": ".vault_connectionsettingsallop",
    "VaultConnectionSettingsUpdateGlobals": ".vault_connectionsettingsupdateop",
    "VaultConnectionSettingsUpdateGlobalsTypedDict": ".vault_connectionsettingsupdateop",
    "VaultConnectionSettingsUpdateRequest": ".vault_connectionsettingsupdateop",
    "VaultConnectionSettingsUpdateRequestTypedDict": ".vault_connectionsettingsupdateop",
    "VaultConnectionSettingsUpdateResponse": ".vault_connectionsettingsupdateop",
    "VaultConnectionSettingsUpdateResponseTypedDict": ".vault_connectionsettingsupdateop",
    "VaultConnectionsImportGlobals": ".vault_connectionsimportop",
    "VaultConnectionsImportGlobalsTypedDict": ".vault_connectionsimportop",
    "VaultConnectionsImportRequest": ".vault_connectionsimportop",
    "VaultConnectionsImportRequestTypedDict": ".vault_connectionsimportop",
    "VaultConnectionsImportResponse": ".vault_connectionsimportop",
    "VaultConnectionsImportResponseTypedDict": ".vault_connectionsimportop",
    "VaultConnectionsOneGlobals": ".vault_connectionsoneop",
    "VaultConnectionsOneGlobalsTypedDict": ".vault_connectionsoneop",
    "VaultConnectionsOneRequest": ".vault_connectionsoneop",
    "VaultConnectionsOneRequestTypedDict": ".vault_connectionsoneop",
    "VaultConnectionsOneResponse": ".vault_connectionsoneop",
    "VaultConnectionsOneResponseTypedDict": ".vault_connectionsoneop",
    "VaultConnectionsTokenGlobals": ".vault_connectionstokenop",
    "VaultConnectionsTokenGlobalsTypedDict": ".vault_connectionstokenop",
    "VaultConnectionsTokenRequest": ".vault_connectionstokenop",
    "VaultConnectionsTokenRequestBody": ".vault_connectionstokenop",
    "VaultConnectionsTokenRequestBodyTypedDict": ".vault_connectionstokenop",
    "VaultConnectionsTokenRequestTypedDict": ".vault_connectionstokenop",
    "VaultConnectionsTokenResponse": ".vault_connectionstokenop",
    "VaultConnectionsTokenResponseTypedDict": ".vault_connectionstokenop",
    "VaultConnectionsUpdateGlobals": ".vault_connectionsupdateop",
    "VaultConnectionsUpdateGlobalsTypedDict": ".vault_connectionsupdateop",
    "VaultConnectionsUpdateRequest": ".vault_connectionsupdateop",
    "VaultConnectionsUpdateRequestTypedDict": ".vault_connectionsupdateop",
    "VaultConnectionsUpdateResponse": ".vault_connectionsupdateop",
    "VaultConnectionsUpdateResponseTypedDict": ".vault_connectionsupdateop",
    "VaultConsumerRequestCountsAllGlobals": ".vault_consumerrequestcountsallop",
    "VaultConsumerRequestCountsAllGlobalsTypedDict": ".vault_consumerrequestcountsallop",
    "VaultConsumerRequestCountsAllRequest": ".vault_consumerrequestcountsallop",
    "VaultConsumerRequestCountsAllRequestTypedDict": ".vault_consumerrequestcountsallop",
    "VaultConsumerRequestCountsAllResponse": ".vault_consumerrequestcountsallop",
    "VaultConsumerRequestCountsAllResponseTypedDict": ".vault_consumerrequestcountsallop",
    "VaultConsumersAddGlobals": ".vault_consumersaddop",
    "VaultConsumersAddGlobalsTypedDict": ".vault_consumersaddop",
    "VaultConsumersAddRequest": ".vault_consumersaddop",
    "VaultConsumersAddRequestTypedDict": ".vault_consumersaddop",
    "VaultConsumersAddResponse": ".vault_consumersaddop",
    "VaultConsumersAddResponseTypedDict": ".vault_consumersaddop",
    "VaultConsumersAllGlobals": ".vault_consumersallop",
    "VaultConsumersAllGlobalsTypedDict": ".vault_consumersallop",
    "VaultConsumersAllRequest": ".vault_consumersallop",
    "VaultConsumersAllRequestTypedDict": ".vault_consumersallop",
    "VaultConsumersAllResponse": ".vault_consumersallop",
    "VaultConsumersAllResponseTypedDict": ".vault_consumersallop",
    "VaultConsumersDeleteGlobals": ".vault_consumersdeleteop",
    "VaultConsumersDeleteGlobalsTypedDict": ".vault_consumersdeleteop",
    "VaultConsumersDeleteRequest": ".vault_consumersdeleteop",
    "VaultConsumersDeleteRequestTypedDict": ".vault_consumersdeleteop",
    "VaultConsumersDeleteResponse": ".vault_consumersdeleteop",
    "VaultConsumersDeleteResponseTypedDict": ".vault_consumersdeleteop",
    "VaultConsumersOneGlobals": ".vault_consumersoneop",
    "VaultConsumersOneGlobalsTypedDict": ".vault_consumersoneop",
    "VaultConsumersOneRequest": ".vault_consumersoneop",
    "VaultConsumersOneRequestTypedDict": ".vault_consumersoneop",
    "VaultConsumersOneResponse": ".vault_consumersoneop",
    "VaultConsumersOneResponseTypedDict": ".vault_consumersoneop",
    "VaultConsumersUpdateGlobals": ".vault_consumersupdateop",
    "VaultConsumersUpdateGlobalsTypedDict": ".vault_consumersupdateop",
    "VaultConsumersUpdateRequest": ".vault_consumersupdateop",
    "VaultConsumersUpdateRequestTypedDict": ".vault_consumersupdateop",
    "VaultConsumersUpdateResponse": ".vault_consumersupdateop",
    "VaultConsumersUpdateResponseTypedDict": ".vault_consumersupdateop",
    "VaultCreateCallbackStateGlobals": ".vault_createcallbackstateop",
    "VaultCreateCallbackStateGlobalsTypedDict": ".vault_createcallbackstateop",
    "VaultCreateCallbackStateRequest": ".vault_createcallbackstateop",
    "VaultCreateCallbackStateRequestTypedDict": ".vault_createcallbackstateop",
    "VaultCreateCallbackStateResponse": ".vault_createcallbackstateop",
    "VaultCreateCallbackStateResponseTypedDict": ".vault_createcallbackstateop",
    "VaultCustomFieldsAllGlobals": ".vault_customfieldsallop",
    "VaultCustomFieldsAllGlobalsTypedDict": ".vault_customfieldsallop",
    "VaultCustomFieldsAllRequest": ".vault_customfieldsallop",
    "VaultCustomFieldsAllRequestTypedDict": ".vault_customfieldsallop",
    "VaultCustomFieldsAllResponse": ".vault_customfieldsallop",
    "VaultCustomFieldsAllResponseTypedDict": ".vault_customfieldsallop",
    "VaultCustomMappingsAllGlobals": ".vault_custommappingsallop",
    "VaultCustomMappingsAllGlobalsTypedDict": ".vault_custommappingsallop",
    "VaultCustomMappingsAllRequest": ".vault_custommappingsallop",
    "VaultCustomMappingsAllRequestTypedDict": ".vault_custommappingsallop",
    "VaultCustomMappingsAllResponse": ".vault_custommappingsallop",
    "VaultCustomMappingsAllResponseTypedDict": ".vault_custommappingsallop",
    "VaultLogsAllGlobals": ".vault_logsallop",
    "VaultLogsAllGlobalsTypedDict": ".vault_logsallop",
    "VaultLogsAllRequest": ".vault_logsallop",
    "VaultLogsAllRequestTypedDict": ".vault_logsallop",
    "VaultLogsAllResponse": ".vault_logsallop",
    "VaultLogsAllResponseTypedDict": ".vault_logsallop",
    "VaultSessionsCreateGlobals": ".vault_sessionscreateop",
    "VaultSessionsCreateGlobalsTypedDict": ".vault_sessionscreateop",
    "VaultSessionsCreateRequest": ".vault_sessionscreateop",
    "VaultSessionsCreateRequestTypedDict": ".vault_sessionscreateop",
    "VaultSessionsCreateResponse": ".vault_sessionscreateop",
    "VaultSessionsCreateResponseTypedDict": ".vault_sessionscreateop",
    "VaultValidateConnectionStateGlobals": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateGlobalsTypedDict": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateRequest": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateRequestBody": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateRequestBodyTypedDict": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateRequestTypedDict": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateResponse": ".vault_validateconnectionstateop",
    "VaultValidateConnectionStateResponseTypedDict": ".vault_validateconnectionstateop",
    "DisabledReason": ".webhook",
    "Webhook": ".webhook",
    "WebhookTypedDict": ".webhook",
    "WebhookEventLogsAllGlobals": ".webhook_eventlogsallop",
    "WebhookEventLogsAllGlobalsTypedDict": ".webhook_eventlogsallop",
    "WebhookEventLogsAllRequest": ".webhook_eventlogsallop",
    "WebhookEventLogsAllRequestTypedDict": ".webhook_eventlogsallop",
    "WebhookEventLogsAllResponse": ".webhook_eventlogsallop",
    "WebhookEventLogsAllResponseTypedDict": ".webhook_eventlogsallop",
    "WebhookWebhooksAddGlobals": ".webhook_webhooksaddop",
    "WebhookWebhooksAddGlobalsTypedDict": ".webhook_webhooksaddop",
    "WebhookWebhooksAddRequest": ".webhook_webhooksaddop",
    "WebhookWebhooksAddRequestTypedDict": ".webhook_webhooksaddop",
    "WebhookWebhooksAddResponse": ".webhook_webhooksaddop",
    "WebhookWebhooksAddResponseTypedDict": ".webhook_webhooksaddop",
    "WebhookWebhooksAllGlobals": ".webhook_webhooksallop",
    "WebhookWebhooksAllGlobalsTypedDict": ".webhook_webhooksallop",
    "WebhookWebhooksAllRequest": ".webhook_webhooksallop",
    "WebhookWebhooksAllRequestTypedDict": ".webhook_webhooksallop",
    "WebhookWebhooksAllResponse": ".webhook_webhooksallop",
    "WebhookWebhooksAllResponseTypedDict": ".webhook_webhooksallop",
    "WebhookWebhooksDeleteGlobals": ".webhook_webhooksdeleteop",
    "WebhookWebhooksDeleteGlobalsTypedDict": ".webhook_webhooksdeleteop",
    "WebhookWebhooksDeleteRequest": ".webhook_webhooksdeleteop",
    "WebhookWebhooksDeleteRequestTypedDict": ".webhook_webhooksdeleteop",
    "WebhookWebhooksDeleteResponse": ".webhook_webhooksdeleteop",
    "WebhookWebhooksDeleteResponseTypedDict": ".webhook_webhooksdeleteop",
    "WebhookWebhooksOneGlobals": ".webhook_webhooksoneop",
    "WebhookWebhooksOneGlobalsTypedDict": ".webhook_webhooksoneop",
    "WebhookWebhooksOneRequest": ".webhook_webhooksoneop",
    "WebhookWebhooksOneRequestTypedDict": ".webhook_webhooksoneop",
    "WebhookWebhooksOneResponse": ".webhook_webhooksoneop",
    "WebhookWebhooksOneResponseTypedDict": ".webhook_webhooksoneop",
    "WebhookWebhooksUpdateGlobals": ".webhook_webhooksupdateop",
    "WebhookWebhooksUpdateGlobalsTypedDict": ".webhook_webhooksupdateop",
    "WebhookWebhooksUpdateRequest": ".webhook_webhooksupdateop",
    "WebhookWebhooksUpdateRequestTypedDict": ".webhook_webhooksupdateop",
    "WebhookWebhooksUpdateResponse": ".webhook_webhooksupdateop",
    "WebhookWebhooksUpdateResponseTypedDict": ".webhook_webhooksupdateop",
    "Attempts": ".webhookeventlog",
    "AttemptsTypedDict": ".webhookeventlog",
    "WebhookEventLog": ".webhookeventlog",
    "WebhookEventLogService": ".webhookeventlog",
    "WebhookEventLogServiceTypedDict": ".webhookeventlog",
    "WebhookEventLogTypedDict": ".webhookeventlog",
    "WebhookEventLogsFilter": ".webhookeventlogsfilter",
    "WebhookEventLogsFilterService": ".webhookeventlogsfilter",
    "WebhookEventLogsFilterServiceTypedDict": ".webhookeventlogsfilter",
    "WebhookEventLogsFilterTypedDict": ".webhookeventlogsfilter",
    "WebhookEventType": ".webhookeventtype",
    "WebhookSubscription": ".webhooksubscription",
    "WebhookSubscriptionTypedDict": ".webhooksubscription",
    "ManagedVia": ".webhooksupport",
    "Mode": ".webhooksupport",
    "RequestRate": ".webhooksupport",
    "RequestRateTypedDict": ".webhooksupport",
    "SubscriptionLevel": ".webhooksupport",
    "Unit": ".webhooksupport",
    "VirtualWebhooks": ".webhooksupport",
    "VirtualWebhooksTypedDict": ".webhooksupport",
    "WebhookSupport": ".webhooksupport",
    "WebhookSupportResources": ".webhooksupport",
    "WebhookSupportResourcesTypedDict": ".webhooksupport",
    "WebhookSupportTypedDict": ".webhooksupport",
    "Website": ".website",
    "WebsiteType": ".website",
    "WebsiteTypedDict": ".website",
}


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = import_module(module_name, __package__)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)
